<?php
/**
 * File Description
 *
 * @package file draft-notify.php is a part of Simple Podcast Press and contains proprietary code - simplepodcastpress.com
 * @subpackage Subpackage
 * @since x.x.x (if applicable)
 */

/**
 * Process Something
 *
 * Description: This function processes email.
 *
 * @since x.x.x (replace with the version when this function was introduced, if applicable)
 *
 * @param int $id The ID used to determine what needs to be processed.
 * @return bool|array Returns true on success, false on failure. If additional data needs to be returned, use an array.
 */
function spp_process( $id ) {

	global $wpdb;

	$prefix = $wpdb->prefix;

	$podcastid = get_post_meta( $id, '_audiourl', true );

	// emails anyone on or above this level, unless a specific email address is entered.
	$email_address = get_option( 'admin_email' );

	// Author's first and last name are found in the metadata table.
	$result = $wpdb->get_row( // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
		$wpdb->prepare(
			"SELECT post_status, post_title, post_modified, user_login, display_name, user_email, {$prefix}usermeta.meta_value as fname, meta2.meta_value as lname 
            FROM {$wpdb->posts}, {$wpdb->users}
            LEFT JOIN {$wpdb->usermeta} ON ({$wpdb->usermeta}.user_id = {$wpdb->users}.ID AND {$wpdb->usermeta}.meta_key = 'first_name' )
            LEFT JOIN {$wpdb->usermeta} as meta2 ON (meta2.user_id = {$wpdb->users}.ID AND meta2.meta_key = 'last_name' )
            WHERE {$wpdb->posts}.post_author = {$wpdb->users}.ID AND {$wpdb->posts}.ID = %d",
			$id
		)
	);

	if ( ( ( 'draft' === $result->post_status ) || ( 'pending' === $result->post_status ) ) && 0 === $result->post_parent && ! empty( $podcastid ) ) {

		$can_email = true;
		// only check if the setting says to ignore revisions.
		$revision_check = $wpdb->get_results( // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
			$wpdb->prepare(
				"
                SELECT * 
                FROM {$wpdb->posts}
                WHERE {$wpdb->posts}.post_parent = %d AND 
                      {$wpdb->posts}.post_name != %s",
				$id,
				$wpdb->esc_like( "{$id}-autosave%" ) . '%' // Escape and like sanitize the post_name pattern.
			)
		);

		if ( count( $revision_check ) > 0 ) {
			$can_email = false;
		}
		if ( isset( $_POST['post_type'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$can_email = false;
		}

		// also do the manual check for the autosave of the first draft.
		$already_emailed_list = get_option( 'draftnotifyemailedlist' );

		$emailed_ex = explode( '|', $already_emailed_list );

		if ( in_array( $id, $emailed_ex, true ) ) {
			$can_email = false;
		}

		if ( $can_email ) {
			$message                              = "Great News!<br /><br />Simple Podcast Press has created a new DRAFT podcast blog post from your latest podcast episode.\n<br /><br />";
			$message                             .= "Your new podcast blog post is called <a href='" . home_url( '/' ) . 'wp-admin/post.php?post=' . $id . "&action=edit' target='_blank'>" . $result->post_title . "</a>\n<br /><br />";
			$message                             .= "Whenever you are ready, you can click here to <a href='" . home_url( '/' ) . 'wp-admin/post.php?post=' . $id . "&action=edit' target='_blank'>" . 'to edit and publish' . "</a> this podcast blog post and share it with the world.\n<br /><br />";
			$message                             .= "Keep up the great work...\n<br />";
			$message                             .= "Your Simple Podcast Press Team\n<br />";
			$saved_email_post_link_val            = get_option( 'draftnotifyemailpostlink' );
			$saved_email_author_name_val          = get_option( 'draftnotifyemailauthorname' );
			$saved_email_author_email_address_val = get_option( 'draftnotifyemailauthoremail' );
			if ( $saved_email_author_name_val ) {
				$message .= "Author's Real Name: {$result->fname} {$result->lname }\n<br>";
			}
			if ( $saved_email_author_email_address_val ) {
				$message .= "Author's Email Address: " . $result->user_email . "\n<br>";
			}
			if ( $saved_email_post_link_val ) {
				$message .= "\n<br>Link to Pending Posts: <a href='" . home_url( '/' ) . "wp-admin/edit.php?post_status=pending&post_type=post' target='_blank'>here</a>\n<br>";
			}
			$subject   = "[Simple Podcast Press] A New Draft Podcast Post Is Ready On '" . get_bloginfo( 'name' ) . "'";
			$recipient = $email_address;
			$headers   = 'MIME-Version: 1.0' . "\r\n";
			$headers  .= 'Content-type: text/html; charset=iso-8859-1' . "\r\n";
			wp_mail( $recipient, $subject, $message, $headers );
			if ( ! in_array( $id, $emailed_ex, true ) ) {
				update_option( 'draftnotifyemailedlist', "{$already_emailed_list}|{$id}" );
			}
		}
	}
}
$spp_email_on_draft = get_option( 'spp_email_on_draft' );
if ( 1 === $spp_email_on_draft ) {
	add_action( 'save_post', 'spp_process' );
}
