<?php
/**
 * Simple Podcast Enqueue Resources File.
 *
 * This file is a part of Simple Podcast Press and contains proprietary code.
 * Website: simplepodcastpress.com
 *
 * @package SimplePodcastPress
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Enqueue resources for the G7G configuration in the admin area.
 *
 * This function is responsible for enqueueing CSS and JavaScript resources
 * needed for the G7G configuration in the WordPress admin area.
 *
 * @return void
 */
function g7g_cfg_enqueue_resources_admin() {

	$screen = get_current_screen();

	if ( ! property_exists( $screen, 'id' ) ) {
		return;
	}

	if ( 'settings_page_g7g-cfg' === $screen->id ) {

		$js_deps = array( 'jquery', 'jquery-ui-core', 'jquery-ui-dialog' );

		wp_enqueue_style( 'wp-jquery-ui-dialog' );

		// Use a meaningful version number, for example, based on your theme or plugin version.
		$g7g_cfg_version = g7g_cfg_random_string();

		wp_enqueue_style( 'g7g-cfg-font-icons', G7G_CFG_URL . 'css/font-icons.css', array(), $g7g_cfg_version );

		wp_enqueue_style( 'g7g-cfg-settings', G7G_CFG_URL . 'css/settings.css', array(), $g7g_cfg_version );

		wp_enqueue_script( 'g7g-cfg-settings', G7G_CFG_URL . 'js/settings.js', $js_deps, $g7g_cfg_version, true );

	}
}

/**
 * Load assets for the G7G configuration in the editor.
 *
 * This function is responsible for enqueuing CSS and JavaScript assets
 * specifically intended for the G7G configuration in the WordPress editor.
 * It ensures that the necessary styles and scripts are available when
 * working with G7G configuration in the editor.
 *
 * @return void
 */
function g7g_cfg_load_assets_editor() {

	$random = g7g_cfg_random_string();

	wp_enqueue_style( 'g7g-cfg-load-assets', G7G_CFG_URL . 'blocks/blocks.css', array(), $random );
}

/**
 * Print scripts for the G7G configuration in the WordPress admin.
 *
 * This function is responsible for printing JavaScript scripts
 * specifically intended for the G7G configuration in the WordPress admin.
 * It is typically hooked into the admin_print_scripts action to ensure
 * the necessary scripts are available on the admin pages related to G7G configuration.
 *
 * @return void
 */
function g7g_cfg_admin_print_scripts() {
	?>
	
	<script type="text/javascript">
		var 
		alert_reset_options_title   = '<?php esc_html_e( 'Confirm Reset', 'custom-fields-gutenberg' ); ?>',
		alert_reset_options_message = '<?php esc_html_e( 'Restore default options?', 'custom-fields-gutenberg' ); ?>',
		alert_reset_options_true    = '<?php esc_html_e( 'Yes, make it so.', 'custom-fields-gutenberg' ); ?>',
		alert_reset_options_false   = '<?php esc_html_e( 'No, abort mission.', 'custom-fields-gutenberg' ); ?>';
	</script>
	
	<?php
}