<?php // phpcs:ignore WordPress.Files.FileName.InvalidClassFileName
/**
 * Simple Podcast Custom Fields Display.
 *
 * This file is a part of Simple Podcast Press and contains proprietary code.
 * Website: simplepodcastpress.com
 *
 * @package SimplePodcastPress
 */

if ( ! defined( 'ABSPATH' ) ) {
	die();
}

if ( ! class_exists( 'G7G_CFG_CustomFields' ) ) {
	/**
	 * Class G7G_CFG_CustomFields
	 *
	 * Handles the configuration of custom fields in the WordPress admin.
	 *
	 * @since 1.0.0
	 */
	class G7G_CFG_CustomFields {

		/**
		 * Constructor method.
		 *
		 * Initializes the class and sets up necessary actions.
		 *
		 * @since 1.0.0
		 */
		public function __construct() {

			$this->constants();
			$this->includes();

			add_action( 'admin_print_scripts', 'g7g_cfg_admin_print_scripts' );

			add_action( 'enqueue_block_editor_assets', 'g7g_cfg_load_assets_editor' );
			add_action( 'add_meta_boxes', 'g7g_cfg_add_meta_box' );
			add_action( 'save_post', array( $this, 'spp_cfg_metabox_save' ) );
		}

		/**
		 * Define constants used by the class.
		 *
		 * @since 1.0.0
		 */
		public function constants() {

			if ( ! defined( 'G7G_CFG_URL' ) ) {
				define( 'G7G_CFG_URL', plugin_dir_url( __FILE__ ) );
			}
			if ( ! defined( 'G7G_CFG_DIR' ) ) {
				define( 'G7G_CFG_DIR', plugin_dir_path( __FILE__ ) );
			}
			if ( ! defined( 'G7G_CFG_FILE' ) ) {
				define( 'G7G_CFG_FILE', plugin_basename( __FILE__ ) );
			}
			if ( ! defined( 'G7G_CFG_SLUG' ) ) {
				define( 'G7G_CFG_SLUG', basename( __DIR__ ) );
			}
		}

		/**
		 * Include necessary files.
		 *
		 * @since 1.0.0
		 */
		public function includes() {

			if ( is_admin() ) {

				require_once G7G_CFG_DIR . 'inc/plugin-core.php';
				require_once G7G_CFG_DIR . 'inc/resources-enqueue.php';
			}
		}

		/**
		 * Save post meta callback.
		 *
		 * @param int   $post_id Post ID.
		 * @param bolem $cron true|false.
		 *
		 * @since 1.0.0
		 */
		public function spp_cfg_metabox_save( $post_id, $cron = false ) {
			if ( ! $cron ) {
				if ( ( empty( $_POST['meta[303153][value]'] ) ) || ( ! get_post_meta( $post_id, '_audiourl', true ) ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
					return;
				}

				$audio_url = isset( $_POST['meta'][303153]['value'] ) ? sanitize_text_field( wp_unslash( $_POST['meta'][303153]['value'] ) ) : ''; // phpcs:ignore WordPress.Security.NonceVerification.Missing
				update_post_meta( $post_id, '_audiourl', $audio_url );
			}
		}
	}

	$G7G_CFG_CustomFields = new G7G_CFG_CustomFields(); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

}
