<?php // phpcs:ignore Squiz.Commenting.FileComment.Missing
/**
 * Retrieves the file size of a remote file by fetching the headers.
 *
 * @param string $url The URL of the remote file.
 *
 * @return int|false The size of the remote file in bytes, or false on failure.
 */
function get_remote_filesize( $url ) {
	$head = array_change_key_case( get_headers( $url, 1 ) );

	// content-length of download (in bytes), read from Content-Length: field.
	$clen = isset( $head['content-length'] ) ? $head['content-length'] : 0;

	$value = max( $clen );
	$key   = array_search( $value, $clen, true );

	// cannot retrieve file size, return "-1".
	if ( ! $clen ) {
		return -1;
	}

	return $clen[ $key ]; // return size in bytes.
}


// Perform nonce verification here.

// Check if the 'file' index exists.
if ( isset( $_GET['file'] ) && ! empty( $_GET['file'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
	// Unslash and sanitize the 'file' value.
	$full_path = wp_unslash( sanitize_text_field( $_GET['file'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended,WordPress.Security.ValidatedSanitizedInput.MissingUnslash
	// Now you can safely use $full_path.
} else {
	// Handle the case where 'file' is not set or empty.
	die( 'File type is not correct' );
}

if ( $full_path ) {

	$fsize      = get_remote_filesize( $full_path );
	$path_parts = pathinfo( $full_path );
	$filepath   = $path_parts['basename'];
		header( 'Content-Description: File Transfer' );
		header( 'Content-Type: application/octet-stream' );
		header( 'Content-Disposition: attachment; filename="' . $filepath . '"' );
		header( 'Expires: 0' );
		header( 'Cache-Control: must-revalidate' );
		header( 'Pragma: public' );
	if ( $fsize ) {// checking if file size exist.
		header( "Content-length: $fsize" );
	}
		flush(); // Flush system output buffer.
	readfile( $full_path ); // phpcs:ignore WordPress.WP.AlternativeFunctions.file_system_operations_readfile
	exit;
}
