<?php // phpcs:ignore WordPress.Files.FileName.InvalidClassFileName
/**
 * Responsive-audio-player.php
 *
 * This file is a part of Simple Podcast Press and contains proprietary code.
 * Website: simplepodcastpress.com
 *
 * @package SimplePodcastPress
 */

add_action(
	'plugins_loaded',
	array( SPPress_Audio_Player::get_instance(), 'plugin_setup' )
);
/**
 * Class SPPress_Audio_Player
 *
 * This class handles the functionality of an audio player for the Simple Podcast Press plugin.
 *
 * @package SimplePodcastPress
 */
class SPPress_Audio_Player {

	/**
	 * Plugin instance.
	 *
	 * @see SPPress_Audio_Player::get_instance()
	 * @var object|null
	 */
	protected static $instance = null;

	/**
	 * URL to this plugin's directory.
	 *
	 * @var string
	 */
	public $plugin_url = '';

	/**
	 * Path to this plugin's directory.
	 *
	 * @var string
	 */
	public $plugin_path = '';

	/**
	 * Access this plugin’s working instance
	 *
	 * @wp-hook plugins_loaded
	 * @since   2012.09.13
	 * @return  object of this class
	 */
	public static function get_instance() {
		null === self::$instance && self::$instance = new self();
		return self::$instance;
	}

	/**
	 * Used for regular plugin work.
	 *
	 * @wp-hook plugins_loaded
	 * @since   2012.09.10
	 * @return  void
	 */
	public function plugin_setup() {

		$this->plugin_url  = plugins_url( '/', __FILE__ );
		$this->plugin_path = plugin_dir_path( __FILE__ );
		$this->load_language( 'b5f-rap' );

		$is_license_valid = get_option( 'sppress_ls' );

		if ( ( 'valid' !== $is_license_valid ) && ( 'trial' !== $is_license_valid ) ) {
			return;
		} else {
			// Only If license is valid, then load the shortcodes and css/jquery.
			add_action( 'wp_default_scripts', array( $this, 'enqueue_jquery' ) );

			add_shortcode( 'spp-audioonly', array( $this, 'spp_audioplayeronly_fn' ) );
			if ( ! is_admin() ) {
				add_shortcode( 'spp-playlist', array( $this, 'spp_audioplayerwithplaylist_fn' ) );
			}
			add_shortcode( 'spp-ctabuttons', array( $this, 'spp_ctabuttons_shortcode_fn' ) );
			add_shortcode( 'spp-optin', array( $this, 'spp_optin_shortcode_fn' ) );
			add_shortcode( 'spp-audio', array( $this, 'spp_fullplayer_fn' ) );
			add_shortcode( 'spp-player', array( $this, 'spp_fullplayer_fn' ) );
			add_shortcode( 'spp-poweredby', array( $this, 'spp_poweredby_fn' ) );
			add_shortcode( 'spp-timestamp', array( $this, 'spp_timestamp_fn' ) );
			add_shortcode( 'spp-episodes', array( $this, 'spp_episodes_fn' ) );
			add_shortcode( 'spp-featured-image', array( $this, 'spp_featured_image_fn' ) );
			add_shortcode( 'spp-transcript', array( $this, 'spp_transcript_fn' ) );

		}
	}

	/**
	 * Constructor. Intentionally left empty and public.
	 *
	 * @see plugin_setup()
	 * @since 2012.09.12
	 */
	public function __construct() {}

	/**
	 * Display an audio player with a playlist based on the provided attributes.
	 *
	 * This function generates and outputs the HTML for an audio player with a playlist using the provided attributes.
	 *
	 * @param array|null $atts Optional. Attributes for configuring the audio player. Default is null.
	 */
	public function spp_audioplayerwithplaylist_fn( $atts = null ) {

		if ( is_feed() ) {
			return;
		}

		$html           = '';
		$ctabuttonspart = '';
		$optinpart      = '';
		$poweredby      = '';
		global $post;
		global $wpdb;
		$max = null;

		$width = isset( $atts['width'] ) ? " style='max-width:{$atts['width']}; float:right; margin-bottom:10px;'" : '';

		$select_audio_player_skin = Wp_Simplepodcastpress::spp_get_option( 'select_audio_player_skin' );
		if ( 'soundmanager2_player' === $select_audio_player_skin ) {

				$order = '';

			if ( isset( $atts['max'] ) ) {
				$max = $atts['max'];
			}

			if ( isset( $atts['order'] ) ) {
				$order = $atts['order'];
			}

			if ( isset( $atts['title'] ) ) {
				$title = $atts['title'];
			} else {
				$title = get_the_title( $post->ID );
			}

			if ( isset( $atts['ep_date'] ) ) {
				$ep_date = $atts['ep_date'];
			} else {
				$ep_date = 'off';
			}

			if ( isset( $atts['ep_duration'] ) ) {
				$ep_duration = $atts['ep_duration'];
			} else {
				$ep_duration = 'off';
			}

			if ( isset( $atts['list'] ) && 'off' === $atts['list'] ) {
				$playlistopen = '';
			} else {
				$playlistopen = 'playlist-open';
			}

			if ( isset( $atts['image'] ) ) {
				$player_image = $atts['image'];
			}

			$audio_player_image = Wp_Simplepodcastpress::spp_get_option( 'audio_player_image' );

			switch ( $audio_player_image ) {
				case 'itunes':
						$player_image = Wp_Simplepodcastpress::spp_get_option( 'channel_image' );
					break;

				case 'featured':
					if ( has_post_thumbnail( $post->ID ) ) {
						$player_image = wp_get_attachment_url( get_post_thumbnail_id( $post->ID ) );
					} else {
						$player_image = '';
					}
					break;

				case 'disabled':
					$player_image = '';
					break;
			}
				$player_image          = $this->spp_get_image_thumb( $player_image );
				$spp_rewind_button     = Wp_Simplepodcastpress::spp_get_option( 'spp_rewind_button' );
				$rewind_button_display = ( 1 === $spp_rewind_button ) ? 'display:none;' : '';

				$spp_speed_button = Wp_Simplepodcastpress::spp_get_option( 'spp_speed_button' );
				$spp_speed_button = ( 1 === $spp_speed_button ) ? 'display:none;' : '';

				$spp_volume_button = Wp_Simplepodcastpress::spp_get_option( 'spp_volume_button' );
				$spp_volume_button = ( 0 == $spp_volume_button ) ? 'display:none;' : '';

				$table_spp_podcast = $wpdb->prefix . 'spp_podcast';
			if ( $wpdb->get_var( "SHOW TABLES LIKE '$table_spp_podcast'" ) != $table_spp_podcast ) {
				return;
			}

			$podcast_items = $wpdb->get_results( 'SELECT * FROM ' . $table_spp_podcast . " ORDER BY STR_TO_DATE(pc_published_date, 'D, d M Y H:i:s O') DESC" );

			$count = 0;

			foreach ( $podcast_items as $row ) {
				if ( ( $max ) && ( $count == $max ) ) {
					break;
				}

				$result_array[] = (array) $row;
				++$count;
			}

			// print_r($result_array);
			if ( ! function_exists( 'sort_function' ) ) {
				function sort_function( $a, $b ) { // phpcs:ignore Squiz.Commenting.FunctionComment.Missing
					return strtotime( $a['pc_published_date'] ) - strtotime( $b['pc_published_date'] );
				}
			}
			
			if (is_array($result_array)) {
				usort($result_array, 'sort_function');
			}

			if ( 'reverse' !== $order ) {
				$result_array = array_reverse( $result_array );
			}

			if ( isset( $atts['cat'] ) ) {

				$new_result_array = array();

				$catslug = $atts['cat'];
				$catObj  = get_category_by_slug( $catslug );// phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$catid   = $catObj->term_id; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

				$args     = array(
					'post_status'    => 'publish',
					'posts_per_page' => -1,
					'category'       => $catid,
				);
				$postlist = get_posts( $args );

				foreach ( $postlist as $post_item ) {
					$title    = $post_item->post_title;
					$audiourl = get_post_meta( $post_item->ID, '_audiourl', true );
					if ( empty( $audiourl ) ) {
						$audiourl = get_post_meta( $post_item->ID, '_useraudiourl', true );
					}
					if ( empty( $audiourl ) ) {
						continue;
					}
					$temp_array = array(
						'pc_title'      => $title,
						'pc_audio_file' => $audiourl,
					);
					array_push( $new_result_array, $temp_array );
				}
				$result_array = array();

				$result_array = $new_result_array;
			}
			if ( empty( $result_array ) ) {
				echo 'Playlist not found';
				return;
			}
				$this->sppress_enqueue_scripts_css();
				$this->sppress_enqueue_scripts_js();
				$this->two_step_optin_fn();

				$audiopart = self::player_html_fn( $post->ID, $result_array, $player_image, $spp_speed_button, $rewind_button_display, $spp_volume_button, false, true,  $playlistopen, '', $ep_date, $ep_duration );
			$js            = self::player_js_fn( $post->ID, false, true, true );

		} else {
			$this->sppress_enqueue_scripts_css();
			$this->sppress_enqueue_scripts_js();
			$this->two_step_optin_fn();

			$js = self::player_js_fn( $post->ID, false, false );
		}

		$spp_optin_box = Wp_Simplepodcastpress::spp_get_option( 'spp_optin_box' );

		if ( isset( $atts['optin'] ) ) {
			$optin = $atts['optin'];
			if ( 'off' !== $optin ) {
				$optinpart = $this->spp_optin_fn( $atts );
			}
		} elseif ( 1 === $spp_optin_box ) {
				$optinpart = $this->spp_optin_fn( $atts );
		}

		if ( isset( $atts['ctabuttons'] ) ) {
			$ctabuttons = $atts['ctabuttons'];
			if ( 'off' !== $ctabuttons ) {
				$ctabuttonspart = $this->spp_ctabuttons_fn( $atts );
			}
		} else {
			$ctabuttonspart = $this->spp_ctabuttons_fn( $atts );
		}

		if ( isset( $atts['poweredby'] ) ) {
			$poweredbylink = $atts['poweredby'];
			if ( 'off' !== $poweredbylink ) {
				$poweredby = $this->spp_poweredby_fn( $atts );
			}
		} else {
			$poweredby = $this->spp_poweredby_fn( $atts );
		}

		$container_width     = Wp_Simplepodcastpress::spp_get_option( 'container_width' );
		$disable_button_home = Wp_Simplepodcastpress::spp_get_option( 'hide_elements_athome' );

		if ( $container_width ) {
			$container_width = 'style="max-width:' . $container_width . 'px;"';
		}

		if ( $disable_button_home ) {
			if ( is_single() && is_main_query() ) {
				$html = '<div class="player_container player-' . $post->ID . '""' . $container_width . ' >' . $audiopart . $ctabuttonspart . $optinpart . $poweredby . '</div>';
			} else {
				$html = '<div class="player_container player-' . $post->ID . '"' . $container_width . ' >' . $audiopart . '</div>';
			}
		} else {
			$html = '<div class="player_container player-' . $post->ID . '"' . $container_width . ' >' . $audiopart . $ctabuttonspart . $optinpart . $poweredby . '</div>';
		}

		$this->sppress_enqueue_scripts_css();
		$this->sppress_enqueue_scripts_js();
		$this->two_step_optin_fn();
		return $html;
	}

	/**
	 * Get the thumbnail URL for the given image URL.
	 *
	 * This function takes an image URL, retrieves the corresponding attachment ID,
	 * and then retrieves the thumbnail size of the image using WordPress functions.
	 *
	 * @param string $image_url The URL of the image.
	 * @return string|false The URL of the thumbnail or false if no thumbnail is found.
	 */
	public function spp_get_image_thumb( $image_url ) {
			$image_id    = attachment_url_to_postid( $image_url );
			$image_thumb = wp_get_attachment_image_src( $image_id, 'thumbnail' );

		if ( empty( $image_thumb[0] ) ) {
			return $image_url;
		} else {
			return $image_thumb[0];
		}
	}

	/**
	 * Display the audio player without a playlist based on the provided attributes.
	 *
	 * This function generates and displays the audio player for a podcast episode
	 * without including a playlist. It utilizes the provided attributes to customize
	 * the appearance and behavior of the audio player.
	 *
	 * @param array $atts Optional. Attributes for customizing the audio player.
	 * @return string The HTML markup for the audio player.
	 */
	public function spp_audioplayeronly_fn( $atts = null ) {
		global $post;
		if ( ! isset( $post->ID ) && ! isset( $atts['url'] ) ) {
			return;
		}

		$css_version  = rand( 10, 99 ) / 10;
		$html         = '';
		$duration     = '';
		$videoPodcast = false;
		$video_file   = '';
		$audio_file   = '';
		if ( isset( $_GET['postid'] ) ) {
			$postid = $_GET['postid'];
			$post   = get_post( $postid, OBJECT );
			setup_postdata( $post );
		}
		$width = isset( $atts['width'] ) ? " style='max-width:{$atts['width']}; float:right; margin-bottom:10px;'" : '';

		if ( ! isset( $atts['widget'] ) ) {

			if ( isset( $atts['url'] ) ) {
				$audio_file = $atts['url'];

					$spp_line                     = '|';
					$filtered_audio_file_location = strpos( $audio_file, $spp_line );

				if ( $filtered_audio_file_location ) {
					$audio_file = substr( $audio_file, 0, $filtered_audio_file_location );
				}

				update_post_meta( $post->ID, '_useraudiourl', $audio_file );
			} else {
				global $post;
				if ( ! is_object( $post ) ) {
					return;
				}
				$audio_file = get_post_meta( $post->ID, '_audiourl', true );
				$duration   = get_post_meta( $post->ID, '_audioduration', true );
				update_post_meta( $post->ID, '_useraudiourl', '' );
			}
		} else {

			if ( isset( $atts['url'] ) ) {
				$audio_file = $atts['url'];
			}
				// $duration = get_post_meta( $post->ID, '_audioduration', true );
		}
		// If not SPP posts or atts then check PowerPress and PodPress.
		if ( empty( $audio_file ) ) {
			global $post;
			if ( ! is_object( $post ) ) {
					return;
			}
			$PPGeneral = get_option( 'powerpress_general' );

			$MetaData = get_post_meta( $post->ID, 'enclosure', true );
			if ( $MetaData ) {
				$MetaParts  = explode( "\n", $MetaData, 4 );
				$audio_file = trim( $MetaParts[0] );
				if ( isset( $MetaParts[3] ) ) {
					$audioduration = unserialize( $MetaParts[3] );
				}
				if ( isset( $audioduration['duration'] ) ) {
					$duration = $audioduration['duration'];
				}
				unset( $MetaParts );

				$audio_url_parts = parse_url( $audio_file );

				if ( ! empty( $PPGeneral['redirect1'] ) ) {
					$audio_file = $PPGeneral['redirect1'] . $audio_url_parts['host'] . $audio_url_parts['path'];
				}
			} else {
				global $post;
				if ( ! is_object( $post ) ) {
					return;
				}
				$PodPressMetaData = get_post_meta( $post->ID, '_podPressMedia', true );
				if ( $PodPressMetaData ) {
					$audio_file = $PodPressMetaData[0]['URI'];
					unset( $PodPressMetaData );
				}
			}

			if ( ! empty( $PPGeneral['custom_feeds'] ) ) {
				$custom_feed = $PPGeneral['custom_feeds'];
				foreach ( $custom_feed as $key => $value ) {
					$video_channel_id = $key;
					if ( $video_channel_id !== 'podcast' ) {
						break;
					}
				}

				$PPCustomFeedMetaData = get_post_meta( $post->ID, '_' . $video_channel_id . ':enclosure', true );
				if ( $PPCustomFeedMetaData ) {
					$PPCustomFeedMetaData = explode( "\n", $PPCustomFeedMetaData, 4 );
					$video_file           = trim( $PPCustomFeedMetaData[0] );
					$ext                  = pathinfo( $video_file, PATHINFO_EXTENSION );
					if ( ( $ext = 'mp3' ) || ( $ext = 'm4a' ) || ( $ext = 'wav' ) ) {
						$videoPodcast = false;
						$audio_file   = $video_file;
						$video_file   = '';
					} else {
						$videoPodcast = true;
					}

					unset( $PPCustomFeedMetaData );

				}
			}

			unset( $PPGeneral );
		}

		// No audio in this post or page
		if ( empty( $audio_file ) && empty( $video_file ) ) {
			return false;
		}

		$select_audio_player_skin = wp_simplepodcastpress::spp_get_option( 'select_audio_player_skin' );

		// If the extension is not MP3
		$audio_array = explode( '.', $audio_file );
		$audio_ext   = end( $audio_array );

		$resolve = wp_simplepodcastpress::spp_get_option( 'spp_resolve_player_conflict' );
		// if (($audio_ext === 'mp4') and ($resolve))
		if ( ( $audio_ext !== 'mp3' ) and ( $resolve ) ) {
			return false;
		}

		if ( ( $videoPodcast !== true ) and ( $audio_ext === 'mp4' ) ) {
			$videoPodcast = true;
			$video_file   = $audio_file;
			$audio_file   = false;

		}

		// remove any query arguments in MP3 file ONLY for Classic player only
		if ( ( $select_audio_player_skin == 'original_player' ) && ( strpos( $audio_file, '?' ) ) ) {
			$audio_file = substr( $audio_file, 0, strpos( $audio_file, '?' ) );
		}

		$mp3  = '<source src="' . $audio_file . '" />';
		$text = __( "This text displays if the audio tag isn't supported.", 'b5f-rap' );

		$audio_player_image = wp_simplepodcastpress::spp_get_option( 'audio_player_image' );
		switch ( $audio_player_image ) {
			case 'itunes':
				$player_image = wp_simplepodcastpress::spp_get_option( 'channel_image' );
				break;

			case 'featured':
				if ( ! is_object( $post ) ) {
							return;
				}
				if ( has_post_thumbnail( $post->ID ) ) {
					$player_image = wp_get_attachment_url( get_post_thumbnail_id( $post->ID ) );
				} else {
					$player_image = ' ';
				}
				break;

			case 'disabled':
				$player_image = '';
				break;
		}

		$spp_rewind_button     = wp_simplepodcastpress::spp_get_option( 'spp_rewind_button' );
		$rewind_button_display = ( $spp_rewind_button == 1 ) ? 'display:none;' : '';

		$spp_speed_button = wp_simplepodcastpress::spp_get_option( 'spp_speed_button' );
		$spp_speed_button = ( $spp_speed_button == 1 ) ? 'display:none;' : '';

		$spp_volume_button = wp_simplepodcastpress::spp_get_option( 'spp_volume_button' );
		$spp_volume_button = ( $spp_volume_button == 0 ) ? 'display:none;' : '';

		$spp_volume_button = wp_simplepodcastpress::spp_get_option( 'spp_volume_button' );

		// The widget player for either Classic of Pro player.
		if ( isset( $atts['widget'] ) ) {
			$spp_preroll  = '';
			$postid       = is_object( $post ) ? $post->ID : '';
			$player_image = $atts['player_image'];
			$player_image = $this->spp_get_image_thumb( $player_image );

			if ( $select_audio_player_skin == 'soundmanager2_player' ) {
				if ( isset( $atts['title'] ) ) {
					$title = $atts['title'];
				} else {
					$title = get_the_title( $postid );
				}

					$duration = $atts['duration'];

					$force_ssl_for_audio = wp_simplepodcastpress::spp_get_option( 'force_ssl_for_audio' );
				if ( $force_ssl_for_audio ) {
					$audio_file = str_replace( 'http://', 'https://', $audio_file );
				}
					$podcast_items[] = array(
						'pc_audio_file' => $audio_file,
						'pc_title'      => $title,
					);
						$this->sppress_enqueue_scripts_css();
						$this->sppress_enqueue_scripts_js();
						$this->two_step_optin_fn();

						$html = self::player_html_fn( $postid, $podcast_items, $player_image, $spp_speed_button, $rewind_button_display, $spp_volume_button, true, false, $playlistopen = '', $this->seconds_to_time( $duration ) );
					$js       = self::player_js_fn( $postid, true, true );
			}

			// Default to the classic player
			else {
				$html = '<div>
			<audio class="sppaudioplayer_widget" controls preload="none"' . $spp_preroll . '>' . $mp3 . '
			</audio>
		</div>
        ';

			}
		} else {

			if ( ! empty( $duration ) ) {
				$durationinplayer = $this->seconds_to_time( $duration );
				$duration         = '(' . $this->seconds_to_time( $duration ) . ')';
			} else {
				$durationinplayer = '0:00';
			}

			$spp_pre_roll_checkbox = wp_simplepodcastpress::spp_get_option( 'spp_pre_roll_checkbox' );
			$spp_pre_roll_url      = wp_simplepodcastpress::spp_get_option( 'spp_pre_roll_url' );

			if ( $spp_pre_roll_checkbox ) {
				$spp_preroll = 'preroll=' . $spp_pre_roll_url;
			} else {
				$spp_preroll = '';
			}

			$spp_remove_timecode = wp_simplepodcastpress::spp_get_option( 'spp_remove_timecode' );
			$spp_listen_text     = wp_simplepodcastpress::spp_get_option( 'spp_listen_text' );
			if ( $spp_remove_timecode or ! $spp_listen_text ) {
				$duration = '';
			}

			if ( isset( $atts['textabove'] ) ) {
				$textabove = $atts['textabove'];
				if ( $textabove == 'off' ) {
					$listen_text = '';
				} else {
					$listen_text = $textabove;
				}
			} else {
				$listen_text = wp_simplepodcastpress::spp_get_option( 'spp_listen_text' );
			}

			$listen_text = '<div class="spp_player_textabove"><b>' . $listen_text . ' ' . $duration . ' </b></div>';

			if ( ! empty( $video_file ) ) {
				wp_enqueue_style( 'resp-vidplayer-css', SPPRESS_PLUGIN_URL . '/responsive_audio_player/videojs/video-js.css', false, $css_version, 'all' );
				wp_enqueue_script( 'resp-vidplayer-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/videojs/video.js', '', false, false );

				$videoPoster = wp_simplepodcastpress::spp_get_option( 'channel_image' );
				$videoPlayer = '<center><video id="sppvideo" class="video-js vjs-default-skin vjs-big-play-centered vjs-big-play-button"  width="640px" height="360px" controls preload="none" poster="' . $videoPoster . '" data-setup=\'{ "aspectRatio":"640:360", "playbackRates": [1, 1.5, 2] }\'><source src="' . $video_file . '" type="video/mp4" /></video></center>';
				$html       .= $videoPlayer;
			}

			if ( $audio_file ) {
				// Add a paragraph space if both video and audio exist
				if ( $videoPodcast ) {
					$html .= '<p>';
				}

				$html .= $listen_text;

				if ( $select_audio_player_skin == 'soundmanager2_player' ) {

					if ( isset( $atts['title'] ) ) {
							$title = $atts['title'];
					} else {
						$title = get_the_title( $post->ID );
					}

					if ( isset( $atts['image'] ) ) {
						$player_image = $atts['image'];
					}

					$player_image = $this->spp_get_image_thumb( $player_image );

					$force_ssl_for_audio = wp_simplepodcastpress::spp_get_option( 'force_ssl_for_audio' );

					if ( $force_ssl_for_audio ) {
							$audio_file = str_replace( 'http://', 'https://', $audio_file );
					}
					$podcast_items[] = array(
						'pc_audio_file' => $audio_file,
						'pc_title'      => $title,
					);
					$this->sppress_enqueue_scripts_css();
					$this->sppress_enqueue_scripts_js();
					if ( ! is_admin() ) {
						add_action( 'wp_enqueue_scripts', array( $this, 'sppress_enqueue_scripts_css' ), 99 );
						add_action( 'wp_enqueue_scripts', array( $this, 'sppress_enqueue_scripts_js' ), 99 );
						add_action( 'wp_footer', array( &$this, 'two_step_optin_fn' ) );
					}
					$player = self::player_html_fn( $post->ID, $podcast_items, $player_image, $spp_speed_button, $rewind_button_display, $spp_volume_button, false, false, $playlistopen = '', $durationinplayer );

				} else {
					$player = '
				<div>
				<audio id="spp-audio-player" class="sppaudioplayer" controls preload="none"' . $spp_preroll . '>' . $mp3 . '
				</audio>
				</div>
				';
				}

				$select_audio_player = wp_simplepodcastpress::spp_get_option( 'select_audio_player' );
				$player_text_color   = wp_simplepodcastpress::spp_get_option( 'player_text_color' );
				if ( ! $player_text_color ) {
					$player_text_color = '#ffffff';
				}

				switch ( $select_audio_player ) {

					case 'simplepodcastpressblack':
						$color = '#fff';
						break;
					case 'simplepodcastpresswhite':
						$color = '#222';
						break;
					case 'simplepodcastpresscustomcolor':
						$color = $player_text_color;
						break;
				}

				if ( ! isset( $atts['widget'] ) ) {
					$player = apply_filters( 'spp_playeronly_html', $player, $audio_file );
				}

				$html .= $player;

			}

			unset( $audio_array );

		}

		$select_audio_player_skin = wp_simplepodcastpress::spp_get_option( 'select_audio_player_skin' );
		global $post;
		if ( ( isset( $post->ID ) ) && ( $select_audio_player_skin == 'soundmanager2_player' ) ) {

			$js = self::player_js_fn( $post->ID, false, true );

		} else {

			$js = self::player_js_fn( $post->ID, false, false );

		}

		// add_action( 'wp_footer', array( $this, 'sppress_enqueue_scripts_js' ),99);
		return $html;
	}


	/**
	 * Convert seconds to a formatted time string.
	 *
	 * This function takes a duration in seconds and converts it to a formatted time string
	 * in the format HH:MM:SS.
	 *
	 * @param int $seconds The duration in seconds to be converted.
	 * @return string Formatted time string in the format HH:MM:SS.
	 */
	public function seconds_to_time( $seconds ) {

			// If contains : then assume it is already in HH:MM and return same valude.
		if ( strpos( $seconds, ':' ) !== false ) {
			return $seconds;
		}

		if ( ! is_numeric( $seconds ) ) {
			return '0:00';
		}

			// extract hours.
			$hours = floor( $seconds / ( 60 * 60 ) );

			// extract minutes.
			$divisor_for_minutes = $seconds % ( 60 * 60 );
			$minutes             = floor( $divisor_for_minutes / 60 );

			// extract the remaining seconds.
			$divisor_for_seconds = $divisor_for_minutes % 60;
			$seconds             = ceil( $divisor_for_seconds );

		if ( $seconds < 10 ) {
			$seconds = '0' . $seconds;
		}

		if ( 0 !== $hours ) {
			// create string HH:MM:SS.
			$ret = $hours . ':' . $minutes . ':' . $seconds;
		} else {
			$ret = $minutes . ':' . $seconds;
		}

		return( $ret );
	}

	/**
	 * Generate content to be displayed before the podcast audio player based on the provided attributes.
	 *
	 * This static function generates content that is intended to be displayed before
	 * the podcast audio player. It utilizes the provided attributes to customize the
	 * appearance and behavior of the content.
	 *
	 * @param array $atts Optional. Attributes for customizing the content before the audio player.
	 * @return string The HTML markup for the content before the audio player.
	 */
	public static function spp_beforeplayer_fn( $atts = null ) {

		global $post;
		if ( ! isset( $post->ID ) ) {
				return;
		}
		if ( isset( $atts['textabove'] ) ) {
			$textabove = $atts['textabove'];
			if ( 'off' === $textabove ) {
				$listen_text = '';
			} else {
				$listen_text = $textabove;
			}
		} else {
				$listen_text = Wp_Simplepodcastpress::spp_get_option( 'spp_listen_text' );
		}

		$container_width = Wp_Simplepodcastpress::spp_get_option( 'container_width' );
		if ( $container_width ) {
			$container_width = 'style="max-width:' . $container_width . 'px;"';
		}

		$html  = '<div class="player_container player-' . $post->ID . '" ' . $container_width . ' >';
		$html .= '<div class="spp_player_textabove"><b>' . $listen_text . '</b></div>';

		return $html;
	}

	/**
	 * Generate content to be displayed after the podcast audio player based on the provided attributes.
	 *
	 * This static function generates content that is intended to be displayed after
	 * the podcast audio player. It utilizes the provided attributes to customize the
	 * appearance and behavior of the content.
	 *
	 * @param array $atts Optional. Attributes for customizing the content after the audio player.
	 * @return string The HTML markup for the content after the audio player.
	 */
	public static function spp_afterplayer_fn( $atts = null ) {
		$spp_buttons = self::spp_ctabuttons_fn( $atts );

		$html = $spp_buttons;

		$spp_optin_box = Wp_Simplepodcastpress::spp_get_option( 'spp_optin_box' );

		if ( $spp_optin_box ) {
			$html .= self::spp_optin_fn( $atts );
		}
		$spp_poweredby = self::spp_poweredby_fn( $atts );

		$html .= $spp_poweredby . '</div>';

		return $html;
	}
	/**
	 * Generate the shortcode for displaying call-to-action buttons based on the provided attributes.
	 *
	 * This static function generates the shortcode for displaying call-to-action buttons.
	 * It utilizes the provided attributes to customize the appearance and behavior of the buttons.
	 *
	 * @param array $atts Optional. Attributes for customizing the call-to-action buttons.
	 * @return string The shortcode for displaying call-to-action buttons.
	 */
	public static function spp_ctabuttons_shortcode_fn( $atts = null ) {
		if ( isset( $atts['noplayer'] ) ) {
			$atts['noplayer'] = true;
		}
		$html = self::spp_ctabuttons_fn( $atts );
		return $html;
	}
	/**
	 * Generate HTML markup for displaying call-to-action buttons based on the provided attributes.
	 *
	 * This static function generates HTML markup for displaying call-to-action buttons.
	 * It utilizes the provided attributes to customize the appearance and behavior of the buttons.
	 *
	 * @param array $atts Optional. Attributes for customizing the call-to-action buttons.
	 * @return string The HTML markup for displaying call-to-action buttons.
	 */
	public static function spp_ctabuttons_fn( $atts = null ) {
		global $post;
		if ( ! isset( $post->ID ) ) {
				return;
		}
		$html = '';

		$itunes_url = Wp_Simplepodcastpress::spp_get_option( 'btn_itunes_url' );
		if ( empty( $itunes_url ) ) {
			$itunes_url = Wp_Simplepodcastpress::spp_get_option( 'itunes_url' );
		}

		if ( strpos( $itunes_url, '?' ) ) {
			$itunes_url = substr( $itunes_url, 0, strpos( $itunes_url, '?' ) ) . '?ls=1';
		} else {
			$itunes_url = $itunes_url . '?ls=1';
		}

		$btn_g_play     = Wp_Simplepodcastpress::spp_get_option( 'btn_g_play' );
		$btn_share      = Wp_Simplepodcastpress::spp_get_option( 'btn_share' );
		$btn_embed      = Wp_Simplepodcastpress::spp_get_option( 'btn_embed' );
		$btn_download   = Wp_Simplepodcastpress::spp_get_option( 'btn_download' );
		$btn_spplisten  = Wp_Simplepodcastpress::spp_get_option( 'btn_spplisten' );
		$btn_spprss     = Wp_Simplepodcastpress::spp_get_option( 'btn_spprss' );
		$btn_sppreview  = Wp_Simplepodcastpress::spp_get_option( 'btn_sppreview' );
		$btn_sppandroid = Wp_Simplepodcastpress::spp_get_option( 'btn_sppandroid' );
		$btn_itunes     = Wp_Simplepodcastpress::spp_get_option( 'btn_itunes' );
		$btn_stiticher  = Wp_Simplepodcastpress::spp_get_option( 'btn_stiticher' );
		$btn_soundcloud = Wp_Simplepodcastpress::spp_get_option( 'btn_soundcloud' );
		$btn_spotify    = Wp_Simplepodcastpress::spp_get_option( 'btn_spotify' );

		$btn_stiticher_url  = Wp_Simplepodcastpress::spp_get_option( 'btn_stiticher_url' );
		$btn_soundcloud_url = Wp_Simplepodcastpress::spp_get_option( 'btn_soundcloud_url' );
		$btn_spotify_url    = Wp_Simplepodcastpress::spp_get_option( 'btn_spotify_url' );

		$btn_spp_custom1 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom1' );
		$btn_spp_custom2 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom2' );
		$btn_spp_custom3 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom3' );
		$btn_spp_custom4 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom4' );
		$btn_spp_custom5 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom5' );
		$btn_spp_custom6 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom6' );

		if ( ( 0 === $btn_g_play ) && ( 0 === $btn_share ) && ( 0 === $btn_download ) && ( 0 === $btn_itunes ) && ( 0 === $btn_stiticher ) && ( 0 === $btn_soundcloud ) && ( 0 === $btn_spotify ) && ( 0 === $btn_spplisten ) && ( 0 === $btn_spprss ) && ( 0 === $btn_sppreview ) && ( 0 === $btn_sppandroid ) && ( 0 === $btn_spp_custom1 ) && ( 0 === $btn_spp_custom2 ) && ( 0 === $btn_spp_custom3 ) && ( 0 === $btn_spp_custom4 ) && ( 0 === $btn_spp_custom5 ) && ( 0 === $btn_spp_custom6 ) ) {
			$allbtn_onoff = 'display:none !important;';
		} else {
			$allbtn_onoff = '';
		}

		$btn_g_play_name     = Wp_Simplepodcastpress::spp_get_option( 'btn_g_play_name' );
		$btn_share_name      = Wp_Simplepodcastpress::spp_get_option( 'btn_share_name' );
		$btn_download_name   = Wp_Simplepodcastpress::spp_get_option( 'btn_download_name' );
		$btn_itunes_name     = Wp_Simplepodcastpress::spp_get_option( 'btn_itunes_name' );
		$btn_stitcher_name   = Wp_Simplepodcastpress::spp_get_option( 'btn_stitcher_name' );
		$btn_soundcloud_name = Wp_Simplepodcastpress::spp_get_option( 'btn_soundcloud_name' );
		$btn_spotify_name    = Wp_Simplepodcastpress::spp_get_option( 'btn_spotify_name' );
		$btn_spplisten_name  = Wp_Simplepodcastpress::spp_get_option( 'btn_spplisten_name' );
		$btn_sppreview_name  = Wp_Simplepodcastpress::spp_get_option( 'btn_sppreview_name' );
		$btn_spprss_name     = Wp_Simplepodcastpress::spp_get_option( 'btn_spprss_name' );
		$btn_sppandroid_name = Wp_Simplepodcastpress::spp_get_option( 'btn_sppandroid_name' );
		$btn_embed_name      = Wp_Simplepodcastpress::spp_get_option( 'btn_embed_name' );

		$google_play_text = ( ! empty( $btn_g_play_name ) ? $btn_g_play_name : 'Google Podcasts' );
		$share_text       = ( ! empty( $btn_share_name ) ? $btn_share_name : 'Share' );
		$download_text    = ( ! empty( $btn_download_name ) ? $btn_download_name : 'Download' );
		$i_tunes_text     = ( ! empty( $btn_itunes_name ) ? $btn_itunes_name : 'Apple Podcasts' );
		$stitcher_text    = ( ! empty( $btn_stitcher_name ) ? $btn_stitcher_name : 'Stitcher' );
		$sound_cloud_text = ( ! empty( $btn_soundcloud_name ) ? $btn_soundcloud_name : 'SoundCloud' );
		$spotify_text     = ( ! empty( $btn_spotify_name ) ? $btn_spotify_name : 'Spotify' );

		$listen_text  = ( ! empty( $btn_spplisten_name ) ? $btn_spplisten_name : 'Listen in a New Window' );
		$review_text  = ( ! empty( $btn_sppreview_name ) ? $btn_sppreview_name : 'Leave a Review' );
		$r_s_s_text   = ( ! empty( $btn_spprss_name ) ? $btn_spprss_name : 'Subscribe via RSS' );
		$android_text = ( ! empty( $btn_sppandroid_name ) ? $btn_sppandroid_name : 'Subscribe on Android' );
		$embed_text   = ( ! empty( $btn_embed_name ) ? $btn_embed : 'Player Embed' );

		$btn_soundcloud    = ( 0 === $btn_soundcloud ) ? 'display:none !important;' : '';
		$btn_embed_display = ( 0 === $btn_embed ) ? 'display:none !important;' : '';
		$btn_g_play        = ( 0 === $btn_g_play ) ? 'display:none !important;' : '';
		$btn_stiticher     = ( 0 === $btn_stiticher ) ? 'display:none !important;' : '';
		$btn_share         = ( 0 === $btn_share ) ? 'display:none !important;' : '';
		$btn_download      = ( 0 === $btn_download ) ? 'display:none !important;' : '';
		$btn_itunes        = ( 0 === $btn_itunes ) ? 'display:none !important;' : '';
		$btn_spotify       = ( 0 === $btn_spotify ) ? 'display:none !important;' : '';
		$btn_spplisten     = ( 0 === $btn_spplisten ) ? 'display:none !important;' : '';
		$btn_sppreview     = ( 0 === $btn_sppreview ) ? 'display:none !important;' : '';
		$btn_spprss        = ( 0 === $btn_spprss ) ? 'display:none !important;' : '';
		$btn_sppandroid    = ( 0 === $btn_sppandroid ) ? 'display:none !important;' : '';

		$useraudio_file = get_post_meta( $post->ID, '_useraudiourl', true );
		if ( empty( $useraudio_file ) ) {
			$audio_file = get_post_meta( $post->ID, '_audiourl', true );
		} else {
			$audio_file = $useraudio_file;
		}

		if ( empty( $audio_file ) ) {

				$meta_data = get_post_meta( $post->ID, 'enclosure', true );
			if ( $meta_data ) {
					$meta_parts = explode( "\n", $meta_data, 4 );
					$audio_file = trim( $meta_parts[0] );
				if ( isset( $meta_parts[3] ) ) {
					$audioduration = unserialize( $meta_parts[3] ); // phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.serialize_unserialize
				}
				if ( isset( $audioduration['duration'] ) ) {
					$duration = $audioduration['duration'];
				}
					unset( $meta_parts );
			} else {
				$pod_press_meta_data = get_post_meta( $post->ID, '_podPressMedia', true );
				if ( $pod_press_meta_data ) {
						$audio_file = $pod_press_meta_data[0]['URI'];
						unset( $pod_press_meta_data );
				}
			}

				$audio_url_parts = wp_parse_url( $audio_file );

				$p_p_general = get_option( 'powerpress_general' );
			if ( ( ! empty( $p_p_general['redirect1'] ) ) && isset( $audio_url_parts['host'] ) ) {
				$audio_file = $p_p_general['redirect1'] . $audio_url_parts['host'] . $audio_url_parts['path'];
			}

				unset( $p_p_general );
		}

		$btn_spp_custom1_display = ( 0 === $btn_spp_custom1 ) ? 'display:none !important;' : '';
		$btn_spp_custom2_display = ( 0 === $btn_spp_custom2 ) ? 'display:none !important;' : '';
		$btn_spp_custom3_display = ( 0 === $btn_spp_custom3 ) ? 'display:none !important;' : '';
		$btn_spp_custom4_display = ( 0 === $btn_spp_custom4 ) ? 'display:none !important;' : '';
		$btn_spp_custom5_display = ( 0 === $btn_spp_custom5 ) ? 'display:none !important;' : '';
		$btn_spp_custom6_display = ( 0 === $btn_spp_custom6 ) ? 'display:none !important;' : '';
		// Get Custom Buttons Name.
		$btn_spp_custom_name1 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom_name1' );
		$btn_spp_custom_name2 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom_name2' );
		$btn_spp_custom_name3 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom_name3' );
		$btn_spp_custom_name4 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom_name4' );
		$btn_spp_custom_name5 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom_name5' );

		$btn_spp_custom_name_default = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom_name6' );
		$btn_spp_custom_name_post    = get_post_meta( $post->ID, '_btn_spp_custom_name6', true );
		$btn_spp_custom_name6        = ( empty( $btn_spp_custom_name_post ) ? $btn_spp_custom_name_default : $btn_spp_custom_name_post );

		if ( ! empty( $btn_spp_custom6_display ) ) {
			$btn_spp_custom6_display = ( empty( $btn_spp_custom_name_post ) ? 'display:none !important;' : '' );
		}
		$btn_spp_custom_url1 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom_url1' );
		$btn_spp_custom_url2 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom_url2' );
		$btn_spp_custom_url3 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom_url3' );
		$btn_spp_custom_url4 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom_url4' );
		$btn_spp_custom_url5 = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom_url5' );

		$btn_spp_custom_url_default = Wp_Simplepodcastpress::spp_get_option( 'btn_spp_custom_url6' );
		$btn_spp_custom_url_post    = get_post_meta( $post->ID, '_btn_spp_custom_url6', true );
		$btn_spp_custom_url6        = ( empty( $btn_spp_custom_url_post ) ? $btn_spp_custom_url_default : $btn_spp_custom_url_post );

		$spp_lead_box_btn_code   = '';
		$btn_sppleadbox_checkbox = Wp_Simplepodcastpress::spp_get_option( 'spp_LeadBox_btn_checkbox' );
		if ( $btn_sppleadbox_checkbox ) {
			$spp_lead_box_btn_code = Wp_Simplepodcastpress::spp_get_option( 'spp_LeadBox_btn_code' );
			$spp_lead_box_btn_code = htmlspecialchars_decode( $spp_lead_box_btn_code, ENT_QUOTES );
			$spp_lead_box_btn_code = str_replace( '<a', '<a class="spp-button-leadbox"', $spp_lead_box_btn_code );
		}

		if ( has_post_thumbnail() ) {
			$channel_image = wp_get_attachment_url( get_post_thumbnail_id( $post->ID ) );
		} else {
			$channel_image = Wp_Simplepodcastpress::spp_get_option( 'channel_image' );
		}

		$link      = get_permalink( $post->ID );
		$posttitle = htmlspecialchars( rawurlencode( html_entity_decode( get_the_title(), ENT_COMPAT, 'UTF-8' ) ), ENT_COMPAT, 'UTF-8' );

		$fullpostcontent = get_the_content();
		$postcontent     = $link . ' - ' . strip_shortcodes( $fullpostcontent );
		$postcontent     = str_replace( '&nbsp;', '', $postcontent );
		$postcontent     = preg_replace( "/[\r\n]+/", "\n", $postcontent );
		$postcontent     = preg_replace( '/\s+/', ' ', $postcontent );
		$postcontent     = html_entity_decode( wp_strip_all_tags( $postcontent ) );
		$postcontent     = substr( $postcontent, 0, 540 ) . ' ...';
		$postcontent     = rawurlencode( $postcontent );

		$direct_download_button = Wp_Simplepodcastpress::spp_get_option( 'direct_download_button' );
		if ( $direct_download_button ) {
			$audiodownloadurl    = SPPRESS_PLUGIN_URL . '/responsive_audio_player/downloadaudio.php?file=' . $audio_file;
			$spp_download_target = '';
		} else {
			$audiodownloadurl    = $audio_file;
			$spp_download_target = '_blank';
		}
		$itunes_id = Wp_Simplepodcastpress::spp_get_option( 'itunes_id' );
		if ( empty( $itunes_id ) ) {
			$btn_itunes        = 'display:none !important;';
			$btn_sppreview_url = '';
			$btn_sppreview     = 'display:none !important;';
		} else {
			$btn_sppreview_url = 'http://getpodcast.reviews/id/' . $itunes_id;
		}

		$rss_feed = Wp_Simplepodcastpress::spp_get_option( 'podcast_url' );
		// If user didn't specify podcast feed or iTunes URL at start.
		if ( 'none' === $rss_feed || empty( $rss_feed ) ) {
			$btn_sppandroid     = 'display:none !important;';
			$btn_spprss         = 'display:none !important;';
			$btn_sppandroid_url = '';
			$btn_spprss_url     = '';
			$btn_g_play         = 'display:none !important;';
			$btn_g_play_url     = '';
		} else {
			$btn_spprss_url     = $rss_feed;
			$android_rss_feed   = wp_parse_url( $rss_feed );
			$btn_sppandroid_url = 'http://subscribeonandroid.com/' . $android_rss_feed['host'] . $android_rss_feed['path'];
			$btn_g_play_url     = 'https://podcasts.google.com/?feed=' . base64_encode( $rss_feed ); // phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.obfuscation_base64_encode
		}

		if ( isset( $atts['popoutplayer'] ) ) {
			$spplisten = $atts['popoutplayer'];
		} else {
			$spplisten = $audio_file;
		}

		$spp_optin_hide_chk = Wp_Simplepodcastpress::spp_get_option( 'spp_optin_hide_chk' );
		if ( isset( $atts['noplayer'] ) ) {
			$noplayer = $atts['noplayer'];
		} else {
			$noplayer = false;
		}

		$style = '';
		if ( $spp_optin_hide_chk && ! $noplayer ) {
			$allbtn_onoff = 'display:none !important;';
		}

		$share_link  = get_permalink( $post->ID );
		$share_title = '';
		if ( extension_loaded( 'mbstring' ) ) {
			$share_link = rawurlencode( wp_strip_all_tags( mb_convert_encoding( get_the_title( $post->ID ), 'UTF-8', 'HTML-ENTITIES' ) ) );
		} else {
			$share_title = rawurlencode( wp_strip_all_tags( get_the_title( $post->ID ) ) );
		}
		$spp_post_id                      = get_the_ID( $post->ID );
		$buttons_array['btn_itunes']      = '<a class="button-itunes" target="_blank" style="' . $btn_itunes . '" href="' . $itunes_url . '">' . $i_tunes_text . '</a>';
		$buttons_array['btn_stiticher']   = '<a class="button-stitcher" target="_blank" style="' . $btn_stiticher . '" href="' . $btn_stiticher_url . '">' . $stitcher_text . '</a>';
		$buttons_array['btn_soundcloud']  = '<a class="button-soundcloud" target="_blank" style="' . $btn_soundcloud . '" href="' . $btn_soundcloud_url . '">' . $sound_cloud_text . '</a>';
		$buttons_array['btn_sppreview']   = '<a class="button-sppreview" target="_blank" style="' . $btn_sppreview . '" href="' . $btn_sppreview_url . '">' . $review_text . '</a>';
		$buttons_array['btn_spotify']     = '<a class="button-spotify" target="_blank" style="' . $btn_spotify . '" href="' . $btn_spotify_url . '">' . $spotify_text . '</a>';
		$buttons_array['btn_spprss']      = '<a class="button-spprss" target="_blank" style="' . $btn_spprss . '" href="' . $btn_spprss_url . '">' . $r_s_s_text . '</a>';
		$buttons_array['btn_sppandroid']  = '<a class="button-sppandroid" target="_blank" style="' . $btn_sppandroid . '" href="' . $btn_sppandroid_url . '">' . $android_text . '</a>';
		$buttons_array['btn_g_play']      = '<a class="button-g-play" target="_blank" style="' . $btn_g_play . '" href="' . $btn_g_play_url . '">' . $google_play_text . '</a>';
		$buttons_array['btn_share']       = '<a class="button-share" data-target="btnshare" style="' . $btn_share . '" href="#">' . $share_text . '</a>
				<div id="btnshare" class="webui-popover-content share-buttons">
						<a href="https://www.facebook.com/sharer/sharer.php?u=' . $share_link . '" title="Share on Facebook" target="_blank" class="facebook"><i class="fa fa-facebook"></i></a>
						<a href="https://twitter.com/share?text=' . $share_title . ' -&url=' . $share_link . '" title="Share on Twitter" target="_blank" class="twitter"><i class="fa fa-twitter"></i></a>
				</div>';
		$buttons_array['btn_spp_custom1'] = '<a class="spp-button-custom1" target="_blank" style="' . $btn_spp_custom1_display . '" href="' . $btn_spp_custom_url1 . '">' . $btn_spp_custom_name1 . '</a>';
		$buttons_array['btn_spp_custom2'] = '<a class="spp-button-custom2" target="_blank" style="' . $btn_spp_custom2_display . '" href="' . $btn_spp_custom_url2 . '">' . $btn_spp_custom_name2 . '</a>';
		$buttons_array['btn_spp_custom3'] = '<a class="spp-button-custom3" target="_blank" style="' . $btn_spp_custom3_display . '" href="' . $btn_spp_custom_url3 . '">' . $btn_spp_custom_name3 . '</a>';
		$buttons_array['btn_spp_custom4'] = '<a class="spp-button-custom4" target="_blank" style="' . $btn_spp_custom4_display . '" href="' . $btn_spp_custom_url4 . '">' . $btn_spp_custom_name4 . '</a>';
		$buttons_array['btn_spp_custom5'] = '<a class="spp-button-custom5" target="_blank" style="' . $btn_spp_custom5_display . '" href="' . $btn_spp_custom_url5 . '">' . $btn_spp_custom_name5 . '</a>';
		$buttons_array['btn_spp_custom6'] = '<a class="spp-button-custom6" target="_blank" style="' . $btn_spp_custom6_display . '" href="' . $btn_spp_custom_url6 . '">' . $btn_spp_custom_name6 . '</a>';
		$buttons_array['btn_spplisten']   = '<a class="button-spplisten" style="' . $btn_spplisten . '" target="_blank" href="javascript:window.open(\'' . $spplisten . '\',\'\',\'width=500,height=250\')">' . $listen_text . '</a>';
		$buttons_array['btn_download']    = '<a class="button-download" target="' . $spp_download_target . '" style="' . $btn_download . '" href="' . $audiodownloadurl . '">' . $download_text . '</a>';
		$buttons_array['btn_embed']       = '<a class="button-embed" rel="modal:open"  id="button-embed" style="' . $btn_embed_display . '" href="#ex1">' . $embed_text . '</a>';

		if ( $btn_embed ) {
			$postid     = $post->ID;
			$siteurl    = get_site_url();
			$iframehtml = '<iframe width="320" src="' . $siteurl . '/wp-content/plugins/simple-podcast-press/responsive_audio_player/embedplayer.php?postid=' . $postid . '&frameborder="0" scrolling="no"></iframe>';
			$html      .= '<div id="ex1" class="modal" style="display:none; z-index:1000000;">
				<textarea id="spp_embed_code" name="spp_embed_code" style="width: 100%; height: 150px;" class="span11">' . htmlspecialchars_decode( $iframehtml, ENT_QUOTES ) . '</textarea>
			</div>';
		}
		$buttons_array['spp_LeadBox_btn_checkbox'] = $spp_lead_box_btn_code;

		$buttons_order = Wp_Simplepodcastpress::spp_get_option( 'spp_button_order' );
		$buttons_order = str_replace( 'btn_ClammrIT_checkbox,', '', $buttons_order );

		if ( empty( $buttons_order ) ) {
			$buttons_order = 'btn_itunes,btn_g_play,btn_embed,btn_share,btn_sppreview,btn_spplisten,btn_download,btn_soundcloud,btn_stiticher,btn_sppandroid,btn_spprss,btn_spotify,btn_spp_custom1,btn_spp_custom2,btn_spp_custom3,btn_spp_custom4,btn_spp_custom5,btn_spp_custom6,spp_LeadBox_btn_checkbox';
		}
		$button_order   = explode( ',', $buttons_order );
		$buttons_sorted = array_replace( array_flip( $button_order ), $buttons_array );

		$html .= '<div class="sppbuttons" style="' . $allbtn_onoff . '">';

		foreach ( $buttons_sorted as $button => $key ) {
			$html .= $key;
		}

		$html .= '</div>';

		$js = <<<HTML
jQuery(document).ready(function($){
    if ( $('.button-share').webuiPopover )
        $('.button-share').webuiPopover({placement:'bottom',url:'#btnshare'});
});
HTML;

		wp_enqueue_script( 'inline-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/inline.js', array( 'jquery' ), 1, false );
		wp_add_inline_script( 'inline-js', $js, 'after' );

		return $html;
	}

	/**
	 * Generate the "Powered By" shortcode output.
	 *
	 * This function processes the attributes and generates the HTML output for the "Powered By" shortcode.
	 *
	 * @param array $atts An array of attributes passed to the shortcode.
	 * @return string The HTML output for the "Powered By" shortcode.
	 */
	public static function spp_poweredby_fn( $atts = null ) {
		$html               = '';
		$display_style      = '';
		$disable_powered_by = Wp_Simplepodcastpress::spp_get_option( 'spp_disable_poweredby' );
		$ref_url            = Wp_Simplepodcastpress::spp_get_option( 'spp_poweredby_url' );
		$ref_url            = filter_var( $ref_url, FILTER_SANITIZE_URL );

		// Validate url.
		if ( ! filter_var( $ref_url, FILTER_VALIDATE_URL ) === false ) {
			$ref_url = $ref_url;
		} else {
			$ref_url = 'https://simplepodcastpress.com/?ref=' . $ref_url;
		}

		$spp_optin_hide_chk = Wp_Simplepodcastpress::spp_get_option( 'spp_optin_hide_chk' );
		$style              = '';
		if ( isset( $atts['noplayer'] ) ) {
			$noplayer = $atts['noplayer'];
		} else {
			$noplayer = false;
		}

		if ( $spp_optin_hide_chk && ! $noplayer ) {

			$display_style = 'display:none !important;';

		}

		if ( ! $disable_powered_by ) {
			$html .= '
            <div class="spp-powered-by" style="font-size:12px; ' . $display_style . '"><center>Powered by the <a target="_blank" href="' . $ref_url . '">Simple Podcast Press</a> Player</center></div>
            ';
		} else {
			$html .= '';
		}

		return $html;
	}
	/**
	 * Generate the Opt-in Form shortcode output.
	 *
	 * This function processes the attributes and generates the HTML output for the Opt-in Form shortcode.
	 *
	 * @param array $atts An array of attributes passed to the shortcode.
	 * @return string The HTML output for the Opt-in Form shortcode.
	 */
	public static function spp_optin_shortcode_fn( $atts = null ) {
		if ( isset( $atts['noplayer'] ) ) {
			$atts['noplayer'] = true;
		}

		$html = self::spp_optin_fn( $atts );
		return $html;
	}

	/**
	 * Process and render the Opt-in Form.
	 *
	 * This function processes the attributes, performs the necessary actions, and renders the Opt-in Form.
	 *
	 * @param array $atts An array of attributes passed to the function.
	 * @return string The HTML output for the rendered Opt-in Form.
	 */
	public static function spp_optin_fn( $atts = null ) {
		$hide_email = '';
		$html       = '';
		global $post;
		if ( ! isset( $post->ID ) ) {
				return;
		}
		// Get Default Values.
		$spp_auto_resp_url_default           = Wp_Simplepodcastpress::spp_get_option( 'spp_auto_resp_url' );
		$spp_auto_resp_heading_default       = Wp_Simplepodcastpress::spp_get_option( 'spp_auto_resp_heading' );
		$spp_auto_resp_sub_heading_default   = Wp_Simplepodcastpress::spp_get_option( 'spp_auto_resp_sub_heading' );
		$spp_auto_resp_hidden_default        = Wp_Simplepodcastpress::spp_get_option( 'spp_auto_resp_hidden' );
		$spp_auto_resp_name_default          = Wp_Simplepodcastpress::spp_get_option( 'spp_auto_resp_name' );
		$spp_auto_resp_email_default         = Wp_Simplepodcastpress::spp_get_option( 'spp_auto_resp_email' );
		$spp_auto_resp_email_submitt_default = Wp_Simplepodcastpress::spp_get_option( 'spp_auto_resp_submitt' );

		// Get Values from Post Meta.
		$spp_auto_resp_url_post         = get_post_meta( $post->ID, 'spp_auto_resp_url', true );
		$spp_auto_resp_heading_post     = get_post_meta( $post->ID, 'spp_auto_resp_heading', true );
		$spp_auto_resp_sub_heading_post = get_post_meta( $post->ID, 'spp_auto_resp_sub_heading', true );
		$spp_auto_resp_hidden_post      = get_post_meta( $post->ID, 'spp_auto_resp_hidden', true );
		$spp_auto_resp_name_post        = get_post_meta( $post->ID, 'spp_auto_resp_name', true );
		$spp_auto_resp_email_post       = get_post_meta( $post->ID, 'spp_auto_resp_email', true );
		$spp_auto_resp_submitt_post     = get_post_meta( $post->ID, 'spp_auto_resp_submitt', true );

		$spp_auto_resp_url_get         = ( empty( $spp_auto_resp_url_post ) ? $spp_auto_resp_url_default : $spp_auto_resp_url_post );
		$spp_auto_resp_heading_get     = ( empty( $spp_auto_resp_heading_post ) ? $spp_auto_resp_heading_default : $spp_auto_resp_heading_post );
		$spp_auto_resp_sub_heading_get = ( empty( $spp_auto_resp_sub_heading_post ) ? $spp_auto_resp_sub_heading_default : $spp_auto_resp_sub_heading_post );
		$spp_auto_resp_hidden_get      = ( empty( $spp_auto_resp_hidden_post ) ? $spp_auto_resp_hidden_default : $spp_auto_resp_hidden_post );
		$spp_auto_resp_name_get        = ( empty( $spp_auto_resp_name_post ) ? $spp_auto_resp_name_default : $spp_auto_resp_name_post );
		$spp_auto_resp_email_get       = ( empty( $spp_auto_resp_email_post ) ? $spp_auto_resp_email_default : $spp_auto_resp_email_post );
		$spp_auto_resp_email_submitt   = ( empty( $spp_auto_resp_submitt_post ) ? $spp_auto_resp_email_submitt_default : $spp_auto_resp_submitt_post );

		$spp_optin_box      = Wp_Simplepodcastpress::spp_get_option( 'spp_optin_box' );
		$spp_two_step_optin = Wp_Simplepodcastpress::spp_get_option( 'spp_two_step_optin' );

		switch ( $spp_two_step_optin ) {
			case 1:
					$hide_first_name = '';
				break;
			case 2:
					$hide_first_name = 'display:none !important;';
				break;
			case 3:
					$hide_first_name = 'display:none !important;';
					$hide_email      = 'display:none !important;';
				break;
			case 4:
					$hide_first_name          = 'display:none !important;';
					$hide_email               = 'display:none !important;';
					$hide_first_name_two_step = 'display:none !important;';
				break;

		}

		$spp_optin_hide_chk = Wp_Simplepodcastpress::spp_get_option( 'spp_optin_hide_chk' );
		$style              = '';
		if ( isset( $atts['noplayer'] ) ) {
			$noplayer = $atts['noplayer'];
		} else {
			$noplayer = false;
		}

		if ( $spp_optin_hide_chk && ! $noplayer ) {
			$style = "style='display:none !important;' ";
		}
		$spp_privacy_page = Wp_Simplepodcastpress::spp_get_option( 'spp_privacy_page' );

		$s = '';
		$d = '';
		if ( ( 'disabled' === $spp_privacy_page ) || ( '' === $spp_privacy_page ) ) {
			$d = 'display:none;';
		} else {
			$s = 'disabled=disabled';
		}
		$pp_text = Wp_Simplepodcastpress::spp_get_option( 'privacy_policy_text' );

		$pp_page = '<a href="' . get_the_permalink( $spp_privacy_page ) . '" target="_blank">Privacy Policy</a>';
		$parse   = wp_parse_url( $spp_auto_resp_url_get );

		$method     = '';
		$data       = '';
		$script     = '';
		$action_url = isset( $parse['host'] ) ? $parse['host'] : '';

		if ( 'app.convertkit.com' !== $action_url ) {
			$method = 'method="post"';
		} elseif ( 'app.convertkit.com' === $action_url && stripos( false === $spp_auto_resp_url_get, 'landing_pages' ) ) {
			$data   = 'data-sv-form="2061587" data-options=\'{"settings":{"after_subscribe":{"action":"redirect","success_message":"Success! Now check your email to confirm your subscription.","redirect_url":"https://www.cybertraps.com/thank-you-for-subscribing/"},"analytics":{"google":null,"facebook":null,"segment":null,"pinterest":null,"sparkloop":null,"googletagmanager":null},"modal":{"trigger":"timer","scroll_percentage":null,"timer":5,"devices":"all","show_once_every":15},"powered_by":{"show":true,"url":"https://convertkit.com?utm_source=dynamic&utm_medium=referral&utm_campaign=poweredby&utm_content=form"},"recaptcha":{"enabled":false},"return_visitor":{"action":"show","custom_content":""},"slide_in":{"display_in":"bottom_right","trigger":"timer","scroll_percentage":null,"timer":5,"devices":"all","show_once_every":15},"sticky_bar":{"display_in":"top","trigger":"timer","scroll_percentage":null,"timer":5,"devices":"all","show_once_every":15}},"version":"5"}\'';
			$script = '<script src="https://f.convertkit.com/ckjs/ck.5.js"></script>'; // phpcs:ignore WordPress.WP.EnqueuedResources.NonEnqueuedScript
		}
		$pp_text = str_replace( 'Privacy Policy', $pp_page, $pp_text );
		$html   .= '<div id="spp-box-below-video" class="spp-optin-box" ' . $style . '>
				<div class="spp-optin-box-padding">
				<div class="spp-optin-box-content">
				<div class="spp-optin-box-headline">' . stripslashes( $spp_auto_resp_heading_get ) . '</div>
				<div class="spp-optin-box-subheadline">' . stripslashes( $spp_auto_resp_sub_heading_get ) . '</div>
				<div class="spp-optin-box-form-wrap">
				' . $script . '
				<form accept-charset="utf-8" action="' . $spp_auto_resp_url_get . '" ' . $data . ' ' . $method . ' target="_blank">
				' . htmlspecialchars_decode( $spp_auto_resp_hidden_get, ENT_QUOTES ) . '
		
				<div class="spp-optin-box-field" style="' . $hide_first_name . '">
				 <input placeholder="First Name" type="text" data-role="none" name="' . $spp_auto_resp_name_get . '"></div>
				<div class="spp-optin-box-field" style="' . $hide_email . '">
				 <input placeholder="Email" type="text" data-role="none" name="' . $spp_auto_resp_email_get . '"></div><br>
				 <div style="' . $d . '"><input type="checkbox" name="checkbox" value="check" id="consent" /> ' . $pp_text . ' <br><br></div>';
		if ( 3 === $spp_two_step_optin || 4 === $spp_two_step_optin ) {
			$html .= '<a class="spp-optin-box-submit" data-reveal-id="spp-two-step-optin"  href="#">' . stripslashes( $spp_auto_resp_email_submitt ) . '</a>';
		} else {
			$html .= '<div class="spp-optin-box-field-submit"><input type="submit" data-role="none" name="submit" class="spp-optin-box-submit" id="sppoptinboxsubmit" ' . $s . ' value=" ' . stripslashes( $spp_auto_resp_email_submitt ) . '"></div>';
		}
			$html .= '
				</form>
				</div>
				</div>
				</div>
				</div>';
		if ( isset( $atts['noplayer'] ) ) {
			$noplayer = $atts['noplayer'];
		} else {
			$noplayer = false;
		}

		if ( $spp_optin_hide_chk && ! $noplayer ) {
			$spp_optshow_xseconds = Wp_Simplepodcastpress::spp_get_option( 'spp_optshow_xseconds' );
			if ( ! $spp_optshow_xseconds ) {
				$spp_optshow_xseconds = 3;
			}
			$spp_optshow_xseconds     = $spp_optshow_xseconds * 1000;
			$select_audio_player_skin = Wp_Simplepodcastpress::spp_get_option( 'select_audio_player_skin' );
			if ( 'soundmanager2_player' === $select_audio_player_skin ) {
				$js = <<<HTML

		jQuery(".player-$post->ID .play-pause").click(function(){
				setTimeout(function (){
						jQuery(".player-$post->ID .sppbuttons").slideDown('slow');
						setTimeout(function (){
						jQuery(".player-$post->ID .spp-optin-box").slideDown('slow');
						;}, 300);
						setTimeout(function (){
						jQuery(".player-$post->ID .spp-powered-by").slideDown('slow');
						;}, 200);


				;}, $spp_optshow_xseconds);
		});

HTML;
				wp_enqueue_script( 'inline-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/inline.js', array( 'jquery' ), 1.1, false );
				wp_add_inline_script( 'inline-js', $js, 'after' );

			} else {
				$js = <<<HTML

		jQuery(".player-$post->ID .audioplayer-playpause").click(function(){
				setTimeout(function (){
						jQuery(".player-$post->ID .sppbuttons").slideDown('slow');
						setTimeout(function (){
						jQuery(".player-$post->ID .spp-optin-box").slideDown('slow');
						;}, 300);
						setTimeout(function (){
						jQuery(".player-$post->ID .spp-powered-by").slideDown('slow');
						;}, 200);


				;}, $spp_optshow_xseconds);
		});

HTML;
				wp_enqueue_script( 'inline-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/inline.js', array( 'jquery' ), 1.1, false );
				wp_add_inline_script( 'inline-js', $js, 'after' );

			}
		}

		return apply_filters( 'spp_optin_html', $html );
	}

	/**
	 * Process and render the Two-Step Opt-in Form.
	 *
	 * This function processes the attributes, performs the necessary actions, and renders the Two-Step Opt-in Form.
	 *
	 * @param array $atts An array of attributes passed to the function.
	 * @return void The HTML output for the rendered Two-Step Opt-in Form.
	 */
	public function two_step_optin_fn( $atts = null ) { // phpcs:ignore Generic.CodeAnalysis.UnusedFunctionParameter.Found
		$html                          = '';
		$hide_first_name_two_step      = '';
		$spp_auto_resp_url_get         = Wp_Simplepodcastpress::spp_get_option( 'spp_auto_resp_url' );
		$spp_auto_resp_heading_get     = Wp_Simplepodcastpress::spp_get_option( 'spp_auto_resp_heading' );
		$spp_auto_resp_sub_heading_get = Wp_Simplepodcastpress::spp_get_option( 'spp_auto_resp_sub_heading' );
		$spp_auto_resp_hidden_get      = Wp_Simplepodcastpress::spp_get_option( 'spp_auto_resp_hidden' );
		$spp_auto_resp_name_get        = Wp_Simplepodcastpress::spp_get_option( 'spp_auto_resp_name' );
		$spp_auto_resp_email_get       = Wp_Simplepodcastpress::spp_get_option( 'spp_auto_resp_email' );
		$spp_auto_resp_email_submitt   = Wp_Simplepodcastpress::spp_get_option( 'spp_auto_resp_submitt' );
		$spp_optin_box                 = Wp_Simplepodcastpress::spp_get_option( 'spp_optin_box' );
		$spp_two_step_optin            = Wp_Simplepodcastpress::spp_get_option( 'spp_two_step_optin' );
		switch ( $spp_two_step_optin ) {
			case 1:
					$hide_first_name = '';
				break;
			case 2:
					$hide_first_name = 'display:none !important;';
				break;
			case 3:
					$hide_first_name = 'display:none !important;';
				break;
			case 4:
					$hide_first_name          = 'display:none !important;';
					$hide_email               = 'display:none !important;';
					$hide_first_name_two_step = 'display:none !important;';
				break;

		}
		$parse      = wp_parse_url( $spp_auto_resp_url_get );
		$action_url = isset( $parse['host'] ) ? $parse['host'] : '';
		if ( 'app.convertkit.com' !== $action_url ) {
			$method = 'method="post"';
		}

		if ( ( 3 === $spp_two_step_optin ) || ( 4 === $spp_two_step_optin ) ) {
			$html .= '
             <div id="spp-two-step-optin" class="reveal-modal">
                 <a class="close-reveal-modal">&#215;</a>
                <div id="spp-box-below-video" class="spp-optin-box">
				<div class="spp-optin-box-padding">
				<div class="spp-optin-box-content">
				<div class="spp-optin-box-headline">' . stripslashes( $spp_auto_resp_heading_get ) . '</div>
				<div class="spp-optin-box-subheadline">' . stripslashes( $spp_auto_resp_sub_heading_get ) . '</div>
				<div class="spp-optin-box-form-wrap">
				<form accept-charset="utf-8" action="' . $spp_auto_resp_url_get . '" ' . $method . '  target="_blank">
				' . htmlspecialchars_decode( $spp_auto_resp_hidden_get, ENT_QUOTES ) . '
				<div class="spp-optin-box-field" style="' . $hide_first_name_two_step . '">
				 <input placeholder="First Name" data-role="none" type="text" name="' . $spp_auto_resp_name_get . '"></div>
				<div class="spp-optin-box-field">
				 <input placeholder="Email" type="text" data-role="none" name="' . $spp_auto_resp_email_get . '"></div>
				<div class="spp-optin-box-field-submit"><input type="submit" data-role="none" name="submit" class="spp-optin-box-submit" value=" ' . stripslashes( $spp_auto_resp_email_submitt ) . '"></div>
				</form>
				</div>
				</div>
				</div>
				</div>
			</div>
                ';
		}

		echo apply_filters( 'two_step_optin', $html ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
	}
	/**
	 * Render the Full Podcast Player.
	 *
	 * This function processes the attributes, performs any necessary actions, and renders the Full Podcast Player.
	 *
	 * @param array $atts An array of attributes passed to the function.
	 * @return string The HTML output for the rendered Full Podcast Player.
	 */
	public function spp_fullplayer_fn( $atts = null ) {
		// Prevents SPP Player from showing up in feeds. Not necessary it seems with latest PowerPress.
		if ( is_feed() ) {
			return;
		}

		$html           = '';
		$ctabuttonspart = '';
		$optinpart      = '';
		$poweredby      = '';
		global $post;

		if ( ! isset( $post->ID ) && ! isset( $atts['url'] ) ) {
				return;
		}
		if ( isset( $atts['url'] ) ) {

			$audio_file = $atts['url'];

			$spp_line                     = '|';
			$filtered_audio_file_location = strpos( $audio_file, $spp_line );

			if ( $filtered_audio_file_location ) {
				$audio_file = substr( $audio_file, 0, $filtered_audio_file_location );
			}
		} elseif ( isset( $atts['src'] ) ) {
			$audio_file = $atts['src'];
		}

		if ( isset( $atts['hide'] ) ) {
			if ( ( 'true' === $atts['hide'] ) || ( 'TRUE' === $atts['hide'] ) ) {
				return;
			}
		}

		$audiopart = $this->spp_audioplayeronly_fn( $atts );
		if ( false === $audiopart ) {
			return false;
		}

		$spp_optin_box = Wp_Simplepodcastpress::spp_get_option( 'spp_optin_box' );
		if ( isset( $atts['optin'] ) ) {
			$optin = $atts['optin'];
			if ( 'off' !== $optin ) {
				$optinpart = $this->spp_optin_fn( $atts );
			}
		}
		if ( 1 === $spp_optin_box ) {
			// If Optin Option is disabled, don't add it to the player.
				$optinpart = $this->spp_optin_fn( $atts );
		}

		if ( isset( $atts['ctabuttons'] ) ) {
			$ctabuttons = $atts['ctabuttons'];
			if ( 'off' !== $ctabuttons ) {
				$ctabuttonspart = $this->spp_ctabuttons_fn( $atts );
			}
		} else {
			$ctabuttonspart = $this->spp_ctabuttons_fn( $atts );
		}

		if ( isset( $atts['poweredby'] ) ) {
			$poweredbylink = $atts['poweredby'];
			if ( 'off' !== $poweredbylink ) {
				$poweredby = $this->spp_poweredby_fn( $atts );
			}
		} else {
			$poweredby = $this->spp_poweredby_fn( $atts );
		}

		$container_width     = Wp_Simplepodcastpress::spp_get_option( 'container_width' );
		$disable_button_home = Wp_Simplepodcastpress::spp_get_option( 'hide_elements_athome' );

		if ( $container_width ) {
			$container_width = 'style="max-width:' . $container_width . 'px;"';
		}

		if ( $disable_button_home ) {
			if ( is_single() && is_main_query() ) {
				$html = '<div class="player_container player-' . $post->ID . '""' . $container_width . ' >' . $audiopart . $ctabuttonspart . $optinpart . $poweredby . '</div>';
			} else {
				$html = '<div class="player_container player-' . $post->ID . '"' . $container_width . ' >' . $audiopart . '</div>';
			}
		} else {
			$html = '<div class="player_container player-' . $post->ID . '"' . $container_width . ' >' . $audiopart . $ctabuttonspart . $optinpart . $poweredby . '</div>';
		}

		return $html;
	}
	/**
	 * Render the Podcast Transcript.
	 *
	 * This function processes the attributes, performs any necessary actions, and renders the Podcast Transcript.
	 *
	 * @param array  $atts    An array of attributes passed to the function.
	 * @param string $content The content inside the shortcode.
	 * @return string The HTML output for the rendered Podcast Transcript.
	 */
	public function spp_transcript_fn( $atts = null, $content = null ) {

		global $post;
		$title = Wp_Simplepodcastpress::spp_get_option( 'transcript_txt' );

		if ( empty( $title ) ) {
			$title = 'Read Full Transcript';
		}

		if ( null == $content ) {
			$spptranscript = get_post_meta( $post->ID, '_spptranscript', true );
		} else {
			$spptranscript = $content;
		}

		$spptranscript = do_shortcode( $spptranscript );

			$html = <<<HTML
<div class="transcript-box" style="float:none !important;">
<div class="accordion-container">
		<a href="#" class="accordion-toggle">$title<span class="toggle-icon"><i class="fa fa-angle-double-down"></i></span></a>
		<div class="accordion-accordion_content">
			<p>$spptranscript</p>
		</div>
		<!--/.accordion-accordion_content-->
	</div>
</div>

HTML;

		$this->sppress_enqueue_scripts_css();
		$this->sppress_enqueue_scripts_js();
		return $html;
	}
	/**
	 * Render the Podcast Timestamp.
	 *
	 * This function processes the attributes, performs any necessary actions, and renders the Podcast Timestamp.
	 *
	 * @param array $atts An array of attributes passed to the function.
	 * @return string The HTML output for the rendered Podcast Timestamp.
	 */
	public function spp_timestamp_fn( $atts = null ) {

		if ( isset( $atts['time'] ) ) {
			$time = $atts['time'];
		}
		$arr = explode( ':', $time );
		if ( ! isset( $arr[2] ) ) {
				$m        = (int) $arr[0];
				$s        = (int) $arr[1];
				$interval = $m * 60 + $s;
		} else {
				$h        = (int) $arr[0];
				$m        = (int) $arr[1];
				$s        = (int) $arr[2];
				$interval = $h * 3600 + $m * 60 + $s;
		}
		$spp_remove_brackets_timestamp = Wp_Simplepodcastpress::spp_get_option( 'spp_remove_brackets_timestamp' );

		if ( ( is_single() && is_main_query() ) || is_page() ) {
			if ( ! $spp_remove_brackets_timestamp ) {
				return '<a class="spp-timestamp" time="' . $interval . '">[' . $time . ']</a>';
			} else {
				return '<a class="spp-timestamp" time="' . $interval . '">' . $time . '</a>';
			}
		} elseif ( ! $spp_remove_brackets_timestamp ) {
				return '<span>[' . $time . ']</span>';
		} else {
			return '<span>' . $time . '</span>';
		}
	}


	/**
	 * Custom sort function for sorting podcasts by published date.
	 *
	 * This function is used as a callback for sorting an array of podcast items based on their published date.
	 *
	 * @param array $a First podcast item to compare.
	 * @param array $b Second podcast item to compare.
	 * @return int Returns a negative value if $a is older than $b, a positive value if $a is newer than $b, and 0 if they are equal.
	 */
	public function sort_function( $a, $b ) {
		return strtotime( $b['post_date'] ) - strtotime( $a['post_date'] );
	}

	/**
	 * Display a list of podcast episodes.
	 *
	 * This function generates and outputs a list of podcast episodes based on the provided attributes.
	 *
	 * @param array $atts Shortcode attributes (if any).
	 * @return string HTML content representing the list of podcast episodes.
	 */
	public function spp_episodes_fn( $atts = null ) {
		global $wpdb;
		$max        = null;
		$table_name = $wpdb->prefix . 'spp_episode';
		if ( isset( $atts['cat'] ) ) {
			$catid = $atts['cat'];
		}

		if ( isset( $atts['max'] ) ) {
			$max = $atts['max'];
		}

		$episodes = $wpdb->get_results( 'SELECT * FROM ' . $table_name ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching

		if ( empty( $episodes ) ) {
			$html = '<p>No podcast episodes found</p>';
			return $html;
		}

		$audiodownloadurl = '';

		$direct_download_button = Wp_Simplepodcastpress::spp_get_option( 'direct_download_button' );

		if ( $direct_download_button ) {
			$audiodownloadurl = SPPRESS_PLUGIN_URL . '/responsive_audio_player/downloadaudio.php?file=';
		}

		$html = '		
					
					<table id="grid-basic" class="table table-condensed table-hover table-striped" style="font-size: 14px;" >
							<thead>
									<tr>
											<td class="spp-episodes-released-title" data-column-id="released" data-header-css-class="releasedColumn">Released</th>
											<td class="spp-episodes-podcast-title" data-column-id="name">Episode</th>
											<td class="spp-episodes-download-title" data-column-id="link" data-header-css-class="downloadColumn" data-formatter="link" data-sortable="false"></th>
									</tr>
							</thead>
							<tbody>';
		foreach ( $episodes as $episode ) {
			$episode_array[] = (array) $episode;
			usort( $episode_array, array( $this, 'sort_function' ) );
		}

		$count = 0;

		foreach ( $episode_array as $latest_episode ) {

			if ( ( $max ) && ( $count === $max ) ) {
				break;
			}

			$postid = $latest_episode['postid'];
			if (!is_int($postid) || $postid <= 0) {
				$postlink = get_permalink( $postid );
				if ( isset( $catid ) ) {
					$category = get_the_category( $postid );
					if (!empty($category) && isset($category->slug) && $category->slug != null) {
						if ($catid !== $category->slug) {
							continue;
						}
					} 
				}
			}

			if ( ! empty( $postlink ) ) {
				$title = '<a href="' . $postlink . '">' . $latest_episode['posttitle'] . '</a>';
			} else {

				$title = $latest_episode['posttitle'];
			}

			$html                 .= '
			    <tr>
							<td class="spp-episodes-released" >';
							$arr   = explode( ' ', $latest_episode['post_date'] );
							$html .= $arr[1] . ' &nbsp;' . $arr[2] . ' &nbsp;' . $arr[3];
							$html .= '</td>
							<td class="spp-episodes-podcast" >' . $title . '</td>
							<td class="spp-episodes-download" ><a href="' . $latest_episode['downloadurl'] . '">Download</a></td>
					</tr> ';
			++$count;
		}
		$html .= '</tbody></table>';

		return $html;
	}

	/**
	 * Retrieve the featured image for a podcast episode.
	 *
	 * @param array $atts Shortcode attributes (if any).
	 * @return string HTML markup for the featured image.
	 */
	public function spp_featured_image_fn( $atts = null ) { // phpcs:ignore Generic.CodeAnalysis.UnusedFunctionParameter.Found
			global $post;
		if ( has_post_thumbnail() ) {
			$spp_imgsize_select  = Wp_Simplepodcastpress::spp_get_option( 'spp_imgsize_select' );
			$spp_imgalign_select = Wp_Simplepodcastpress::spp_get_option( 'spp_imgalign_select' );

			if ( 'thumbnail' === $spp_imgsize_select ) {
				$spp_imgsize = '150';
			} elseif ( 'medium' === $spp_imgsize_select ) {
				$spp_imgsize = '300';
			} elseif ( 'large' === $spp_imgsize_select ) {
				$spp_imgsize = '640';
			} else {
				$spp_imgsize = '';
			}

			$image               = wp_get_attachment_image_src( get_post_thumbnail_id( $post->ID ), 'single-post-thumbnail' );
			$spp_featured_image  = '<img width="' . $spp_imgsize . '"';
			$spp_featured_image .= ' height="' . $spp_imgsize . '"';
			$spp_featured_image .= ' class="' . $spp_imgalign_select . ' wp-post-image"';
			$spp_featured_image .= ' src="' . $image[0] . '" >';
			return $spp_featured_image;
		}
	}
	/**
	 * Enqueues JavaScript scripts for Simple Podcast Press.
	 *
	 * This function is responsible for adding JavaScript scripts to the WordPress
	 * site for Simple Podcast Press. It ensures the necessary scripts are loaded
	 * to enhance functionality.
	 *
	 * @since 1.0.0
	 */
	public function sppress_enqueue_scripts_js() {
		global $post;

		$css_version              = wp_rand( 10, 99 ) / 10;
		$select_audio_player_skin = Wp_Simplepodcastpress::spp_get_option( 'select_audio_player_skin' );

		if ( 'soundmanager2_player' === $select_audio_player_skin ) {

			wp_enqueue_script( 'soundmanager2-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/soundmanager2.js', array( 'jquery' ), 1.0, false );
			if ( is_single() || is_page() ) {
				wp_enqueue_script( 'resp-player-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/bar-ui-content.js', array( 'jquery' ), 1.0, false );
			} else {
				wp_enqueue_script( 'resp-player-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/bar-ui.js', array( 'jquery' ), 1.0, false );
			}
			wp_enqueue_script( 'timestamp-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/sm_timestamp.js', array( 'jquery' ), 1.0, false );

			if ( isset( $_GET['t'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
				$time   = isset( $_GET['t'] ) ? sanitize_text_field( wp_unslash( $_GET['t'] ) ) : ''; // phpcs:ignore WordPress.Security.NonceVerification.Recommended
				$params = array(
					'time' => $time,
				);
				wp_localize_script( 'timestamp-js', 'timestampget', $params );

			}
			if ( shortcode_exists( 'spp-transcript' ) ) {
				wp_enqueue_script( 'transcript-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/transcript.js', array( 'jquery' ), 1.0, false );
			}
		} else {
			wp_enqueue_script( 'resp-player-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/audio-player.js', array( 'jquery' ), 1.0, false );
			wp_enqueue_script( 'timestamp-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/timestamp.js', array( 'jquery' ), 1.0, false );
			wp_enqueue_script( 'transcript-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/transcript.js', array( 'jquery' ), 1.0, false );
		}
		global $post;
		if ( ! isset( $post->ID ) ) {
				return;
		}
		wp_enqueue_script( 'popover-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/jquery.webui-popover.min.js', array( 'jquery' ), 1.0, false );

		wp_enqueue_script( 'jquery-reveal', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/jquery.reveal.js', array( 'jquery' ), 1.0, false );
	}
	/**
	 * Enqueue CSS scripts for Simple Podcast Press.
	 *
	 * This function is responsible for enqueuing two different CSS files for Simple Podcast Press.
	 * It ensures that the stylesheets are properly loaded on the site.
	 *
	 * @since 1.0.0
	 */
	public function sppress_enqueue_scripts_css() {
		$css_version = wp_rand( 10, 99 ) / 10;
		global $post;

		// Always load all scripts and js.
		// Handle the location of the CSS file for multisite for css files we write when saving only.
		if ( is_multisite() ) {
			$uploads            = wp_upload_dir();
			$spp_simple_player  = trailingslashit( $uploads['baseurl'] ) . 'audio-player.css';
			$spp_classic_player = trailingslashit( $uploads['baseurl'] ) . 'classic-player.css';
			$player_buttons     = trailingslashit( $uploads['baseurl'] ) . 'player-buttons.css';
			$bar_ui             = trailingslashit( $uploads['baseurl'] ) . 'bar-ui.css';
		} else {
			$spp_simple_player  = SPPRESS_PLUGIN_URL . '/responsive_audio_player/css/audio-player.css';
			$spp_classic_player = SPPRESS_PLUGIN_URL . '/responsive_audio_player/css/classic-player.css';
			$player_buttons     = SPPRESS_PLUGIN_URL . '/responsive_audio_player/css/player-buttons.css';
			$bar_ui             = SPPRESS_PLUGIN_URL . '/responsive_audio_player/css/bar-ui.css';

		}

		// Load these from same place regardless of whether multisite or not.
		$modelcss     = SPPRESS_PLUGIN_URL . '/responsive_audio_player/css/jquery.modal.min.css';
		$font_awesome = SPPRESS_PLUGIN_URL . '/responsive_audio_player/css/font-awesome.css';

		$select_audio_player_skin = Wp_Simplepodcastpress::spp_get_option( 'select_audio_player_skin' );

		if ( 'soundmanager2_player' === $select_audio_player_skin ) {

			wp_enqueue_style( 'font-awesome', $font_awesome, false, $css_version, 'all' );
			wp_enqueue_style( 'player-buttons', $player_buttons, false, $css_version, 'all' );
			wp_enqueue_style( 'bar-ui', $bar_ui, false, $css_version, 'all' );
			$disable_jquery_model = Wp_Simplepodcastpress::spp_get_option( 'disable_jquery_model' );
			if ( ! $disable_jquery_model ) {
				wp_enqueue_style( 'model-ui', $modelcss, false, $css_version, 'all' );
			}
		} else {
			wp_enqueue_style( 'resp-player-css', $spp_classic_player, false, $css_version, 'all' );
			wp_enqueue_style( 'player-buttons', $player_buttons, false, $css_version, 'all' );
		}
		global $post;
		if ( ! isset( $post->ID ) ) {
				return;
		}

		wp_enqueue_style( 'popover-css', SPPRESS_PLUGIN_URL . '/responsive_audio_player/css/jquery.webui-popover.min.css', false, $css_version, 'all' );
		wp_enqueue_style( 'jquery-reveal-css', SPPRESS_PLUGIN_URL . '/responsive_audio_player/css/reveal.css', false, $css_version, 'all' );
		wp_enqueue_style( 'spp-clickable-tweet', SPPRESS_PLUGIN_URL . '/responsive_audio_player/css/spp-tweet-styles.css', false, $css_version, 'all' );

		$disable_jquery_model = Wp_Simplepodcastpress::spp_get_option( 'disable_jquery_model' );
		if ( ! $disable_jquery_model ) {
			wp_enqueue_script( 'jquery-model', $this->plugin_url . 'js/jquery.modal.min.js', array( 'jquery' ), $css_version, true );
		}
	}
	/**
	 * Check if a specific shortcode exists in the given content.
	 *
	 * This function searches for a specific shortcode within the provided content string.
	 * It returns a boolean indicating whether the shortcode is present or not.
	 *
	 * @since 1.0.0
	 *
	 * @param string $shortcodestr The content string to search for the shortcode.
	 * @return bool Whether the specified shortcode is found in the content.
	 */
	public function spp_if_shorcode( $shortcodestr ) {
			global $post, $wpdb;
		if ( ! isset( $post->ID ) ) {
			return;
		}
			// Determine whether this page contains "my_shortcode" shortcode.
			$shortcode_found = false;
		if ( has_shortcode( $post->post_content, $shortcodestr ) ) {
			$shortcode_found = true;
		} elseif ( isset( $post->ID ) ) {
			$result          = $wpdb->get_var( // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
				$wpdb->prepare(
					"SELECT count(*) FROM $wpdb->postmeta " .
					'WHERE post_id = %d and meta_value LIKE %s',
					$post->ID,
					$shortcodestr
				)
			);
			$shortcode_found = ! empty( $result ) ? true : false;
		}
		return $shortcode_found;
	}

	/**
	 * Enqueue jQuery script.
	 *
	 * This function adds jQuery to the list of registered scripts to be included in WordPress.
	 * It leverages the $scripts object to enqueue the jQuery script.
	 *
	 * @since 1.0.0
	 *
	 * @param WP_Scripts $scripts The global WP_Scripts object for managing scripts.
	 */
	public function enqueue_jquery( &$scripts ) {

			$scripts->add_data( 'jquery', 'group', 1 );
	}

	/**
	 * Check if the content contains a specific shortcode.
	 *
	 * This method checks whether the given content contains the specified shortcode.
	 *
	 * @since 1.0.0
	 *
	 * @param string $shortcode The name of the shortcode to check.
	 *
	 * @return bool Whether the content contains the specified shortcode.
	 */
	private function has_shortcode( $shortcode = '' ) {
		$post_to_check = get_post( get_the_ID() );
		$found         = false;
		if ( ! $shortcode ) {
			return $found;
		}

		if ( stripos( $post_to_check->post_content, '[' . $shortcode ) !== false ) {
			$found = true;
		}

		return $found;
	}
	/**
	 * Generate the HTML for the podcast player.
	 *
	 * This method generates the HTML markup for the podcast player based on the provided parameters.
	 *
	 * @since 1.0.0
	 *
	 * @param int    $postid               The ID of the podcast post.
	 * @param string $player_image         URL or name of the player image.
	 * @param bool   $spp_speed_button     Whether to include the speed control button.
	 * @param bool   $rewind_button_display Whether to display the rewind button.
	 * @param bool   $spp_volume_button    Whether to include the volume control button.
	 * @param bool   $sidebar              Whether to display the player in the sidebar.
	 * @param bool   $playlist             Whether to include a playlist.
	 * @param array  $podcast_items        Array of podcast items.
	 * @param string $playlistopen         Playlist opening state.
	 * @param string $duration             Duration of the podcast.
	 * @param string $ep_date              Whether to display episode date.
	 * @param string $ep_duration          Whether to display episode duration.
	 *
	 * @return string Generated HTML for the podcast player.
	 */
	public static function player_html_fn( $postid, $podcast_items, $player_image, $spp_speed_button, $rewind_button_display, $spp_volume_button, $sidebar = false, $playlist = false, $playlistopen = '', $duration = '', $ep_date = 'off', $ep_duration = 'off' ) {
		if ( $playlist ) {
			$playerhtml = '<div id="sm2-' . $postid . '" class="sm2-' . $postid . ' sm2-bar-ui compact flat full-width playlist full-width-player ' . $playlistopen . '">';
		} else {
			$playerhtml = '<div id="sm2-' . $postid . '" class="sm2-' . $postid . ' playercontent sm2-bar-ui compact flat full-width full-width-player">';
		}
		if ( $sidebar ) {
			$playerhtml = '<div id="spp-pro-sidebar" class="sm2-' . $postid . ' sm2-bar-ui compact flat full-width">';
		}
		$playerhtml .= '<div class="bd sm2-main-controls">
									<div class="sm2-inline-element sm2-button-element">
										<div class="sm2-button-bd" style="background: transparent url(' . $player_image . ') repeat scroll 0% 0% / cover ;">
										<div id="spp-player-background"> </div>
										<a href="#play" class="sm2-inline-button play-pause"></a>
										<div class="loader" style="display:none;"></div>
										<div class="eq">
											<span class="bar bar-1"></span>
											<span class="bar bar-2"></span>
											<span class="bar bar-3"></span>
											<span class="bar bar-4"></span>
										</div>
									</div>
								</div>';
		if ( $sidebar ) {
			$playerhtml .= '<div class="sm2-inline-element sm2-inline-status">';
		} else {
			$playerhtml .= '<div class="sm2-inline-element sm2-inline-status progressbar">';

		}

		if ( ! $sidebar ) {
			$playerhtml .= '			
									<div class="sm2-button-bd" style="' . $spp_speed_button . '">
										<div class="playerpitch0" style="display:none;">0.75x</div>									
										<div class="playerpitch1">1x</div>
										<div class="playerpitch2" style="display:none;">1.25x</div>										
										<div class="playerpitch3" style="display:none;">1.5x</div>
										<div class="playerpitch4" style="display:none;">2x</div>
									</div>
									<div class="sm2-button-bd" style="' . $rewind_button_display . '">
										<div class="secrewind"></div>
									</div>										
									';
		}
							$playerhtml .= '
									<div class="sm2-playlist">
										<div class="sm2-playlist-target"></div>								
									</div>
									<div class="sm2-progress">
											<div class="sm2-progress-bd">
												<div class="sm2-progress-track">
													<div class="sm2-progress-bar1" style="display:none;"></div>
													<div class="sm2-progress-bar"></div>
													<div class="sm2-progress-ball"><div class="icon-overlay"></div></div>
												</div>
											</div>
												
											';
		if ( $playlist || '0:00' === $duration ) {
			$playerhtml .= '<div class="sm2-row timestamp timetext" style="display:none;">';
		} else {
			$playerhtml .= '<div class="sm2-row timestamp timetext">';
		}

		if ( '0:00' !== $duration ) {
			$playerhtml .= '<div class="sm2-inline-time timestamp">0:00</div>';
			if ( $playlist ) {
				$playerhtml .= '<div class="sm2-inline-duration timestamp">0:00</div>';
			} else {
				$playerhtml .= '<div class="sm2-inline-duration timestamp">' . $duration . '</div>';
			}
		} else {
			$playerhtml .= '<div class="sm2-inline-time timestamp">0:00</div>';
			$playerhtml .= '<div class="sm2-inline-duration timestamp">...</div>';
		}

			$playerhtml .= '</div>
									</div>
								</div>';
		if ( $sidebar ) {
			$playerhtml .= '</div>';
		}
		if ( $playlist ) {
			$playerhtml .= '
									<div class="sm2-inline-element sm2-button-element sm2-menu">
										<div class="sm2-button-bd">
										 <a href="#menu" class="sm2-inline-button menu">menu</a>
										</div>
									</div>';
		}
		if ( ! $sidebar ) {
			$playerhtml .= '
								<div class="sm2-inline-element sm2-button-element sm2-volume" style="' . $spp_volume_button . '">
									   <div class="sm2-button-bd">
										<span class="sm2-inline-button sm2-volume-control volume-shade"></span>
										<a href="#volume" class="sm2-inline-button sm2-volume-control" style=""></a>
									   </div>
								  </div>
							</div>
							<div class="bd sm2-playlist-drawer sm2-element">
							';
		}

		if ( $sidebar ) {
			$playerhtml .= '
								<div class="bd sm2-playlist-drawer sm2-element">							
								<div class="sm2-inline-texture">
									<div class="sm2-box-shadow"></div>
								</div>';
		}
			$playerhtml         .= '
								<div class="sm2-playlist-wrapper">
									<ul class="sm2-playlist-bd">';
			$force_ssl_for_audio = Wp_Simplepodcastpress::spp_get_option( 'force_ssl_for_audio' );
			$_date               = '';
			$_duration           = '';
		foreach ( $podcast_items as $podcast_item ) {
			$audio_file = $podcast_item['pc_audio_file'];
			$title      = $podcast_item['pc_title'];
			if ( 'off' !== $ep_date ) {
				$pc_published_date = $podcast_item['pc_published_date'];
				$publishedate      = strtotime( $pc_published_date ) - Wp_Simplepodcastpress::get_unix_timeoffset();
				$publish_date      = gmdate( 'Y-m-d', $publishedate );
				$_date             = ' - (' . $publish_date . ')';
			}
			if ( 'off' !== $ep_duration ) {
				$_duration = ! empty( $podcast_item['pc_audio_duration'] ) ? $podcast_item['pc_audio_duration'] : '';
				$_duration = ' - (' . $_duration . ')';
			}
			if ( $force_ssl_for_audio ) {
				$audio_file = str_replace( 'http://', 'https://', $audio_file );
			}
			if ( $playlist ) {
				$playerhtml .= '<li><a href="' . $audio_file . '">' . $title . '' . $_duration . '' . $_date . '</a></li>';
			} else {
				$playerhtml .= '<li><a href="' . $audio_file . '">' . $title . '</a></li>';
			}
		}
				$playerhtml .= '</ul>
								</div>
							</div>
						</div>';
		return $playerhtml;
	}
	/**
	 * Generate JavaScript for the podcast player.
	 *
	 * This method generates the JavaScript code needed for the podcast player based on the provided parameters.
	 *
	 * @since 1.0.0
	 *
	 * @param int  $postid       The ID of the podcast post.
	 * @param bool $sidebar      Whether to display the player in the sidebar.
	 * @param bool $soundmanager Whether to use SoundManager for audio playback.
	 * @param bool $playlist     Whether to include a playlist.
	 *
	 * @return void Generated JavaScript for the podcast player.
	 */
	public static function player_js_fn( $postid, $sidebar = false, $soundmanager = false, $playlist = false ) {

		$js = '';

		if ( $soundmanager ) {
			if ( ! $sidebar ) {
				$js = 'jQuery(document).ready(function($) {
					jQuery("#sm2-' . $postid . ' .play-pause").on("click",function(){
							jQuery("#sm2-' . $postid . '").find(".timetext").css("display","table");
					});';
				if ( $playlist ) {
					$direct_download_button = wp_simplepodcastpress::spp_get_option( 'direct_download_button' );
					$audiodownloadurl       = '';
					if ( $direct_download_button ) {
						$audiodownloadurl = SPPRESS_PLUGIN_URL . '/responsive_audio_player/downloadaudio.php?file=';
					}

					$js .= '	
						var aurl = $("#sm2-' . $postid . ' .sm2-playlist-bd li:first").find("a").attr("href");
						aurl = "' . $audiodownloadurl . '"+aurl;

						jQuery(".button-download").attr("href", aurl);
						jQuery("#sm2-' . $postid . ' .sm2-playlist-bd li").on("click",function(){
								jQuery("#sm2-' . $postid . '").find(".timetext").css("display","table");
								var audiourl = jQuery(this).find("a").attr("href");
								audiourl = "' . $audiodownloadurl . '"+audiourl;
								jQuery(".button-download").attr("href", audiourl);
						});';
				}
			} else {

						$js = 'jQuery(document).ready(function($) {
					jQuery("#spp-pro-sidebar .play-pause").on("click",function(){
							jQuery("#spp-pro-sidebar").find(".timetext").css("display","table");
					});
					jQuery("#spp-pro-sidebar .sm2-playlist-bd li").on("click",function(){
							jQuery("#spp-pro-sidebar").find(".timetext").css("display","table");
					});
		';

			}
			if ( ! $sidebar ) {
				$js .= '

		$(".sm2-inline-button .menu").click(function() {
			$(".sm2-bar-ui .sm2-inline-element:first-of-type").css("border-bottom-left-radius", "0px");
		  });
	$(".playlist-open .sm2-inline-button .menu").click(function() {
			$(".sm2-bar-ui .sm2-inline-element:first-of-type").css("border-bottom-left-radius", "6px");
		  });
		jQuery(".timestamp").removeAttr("href");
		jQuery("#sm2-' . $postid . ' .playerpitch1").on("click",function(){
		  jQuery(this).css("display", "none");
		  jQuery("#sm2-' . $postid . ' .playerpitch2").css("display", "block");
			for (i = window.soundManager.soundIDs.length-1; i >= 0; i--) {
				 soundId = window.soundManager.soundIDs[i];
				 soundManager.setPlaybackRate(soundId, 1.25);
			}
		});
		jQuery("#sm2-' . $postid . ' .playerpitch2").on("click",function(){	  
		  jQuery(this).css("display", "none");
		  jQuery("#sm2-' . $postid . ' .playerpitch3").css("display", "block");
		 for (i = window.soundManager.soundIDs.length-1; i >= 0; i--) {
				 soundId = window.soundManager.soundIDs[i];
					soundManager.setPlaybackRate(soundId, 1.5);
		}
		});
		jQuery("#sm2-' . $postid . ' .playerpitch3").on("click",function(){
		  jQuery(this).css("display", "none");
		  jQuery("#sm2-' . $postid . ' .playerpitch4").css("display", "block");
			for (i = window.soundManager.soundIDs.length-1; i >= 0; i--) {
				 soundId = window.soundManager.soundIDs[i];
				soundManager.setPlaybackRate(soundId, 2);
			}
		});
		jQuery("#sm2-' . $postid . ' .playerpitch4").on("click",function(){
		  jQuery(this).css("display", "none");
		  jQuery("#sm2-' . $postid . ' .playerpitch0").css("display", "block");
			for (i = window.soundManager.soundIDs.length-1; i >= 0; i--) {
				 soundId = window.soundManager.soundIDs[i];
				soundManager.setPlaybackRate(soundId, 0.75);
			}
		});
		jQuery("#sm2-' . $postid . ' .playerpitch0").on("click",function(){
		  jQuery(this).css("display", "none");
		  jQuery("#sm2-' . $postid . ' .playerpitch1").css("display", "block");
			for (i = window.soundManager.soundIDs.length-1; i >= 0; i--) {
				 soundId = window.soundManager.soundIDs[i];
				soundManager.setPlaybackRate(soundId, 1);
			}
		});
		jQuery("#sm2-' . $postid . ' .eq").on("click",function(){	  
				 for (i = window.soundManager.soundIDs.length-1; i >= 0; i--) {
						 soundId = window.soundManager.soundIDs[i];
						soundManager.getSoundById(soundId).pause();
				}
						
		  });
			jQuery("#sm2-' . $postid . ' .secrewind").on("click",function(){	  
				 for (i = window.soundManager.soundIDs.length-1; i >= 0; i--) {
						 soundId = window.soundManager.soundIDs[i];
						 var spp15Ms = 15 * 1000;
						 var sppCurStartTimeMs = soundManager.getSoundById(soundId).position;
						 SetPosition = sppCurStartTimeMs - 	spp15Ms;
						 sound = window.soundManager.getSoundById(soundId);
						 sound.setPosition(SetPosition);
				}
						
		  });
			function fluctuate(bar) {
				var height = Math.floor(Math.random() * 25) + 1;
				//Animate the equalizer bar repeatedly
				bar.animate({
					height: height
				},150, function() {
					fluctuate($(this));
				});
			}

			jQuery(".bar").each(function(i) {
				fluctuate(jQuery(this));
			});
			jQuery(function() {
				jQuery("#consent").click(function() {
					if (jQuery(this).is(":checked")) {
						jQuery("#sppoptinboxsubmit").removeAttr("disabled");
					} else {
						jQuery("#sppoptinboxsubmit").attr("disabled", "disabled");
					}
				});
			});
';
			}
			if ( $playlist ) {
				$js .= '			
	var soundurl;
	soundurl = jQuery("#sm2-' . $postid . ' .sm2-playlist-bd li:first").find("a").attr("href");
	jQuery(\'.button-spplisten\').prop(\'href\', \'javascript:window.open("\'+ soundurl +\'","","width=500,height=250")\');
	jQuery("#sm2-' . $postid . ' .sm2-playlist-bd li").on("click",function(){
		var soundurl1;
		soundurl = jQuery(this).find("a").attr("href");
		//var audiourl = jQuery(this).find("a").attr("href");
		jQuery(".button-spplisten").attr("href", soundurl);
		//jQuery(\'.button-spplisten\').prop(\'href\', \'javascript:window.open("\'+ soundurl1 +\'","","width=500,height=250")\');
	});
	';
			}
			$jsid      = ( $sidebar ) ? '#spp-pro-sidebar' : '#sm2-' . $postid;
			$sidebarid = ( $sidebar ) ? '#spp-pro-sidebar ' : '';
			$js       .= '			
		jQuery(".download-box .eq").on("click",function(){	  
				 for (i = window.soundManager.soundIDs.length-1; i >= 0; i--) {
						 soundId = window.soundManager.soundIDs[i];
						soundManager.getSoundById(soundId).pause();
				}
						
		  });
function secsToTime(e){
					for (ids = window.soundManager.soundIDs.length-1; ids >= 0; ids--) {
						soundId = window.soundManager.soundIDs[ids];
					 //soundId = window.soundManager.soundIDs[0];
					 i = window.soundManager.getSoundById(soundId);

					
							if (null != i) var s = null !== i.duration ? i.duration : i.durationEstimate;
							if (null != i && null !== s && s > 0 &&  i.loaded) {
								var n = jQuery("#sm2-$postid .sm2-progress").outerWidth();
									a = s * e / n / 1e3;
									o = Math.floor(a % 60);
									r = Math.floor(a / 60 % 60);
									l = Math.floor(a / 3600);
									p = r + ":";
								p += o > 9 ? o : "0" + o, l > 0 && (p = r > 9 ? l + ":" + p : l + ":0" + p);
								return p;
							
						}
					}
	}
	var startCoords = {}, endCoords = {}; var totalwidth; var oldXPos;
		jQuery("' . $jsid . ' .sm2-progress-track").bind("touchstart", function(event) {
				
				jQuery("' . $jsid . ' .sm2-inline-time").css("display", "none");
				jQuery("' . $jsid . ' .sm2-inline-time1").css("display", "table-cell");


				jQuery("' . $jsid . ' .sm2-progress-bar").css("display", "none");
				jQuery("' . $jsid . ' .sm2-progress-bar1").css("display", "block");

				for (ids = window.soundManager.soundIDs.length-1; ids >= 0; ids--) {
					soundId = window.soundManager.soundIDs[ids];
					 i = window.soundManager.getSoundById(soundId);
					if (null != i &&  i.loaded) {
						isDragging = true;
					}else{
						isDragging = false;
					}
				}
			
			endCoords = event.originalEvent.targetTouches[0];
			oldXPos  = event.originalEvent.targetTouches[0].pageX;
			startCoords.pageX = event.originalEvent.targetTouches[0].pageX;
			startCoords.pageY = event.originalEvent.targetTouches[0].pageY;
		});

		jQuery("' . $jsid . ' .sm2-progress-track").bind("touchmove", function(event) {
			event.preventDefault();
					 if (isDragging)
					{
						 var touch = event.originalEvent.touches[0];
						  var elm = jQuery(this).offset();
						  var x = touch.pageX - elm.left;
						  var y = touch.pageY - elm.top;
						  totalwidth = jQuery("' . $jsid . ' .sm2-progress").width();
						  var percent =	x  /  totalwidth * 100;
							

						  if(x < jQuery(this).width() && x > 0){
							  if(y < jQuery(this).height() && y > 0){
									jQuery("' . $jsid . ' .sm2-progress-bar1").width(percent + "%");
									 jQuery("' . $jsid . ' .sm2-inline-time1").text(secsToTime(x));
							  }
						  }

								  
					}

			
			
		});

		jQuery("' . $jsid . ' .sm2-progress-track").bind("touchend", function(event) {
						var e = event.originalEvent.changedTouches[event.originalEvent.changedTouches.length - 1];
                                var n = jQuery(this).offset();
                                    pageX = e.pageX - n.left;
									var elm = jQuery(this).offset();
									var x = e.pageX - elm.left;
									var y = e.pageY - elm.top;
						if(x < jQuery(this).width() && x > 0){
							if(y < jQuery(this).height() && y > 0){
								for (ids = window.soundManager.soundIDs.length-1; ids >= 0; ids--) {
									soundId = window.soundManager.soundIDs[ids];
									var outerw = jQuery("' . $jsid . ' .sm2-progress").outerWidth();
									sound = window.soundManager.getSoundById(soundId);
										 
													if (sound.playState === 1) {
														if (null != i) var s = null !== sound.duration ? sound.duration : sound.durationEstimate;
														axes = pageX / outerw;
														sound.setPosition(axes*s);
														setTimeout(function (){
															jQuery("' . $sidebarid . '.sm2-progress-bar").css("display", "block");
															jQuery("' . $sidebarid . '.sm2-progress-bar1").css("display", "none");
															jQuery("' . $sidebarid . '.sm2-inline-time1").css("display", "none");
															jQuery("' . $sidebarid . '.sm2-inline-time").css("display", "table-cell");
															isDragging = false;
														;}, 1500);

													}
										}
									}
							}
		});
	
});
';

		} else {

			$spp_autoplay_podcast = wp_simplepodcastpress::spp_get_option( 'spp_autoplay_podcast' );

			if ( $spp_autoplay_podcast and is_single() ) {

				$js = ' 
			jQuery(document).ready(function($) {   
					setTimeout(function(){
							var audio = $(document).find(".sppaudioplayer" ).get(0);
							audio.play();
						
					   },1000);
			});
			';

			}

			if ( isset( $postid ) ) {
				$js .= '
				jQuery(document).ready(function($) {   
					jQuery(".player-' . $postid . ' .playerpitch").on(\'click\',function(){
					  jQuery(this).css(\'display\', \'none\');
					  jQuery(".player-' . $postid . ' .playerpitch1").css(\'display\', \'block\');
					  jQuery(".player-' . $postid . ' #spp-audio-player").get(0).playbackRate  = 1.5;
					});
					jQuery(".player-' . $postid . ' .playerpitch1").on(\'click\',function(){	  
					  jQuery(this).css(\'display\', \'none\');
					  jQuery(".player-' . $postid . ' .playerpitch2").css(\'display\', \'block\');
					  jQuery(".player-' . $postid . ' #spp-audio-player").get(0).playbackRate  = 2;
					});
					jQuery(".player-' . $postid . ' .playerpitch2").on(\'click\',function(){
					  jQuery(this).css(\'display\', \'none\');
					  jQuery(".player-' . $postid . ' .playerpitch").css(\'display\', \'block\');
					  jQuery(".player-' . $postid . ' #spp-audio-player").get(0).playbackRate  = 1;
					});
			});
			';
			}
		}

		$off_scrolling_text = wp_simplepodcastpress::spp_get_option( 'off_scrolling_text' );

		if ( $off_scrolling_text ) {
			$js .= ' 
			jQuery(window).load(function($) {   
					var s = jQuery(".player-' . $postid . ' .sm2-playlist-target ul li").html();
					if (typeof s != "undefined") {
					jQuery(".player-' . $postid . ' .sm2-playlist-target ul li").html(s.replace(/(<([^>]+)>)/ig,""));
					}
			});';

		}

		$spp_autoplay_podcast = wp_simplepodcastpress::spp_get_option( 'spp_autoplay_podcast' );

		if ( $spp_autoplay_podcast and is_single() ) {

			$js .= '		
					jQuery(document).ready(function($) {   
							setTimeout(function (){
								window.soundManager.getSoundById(window.soundId).play();
							;},1000 );
					});
			';

		}
		$btn_embed = wp_simplepodcastpress::spp_get_option( 'btn_embed' );

		if ( $btn_embed ) {

			$js .= '		
					jQuery(document).ready(function($) {   
						jQuery("#button-embed").click(function () {
							jQuery("#embed-model").dialog({modal: true, width: "auto" });
						});
					});
			';

		}

		wp_enqueue_script( 'inline-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/inline.js', array( 'jquery' ), false, false );
		wp_add_inline_script( 'inline-js', $js, 'after' );
	}

	/**
	 * Load the language files for the plugin.
	 *
	 * This method is responsible for loading the language files associated with the plugin.
	 *
	 * @since 1.0.0
	 *
	 * @param string $domain The text domain to use for translation.
	 */
	public function load_language( $domain ) {
		load_plugin_textdomain(
			$domain,
			false,
			$this->plugin_path . 'languages'
		);
	}
}
