<?php
/**
 * Plugin Name: Simple Podcast Press
 * Description: A Podcast Player Designed To Grow Your Audience and Build Your Email List On Autopilot.
 * Version: 3.0.0.6
 * Author: Hani Mourra
 * Plugin URI: https://www.simplepodcastpress.com/
 *
 * @package SimplePodcastPress
 */

define( 'SPPRESS_PLUGIN_VERSION', '3.0.0.6' );
define( 'SPPRESS_STORE_URL', 'https://simplepodcastpress.com' );
define( 'SPPRESS_ITEM_NAME', 'Simple Podcast Press' );

if ( ! class_exists( 'SPP_SL_Plugin_Updater' ) ) {
	include __DIR__ . '/updater/class-spp-sl-plugin-updater.php';
}

if ( ! function_exists( 'array_replace' ) ) {
	/** 
	 * Replaces elements from passed arrays into the first array.
	 *
	 * @param array &$target_array The target array.
	 * @param array ...$arrays The arrays to replace elements from.
	 *
	 * @return array The modified array.
	 */
	function array_replace( array &$target_array, array &...$arrays ) { // phpcs:ignore Generic.CodeAnalysis.UnusedFunctionParameter.FoundAfterLastUsed
		$args  = func_get_args();
		$count = func_num_args();

		for ( $i = 1; $i < $count; ++$i ) {
			if ( is_array( $args[ $i ] ) ) {
				foreach ( $args[ $i ] as $key => $val ) {
					$target_array[ $key ] = $val;
				}
			} else {
				trigger_error( // phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_trigger_error
					__FUNCTION__ . '(): Argument #' . ( $i + 1 ) . ' is not an array', // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					E_USER_WARNING
				);
				return null;
			}
		}

		return $target_array;
	}
}



/**
 * Update the Simple Podcast Press plugin.
 *
 * This function is responsible for updating the Simple Podcast Press plugin.
 * It retrieves the license key from the database and sets up the updater.
 *
 * @return void
 */
function spp_sl_sppress_plugin_updater() {
	global $pagenow;

	if ( 'plugin.php' === $pagenow ) {
		set_site_transient( 'update_plugins', null );
	}

	$license_key = trim( get_option( 'edd_sppress_license_key' ) );

	$edd_updater = new SPP_SL_Plugin_Updater(
		SPPRESS_STORE_URL,
		__FILE__,
		array(
			'version'   => SPPRESS_PLUGIN_VERSION,
			'license'   => $license_key,
			'item_name' => SPPRESS_ITEM_NAME,
			'author'    => 'Hani Mourra',
		)
	);
}
add_action( 'admin_init', 'spp_sl_sppress_plugin_updater', 0 );

require_once ABSPATH . 'wp-content/plugins/simple-podcast-press/class-wp-simplepodcastpress.php';
require_once ABSPATH . 'wp-includes/pluggable.php';

$ob_wp_simplepodcastpress = new Wp_Simplepodcastpress();

if ( isset( $ob_wp_simplepodcastpress ) ) {
	register_activation_hook( __FILE__, array( $ob_wp_simplepodcastpress, 'simplepodcastpress_activate' ) );
	register_deactivation_hook( __FILE__, array( $ob_wp_simplepodcastpress, 'simplepodcastpress_deactivate' ) );
}