<?php // phpcs:ignore Squiz.Commenting.FileComment.Missing
/*************************************************************************************************************
 * File class-spp-widget.php is a part of Simple Podcast Press and contains proprietary code - simplepodcastpress.com
 *
 * @package SimplePodcastPress
 *************************************************************************************************************/ // phpcs:ignore Squiz.Commenting.ClassComment.SpacingAfter

/**
 * Class Spp_Playlist_Widget
 *
 * This class defines a widget for displaying a playlist in Simple Podcast Press.
 *
 * @package SimplePodcastPress
 */
class Spp_Playlist_Widget extends WP_Widget {

	/**
	 * Default values for widget settings.
	 *
	 * @var array
	 */
	protected $defaults;

	/**
	 * Constructor method for the Spp_Playlist_Widget class.
	 *
	 * Initializes the widget with a unique ID, name, and description.
	 */
	public function __construct() {
		parent::__construct( 'latest-podcast-playlist', 'Latest Podcast Playlist', array( 'description' => 'Showcase Your Latest Podcast Playlist' ) );
	}

	/**
	 * Retrieves the thumbnail URL for a given image URL.
	 *
	 * @param string $image_url The URL of the image.
	 * @return string|false The URL of the thumbnail, or false if no thumbnail is found.
	 */
	public function spp_get_image_thumb( $image_url ) {
		$image_id = attachment_url_to_postid( $image_url );
		// retrieve the thumbnail size of our image.
		$image_thumb = wp_get_attachment_image_src( $image_id, 'thumbnail' );

		// display the image.
		return $image_thumb[0];
	}

	/**
	 * Widget function for displaying the Latest Podcast Playlist.
	 *
	 * @param array $args     Widget arguments.
	 * @param array $instance Widget instance settings.
	 */
	public function widget( $args, $instance ) {

		extract( $args ); // phpcs:ignore WordPress.PHP.DontExtract.extract_extract

		$title = apply_filters( 'widget_title', $instance['title'] );
		echo $before_widget;

		// Display the widget title.
		if ( $title ) {
			echo $before_title . $title . $after_title;
		}

		$spp_widget_art = $instance['select_widget_art_name'];
		$playlistopen   = '';
		global $wpdb;
		global $post;

		$table_spp_podcast = $wpdb->prefix . 'spp_podcast';

		if ( is_feed() ) {
			return;
		}

		$html           = '';
		$ctabuttonspart = '';
		$optinpart      = '';
		$poweredby      = '';
		$audiopart      = 'This feature is not available with your current player.  Switch to the "Pro Player" in the SPP setings to use this playlist widget.';

		$width = isset( $atts['width'] ) ? " style='max-width:{$atts['width']}; float:right; margin-bottom:10px;'" : '';

		$postid = is_object( $post ) ? $post->ID : '';

		$select_audio_player_skin = Wp_Simplepodcastpress::spp_get_option( 'select_audio_player_skin' );

		if ( 'soundmanager2_player' === $select_audio_player_skin ) {

			if ( isset( $atts['title'] ) ) {
				$title = $atts['title'];
			} else {
				$title = is_object( $post ) ? get_the_title( $post->ID ) : '';
			}

			if ( isset( $atts['image'] ) ) {
				$player_image = $atts['image'];
			}

				$widgetlist            = $instance['widgetlist'] ? 'true' : 'false';
				$spp_rewind_button     = Wp_Simplepodcastpress::spp_get_option( 'spp_rewind_button' );
				$rewind_button_display = ( 1 === $spp_rewind_button ) ? 'display:none;' : '';
				$table_spp_podcast     = $wpdb->prefix . 'spp_podcast';
			if ( $wpdb->get_var( "SHOW TABLES LIKE '$table_spp_podcast'" ) !== $table_spp_podcast ) { // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
				return;
			}

				$podcast_items = $wpdb->get_results( 'SELECT * FROM ' . $table_spp_podcast . " ORDER BY STR_TO_DATE(pc_published_date, 'D, d M Y H:i:s O') DESC" ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
				$podcast_array = array_filter( $podcast_items );

			if ( empty( $podcast_array ) ) {
				return;
			}
			foreach ( $podcast_items as $row ) {
						$result_array[] = (array) $row;
			}

			if ( ! function_exists( 'sort_function' ) ) {
				function sort_function( $a, $b ) { // phpcs:ignore Squiz.Commenting.FunctionComment.Missing
					return strtotime( $a['pc_published_date'] ) - strtotime( $b['pc_published_date'] );
				}
			}

				usort( $result_array, 'sort_function' );

			if ( 'on' === $instance['widgetlist'] ) {
						$playlistopen = 'playlist-open';
			}
			if ( 'on' !== $instance['listorder'] ) {
				$result_array = array_reverse( $result_array );
			}

			$mostRecent = current( $result_array ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

			switch ( $spp_widget_art ) {

				case 'no_image':
					$spp_channel_image = '';
					$art_style         = 'display:none;';
					$player_image      = '';

					break;
				case 'podcast_channel_art':
					echo 'channel_art';
					$spp_channel_image = Wp_Simplepodcastpress::spp_get_option( 'channel_image' );
					$player_image      = $this->spp_get_image_thumb( $spp_channel_image );

					break;
				case 'pd_episode_art':
					$spp_channel_image = $mostRecent['pc_episode_image']; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$player_image      = $spp_channel_image;

					break;
				case 'libsyn_image':
					$spp_channel_image = $mostRecent['pc_libsyn_image']; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$player_image      = $spp_channel_image;

					break;

				default:
					$spp_channel_image = Wp_Simplepodcastpress::spp_get_option( 'channel_image' );
					$player_image      = $this->spp_get_image_thumb( $spp_channel_image );

			}

				$audiopart                                   = '<div id="sm2-' . $postid . '" class="sm2-bar-ui compact flat full-width playlist full-width-player ' . $playlistopen . '">
								<div class="bd sm2-main-controls">
									<div class="sm2-inline-element sm2-button-element">
										<div class="sm2-button-bd playpausebutton" style="background: transparent url(' . $player_image . ') repeat scroll 0% 0% / cover ;">
										<div id="spp-player-background"> </div>
										<a href="#play" class="sm2-inline-button play-pause">Play / pause</a>
										<div class="eq">
											<span class="bar bar-1"></span>
											<span class="bar bar-2"></span>
											<span class="bar bar-3"></span>
											<span class="bar bar-4"></span>
										</div>
									</div>
								</div>
								<div class="sm2-inline-element sm2-inline-status progressbar" style="padding-right:0px !important;">

					
									<div class="sm2-playlist">
										<div class="sm2-playlist-target" style="max-width: 100% !important;"></div>								
									</div>
									<div class="sm2-progress">
											<div class="sm2-progress-bd">
												<div class="sm2-progress-track">
													<div class="sm2-progress-bar1" style="display:none;"></div>
													<div class="sm2-progress-bar"></div>
													<div class="sm2-progress-ball"><div class="icon-overlay"></div></div>
												</div>
											</div>
												
											<div class="sm2-row">
											<div class="sm2-inline-time1" style="display:none;">0:00</div>
											<div class="sm2-inline-time">0:00</div>
											<div class="sm2-inline-duration">0:00</div>
											
										</div>
									</div>
								</div>
								<div class="sm2-inline-element sm2-button-element sm2-menu">
							   <div class="sm2-button-bd">
								 <a href="#menu" class="sm2-inline-button menu">menu</a>
							   </div>
							  </div>
							</div>

							<div class="bd sm2-playlist-drawer sm2-element">
								
								<div class="sm2-playlist-wrapper">
									<ul class="sm2-playlist-bd">';
										$force_ssl_for_audio = Wp_Simplepodcastpress::spp_get_option( 'force_ssl_for_audio' );

			foreach ( $result_array as $podcast_item ) {
					$audio_file = $podcast_item['pc_audio_file'];
					$title      = $podcast_item['pc_title'];
				if ( $force_ssl_for_audio ) {
											$audio_file = str_replace( 'http://', 'https://', $audio_file );
				}
					$audiopart .= '<li><a href="' . $audio_file . '">' . $title . '</a></li>';
			}
						$audiopart .= '</ul>
								</div>
							</div>
						</div>';

			$spp_autoplay_podcast = Wp_Simplepodcastpress::spp_get_option( 'spp_autoplay_podcast' );
			$js = '';
			if ( 'pd_episode_art' === $spp_widget_art ) {
				$ajaxurl = admin_url( 'admin-ajax.php' );
				$js      = <<<HTML

		jQuery(document).ready(function($) {
			jQuery("#sm2-$postid .sm2-playlist-bd li").on("click",function(){
					var audiourl = jQuery(this).find("a").attr("href");
					var data = 'audiourl=' + audiourl + '&action=change_image_ajax';
					jQuery.post('$ajaxurl', data, function(response) {
  				  		// Log the response to the console
						$(".playpausebutton").css("background-image", "url(" + response + ")");
					});
			});

		});

HTML;
				wp_enqueue_script( 'inline-new-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/inline-new.js', array( 'jquery' ), 1.0, false );
				wp_add_inline_script( 'inline-new-js', $js, 'after' );
			}
			if ( $spp_autoplay_podcast ) {

			$js .= <<<HTML
		jQuery(document).ready(function($) {   
				setTimeout(function (){
						 soundId = window.soundManager.soundIDs[0];
						  //soundManager.getSoundById(soundId).play();
				;},1000 );
		});
HTML;

				wp_enqueue_script( 'inline-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/inline.js', array( 'jquery' ), 1, false );
				wp_add_inline_script( 'inline-js', $js, 'after' );

			}

			$js         = <<<HTML
	jQuery(document).ready(function($) {   
		
		jQuery("#sm2-$postid .eq").on('click',function(){	  
				 for (i = window.soundManager.soundIDs.length-1; i >= 0; i--) {
						 soundId = window.soundManager.soundIDs[i];
						soundManager.getSoundById(soundId).pause();
				}
						
		  });
			jQuery("#sm2-$postid .secrewind").on('click',function(){	  
				 
				 for (i = window.soundManager.soundIDs.length-1; i >= 0; i--) {
						 soundId = window.soundManager.soundIDs[i];
						 var spp15Ms = 7.5 * 1000;
						 var sppCurStartTimeMs = soundManager.getSoundById(soundId).position;
						 SetPosition =  sppCurStartTimeMs - spp15Ms;
						 sound = window.soundManager.getSoundById(soundId);
						 sound.setPosition(SetPosition);
				}
						
		  });
			function fluctuate(bar) {
				var height = Math.floor(Math.random() * 25) + 1;
				//Animate the equalizer bar repeatedly
				bar.animate({
					height: height
				},150, function() {
					fluctuate($(this));
				});
			}

			jQuery(".bar").each(function(i) {
				fluctuate(jQuery(this));
			});
function secsToTime(e){
					for (ids = window.soundManager.soundIDs.length-1; ids >= 0; ids--) {
						soundId = window.soundManager.soundIDs[ids];
					 //soundId = window.soundManager.soundIDs[0];
					 i = window.soundManager.getSoundById(soundId);

					
							if (null != i) var s = null !== i.duration ? i.duration : i.durationEstimate;
							if (null != i && null !== s && s > 0 &&  i.loaded) {
								var n = jQuery("#sm2-$postid .sm2-progress").outerWidth();
									a = s * e / n / 1e3;
									o = Math.floor(a % 60);
									r = Math.floor(a / 60 % 60);
									l = Math.floor(a / 3600);
									p = r + ":";
								p += o > 9 ? o : "0" + o, l > 0 && (p = r > 9 ? l + ":" + p : l + ":0" + p);
								return p;
							
						}
					}
	}
	var startCoords = {}, endCoords = {}; var totalwidth; var oldXPos;
		jQuery("#sm2-$postid .sm2-progress-track").bind("touchstart", function(event) {
				
				jQuery('.sm2-inline-time').css('display', 'none');
				jQuery('.sm2-inline-time1').css('display', 'table-cell');


				jQuery('.sm2-progress-bar').css('display', 'none');
				jQuery('.sm2-progress-bar1').css('display', 'block');

				for (ids = window.soundManager.soundIDs.length-1; ids >= 0; ids--) {
					soundId = window.soundManager.soundIDs[ids];
					 i = window.soundManager.getSoundById(soundId);
					if (null != i &&  i.loaded) {
						isDragging = true;
					}else{
						isDragging = false;
					}
				}
			
			endCoords = event.originalEvent.targetTouches[0];
			oldXPos  = event.originalEvent.targetTouches[0].pageX;
			startCoords.pageX = event.originalEvent.targetTouches[0].pageX;
			startCoords.pageY = event.originalEvent.targetTouches[0].pageY;
		});

		jQuery("#sm2-$postid .sm2-progress-track").bind("touchmove", function(event) {
			event.preventDefault();
					 if (isDragging)
					{
						 var touch = event.originalEvent.touches[0];
						  var elm = jQuery(this).offset();
						  var x = touch.pageX - elm.left;
						  var y = touch.pageY - elm.top;
						  totalwidth = jQuery('#sm2-$postid .sm2-progress').width();
						  var percent =	x  /  totalwidth * 100;
							

						  if(x < jQuery(this).width() && x > 0){
							  if(y < jQuery(this).height() && y > 0){
									jQuery('#sm2-$postid .sm2-progress-bar1').width(percent + '%');
									 jQuery('#sm2-$postid .sm2-inline-time1').text(secsToTime(x));
							  }
						  }

								  
					}

			
			
		});

		jQuery("#sm2-$postid .sm2-progress-track").bind("touchend", function(event) {
						var e = event.originalEvent.changedTouches[event.originalEvent.changedTouches.length - 1];
                                var n = jQuery(this).offset();
                                    pageX = e.pageX - n.left;
									var elm = jQuery(this).offset();
									var x = e.pageX - elm.left;
									var y = e.pageY - elm.top;
						if(x < jQuery(this).width() && x > 0){
							if(y < jQuery(this).height() && y > 0){
								for (ids = window.soundManager.soundIDs.length-1; ids >= 0; ids--) {
									soundId = window.soundManager.soundIDs[ids];
									var outerw = jQuery("#sm2-$postid .sm2-progress").outerWidth();
									sound = window.soundManager.getSoundById(soundId);
										 
													if (sound.playState === 1) {
														if (null != i) var s = null !== sound.duration ? sound.duration : sound.durationEstimate;
														axes = pageX / outerw;
														sound.setPosition(axes*s);
														setTimeout(function (){
															jQuery('.sm2-progress-bar').css('display', 'block');
															jQuery('.sm2-progress-bar1').css('display', 'none');
															jQuery('.sm2-inline-time1').css('display', 'none');
															jQuery('.sm2-inline-time').css('display', 'table-cell');
															isDragging = false;
														;}, 1500);

													}
										}
									}
							}
		});

});
HTML;
			$spp_player = new SPPress_Audio_Player();
			$spp_player->sppress_enqueue_scripts_css();
			$spp_player->sppress_enqueue_scripts_js();
			$spp_player->two_step_optin_fn();
			wp_enqueue_script( 'inline-js', SPPRESS_PLUGIN_URL . '/responsive_audio_player/js/inline.js', array( 'jquery' ), 1.1, false );
			wp_add_inline_script( 'inline-js', $js, 'after' );
		}
		echo $audiopart; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		echo $after_widget;
	}

	/**
	 * Updates a widget instance after modifying the widget settings.
	 *
	 * @param array $new_instance The new widget settings.
	 * @param array $old_instance The previous widget settings.
	 * @return array The updated widget settings.
	 */
	public function update( $new_instance, $old_instance ) {
		$instance = $old_instance;
		// The update for the variable of the checkbox.
		$instance['title'] = ( ! empty( $new_instance['title'] ) ) ? wp_strip_all_tags( $new_instance['title'] ) : '';

		if ( isset( $new_instance['listorder'] ) ) {
			$instance['listorder'] = $new_instance['listorder'];
		} else {
			$instance['listorder'] = '';
		}

		if ( isset( $new_instance['widgetlist'] ) ) {
			$instance['widgetlist'] = $new_instance['widgetlist'];
		} else {
			$instance['widgetlist'] = '';
		}
		if ( isset( $new_instance['select_widget_art_name'] ) ) {
			$instance['select_widget_art_name'] = $new_instance['select_widget_art_name'];
		} else {
			$instance['select_widget_art_name'] = '';
		}

		return $instance;
	}

	/**
	 * Generates the widget form in the WordPress admin area.
	 *
	 * @param array $instance The current widget settings.
	 */
	public function form( $instance ) {

		$defaults = array(
			'title'      => __( 'Latest Podcast Episode', 'wptheme' ),
			'listorder'  => 'off',
			'widgetlist' => 'off',
		);
		if ( isset( $instance['title'] ) ) {
			$title = $instance['title'];
		} else {
			$title = 'Latest Podcast Episode';
		}
		$instance           = wp_parse_args( (array) $instance, $defaults );
		$itunes_widget_id   = $this->get_field_id( 'selected_itunes_id' );
		$itunes_widget_name = $this->get_field_name( 'selected_itunes_id' );
		if ( isset( $instance['select_widget_art_name'] ) ) {
			$select_widget_art = $instance['select_widget_art_name'];
		} else {
			$select_widget_art = '';
		}
		$select_widget_art_id   = $this->get_field_id( 'select_widget_art_id' );
		$select_widget_art_name = $this->get_field_name( 'select_widget_art_name' );
		?>
		<!-- The checkbox -->
		<p>
		<label for="<?php echo esc_html( $this->get_field_id( 'title' ) ); ?>"><?php esc_html_e( 'Title:' ); ?></label> 
		<input class="widefat" id="<?php echo esc_html( $this->get_field_id( 'title' ) ); ?>" name="<?php echo esc_html( $this->get_field_name( 'title' ) ); ?>" type="text" value="<?php echo esc_attr( $title ); ?>" />
		</p>
	<p>
		<input class="checkbox" type="checkbox" <?php checked( $instance['widgetlist'], 'on' ); ?> id="<?php echo esc_html( $this->get_field_id( 'widgetlist' ) ); ?>" name="<?php echo esc_html( $this->get_field_name( 'widgetlist' ) ); ?>" /> 
		<label for="<?php echo esc_html( $this->get_field_id( 'widgetlist' ) ); ?>">Display Open Playlist</label>
	</p>
		<p>
		<input class="checkbox" type="checkbox" <?php checked( $instance['listorder'], 'on' ); ?> id="<?php echo $this->get_field_id( 'listorder' ); ?>" name="<?php echo $this->get_field_name( 'listorder' ); ?>" /> <?php // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped,WordPress.Security.EscapeOutput.OutputNotEscaped ?>
		<label for="<?php echo esc_html( $this->get_field_id( 'listorder' ) ); ?>">Reverse Playlist (Oldest Episodes on Top)</label>
	</p>
		<select name="<?php echo $select_widget_art_name; ?>" id="<?php echo $select_widget_art_id; ?>" class="postform span10" style="padding: 0px ! important; width: 100%;" > <?php // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
		<option  value="no_image" 
		<?php
		if ( 'no_image' === $select_widget_art ) {
			echo 'selected=selected';}
		?>
		>No Image</option>
		<option  value="podcast_channel_art" 
		<?php
		if ( 'podcast_channel_art' === $select_widget_art ) {
			echo 'selected=selected'; }
		?>
		>Podcast channel art</option>
		<option  value="pd_episode_art"  
		<?php
		if ( 'pd_episode_art' === $select_widget_art ) {
			echo 'selected=selected'; }
		?>
		>Podcast Episode Art from Feed</option>
		<option  value="libsyn_image" 
		<?php
		if ( 'libsyn_image' === $select_widget_art ) {
			echo 'selected=selected'; }
		?>
		>Libsyn Page Image</option>
		</select>
		<?php
	}
}
?>