<?php
/**
 * Simple Podcast Press Widget File
 *
 * This file is a part of Simple Podcast Press and contains proprietary code.
 * Website: simplepodcastpress.com
 *
 * @package SimplePodcastPress
 */

/**
 * Simple Podcast Press Widget Class
 *
 * This class extends the WordPress WP_Widget class to create a widget for displaying the latest podcast episode.
 *
 * @package SimplePodcastPress
 */
class Spp_Widget extends WP_Widget {
	/**
	 * Default settings for the widget.
	 *
	 * @var array
	 */
	protected $defaults;
	/**
	 * Constructor method for the spp_widget class.
	 */
	public function __construct() {
		parent::__construct( 'latest-podcast', 'Latest Podcast Episode', array( 'description' => 'Showcase Your Latest Podcast Episode' ) );
		$this->defaults = array(
			'select_widget_art_name' => 'No Image',
		);
	}
	/**
	 * Get the URL of the medium-sized thumbnail for a given image URL.
	 *
	 * @param string $image_url The URL of the image.
	 * @return string The URL of the medium-sized thumbnail, or the original image URL if no thumbnail is found.
	 */
	public function spp_get_image_thumb( $image_url ) {
		$postid = attachment_url_to_postid( $image_url );

		// Retrieve the medium-sized thumbnail for our image.
		$image_thumb = wp_get_attachment_image_src( $postid, 'medium' );

		// If medium-sized thumbnail is not available, return the original image URL.
		if ( empty( $image_thumb[0] ) ) {
			return $image_url;
		} else {
			return $image_thumb[0];
		}
	}

	/**
	 * Outputs the content of the widget.
	 *
	 * @param array $args     The widget arguments.
	 * @param array $instance The widget instance settings.
	 */
	public function widget( $args, $instance ) {
		extract( $args ); // phpcs:ignore WordPress.PHP.DontExtract.extract_extract

		$title = apply_filters( 'widget_title', $instance['title'] );

		echo $before_widget;

		// Display the widget title.
		if ( $title ) {
			echo $before_title . $title . $after_title;
		}

		global $wpdb;

		$spp_widget_art    = $instance['select_widget_art_name'];
		$table_spp_podcast = $wpdb->prefix . 'spp_podcast';
		if ( $wpdb->get_var( "SHOW TABLES LIKE '$table_spp_podcast'" ) !== $table_spp_podcast ) { // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
					return;
		}
		$itunes_url = Wp_Simplepodcastpress::spp_get_option( 'btn_itunes_url' );
		if ( empty( $itunes_url ) ) {
			$itunes_url = Wp_Simplepodcastpress::spp_get_option( 'itunes_url' );
		}

		$query = "SELECT * FROM {$table_spp_podcast}";

		$results = $wpdb->get_results( $query ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.NoCaching,WordPress.DB.DirectDatabaseQuery.DirectQuery
		if ( empty( $results ) ) {
			$html = 'No podcast episodes found';
			echo $html; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			echo $after_widget;
			return;
		}

		foreach ( $results as $row ) {
			$result_array[] = (array) $row;
		}

		if ( ! function_exists( 'sortFunction' ) ) {
			function sortFunction( $a, $b ) { // phpcs:ignore WordPress.NamingConventions.ValidFunctionName.FunctionNameInvalid,Squiz.Commenting.FunctionComment.Missing
				return strtotime( $a['pc_published_date'] ) - strtotime( $b['pc_published_date'] );
			}
		}

		$art_style = '';
		usort( $result_array, 'sortFunction' );

		$most_recent = end( $result_array );
		switch ( $spp_widget_art ) {

			case 'no_image':
				$spp_channel_image = '';
				$art_style         = 'display:none;';

				break;
			case 'podcast_channel_art':
				$spp_channel_image = Wp_Simplepodcastpress::spp_get_option( 'channel_image' );

				break;
			case 'pd_episode_art':
				$spp_channel_image = $most_recent['pc_episode_image'];

				break;
			case 'libsyn_image':
				$spp_channel_image = $most_recent['pc_libsyn_image'];

				break;

			default:
				$spp_channel_image = Wp_Simplepodcastpress::spp_get_option( 'channel_image' );

		}
		$postid = attachment_url_to_postid( $spp_channel_image );

		$spp_channel_image_url = $this->spp_get_image_thumb( $spp_channel_image );

		$audio_file = $most_recent['pc_audio_file'];
		$title      = $most_recent['pc_title'];
		$duration   = $most_recent['pc_audio_duration'];

		if ( empty( $audio_file ) ) {

				$p_p_general = Wp_Simplepodcastpress::spp_get_option( 'powerpress_general' );
				$meta_data   = get_post_meta( $post->ID, 'enclosure', true );
				$meta_parts  = explode( "\n", $meta_data, 4 );
				$audio_file  = trim( $meta_parts[0] );

				$audio_url_parts = wp_parse_url( $audio_file );

			if ( ! empty( $p_p_general['redirect1'] ) ) {

				$audio_file = $p_p_general['redirect1'] . 'p/' . $audio_url_parts['host'] . $audio_url_parts['path'];

			}
				$duration = unserialize( $meta_parts[3] ); // phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.serialize_unserialize

				$duration = $duration['duration'];

		}
				$title = str_replace( '[', '&#91;', $title );

				$force_ssl_for_audio = Wp_Simplepodcastpress::spp_get_option( 'force_ssl_for_audio' );
		if ( $force_ssl_for_audio ) {
			$audio_file = str_replace( 'http://', 'https://', $audio_file );
		}
		?>
			<div class="download-box"><?php echo do_shortcode( '[spp-audioonly postid="' . $postid . '" url="' . $audio_file . '" title="' . htmlspecialchars( $title ) . '" image="false" widget="true" player_image="' . $spp_channel_image . '" duration="' . $duration . '"]' ); ?>
			<img style="margin-top: 0px !important; margin-bottom:0px !important; <?php echo esc_html( $art_style ); ?>" width="100%" height="auto" src="<?php echo esc_url( $spp_channel_image_url ); ?>" >
			<a class="button-sppsidebar" target="_blank" href="<?php echo esc_url( $itunes_url ); ?>">Subscribe on Apple Podcasts</a></div>




		<?php
		echo $after_widget;
	}

	/**
	 * Updates a widget instance after modifying the widget settings.
	 *
	 * @param array $new_instance The new widget settings.
	 * @param array $old_instance The previous widget settings.
	 * @return array The updated widget settings.
	 */
	public function update( $new_instance, $old_instance ) {
		$instance = $old_instance;

		// Strip tags from title and name to remove HTML.
		$instance['title']                  = ( ! empty( $new_instance['title'] ) ) ? wp_strip_all_tags( $new_instance['title'] ) : '';
		$instance['select_widget_art_name'] = $new_instance['select_widget_art_name'];
		return $instance;
	}

	/**
	 * Outputs the settings form for the widget.
	 *
	 * @param array $instance The current widget settings.
	 */
	public function form( $instance ) {
		$instance = wp_parse_args( (array) $instance, $this->defaults );
		if ( isset( $instance['title'] ) ) {
			$title = $instance['title'];
		} else {
			$title = 'Latest Podcast Episode';
		}
		$itunes_widget_id       = $this->get_field_id( 'selected_itunes_id' );
		$itunes_widget_name     = $this->get_field_name( 'selected_itunes_id' );
		$select_widget_art      = $instance['select_widget_art_name'];
		$select_widget_art_id   = $this->get_field_id( 'select_widget_art_id' );
		$select_widget_art_name = $this->get_field_name( 'select_widget_art_name' );
		?>
		<p>
		<label for="<?php echo esc_html( $this->get_field_id( 'title' ) ); ?>"><?php esc_html_e( 'Title:' ); ?></label> 
		<input class="widefat" id="<?php echo esc_html( $this->get_field_id( 'title' ) ); ?>" name="<?php echo esc_html( $this->get_field_name( 'title' ) ); ?>" type="text" value="<?php echo esc_attr( $title ); ?>" />
		</p>
		<select name="<?php echo esc_html( $select_widget_art_name ); ?>" id="<?php echo esc_html( $select_widget_art_id ); ?>" class="postform span10" style="padding: 0px ! important; width: 100%;" >
		<option  value="no_image" 
		<?php
		if ( 'no_image' === $select_widget_art ) {
			echo 'selected=selected';}
		?>
		>No Image</option>
		<option  value="podcast_channel_art" 
		<?php
		if ( 'podcast_channel_art' === $select_widget_art ) {
			echo 'selected=selected'; }
		?>
		>Podcast channel art</option>
		<option  value="pd_episode_art"  
		<?php
		if ( 'pd_episode_art' === $select_widget_art ) {
			echo 'selected=selected'; }
		?>
		>Podcast Episode Art from Feed</option>
		<option  value="libsyn_image" 
		<?php
		if ( 'libsyn_image' === $select_widget_art ) {
			echo 'selected=selected'; }
		?>
		>Libsyn Page Image</option>
		</select>

		<?php
	}
}
?>