<?php // phpcs:ignore Squiz.Commenting.FileComment.Missing
/**
 * Class Wp_Simplepodcastpress
 *
 * This is a part of Simple Podcast Press and contains proprietary code - simplepodcastpress.com.
 *
 * @package Your_Plugin_Package
 */
class Wp_Simplepodcastpress {

	/**
	 * The settings for WP Simple Podcast Press.
	 *
	 * @var array
	 */
	public $spp_settings;


	/**
	 * Class Construct.
	 */
	public function __construct() {

		// Global variable to hold the database version for Simple Podcast Press.
		global $spp_db_version;
		$spp_db_version = '1.3';

		// Global variable to hold the current version of Simple Podcast Press.
		global $sppress_current_version;
		$sppress_current_version = SPPRESS_PLUGIN_VERSION;
		define( 'SPPRESS_PLUGIN_URL', plugins_url() . '/simple-podcast-press' );
		set_time_limit( 300 );
		if ( ! defined( 'WP_MEMORY_LIMIT' ) ) {
			define( 'WP_MEMORY_LIMIT', '64M' );
		}
		$dir = plugin_dir_path( __FILE__ );
		define( 'SPPRESS_PLUGIN_PATH', $dir );
		$new_i_nstall       = false;
		$this->spp_settings = get_option( 'spp_settings' );
		// Hani.
		$disable_url_shortner = $this->spp_get_option( 'disable_url_shortner' );
		// If disable URL shortner doesn't exist, treat it as true (i.e disable the spp redirect action).
		if ( '' === $disable_url_shortner ) {
			$new_i_nstall = true;
		}

		if ( ! is_array( $this->spp_settings ) ) {
			$this->spp_settings = array();
			update_option( 'spp_settings', $this->spp_settings );
		}

		if ( is_admin() ) {

			add_action(
				'admin_init',
				array(
					$this,
					'migrate_spp_settings_to_array',
				)
			);
			add_action(
				'admin_menu',
				array(
					$this,
					'spp_admin_menu',
				)
			);
			add_action(
				'wp_ajax_save_spp_settings',
				array(
					$this,
					'spp_save_podcast_settings',
				)
			);
			add_action(
				'wp_ajax_spp_url_shortner',
				array(
					$this,
					'spp_url_shortner_func',
				)
			);
			add_action(
				'wp_ajax_change_image_ajax',
				array(
					$this,
					'change_image_ajax_func',
				)
			);

			add_action(
				'admin_enqueue_scripts',
				array(
					$this,
					'register_spp_admin_scripts',
				)
			);
			add_action(
				'add_meta_boxes',
				array(
					$this,
					'spp_metaboxes',
				)
			);
			add_action(
				'save_post',
				array(
					$this,
					'spp_metabox_save',
				)
			);
			// Actions to take when plugin is updated.
			add_action(
				'admin_init',
				array(
					$this,
					'spp_plugin_update',
				),
				1
			);
			add_action(
				'draft_to_publish',
				array(
					$this,
					'preserve_draft_date',
				),
				10,
				1
			);
			add_action(
				'draft_to_private',
				array(
					$this,
					'preserve_draft_date',
				),
				10,
				1
			);
			add_action(
				'draft_to_pending',
				array(
					$this,
					'preserve_draft_date',
				),
				10,
				1
			);
			add_action(
				'wp_dashboard_setup',
				array(
					$this,
					'reviews_dashboard_widgets',
				)
			);
			add_action(
				'publish_post',
				array(
					$this,
					'manual_spp_action',
				),
				10,
				1
			);
			add_action(
				'save_post',
				array(
					$this,
					'manual_spp_action',
				),
				1,
				1
			);
			add_action(
				'wp_ajax_buttons_order_list_update',
				array(
					$this,
					'botton_order_list',
				)
			);
			add_action(
				'spp_plugin_updated',
				array(
					$this,
					'spp_save_plugin_settings',
				)
			);
			if ( ! $disable_url_shortner ) {
				add_action(
					'save_post',
					array(
						$this,
						'spp_link_metabox_save',
					)
				);
			}
				$this->remove_duplicate_images();

		} else {
			// Add shortcode support in widgets.
			add_filter( 'widget_text', 'do_shortcode', 11 );

			if ( ( ! $disable_url_shortner ) && ( ! $new_i_nstall ) ) {
				add_action(
					'init',
					array(
						$this,
						'spp_redirect',
					)
				); // Redirect.
			}

			// Force player on home page.
			$spp_force_player_home = $this->spp_get_option( 'spp_force_player_home' );
			if ( $spp_force_player_home ) {
				add_filter(
					'the_excerpt',
					array(
						$this,
						'spp_the_content',
					)
				);
			}

				add_filter(
					'the_content',
					array(
						$this,
						'spp_the_content',
					)
				);

			add_filter(
				'wp_audio_shortcode_override',
				array(
					$this,
					'spp_wp_audio_override',
				),
				10,
				4
			);
			add_action(
				'wp_head',
				array(
					$this,
					'spp_open_graph',
				),
				10,
				1
			);

			$replace = $this->spp_get_option( 'replace_pp_with_spp' );
			if ( $replace ) {
				add_action(
					'wp_footer',
					array(
						$this,
						'spp_hide_pp',
					),
					1,
					1
				);
			}
		}
		add_action(
			'simplepodcastpress_reviews_fetch',
			array(
				$this,
				'spp_save_reviews',
			)
		);
		// Set cron regardless since we dont know what type of feed it is at this point.
		add_action(
			'simplepodcastpress_fetch',
			array(
				$this,
				'generate_post_with_cron',
			)
		);
		add_action( 'admin_notices', array( $this, 'spp_plugin_notice' ) );

		if ( ! wp_next_scheduled( 'simplepodcastpress_fetch' ) ) {
			wp_schedule_event( time() + 60, 'hourly', 'simplepodcastpress_fetch' );
		}
		add_action(
			'spp_episodes_cron',
			array(
				$this,
				'spp_episodes_cron_fn',
			)
		);

		add_action(
			'widgets_init',
			array(
				$this,
				'spp_playlist_widget',
			)
		);
		add_action(
			'widgets_init',
			array(
				$this,
				'spp_widget',
			)
		);

		if ( ! wp_next_scheduled( 'spp_episodes_cron' ) ) {
			wp_schedule_event( time() + 60, 'hourly', 'spp_episodes_cron' );
		}

		if ( ! wp_next_scheduled( 'simplepodcastpress_reviews_fetch' ) ) {
			wp_schedule_event( time(), 'daily', 'simplepodcastpress_reviews_fetch' );
		}
		include_once 'incl/custom-fields-gutenberg/spp-custom-fields-display.php';
		include_once 'widget/class-spp-widget.php';
		include_once 'widget/class-spp-playlist-widget.php';
		include_once 'updater/spp-update.php';
		include_once 'draftnotify/draft-notify.php';
		include_once 'spp_tweet/spp-tweet.php';
		include_once 'responsive_audio_player/responsive-audio-player.php';
		include_once 'spp_reviews/spp-reviews.php';
	} //    End of defining constructor

	/**
	 * Register the Simple Podcast Press Widget.
	 *
	 * This function registers a widget class named 'spp_widget'.
	 * Widgets are used to add various features or content blocks to a website's sidebar or other widget areas.
	 */
	public function spp_widget() {
		register_widget( 'Spp_Widget' );
	}
	/**
	 * Registers the SPP Playlist Widget.
	 *
	 * This function registers a widget class named 'spp_playlist_widget'.
	 * Widgets are used to add various features or content blocks to a website's sidebar or other widget areas.
	 */
	public function spp_playlist_widget() {
		register_widget( 'Spp_Playlist_Widget' );
	}

	/**
	 * Function to handle cron job for processing episodes.
	 *
	 * @return integer which is true or false mostly is 1
	 */
	public function spp_episodes_cron_fn() {
		global $wpdb;
		global $post;

		$table_spp_podcast = $wpdb->prefix . 'spp_podcast';
		$table_spp_episode = $wpdb->prefix . 'spp_episode';
		$episodes          = $wpdb->get_results( 'SELECT * FROM ' . $table_spp_podcast ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching,WordPress.DB.DirectDatabaseQuery.DirectQuery
		$audiodownloadurl  = '';
		$direct_download_button = self::spp_get_option( 'direct_download_button' );

		if ( $direct_download_button ) {
			$audiodownloadurl = SPPRESS_PLUGIN_URL . '/responsive_audio_player/downloadaudio.php?file=';
		}

		foreach ( $episodes as $episode ) {

			$audio_file = $episode->pc_audio_file;

			$path_parts = pathinfo( $audio_file );
			$audio_file = $path_parts['basename'];

			if ( strpos( $audio_file, '?' ) ) {
				$audio_file = substr( $audio_file, 0, strpos( $audio_file, '?' ) );
			}

			$postid      = 0;
			$posttitle   = esc_sql( $episode->pc_title );
			$postdate    = $episode->pc_published_date;
			$downloadurl = $audiodownloadurl . $episode->pc_audio_file;
			$pc_id       = $episode->pc_id;
			$post_metas  = $wpdb->get_results( // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
				$wpdb->prepare(
					"SELECT * FROM {$wpdb->prefix}postmeta WHERE meta_value LIKE %s ORDER BY post_id",
					'%' . $wpdb->esc_like( $audio_file ) . '%'
				)
			);
			foreach ( $post_metas as $post_meta ) {
				if ( ( empty( $post_meta->post_id ) ) && ( 'enclousure' !== $post_meta->meta_key ) && ( '_audiourl' !== $post_meta->meta_key ) && ( '_useraudiourl' !== $post_meta->meta_key ) ) {
					continue;
				} elseif ( get_post_status( $post_meta->post_id ) === 'publish' ) {
						$postid = $post_meta->post_id;
				}
			}

			$is_in_db = $wpdb->get_var( $wpdb->prepare( 'SELECT podcast_table_id FROM ' . $table_spp_episode .' WHERE podcast_table_id=%d', $pc_id ) ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching

			if ( ! $is_in_db ) {

				$wpdb->query( // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
					$wpdb->prepare(
						'INSERT INTO ' .$table_spp_episode . ' (postid,post_date,posttitle,downloadurl,podcast_table_id) VALUES(%d,%s,%s,%s,%d)',						
						$postid,
						$postdate,
						$posttitle,
						$downloadurl,
						$pc_id
					)
				);
			}
		}
		return 1;
	}
	/**
	 * Ajax function for change image.
	 *
	 * @return void
	 */
	public function change_image_ajax_func() {
		if ( isset( $_REQUEST['audiourl'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$audio_file = sanitize_text_field( wp_unslash( $_REQUEST['audiourl'] ) );  // phpcs:ignore WordPress.Security.NonceVerification.Recommended,WordPress.Security.NonceVerification.Recommended
			global $wpdb;
			global $post;

			$table_spp_podcast = $wpdb->prefix . 'spp_podcast';

			$record     = $wpdb->get_row( // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
				$wpdb->prepare(
					"SELECT * FROM {$table_spp_podcast} WHERE pc_audio_file LIKE %s",
					'%' . $audio_file . '%'
				)
			);

			if ( $record && isset( $record->pc_episode_image ) ) {
				$pc_episode_image = $record->pc_episode_image;
				echo esc_url( $pc_episode_image );
			}
		}
		exit;
	}
	/**
	 * Plugin Notice function.
	 *
	 * @return void
	 */
	public function spp_plugin_notice() {
		if ( version_compare( PHP_VERSION, '7' ) < 0 ) {

			// Show a message inside the dashboard.
			if ( is_admin() ) {
				?>
			<div class="error below-h2">
				<p>
				<?php
				printf( __( 'Simple Podcast Press requires at least PHP 7. You have %s, Contact with your hosting comapny to update PHP Version.' ), PHP_VERSION ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped,WordPress.WP.I18n.MissingTranslatorsComment
				?>
				</p>
			</div>
				<?php
			}
		}
	}

	/**
	 * Player embed function.
	 *
	 * @param string $page_template The template for the page.
	 * @return string $page_template The modified template for the page.
	 */
	public function spp_embed_template( $page_template ) {
		if ( is_page( 'spp-player-embed' ) ) {
			$page_template = __DIR__ . '/spp_template/spp-embed.php';
		}
		return $page_template;
	}

	/**
	 * Register custom post type for SPP.
	 *
	 * @return true // phpcs:ignore Squiz.Commenting.FunctionComment.InvalidReturnVoid
	 */
	public function spp_register_post_type() {
		$posttype_txt_box = $this->spp_get_option( 'posttype_txt_box' );

		if ( post_type_exists( $posttype_txt_box ) || empty( $posttype_txt_box ) ) {
			return false;
		} else {
			register_post_type(
				$posttype_txt_box,
				array(
					'labels'              => array(
						'name'          => ( $posttype_txt_box ),
						'singular_name' => ( $posttype_txt_box ),

					),
					'public'              => true,
					'has_archive'         => true,
					'show_ui'             => true,
					'show_in_menu'        => true,
					'show_in_nav_menus'   => true,
					'show_in_admin_bar'   => true,
					'menu_position'       => 5,
					'can_export'          => true,
					'exclude_from_search' => false,
					'publicly_queryable'  => true,
					'capability_type'     => 'page',
				)
			);

		}
		return true;
	}

	/**
	 * Register custom post type for SPP.
	 *
	 * @return void
	 */
	public function botton_order_list() {
		// Verify nonce.
		if ( ! isset( $_POST['button_order_nonce'] ) || ! wp_verify_nonce( wp_unslash( sanitize_text_field( $_POST['button_order_nonce'] ) ), 'button_order' ) ) { // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.MissingUnslash
			// Nonce verification failed, handle the error or return an error message.
			return;
		}
		if ( isset( $_POST['buttonorder'] ) ) {
			$order                            = sanitize_text_field( wp_unslash( $_POST['buttonorder'] ) );
			$spp_settings['spp_button_order'] = $order;
			$this->spp_save_option( $spp_settings );
		}
	}
	/**
	 * Cleans up shortcodes in the provided content.
	 *
	 * @param string $content The content containing shortcodes.
	 *
	 * @return string Cleaned up content with fixed shortcodes.
	 */
	public function cleanup_shortcode_fix( $content ) {
		$array   = array(
			'<p>['       => '[',
			']</p>'      => ']',
			'<i>['       => '[',
			']</i>'      => ']',
			'<b>['       => '[',
			']</b>'      => ']',
			'<strong>['  => '[',
			']</strong>' => ']',
			'<h1>['      => '[',
			']</h1>'     => ']',
			'<h4>['      => '[',
			']</h4>'     => ']',
			'<h5>['      => '[',
			']</h5>'     => ']',
			'<h2>['      => '[',
			']</h2>'     => ']',
			']<h3>'      => ']',
			']</h3>'     => ']',
		);
		$content = strtr( $content, $array );
		return $content;
	}

	/**
	 * Retrieves an excerpt for the post with the given ID.
	 *
	 * @param int $post_id The ID of the post.
	 *
	 * @return string The generated excerpt.
	 */
	public function get_excerpt_by_id( $post_id ) {
		$the_post    = get_post( $post_id );
		$postcontent = strip_shortcodes( $the_post->post_content );
		$postcontent = str_replace( '&nbsp;', '', $postcontent );
		$postcontent = preg_replace( "/[\r\n]+/", "\n", $postcontent );
		$postcontent = preg_replace( '/\s+/', ' ', $postcontent );
		$postcontent = html_entity_decode( wp_strip_all_tags( $postcontent ) );
		$postcontent = substr( esc_attr( $postcontent ), 0, 500 );
		return $postcontent;
	}


	/**
	 * Generates Open Graph meta tags for single posts.
	 */
	public function spp_open_graph() {
		$post_id           = get_the_ID();
		$disable_opengraph = $this->spp_get_option( 'disable_opengraph' );
		$sppogp            = get_post_meta( $post_id, 'OGP', true );

		if ( ! $disable_opengraph && is_single() && $sppogp ) {
			$sppogp_desc  = $this->get_excerpt_by_id( $post_id );
			$og_title     = get_the_title( $post_id );
			$og_site_name = get_bloginfo( 'site_name' );
			$og_url       = get_permalink( $post_id );

			echo PHP_EOL . PHP_EOL . '<!-- Simple Podcast Press Open Graph Meta -->' . PHP_EOL;
			echo "<meta property='og:url' content='" . esc_url( $og_url ) . "'/>" . PHP_EOL;
			echo "<meta property='og:site_name' content='" . esc_html( $og_site_name ) . "'/>" . PHP_EOL;
			echo "<meta property='og:type' content='website' />" . PHP_EOL;
			echo "<meta property='og:title' content='" . esc_html( $og_title ) . "'/>" . PHP_EOL;
			echo "<meta property='og:description' content='" . esc_html( $sppogp_desc ) . "'/>" . PHP_EOL;

			if ( isset( $sppogp['image'] ) ) {
				echo "<meta property='og:image' content='" . esc_url( $sppogp['image'] ) . "'/>" . PHP_EOL;
				echo "<meta property='og:image:type' content='image/jpeg' />" . PHP_EOL;
			}

			echo '<!-- END of Simple Podcast Press Open Graph Meta -->' . PHP_EOL . PHP_EOL;

				do_action( 'spp_opengraph' );
		}
	}

	/**
	 * Hide PowerPress player based on certain conditions.
	 *
	 * This function checks if the current page is a single post and if it's the main query.
	 * If both conditions are met, it attempts to hide the PowerPress player.
	 * The hiding is done by printing CSS code to hide specific elements related to PowerPress.
	 *
	 * @return void
	 */
	public function spp_hide_pp() {
		if ( is_single() && is_main_query() ) {
			global $post;
			$meta_data = get_post_meta( $post->ID, 'enclosure', true );
			if ( $meta_data ) {
				$meta_parts = explode( "\n", $meta_data, 4 );
				if ( isset( $meta_parts[3] ) ) {
					$display_post_power_press_player = unserialize( $meta_parts[3] ); // phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.serialize_unserialize
				}
				if ( ! isset( $display_post_power_press_player['no_player'] ) ) {
					print( PHP_EOL . "<style type='text/css'>" . PHP_EOL );
					print( 'powerpress_links, p.powerpress_links, .powerpress_links_mp3 {
                        display:none; }' . PHP_EOL );
					print( '</style>' . PHP_EOL );
				}
				unset( $meta_parts );
			}
		} else {
			print( PHP_EOL . "<style type='text/css'>" . PHP_EOL );
			print( 'powerpress_links, p.powerpress_links, .powerpress_links_mp3 {
            display:none; }' . PHP_EOL );
			print( '</style>' . PHP_EOL );
		}
	}

	/**
	 * Override WordPress audio shortcode with Simple Podcast Press player.
	 *
	 * This function checks if there is an enclosure meta data associated with the post.
	 * If there is, it returns early to maintain the default behavior.
	 * Otherwise, it replaces the WordPress audio player with the Simple Podcast Press player.
	 *
	 * @param array  $attr     An array of attributes for the audio shortcode.
	 * @param string $content  The content inside the audio shortcode.
	 *
	 * @return string The modified output of the audio shortcode.
	 */
	public function spp_wp_audio_override( $attr, $content ) {
		global $post;
		if ( ! isset( $post->ID ) ) {
			return;
		}
		$meta_data = get_post_meta( $post->ID, 'enclosure', true );
		if ( ! empty( $meta_data ) ) {
			return;
		}

		$spp_with_pp    = $this->spp_get_option( 'replace_pp_with_spp' );
		$spp_exclude_wp = $this->spp_get_option( 'replace_wp_with_spp' );
		if ( ( 0 === $spp_with_pp ) || ( ( 1 === $spp_with_pp ) && ( 1 === $spp_exclude_wp ) ) ) {
			return $content;
		} else {
			if ( isset( $attr['src'] ) ) {
				$audio_file  = $attr['src'];
				$attr['url'] = $audio_file;
			}
			$spp_player = SPPress_Audio_Player::get_instance();
			$html       = $spp_player->spp_fullplayer_fn( $attr );
			return $html;
		}
	}
	/**
	 * Retrieve Simple Podcast Press links from the database.
	 *
	 * @return array|object|null Database query results or null on failure.
	 */
	public function spp_get_links() {
		global $wpdb;
		$tablespplinks = $wpdb->prefix . 'spp_links';
		$spp_links     = $wpdb->get_results( "SELECT * FROM {$tablespplinks} ORDER BY spp_slug ASC" ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
		return $spp_links;
	}

	/**
	 * Retrieve a specific Simple Podcast Press link from the database.
	 *
	 * @param int $spp_id The ID of the link to retrieve.
	 * @return object|null Database query result or null if link not found.
	 */
	public function spp_get_link( $spp_id ) {
		global $wpdb;
		$tablespplinks = $wpdb->prefix . 'spp_links';
		$the_link      = $wpdb->get_results( "SELECT * FROM {$tablespplinks} WHERE spp_id='" . $spp_id . "'" );
		if ( ! empty( $the_link[0] ) ) {
			return $the_link[0];
		}
		return null;
	}
	/**
	 * Display the meta box for editing Simple Podcast Press links.
	 */
	public function spp_links_meta_box() {
		global $edit_spp_link;
		$spp_name    = isset( $edit_spp_link ) ? esc_attr( $edit_spp_link->spp_name ) : '';
		$spp_url     = isset( $edit_spp_link ) ? esc_url( $edit_spp_link->spp_url ) : '';
		$spp_slug    = isset( $edit_spp_link ) ? esc_attr( $edit_spp_link->spp_slug ) : '';
		$spp_post_id = isset( $edit_spp_link ) ? absint( $edit_spp_link->spp_post_id ) : 0;
		?>
		<p>Post Name: <input type="text" name="spp_name" value="<?php echo esc_html( $spp_name ); ?>" /></p> <?php // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
		<p>Post Url: <input type="text" name="spp_url" value="<?php echo $spp_url; ?>" /></p> <?php // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
		<p>Episode Number: <input type="text" name="spp_slug" value="<?php echo $spp_slug; ?>" /></p> <?php // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
		<p>Post ID: <input type="text" name="spp_post_id" value="<?php echo $spp_post_id; ?>" /></p> <?php // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
		<?php
	}

	/**
	 * Handle form submission for adding/editing Simple Podcast Press links.
	 */
	public function spp_links() {
		global $wpdb;
		$tablespplinks = $wpdb->prefix . 'spp_links';

		if ( isset( $_GET['delete'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$delete_id = absint( $_GET['delete'] ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$wpdb->query( "DELETE FROM {$tablespplinks} WHERE spp_id='" . $_GET['delete'] . "'" );
		}

		if ( isset( $_POST['spp_add_link'] ) && isset( $_POST['spp_name'] ) && isset( $_POST['spp_url'] ) && isset( $_POST['spp_slug'] ) && isset( $_POST['spp_post_id'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing

			$spp_name    = sanitize_text_field( wp_unslash( $_POST['spp_name'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_url     = esc_url_raw( wp_unslash( $_POST['spp_url'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_slug    = sanitize_text_field( $_POST['spp_slug'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.MissingUnslash
			$spp_post_id = absint( $_POST['spp_post_id'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing

			if ( empty( $_POST['spp_id'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
				$wpdb->query( "INSERT INTO {$tablespplinks} (spp_name,spp_url,spp_slug,spp_post_id) VALUES('" . $spp_name . "','" . $spp_url . "','" . $spp_slug . "','" . $spp_post_id . "');" );
			} else {
				$spp_id = absint( $_POST['spp_id'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing
				$wpdb->query( "UPDATE {$tablespplinks} SET spp_name='" . $spp_name . "', spp_url='" . $spp_url . "', spp_slug='" . $spp_slug . "', spp_post_id='" . $spp_post_id . "' WHERE spp_id='" . $spp_id . "'" );
				$admin_url = get_admin_url() . 'admin.php?page=spp-url-shortner';
			}
		}
	}

	/**
	 * Add or edit Simple Podcast Press links.
	 */
	public function spp_add_link() {
		$spp_id = 0;
		if ( $_GET['spp_id'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$spp_id = absint( $_GET['spp_id'] ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended
		}

		global $edit_spp_link;
		if ( $spp_id ) {
			$edit_spp_link = $this->spp_get_link( $spp_id );
		}

		add_meta_box(
			'spp-links-meta',
			__( 'SPP Links' ),
			array(
				$this,
				'spp_links_meta_box',
			),
			'bor',
			'normal',
			'core'
		);
		?>
	<div class="wrap">
		<div id="faq-wrapper">
			<form method="post" action="">
				<h2>
					<?php
					if ( $spp_id == 0 ) {
						$tf_title = __( 'Create New Shortened URL' );
					} else {
						$tf_title = __( 'Update Shortened URL' );
					}
					echo $tf_title;
					?>
				</h2>
				<div id="poststuff" class="metabox-holder">
					<?php do_meta_boxes( 'bor', 'normal', 'low' ); ?>
				</div>
				<input type="hidden" name="spp_id" value="<?php echo (int) $spp_id; ?>" />
				<input type="submit" value="<?php echo $tf_title; ?>" name="spp_add_link" id="spp_add_link" class="button-secondary">
			</form>
		</div>
	</div>
		<?php
	}

	/**
	 * Manage Simple Podcast Press links.
	 */
	public function spp_manage_links() {
		?>
	<div class="wrap">
		<div class="icon32" id="icon-edit"><br></div>
		<form method="post" action="" id="spp_form_action">
			<p>
				<select name="spp_action">
					<option value="actions">
						<?php esc_html_e( 'Actions' ); ?>
					</option>
					<option value="delete">
						<?php esc_html_e( 'Delete' ); ?>
					</option>
				</select>
				<input type="submit" name="spp_form_action_changes" class="button-secondary" value="<?php esc_html_e( 'Apply' ); ?>" />
				<input type="button" class="button-secondary" value="<?php esc_html_e( 'Add new shortened link' ); ?>" onclick="window.location='?page=spp-url-shortner&amp;edit=true'" />
			</p>
			<table class="widefat page fixed" cellpadding="0">
				<thead>
					<tr>
						<th id="cb" class="manage-column column-cb check-column" style="" scope="col">
							<input type="checkbox"/>
						</th>
						<th class="manage-column">
							<?php esc_html_e( 'Post Name' ); ?>
						</th>
						<th class="manage-column">
							<?php esc_html_e( 'Post Url' ); ?>
						</th>
						<th class="manage-column">
							<?php esc_html_e( 'Episode Number' ); ?>
						</th>
						<th class="manage-column">
							<?php esc_html_e( 'Post ID' ); ?>
						</th>
					</tr>
				</thead>
				<tfoot>
					<tr>
						<th id="cb" class="manage-column column-cb check-column" style="" scope="col">
							<input type="checkbox"/>
						</th>
						<th class="manage-column">
							<?php esc_html_e( 'Post Name' ); ?>
						</th>
						<th class="manage-column">
							<?php esc_html_e( 'Post Url' ); ?>
						</th>
						<th class="manage-column">
							<?php esc_html_e( 'Episode Number' ); ?>
						</th>
						<th class="manage-column">
							<?php esc_html_e( 'Post ID' ); ?>
						</th>
					</tr>
				</tfoot>
				<tbody>
					<?php
					$table = $this->spp_get_links();
					if ( $table ) {
						$i = 0;
						foreach ( $table as $link ) {
							++$i;
							?>
							<tr class="<?php echo ( ceil( $i / 2 ) === ( $i / 2 ) ) ? '' : 'alternate'; ?>">
								<th class="check-column" scope="row">
									<input type="checkbox" value="<?php echo $link->spp_id; ?>" name="spp_id[]" /> <?php // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped,WordPress.Security.EscapeOutput.OutputNotEscaped ?>
								</th>
								<td>
									<strong>
										<?php echo esc_html( $link->spp_name ); ?>
									</strong>
									<div class="row-actions-visible">
										<span class="edit"><a href="?page=spp-url-shortner&amp;spp_id=<?php echo $link->spp_id; ?>&amp;edit=true"><?php esc_html_e( 'Edit' ); ?></a> | </span><?php // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
										<span class="delete"><a href="?page=spp-url-shortner&amp;delete=<?php echo $link->spp_id; ?>" onclick="return confirm('<?php esc_html_e( 'Are you sure you want to delete this link?' ); ?>');"><?php esc_html_e( 'Delete' ); ?></a></span><?php // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
									</div>
								</td>
								<td>
									<?php echo esc_html( $link->spp_url ); ?>
								</td>
								<td>
									<?php echo esc_html( $link->spp_slug ); ?>
								</td>
								<td>
									<?php echo esc_html( $link->spp_post_id ); ?>
								</td>
							</tr>
							<?php
						}
					} else {
						?>
						<tr>
							<td colspan="4">
						<?php esc_html_e( 'There are no data.' ); ?>
							</td>
						</tr>  
						<?php
					}
					?>
				</tbody>
			</table>
			<p>
				<select name="spp_action-2">
					<option value="actions">
						<?php esc_html_e( 'Actions' ); ?>
					</option>
					<option value="delete">
						<?php esc_html_e( 'Delete' ); ?>
					</option>
				</select>
				<input type="submit" name="spp_form_action_changes-2" class="button-secondary" value="<?php esc_html_e( 'Apply' ); ?>" />
				<input type="button" class="button-secondary" value="<?php esc_html_e( 'Add a new link' ); ?>" onclick="window.location='?page=spp-url-shortner&amp;edit=true'" />
			</p>
		</form>
	</div>
		<?php
	}

	/**
	 * Handle Simple Podcast Press URL Shortener functionality.
	 */
	public function spp_url_shortner_func() {
		$nonce = isset( $_POST['wpnonce_urlshortner'] ) ? sanitize_text_field( wp_unslash( $_POST['wpnonce_urlshortner'] ) ) : '';

		if ( ! wp_verify_nonce( $nonce, 'urlshortner_settings' ) ) {
			wp_die( esc_html( 'Security check' ) );
		} else {
			global $wpdb;
			$tablespplinks = $wpdb->prefix . 'spp_links';
			$tableposts    = $wpdb->prefix . 'posts';
			$results       = $wpdb->get_results( "SELECT * FROM {$tablespplinks} ORDER BY `spp_post_id`" );

			foreach ( $results as $result ) {
				$spp_post_id       = $result->spp_post_id;
				$check_post_exists = $wpdb->query( "SELECT * FROM {$tableposts} WHERE ID = {$spp_post_id}" );
				if ( ! $check_post_exists ) {
					$delete_record = $wpdb->query( "DELETE FROM {$tablespplinks} WHERE spp_post_id = {$spp_post_id}" );
				}
				$spp_slug      = $result->spp_slug;
				$results_slug  = $wpdb->get_results( "SELECT * FROM {$tablespplinks} WHERE spp_slug = '" . $spp_slug . "' ORDER BY `spp_post_id`" );
				$records_count = count( $results_slug );

				if ( $records_count > 1 ) {
					$counter = 1;
					foreach ( $results_slug as $slug_record ) {
						$spp_post_id  = absint( $slug_record->spp_post_id );
						$check_record = $wpdb->get_row( "SELECT * FROM {$tableposts} WHERE ID = {$spp_post_id}" );

						if ( ! is_object( $check_record ) ) {
							continue;
						}

						if ( 'publish' != $check_record->post_status || $counter > 1 ) {
							$delete_record = $wpdb->query( "DELETE FROM {$tablespplinks} WHERE spp_post_id = {$spp_post_id}" );
						} else {
							++$counter;
						}
					}
				}
			}

			echo 'done';
		}
	}

	/**
	 * Limit the content to a certain number of words.
	 *
	 * @param int    $limit   The maximum number of words to display.
	 * @param string $content The content to limit.
	 * @return string The limited content.
	 */
	public function limit_content_words( $limit, $content ) {
		$content = explode( ' ', $content, $limit );

		if ( count( $content ) >= $limit ) {
			array_pop( $content );
			$content = implode( ' ', $content ) . '...';
		} else {
			$content = implode( ' ', $content );
		}

		return $content;
	}

	/**
	 * Callback function for processing timestamp matches in a regular expression.
	 *
	 * @param array $m The match array.
	 * @return string The replacement string.
	 */
	public function spp_ts_callback( $m ) {
		return empty( $m[1] ) ? '[spp-timestamp time="' . $m[0] . '"]' : $m[0];
	}
	/**
	 * Filter content and integrate SimplePodcastPress player.
	 *
	 * @param string $content The post content.
	 *
	 * @return string The filtered content with integrated player.
	 */
	public function spp_the_content( $content ) {
		$html                       = '';
		$hide_email                 = '';
		$duration                   = '';
		$video_file                 = '';
		$audio_file                 = false;
		$video_podcast              = false;
		$spp_disable_auto_timestamp = $this->spp_get_option( 'spp_disable_auto_timestamp' );

		// Problem: If the_excerpt is used instead of the_content, both the_exerpt and the_content will be called here.
		// Important to note, get_the_excerpt will be called before the_content is called, so we add a simple little hack.
		$hide_player_from_excerpt      = $this->spp_get_option( 'spp_hide_player_from_excerpt' );
		$spp_disable_spp_player_script = $this->spp_get_option( 'spp_disable_spp_player_script' );

		// Assume we are showing the player.
		$spp_disable_all_players = $this->spp_get_option( 'spp_disable_all_players' );
		$spp_force_player_home   = $this->spp_get_option( 'spp_force_player_home' );
		$spp_insert_feat_image   = $this->spp_get_option( 'spp_insert_feat_image' );
		$spp_player_over_image   = $this->spp_get_option( 'spp_player_over_image' );

		$show_player = true;

		if ( ! $spp_disable_auto_timestamp ) {

			$pattern         = '/(<div\s+[^<]*?\bclass="[^<"]*\btimestamp\b[^<]*?>[^<]*)?(?<!:)\d{1,2}:\d{2}(?::\d{2})?(?!(?::\d{2})?\s*[ap]\.?m\.?)(?!.*?[\'])/';
			$replacement     = '[spp-timestamp time="$0"]';
			$found_timestamp = preg_replace_callback( $pattern, array( $this, 'spp_ts_callback' ), $content );

			$found_timestamp = str_replace( '[[spp', '[spp', $found_timestamp );
			$found_timestamp = str_replace( '"]]', '"]', $found_timestamp );
			$found_timestamp = str_replace( ']"]', ']', $found_timestamp );

			if ( $found_timestamp ) {
					$content = $found_timestamp;
			}
		}

		// Prevents SPP Player from showing up in feeds. Not necessary it seems with latest PowerPress.
		if ( is_feed() ) {
			return $content;
		}

		if ( ( $hide_player_from_excerpt ) || ( $spp_disable_spp_player_script ) ) {
			// If on post then show player.
			if ( is_single() && is_main_query() ) {
				$show_player = true;
			} else {
				// Otherwise if on home, blog, archive, or ANY page then hide player.
				$show_player = false;
			}
		}

		if ( $spp_force_player_home ) {
			$show_player = true;
		}

		if ( $spp_disable_all_players ) {
			$show_player = false;
		}

		$is_license_valid = get_option( 'sppress_ls' );

		if ( 'valid' !== $is_license_valid && '' !== $is_license_valid ) {
			return $content;
		}

		global $post;
		if ( ! isset( $post->ID ) ) {
			return;
		}
			$audio_player_position     = ! empty( $this->spp_settings['audio_player_position'] ) ? $this->spp_settings['audio_player_position'] : '';
			$powerpress_active_on_post = false;
			$meta_data                 = get_post_meta( $post->ID, 'enclosure', true );
		if ( $meta_data ) {
					$powerpress_active_on_post = true;
					$p_p_general               = get_option( 'powerpress_general' );
			if ( isset( $p_p_general['display_player'] ) && ( 2 === $p_p_general['display_player'] ) ) {
					$audio_player_position = 'above';
			} elseif ( isset( $p_p_general['display_player'] ) && ( 1 === $p_p_general['display_player'] ) ) {
					$audio_player_position = 'below';
			}
					$meta_parts = explode( "\n", $meta_data, 4 );
			if ( isset( $meta_parts[3] ) ) {
					$display_post_player = unserialize( $meta_parts[3] ); // phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.serialize_unserialize
			}
			if ( isset( $display_post_player['no_player'] ) ) {
				return $content;
			}
					unset( $p_p_general );
					unset( $meta_parts );
		}

		$replace_power_press = $this->spp_get_option( 'replace_pp_with_spp' );

		if ( ( true === $powerpress_active_on_post ) && ( 0 === $replace_power_press ) ) {
			return $content;
		}

		// check if shortcode already on there or not, if yes then ...
		if ( preg_match_all( '/(.?)\[(spp-player)\b(.*?)(?:(\/))?\](?:(.+?)\[\/\2\])?(.?)/s', $content, $matches ) || preg_match_all( '/(.?)\[(powerpress)\b(.*?)(?:(\/))?\](?:(.+?)\[\/\2\])?(.?)/s', $content, $matches ) || preg_match_all( '/(.?)\[(smart_track_player)\b(.*?)(?:(\/))?\](?:(.+?)\[\/\2\])?(.?)/s', $content, $matches ) || preg_match_all( '/(.?)\[(display_podcast)\b(.*?)(?:(\/))?\](?:(.+?)\[\/\2\])?(.?)/s', $content, $matches ) || preg_match_all( '/(.?)\[(spp-playlist)\b(.*?)(?:(\/))?\](?:(.+?)\[\/\2\])?(.?)/s', $content, $matches ) ) {
			$spp_with_pp = $this->spp_get_option( 'replace_pp_with_spp' );

			// Strip out the shortcode before displaying if player is off.
			if ( false === $show_player ) {
				$content = preg_replace( '/\[powerpress.*?\]/', '', $content );
				$content = preg_replace( '/\[spp-player.*?\]/', '', $content );
				return $content;
			} elseif ( 1 === $spp_with_pp ) {
				// Only if Replace PowerPress with SimplePodcastPress Player option is set.
				// This avoids replacing the [powerpress_playlist] shortcode.
				$content = str_replace( '[powerpress]', '[spp-player]', $content );
				$content = str_replace( '[powerpress ', '[spp-player ', $content );
				$content = str_replace( '[display_podcast]', '[spp-player]', $content );

				$s_p_p_exclude_stp = $this->spp_get_option( 'replace_stp_with_spp' );
				if ( ! $s_p_p_exclude_stp ) {
					$content = str_replace( '[smart_track_player ', '[spp-player ', $content );
				}

				return $content;
			} else {
				return $content;

			}
		}
		if ( preg_match_all( '/(.?)\[(audio:)\b(.*?)(?:(\/))?\](?:(.+?)\[\/\2\])?(.?)/s', $content, $matches ) ) {
			$spp_with_pp    = $this->spp_get_option( 'replace_pp_with_spp' );
			$spp_exclude_wp = $this->spp_get_option( 'replace_wp_with_spp' );

			// Only if Replace PowerPress with SimplePodcastPress Player option is set.
			if ( ( ( 1 === $spp_with_pp ) && ( 0 === $spp_exclude_wp ) ) ) {
				$content = str_replace( '[audio:', '[spp-player url=', $content );
				return $content;

			} else {
				return $content;
			}
		}

		if ( $show_player ) {

			$css_version    = wp_rand( 10, 99 ) / 10;
			$player_buttons = SPPRESS_PLUGIN_URL . '/responsive_audio_player/css/player-buttons.css';

			wp_enqueue_style( 'player-buttons', $player_buttons, false, $css_version, 'all' );
			$spp_player = SPPress_Audio_Player::get_instance();
			$html       = $spp_player->spp_fullplayer_fn();

			if ( false === $html ) {
				return $content;
			}

			if ( in_array( 'the_excerpt', $GLOBALS['wp_current_filter'] ) ) { // phpcs:ignore WordPress.PHP.StrictInArray.MissingTrueStrict
				if ( ! has_excerpt( $post->ID ) ) {
					$content = get_the_content( '' );
				}
				$content     = strip_shortcodes( $content );
				$filter_html = wp_strip_all_tags( $html, true );
				$content     = str_replace( $filter_html, '', $content );
				$content     = wp_trim_words( $content );
			}

			if ( has_post_thumbnail() && $spp_insert_feat_image ) {
				$spp_imgsize_select  = $this->spp_settings['spp_imgsize_select'];
				$spp_imgalign_select = $this->spp_settings['spp_imgalign_select'];

				if ( 'thumbnail' === $spp_imgsize_select ) {
					$spp_imgsize = '150';
				} elseif ( 'medium' === $spp_imgsize_select ) {
					$spp_imgsize = '300';
				} elseif ( 'large' === $spp_imgsize_select ) {
					$spp_imgsize = '640';
				} else {
					$spp_imgsize = '';
				}

				$image               = wp_get_attachment_image_src( get_post_thumbnail_id( $post->ID ), 'single-post-thumbnail' );
				$spp_featured_image  = '<img width="' . $spp_imgsize . '"';
				$spp_featured_image .= ' height="' . $spp_imgsize . '"';
				$spp_featured_image .= ' class="' . $spp_imgalign_select . ' wp-post-image"';
				$spp_featured_image .= ' src="' . $image[0] . '" >';
			} else {
				$spp_featured_image = '';
			}

			$html = apply_filters( 'spp_player_html', $html );
			switch ( $audio_player_position ) {
				case 'above':
					if ( $spp_player_over_image ) {
						if ( ( is_single() && is_main_query() ) || is_page() ) {
							$content = $html . $spp_featured_image . $content . PHP_EOL;
						} else {
							$content = $html . $content . PHP_EOL;
						}
					} elseif ( ( is_single() && is_main_query() ) || is_page() ) {
							$content = $spp_featured_image . $html . PHP_EOL . $content;
					} else {
						$content = $html . PHP_EOL . $content;
					}

					break;

				case 'below':
					if ( ( is_single() && is_main_query() ) || is_page() ) {
						$content = $spp_featured_image . $content . PHP_EOL . $html;
					} else {
						$content = $content . PHP_EOL . $html;
					}

					break;

				case 'disabled':
					if ( ( is_single() && is_main_query() ) || is_page() ) {
						$content = $spp_featured_image . $content . PHP_EOL;
					} else {
						$content = $content . PHP_EOL;
					}

					break;

				default:
					if ( ( is_single() && is_main_query() ) || is_page() ) {
						$content = $spp_featured_image . $content . PHP_EOL;
					} else {
						$content = $content . PHP_EOL;
					}
			}
		}// end of if show_player
		return $content;
	}
	/**
	 * Handle plugin update tasks.
	 *
	 * This function includes necessary files, checks for plugin version change,
	 * and performs various update tasks like generating CSS, updating the database, etc.
	 *
	 * @return void
	 * @throws Exception If file inclusion fails.
	 */
	public function spp_plugin_update() {
		include_once ABSPATH . 'wp-admin/includes/file.php';
		global $sppress_current_version;
		$last_savedcss_version = get_option( 'spp-currentcss-version' );

		if ( $last_savedcss_version !== $sppress_current_version ) {
			update_option( 'spp-currentcss-version', $sppress_current_version );
			$is_pro_player_on = $this->spp_get_option( 'select_audio_player_skin', -1 );

			$this->generate_buttons_css();
			$this->generate_options_css();
			$this->generate_barui_css();
			$this->generate_classic_css();
		}

		$this->sppress_updatedb();
		$this->sppress_tcheck();

		return; // phpcs:ignore Squiz.PHP.NonExecutableCode.ReturnNotRequired
	}
	/**
	 * Display trial expiration notice.
	 *
	 * This function checks the trial expiration date and displays a notice accordingly.
	 *
	 * @return void
	 */
	public function trial_expire_soon() {
		$sppress_ed    = get_option( 'sppress_ed' );
		$sppress_vdrdc = get_option( 'sppress_vdrdc' );

		if ( $sppress_vdrdc ) {
			$sppress_ex_dc = ' and <strong>SAVE 25%%</strong> by using discount code <strong>' . esc_html( $sppress_vdrdc ) . '</strong>.';
		} else {
			$sppress_ex_dc = '.';
		}

		$ct                = time();
		$sppress_ex_notice = round( ( $sppress_ed - $ct ) / ( 24 * HOUR_IN_SECONDS ), 0 );

		if ( 0 === $sppress_ex_notice  ) { // phpcs:ignore WordPress.WhiteSpace.ControlStructureSpacing.ExtraSpaceBeforeCloseParenthesis
			$sppress_ex_notice = esc_html__( 'TODAY', 'spp' );
		} else {
			$sppress_ex_notice = esc_html__( 'in', 'spp' ) . ' ' . $sppress_ex_notice . ' ' . esc_html__( 'days', 'your-text-domain' );
		}

		?>
	<div class="error notice">
		<p>
		<?php
		printf(
			esc_html__( 'Your %1$s trial will expire %2$s. Click here to %3$supgrade your license today%4$s', 'your-text-domain' ), // phpcs:ignore WordPress.WP.I18n.MissingTranslatorsComment
			'<strong>' . esc_html__( 'Simple Podcast Press Player', 'spp' ) . '</strong>',
			'<strong>' . $sppress_ex_notice . '</strong>', // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped,WordPress.WP.I18n.MissingTranslatorsComment
			'<a href="https://simplepodcastpress.com/pricing" target="_blank">',
			'</a>' . esc_html( $sppress_ex_dc )
		);
		?>
		</p>
	</div>
		<?php
	}
	/**
	 * Display trial expiration notice for expired trials.
	 *
	 * This function displays a notice for expired trials with an option to upgrade.
	 *
	 * @return void
	 */
	public function trial_expired() {
		$sppress_vdrdc = get_option( 'sppress_vdrdc' );

		if ( $sppress_vdrdc ) {
			$sppress_ex_dc = ' and <strong>SAVE 25%%</strong> by using discount code <strong>' . esc_html( $sppress_vdrdc ) . '</strong>.';
		} else {
			$sppress_ex_dc = '.';
		}

		?>
	<div class="error notice">
		<p>
		<?php
		// Translators: %1$s is a placeholder for the product name, %2$s is a placeholder for the trial status, %3$s and %4$s are placeholders for HTML tags.
		printf(
			esc_html__( 'Your %1$s trial has %2$s. Click here to %3$supgrade your license today%4$s', 'spp' ), // phpcs:ignore WordPress.WP.I18n.MissingTranslatorsComment
			'<strong>' . esc_html__( 'Simple Podcast Press Player', 'spp' ) . '</strong>',
			'<strong>' . esc_html__( 'EXPIRED', 'spp' ) . '</strong>', // phpcs:ignore WordPress.WP.I18n.MissingTranslatorsComment
			'<a href="https://simplepodcastpress.com/pricing" target="_blank">',
			'</a>' . esc_html( $sppress_ex_dc )
		);

		?>
		</p>
	</div>
		<?php
	}

	/**
	 * Linkify URLs in a text.
	 *
	 * @param string $text The text containing URLs.
	 * @return string The text with URLs replaced by anchor tags.
	 */
	public function linkify_text( $text ) {
		$url_re          = '@(https?://([-\w\.]+)+(:\d+)?(/([\w/_\.]*(\?\S+)?)?)?)@';
		$url_replacement = "<a href='$1' target='_blank'>$1</a>";
		return preg_replace( $url_re, $url_replacement, $text );
	}

	/**
	 * Generate and save the bar UI CSS.
	 *
	 * @return bool True if the CSS was generated and saved successfully, false otherwise.
	 */
	public function generate_barui_css() {

		$uploads = wp_upload_dir();
		$css_dir = __DIR__ . '/responsive_audio_player/css/'; // Shorten code, save 1 call.

		/**
		 * Save on different directory if on multisite
		 */
		if ( is_multisite() ) {
			$aq_uploads_dir = trailingslashit( $uploads['basedir'] );
		} else {
			$aq_uploads_dir = $css_dir;
		}

		/**
		 * Capture CSS output
		 */
		ob_start();
		include $css_dir . 'bar-ui.php';
		$barui = ob_get_contents();
		ob_end_clean();

		/**
		 * Write to options.css file
		 */

		global $wp_filesystem;
		// protect if the the global filesystem isn't setup yet.
		if ( is_null( $wp_filesystem ) ) {
			WP_Filesystem();
		}
		if ( ! $wp_filesystem->put_contents( $aq_uploads_dir . 'bar-ui.css', $barui ) ) {
			$spp_settings['spp_writecss'] = true;
			$this->spp_save_option( $spp_settings );

			return false;
		}

		$spp_settings['spp_writecss'] = true;
		$this->spp_save_option( $spp_settings );
		return true;
	}


	/**
	 * Generate and save the classic player CSS.
	 *
	 * @return bool True if the CSS was generated and saved successfully, false otherwise.
	 */
	public function generate_classic_css() {

		// Define some vars.
		$uploads = wp_upload_dir();
		$css_dir = __DIR__ . '/responsive_audio_player/css/'; // Shorten code, save 1 call.

		// Save on different directory if on multisite.
		if ( is_multisite() ) {
			$aq_uploads_dir = trailingslashit( $uploads['basedir'] );
		} else {
			$aq_uploads_dir = $css_dir;
		}

		// Capture CSS output.
		ob_start();
		include $css_dir . 'classic-player.php';
		$classic = ob_get_contents();
		ob_end_clean();

		// Write to options.css file.
		global $wp_filesystem;
		// protect if the the global filesystem isn't setup yet.
		if ( is_null( $wp_filesystem ) ) {
			WP_Filesystem();
		}
		if ( ! $wp_filesystem->put_contents( $aq_uploads_dir . 'classic-player.css', $classic ) ) {
			$spp_settings['spp_writecss'] = false;
			$this->spp_save_option( $spp_settings );
			return false;
		}

		$spp_settings['spp_writecss'] = true;
		$this->spp_save_option( $spp_settings );

		return true;
	}

	/**
	 * Generate and save the player buttons CSS.
	 *
	 * @return bool True if the CSS was generated and saved successfully, false otherwise.
	 */
	public function generate_buttons_css() {
		$uploads = wp_upload_dir();
        $css_dir = dirname(__FILE__) . '/responsive_audio_player/css/'; // Shorten code, save 1 call
        
        /** Save on different directory if on multisite **/
        
        if (is_multisite()) {
            $aq_uploads_dir = trailingslashit($uploads['basedir']);
        } else {
            $aq_uploads_dir = $css_dir;
        }
        /** Capture CSS output **/
        ob_start();
        require($css_dir . 'player-buttons.php');
        $buttons = ob_get_contents();
        ob_end_clean();
        /** Write to options.css file **/
        
        global $wp_filesystem;
        // protect if the the global filesystem isn't setup yet
        if (is_null($wp_filesystem)){
            WP_Filesystem();
		}
		
        if (!$wp_filesystem->put_contents($aq_uploads_dir . 'player-buttons.css', $buttons)) {
            $spp_settings['spp_writecss'] = FALSE;
            $this->spp_save_option($spp_settings);
            return false;
        }
        
        $spp_settings['spp_writecss'] = TRUE;
        $this->spp_save_option($spp_settings);
        return true;
	}
	/**
	 * Generate options CSS and save it to a file.
	 *
	 * @return bool True if CSS was generated and saved successfully, false otherwise.
	 */
	public function generate_options_css() {
		$uploads = wp_upload_dir();
		$css_dir = __DIR__ . '/responsive_audio_player/css/';

		if ( is_multisite() ) {
			$aq_uploads_dir = trailingslashit( $uploads['basedir'] );
		} else {
			$aq_uploads_dir = $css_dir;
		}

		ob_start();
		require $css_dir . 'audio-player.php';
		$css = ob_get_clean();

		global $wp_filesystem;

		// Make sure the global filesystem is set up.
		if ( is_null( $wp_filesystem ) ) {
			WP_Filesystem();
		}

		// Write to options.css file.
		if ( ! $wp_filesystem->put_contents( $aq_uploads_dir . 'audio-player.css', $css ) ) {
			$spp_settings['spp_writecss'] = false;
			$this->spp_save_option( $spp_settings );
			return false;
		}

		$spp_settings['spp_writecss'] = true;
		$this->spp_save_option( $spp_settings );
		return true;
	}
	/**
	 * Display an error message for CSS failure.
	 */
	public function spp_css_failed() {
		?>
	<div class="error">
		<p>
			<?php
			_e( 'ERROR!', 'spp' ); // phpcs:ignore WordPress.Security.EscapeOutput.UnsafePrintingFunction
			?>
		</p>
	</div>
		<?php
	}

	/**
	 * Display a success message for CSS update.
	 */
	public function spp_css_passed() {
		?>
	<div class="updated">
		<p>
			<?php
			_e( 'UPDATED!', 'spp' ); // phpcs:ignore WordPress.Security.EscapeOutput.UnsafePrintingFunction
			?>
		</p>
	</div>
		<?php
	}

	/**
	 * Perform manual action for Simple Podcast Press.
	 */
	public function manual_spp_action() {
		$post_id = get_the_ID();
		$sppogp  = get_post_meta( $post_id, 'OGP', true );

		if ( ! $sppogp ) {
			return;
		}

		if ( has_post_thumbnail( $post_id ) ) {
			$fb_channel_image = wp_get_attachment_url( get_post_thumbnail_id( $post_id ) );
		} else {
			$fb_channel_image = $this->spp_get_option( 'channel_image' );
		}

		if ( isset( $sppogp['image'] ) ) {
			$sppogp['image'] = $fb_channel_image;
			update_post_meta( $post_id, 'OGP', $sppogp );
		}
	}

	/**
	 * Add admin menu items for Simple Podcast Press.
	 */
	public function spp_admin_menu() {
		$icon_image = SPPRESS_PLUGIN_URL . '/icons/spp_icon.png';

		add_menu_page(
			'SimplePodcastPress',
			'Simple Podcast Press',
			'manage_options',
			'spp-podcast-settings',
			array( $this, 'spp_plugin_view' ),
			$icon_image
		);

		add_submenu_page(
			'spp-podcast-settings',
			'Simple Podcast Press',
			'Settings',
			'manage_options',
			'spp-podcast-settings',
			array( $this, 'spp_plugin_view' )
		);

		add_submenu_page(
			'spp-podcast-settings',
			'Simple Podcast Press',
			'Podcast Reviews',
			'manage_options',
			'spp_reviews',
			array( $this, 'spp_plugin_review_page' )
		);

		add_submenu_page(
			'spp-podcast-settings',
			'Simple Podcast Press',
			'URL Shortener',
			'manage_options',
			'spp-url-shortner',
			array( $this, 'spp_url_shortner_page' )
		);

		add_submenu_page(
			'spp-podcast-settings',
			'Simple Podcast Press',
			'Support',
			'manage_options',
			'spp-podcasts-table',
			array( $this, 'spp_spp_podcasts_page' )
		);
	}

	/**
	 * Add dashboard widgets for Simple Podcast Press reviews.
	 */
	public function reviews_dashboard_widgets() {
		global $wp_meta_boxes, $wpdb;
		$table_spp_reviews = $wpdb->prefix . 'spp_reviews';
		if ( $wpdb->get_var( "SHOW TABLES LIKE '$table_spp_reviews'" ) == $table_spp_reviews ) {
			$spp_reviews = $wpdb->get_results( 'SELECT * FROM ' . $table_spp_reviews );
		} else {
			return false;
		}

		$dashboard_current       = $wp_meta_boxes['dashboard']['normal']['core'];
		$totalitunesreviewscount = count( $spp_reviews );
		$itunesreviewstitle      = 'International Podcast Reviews (' . $totalitunesreviewscount . ')';

		wp_add_dashboard_widget(
			'spp_reviews_widget',
			$itunesreviewstitle,
			array( $this, 'rw_widget_fn' )
		);

		$dashboard_podcast = array();
		if ( isset( $dashboard_current['spp_reviews_widget'] ) ) {
			$dashboard_podcast['spp_reviews_widget'] = $dashboard_current['spp_reviews_widget'];
			unset( $dashboard_current['spp_reviews_widget'] );
		}
		if ( count( $dashboard_podcast ) > 0 ) {
			$wp_meta_boxes['dashboard']['normal']['core'] = array_merge( $dashboard_podcast, $dashboard_current ); // phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited
		}
	}


	/**
	 * Display the Simple Podcast Press reviews widget content.
	 */
	public function rw_widget_fn() {
		global $wpdb;
		$table_spp_reviews = $wpdb->prefix . 'spp_reviews';

		if ( $wpdb->get_var( "SHOW TABLES LIKE '$table_spp_reviews'" ) == $table_spp_reviews ) {
			$spp_reviews = $wpdb->get_results( 'SELECT * FROM ' . $table_spp_reviews . ' ORDER BY rw_id DESC' );
		} else {
			return false;
		}

		$counter = 0;
		echo '<table>';
		foreach ( $spp_reviews as $spp_review ) {

			echo '<tr><td>';
			echo '<strong>' . esc_html( $spp_review->rw_title ) . '</strong>&nbsp;&nbsp;&nbsp;&nbsp;';
			$rating = $spp_review->rw_ratings;
			switch ( $rating ) {

				case 5:
					echo '<img src="' . esc_url( SPPRESS_PLUGIN_URL . '/icons/rating_stars/5star.png' ) . '" />';
					break;
				case 4:
					echo '<img src="' . esc_url( SPPRESS_PLUGIN_URL . '/icons/rating_stars/4star.png' ) . '" />';
					break;
				case 3:
					echo '<img src="' . esc_url( SPPRESS_PLUGIN_URL . '/icons/rating_stars/3star.png' ) . '" />';
					break;
				case 2:
					echo '<img src="' . esc_url( SPPRESS_PLUGIN_URL . '/icons/rating_stars/2star.png' ) . '" />';
					break;
				case 1:
					echo '<img src="' . esc_url( SPPRESS_PLUGIN_URL . '/icons/rating_stars/1star.png' ) . '" />';
					break;
				default:
					echo '<img src="' . esc_url( SPPRESS_PLUGIN_URL . '/icons/rating_stars/00star.png' ) . '" />';
					break;
			}

			$publishedat  = $spp_review->rw_published_date;
			$publish_date = date_i18n( 'F j, Y', strtotime( $publishedat ) );
			echo '</td></tr><tr><td>';
			echo esc_html( $publish_date ) . ' by <b>' . esc_html( $spp_review->rw_author ) . '</b> from ' . $this->get_full_country_name( $spp_review->rw_coutry ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			echo '</td></tr><tr><td>';
			echo wp_kses_post( $spp_review->rw_text ) . '</br></br>';
			echo '</td></tr>';
			if ( 4 === $counter ) {
				break;
			}
			++$counter;
		}
		echo '<tr><td><a href="admin.php?page=spp_reviews" style="float:right;">See All Reviews</a></td></tr>';
		echo '</table>';
	}

	/**
	 * Include the Simple Podcast Press plugin view.
	 */
	public function spp_plugin_view() {
		include 'spp_view/spp_plugin_view.php';
	}
	/**
	 * Get the value of a specific setting from the Simple Podcast Press options.
	 *
	 * @param string $settingname The name of the setting.
	 * @return mixed The value of the setting, or an empty string if not found.
	 */
	public static function spp_get_option( $settingname ) {
		$spp_settings = get_option( 'spp_settings' );

		if ( $spp_settings ) {
			return array_key_exists( $settingname, $spp_settings ) ? $spp_settings[ $settingname ] : '';
		} else {
			return '';
		}
	}

	/**
	 * Save the Simple Podcast Press options.
	 *
	 * @param array $spp_settings The settings to be saved.
	 */
	public static function spp_save_option( $spp_settings ) {
		$old_settings = get_option( 'spp_settings' );

		if ( isset( $old_settings ) ) {
			$settings = array_merge( $old_settings, $spp_settings );
			update_option( 'spp_settings', $settings );
		}
	}

	/**
	 * Register the metaboxes for Simple Podcast Press in various post types.
	 */
	public function spp_metaboxes() {
		$args                 = array(
			'public'   => true,
			'_builtin' => false,
		);
		$disable_url_shortner = $this->spp_get_option( 'disable_url_shortner' );

		$array1     = array(
			'post' => 'post',
		);
		$array2     = get_post_types( $args );
		$post_types = array_merge( $array1, $array2 );
		foreach ( $post_types as $post_type ) {

			add_meta_box(
				'spp_audiourl',
				'Simple Podcast Press: Audio URLs',
				array(
					$this,
					'spp_audiourl_metabox_fn',
				),
				$post_type,
				'normal',
				'high'
			);
			if ( ! $disable_url_shortner ) {
				add_meta_box(
					'spp_link_metabox',
					'Simple Podcast Press: URL Shortener',
					array(
						$this,
						'spp_link_metabox_fn',
					),
					$post_type,
					'normal',
					'high'
				);
			}
			add_meta_box(
				'spp_metabox',
				'Simple Podcast Press: Transcript',
				array(
					$this,
					'spp_metabox_fn',
				),
				$post_type,
				'normal',
				'high'
			);
			add_meta_box(
				'spp_settings',
				'Simple Podcast Press: Custom Button 6',
				array(
					$this,
					'spp_setting_metabox_fn',
				),
				$post_type,
				'normal',
				'high'
			);
			add_meta_box(
				'spp_metabox',
				'Simple Podcast Press: Transcript',
				array(
					$this,
					'spp_metabox_fn',
				),
				$post_type,
				'normal',
				'high'
			);

			global $wp_meta_boxes;

			$current_metaboxes = $wp_meta_boxes['post']['normal']['high'];
			if ( isset( $current_metaboxes['spp_metabox'] ) ) {
				$spp_metabox['spp_metabox'] = $current_metaboxes['spp_metabox'];
				unset( $current_metaboxes['spp_metabox'] );
			}
			if ( count( $spp_metabox ) > 0 ) {
				$current_metaboxes = array_merge( $spp_metabox, $current_metaboxes );
			}
		} //end foreach
	}

	/**
	 * Output the HTML for the Simple Podcast Press link metabox.
	 *
	 * @param WP_Post $post The current post object.
	 */
	public function spp_link_metabox_fn( $post ) {

		global $wpdb;
		$table_spp_links = $wpdb->prefix . 'spp_links';

		$spp_slug = '';
		$spp_slug = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$table_spp_links} WHERE spp_post_id = %d", $post->ID ) ); // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching

		if ( isset( $spp_slug ) ) {
			$spp_slug_number = $spp_slug->spp_slug;
		} else {
			$spp_slug_number = '';
		}

		?>
	<div class="mab-spp-link">
		<p>
			<b><?php echo esc_url( get_site_url() ); ?>/</b>
			<input type="text" name="spp_slug" value="<?php echo esc_attr( $spp_slug_number ); ?>" style="width:100px;" />
		</p>
	</div>
		<?php
	}

	/**
	 * Save the Simple Podcast Press link metabox data.
	 *
	 * @param int  $post_id The ID of the current post.
	 * @param bool $cron    Whether the save is triggered by a cron job.
	 */
	public function spp_link_metabox_save( $post_id, $cron = false ) {
		global $wpdb;
		$table_spp_links = $wpdb->prefix . 'spp_links';

		// Security Checks.
		if ( ! $cron ) {
			if ( empty( $_POST['spp_slug'] ) || // phpcs:ignore WordPress.Security.NonceVerification.Missing
			( ! get_post_meta( $post_id, '_audiourl', true ) && ! get_post_meta( $post_id, '_useraudiourl', true ) ) ||
			! current_user_can( 'edit_posts' ) ||
			defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ||
			defined( 'DOING_AJAX' ) && DOING_AJAX ||
			wp_is_post_revision( $post_id ) ||
			in_array( get_post_status( $post_id ), array( 'auto-draft', 'pending' ) ) // phpcs:ignore WordPress.PHP.StrictInArray.MissingTrueStrict
			) {
				return;
			}
		}

		$post_title = html_entity_decode( get_the_title( $post_id ) );
		$slugarray  = preg_split( '/[\s,]+/', $post_title );

		foreach ( $slugarray as $slugelement ) {
			$slug = preg_replace( '~[^0-9]~', '', $slugelement );
			$slug = intval( $slug );
			if ( 0 !== $slug ) {
				break;
			}
		}

		if ( empty( $_POST['spp_slug'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$slug = $this->spp_get_option( 'episode_short_link' ) . $slug;
		} else {
			$slug = sanitize_text_field( wp_unslash( $_POST['spp_slug'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Missing
		}

		$is_available = $this->spp_slugIsAvailable( $slug );
		$permellink   = get_permalink( $post_id );

		$is_in_db = $wpdb->get_var( $wpdb->prepare( "SELECT spp_post_id FROM {$table_spp_links} WHERE spp_post_id = %d", $post_id ) ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching

		if ( empty( $slug ) ) {
			$slug = $post_id;
		}

		if ( ! $is_in_db ) {
			if ( $is_available || ! empty( $slug ) ) {
				$data = array(
					'spp_name'    => $post_title,
					'spp_url'     => $permellink,
					'spp_slug'    => $slug,
					'spp_post_id' => $post_id,
				);

				$wpdb->insert( $table_spp_links, $data ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery
			}
		} else {
			$data = array(
				'spp_name'    => $post_title,
				'spp_url'     => $permellink,
				'spp_slug'    => $slug,
				'spp_post_id' => $post_id,
			);

			$where = array(
				'spp_post_id' => $post_id,
			);

			$wpdb->update( $table_spp_links, $data, $where ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
		}
	}


	/**
	 * Retrieve content from a URL using wp_safe_remote_get.
	 *
	 * @param string $url The URL to retrieve content from.
	 * @return string|WP_Error The response body or a WP_Error if the request fails.
	 */
	public function url_get_contents( $url ) {
		$getresponse = wp_safe_remote_get(
			$url,
			array(
				'timeout'     => 60,
				'redirection' => 20,
				'sslverify'   => false,
			)
		);

		// Check if request was successful.
		if ( ! is_wp_error( $getresponse ) ) {
			$response = wp_remote_retrieve_body( $getresponse );
			return $response;
		} else {
			// Request failed, return a WP_Error.
			return $getresponse;
		}
	}
	/**
	 * Removes the Unicode character fe0f from the given text.
	 *
	 * @param string $text The input text.
	 *
	 * @return string The text with fe0f removed.
	 */
	public function remove_emoji( $text ) {
		$clean_text = '';

		$clean_text = preg_replace( '/[\x{1F3F4}](?:\x{E0067}\x{E0062}\x{E0077}\x{E006C}\x{E0073}\x{E007F})|[\x{1F3F4}](?:\x{E0067}\x{E0062}\x{E0073}\x{E0063}\x{E0074}\x{E007F})|[\x{1F3F4}](?:\x{E0067}\x{E0062}\x{E0065}\x{E006E}\x{E0067}\x{E007F})|[\x{1F3F4}](?:\x{200D}\x{2620}\x{FE0F})|[\x{1F3F3}](?:\x{FE0F}\x{200D}\x{1F308})|[\x{0023}\x{002A}\x{0030}\x{0031}\x{0032}\x{0033}\x{0034}\x{0035}\x{0036}\x{0037}\x{0038}\x{0039}](?:\x{FE0F}\x{20E3})|[\x{1F441}](?:\x{FE0F}\x{200D}\x{1F5E8}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F467}\x{200D}\x{1F467})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F467}\x{200D}\x{1F466})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F467})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F466}\x{200D}\x{1F466})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F466})|[\x{1F468}](?:\x{200D}\x{1F468}\x{200D}\x{1F467}\x{200D}\x{1F467})|[\x{1F468}](?:\x{200D}\x{1F468}\x{200D}\x{1F466}\x{200D}\x{1F466})|[\x{1F468}](?:\x{200D}\x{1F468}\x{200D}\x{1F467}\x{200D}\x{1F466})|[\x{1F468}](?:\x{200D}\x{1F468}\x{200D}\x{1F467})|[\x{1F468}](?:\x{200D}\x{1F468}\x{200D}\x{1F466})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F469}\x{200D}\x{1F467}\x{200D}\x{1F467})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F469}\x{200D}\x{1F466}\x{200D}\x{1F466})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F469}\x{200D}\x{1F467}\x{200D}\x{1F466})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F469}\x{200D}\x{1F467})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F469}\x{200D}\x{1F466})|[\x{1F469}](?:\x{200D}\x{2764}\x{FE0F}\x{200D}\x{1F469})|[\x{1F469}\x{1F468}](?:\x{200D}\x{2764}\x{FE0F}\x{200D}\x{1F468})|[\x{1F469}](?:\x{200D}\x{2764}\x{FE0F}\x{200D}\x{1F48B}\x{200D}\x{1F469})|[\x{1F469}\x{1F468}](?:\x{200D}\x{2764}\x{FE0F}\x{200D}\x{1F48B}\x{200D}\x{1F468})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F9B3})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F9B3})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F9B3})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F9B3})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F9B3})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F9B3})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F9B2})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F9B2})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F9B2})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F9B2})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F9B2})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F9B2})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F9B1})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F9B1})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F9B1})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F9B1})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F9B1})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F9B1})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F9B0})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F9B0})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F9B0})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F9B0})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F9B0})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F9B0})|[\x{1F575}\x{1F3CC}\x{26F9}\x{1F3CB}](?:\x{FE0F}\x{200D}\x{2640}\x{FE0F})|[\x{1F575}\x{1F3CC}\x{26F9}\x{1F3CB}](?:\x{FE0F}\x{200D}\x{2642}\x{FE0F})|[\x{1F46E}\x{1F575}\x{1F482}\x{1F477}\x{1F473}\x{1F471}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F3CC}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{26F9}\x{1F3CB}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93D}\x{1F93E}\x{1F939}](?:\x{1F3FF}\x{200D}\x{2640}\x{FE0F})|[\x{1F46E}\x{1F575}\x{1F482}\x{1F477}\x{1F473}\x{1F471}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F3CC}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{26F9}\x{1F3CB}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93D}\x{1F93E}\x{1F939}](?:\x{1F3FE}\x{200D}\x{2640}\x{FE0F})|[\x{1F46E}\x{1F575}\x{1F482}\x{1F477}\x{1F473}\x{1F471}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F3CC}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{26F9}\x{1F3CB}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93D}\x{1F93E}\x{1F939}](?:\x{1F3FD}\x{200D}\x{2640}\x{FE0F})|[\x{1F46E}\x{1F575}\x{1F482}\x{1F477}\x{1F473}\x{1F471}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F3CC}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{26F9}\x{1F3CB}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93D}\x{1F93E}\x{1F939}](?:\x{1F3FC}\x{200D}\x{2640}\x{FE0F})|[\x{1F46E}\x{1F575}\x{1F482}\x{1F477}\x{1F473}\x{1F471}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F3CC}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{26F9}\x{1F3CB}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93D}\x{1F93E}\x{1F939}](?:\x{1F3FB}\x{200D}\x{2640}\x{FE0F})|[\x{1F46E}\x{1F9B8}\x{1F9B9}\x{1F482}\x{1F477}\x{1F473}\x{1F471}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F9DE}\x{1F9DF}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F46F}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93C}\x{1F93D}\x{1F93E}\x{1F939}](?:\x{200D}\x{2640}\x{FE0F})|[\x{1F46E}\x{1F575}\x{1F482}\x{1F477}\x{1F473}\x{1F471}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F3CC}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{26F9}\x{1F3CB}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93D}\x{1F93E}\x{1F939}](?:\x{1F3FF}\x{200D}\x{2642}\x{FE0F})|[\x{1F46E}\x{1F575}\x{1F482}\x{1F477}\x{1F473}\x{1F471}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F3CC}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{26F9}\x{1F3CB}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93D}\x{1F93E}\x{1F939}](?:\x{1F3FE}\x{200D}\x{2642}\x{FE0F})|[\x{1F46E}\x{1F575}\x{1F482}\x{1F477}\x{1F473}\x{1F471}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F3CC}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{26F9}\x{1F3CB}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93D}\x{1F93E}\x{1F939}](?:\x{1F3FD}\x{200D}\x{2642}\x{FE0F})|[\x{1F46E}\x{1F575}\x{1F482}\x{1F477}\x{1F473}\x{1F471}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F3CC}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{26F9}\x{1F3CB}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93D}\x{1F93E}\x{1F939}](?:\x{1F3FC}\x{200D}\x{2642}\x{FE0F})|[\x{1F46E}\x{1F575}\x{1F482}\x{1F477}\x{1F473}\x{1F471}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F3CC}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{26F9}\x{1F3CB}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93D}\x{1F93E}\x{1F939}](?:\x{1F3FB}\x{200D}\x{2642}\x{FE0F})|[\x{1F46E}\x{1F9B8}\x{1F9B9}\x{1F482}\x{1F477}\x{1F473}\x{1F471}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F9DE}\x{1F9DF}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F46F}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93C}\x{1F93D}\x{1F93E}\x{1F939}](?:\x{200D}\x{2642}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F692})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F692})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F692})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F692})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F692})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F692})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F680})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F680})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F680})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F680})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F680})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F680})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{2708}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{2708}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{2708}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{2708}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{2708}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{200D}\x{2708}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F3A8})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F3A8})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F3A8})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F3A8})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F3A8})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F3A8})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F3A4})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F3A4})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F3A4})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F3A4})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F3A4})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F3A4})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F4BB})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F4BB})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F4BB})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F4BB})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F4BB})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F4BB})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F52C})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F52C})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F52C})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F52C})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F52C})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F52C})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F4BC})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F4BC})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F4BC})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F4BC})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F4BC})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F4BC})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F3ED})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F3ED})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F3ED})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F3ED})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F3ED})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F3ED})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F527})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F527})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F527})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F527})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F527})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F527})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F373})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F373})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F373})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F373})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F373})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F373})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F33E})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F33E})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F33E})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F33E})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F33E})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F33E})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{2696}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{2696}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{2696}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{2696}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{2696}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{200D}\x{2696}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F3EB})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F3EB})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F3EB})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F3EB})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F3EB})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F3EB})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{1F393})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{1F393})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{1F393})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{1F393})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{1F393})|[\x{1F468}\x{1F469}](?:\x{200D}\x{1F393})|[\x{1F468}\x{1F469}](?:\x{1F3FF}\x{200D}\x{2695}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{1F3FE}\x{200D}\x{2695}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{1F3FD}\x{200D}\x{2695}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{1F3FC}\x{200D}\x{2695}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{1F3FB}\x{200D}\x{2695}\x{FE0F})|[\x{1F468}\x{1F469}](?:\x{200D}\x{2695}\x{FE0F})|[\x{1F476}\x{1F9D2}\x{1F466}\x{1F467}\x{1F9D1}\x{1F468}\x{1F469}\x{1F9D3}\x{1F474}\x{1F475}\x{1F46E}\x{1F575}\x{1F482}\x{1F477}\x{1F934}\x{1F478}\x{1F473}\x{1F472}\x{1F9D5}\x{1F9D4}\x{1F471}\x{1F935}\x{1F470}\x{1F930}\x{1F931}\x{1F47C}\x{1F385}\x{1F936}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F483}\x{1F57A}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F6C0}\x{1F6CC}\x{1F574}\x{1F3C7}\x{1F3C2}\x{1F3CC}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{26F9}\x{1F3CB}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93D}\x{1F93E}\x{1F939}\x{1F933}\x{1F4AA}\x{1F9B5}\x{1F9B6}\x{1F448}\x{1F449}\x{261D}\x{1F446}\x{1F595}\x{1F447}\x{270C}\x{1F91E}\x{1F596}\x{1F918}\x{1F919}\x{1F590}\x{270B}\x{1F44C}\x{1F44D}\x{1F44E}\x{270A}\x{1F44A}\x{1F91B}\x{1F91C}\x{1F91A}\x{1F44B}\x{1F91F}\x{270D}\x{1F44F}\x{1F450}\x{1F64C}\x{1F932}\x{1F64F}\x{1F485}\x{1F442}\x{1F443}](?:\x{1F3FF})|[\x{1F476}\x{1F9D2}\x{1F466}\x{1F467}\x{1F9D1}\x{1F468}\x{1F469}\x{1F9D3}\x{1F474}\x{1F475}\x{1F46E}\x{1F575}\x{1F482}\x{1F477}\x{1F934}\x{1F478}\x{1F473}\x{1F472}\x{1F9D5}\x{1F9D4}\x{1F471}\x{1F935}\x{1F470}\x{1F930}\x{1F931}\x{1F47C}\x{1F385}\x{1F936}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F483}\x{1F57A}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F6C0}\x{1F6CC}\x{1F574}\x{1F3C7}\x{1F3C2}\x{1F3CC}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{26F9}\x{1F3CB}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93D}\x{1F93E}\x{1F939}\x{1F933}\x{1F4AA}\x{1F9B5}\x{1F9B6}\x{1F448}\x{1F449}\x{261D}\x{1F446}\x{1F595}\x{1F447}\x{270C}\x{1F91E}\x{1F596}\x{1F918}\x{1F919}\x{1F590}\x{270B}\x{1F44C}\x{1F44D}\x{1F44E}\x{270A}\x{1F44A}\x{1F91B}\x{1F91C}\x{1F91A}\x{1F44B}\x{1F91F}\x{270D}\x{1F44F}\x{1F450}\x{1F64C}\x{1F932}\x{1F64F}\x{1F485}\x{1F442}\x{1F443}](?:\x{1F3FE})|[\x{1F476}\x{1F9D2}\x{1F466}\x{1F467}\x{1F9D1}\x{1F468}\x{1F469}\x{1F9D3}\x{1F474}\x{1F475}\x{1F46E}\x{1F575}\x{1F482}\x{1F477}\x{1F934}\x{1F478}\x{1F473}\x{1F472}\x{1F9D5}\x{1F9D4}\x{1F471}\x{1F935}\x{1F470}\x{1F930}\x{1F931}\x{1F47C}\x{1F385}\x{1F936}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F483}\x{1F57A}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F6C0}\x{1F6CC}\x{1F574}\x{1F3C7}\x{1F3C2}\x{1F3CC}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{26F9}\x{1F3CB}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93D}\x{1F93E}\x{1F939}\x{1F933}\x{1F4AA}\x{1F9B5}\x{1F9B6}\x{1F448}\x{1F449}\x{261D}\x{1F446}\x{1F595}\x{1F447}\x{270C}\x{1F91E}\x{1F596}\x{1F918}\x{1F919}\x{1F590}\x{270B}\x{1F44C}\x{1F44D}\x{1F44E}\x{270A}\x{1F44A}\x{1F91B}\x{1F91C}\x{1F91A}\x{1F44B}\x{1F91F}\x{270D}\x{1F44F}\x{1F450}\x{1F64C}\x{1F932}\x{1F64F}\x{1F485}\x{1F442}\x{1F443}](?:\x{1F3FD})|[\x{1F476}\x{1F9D2}\x{1F466}\x{1F467}\x{1F9D1}\x{1F468}\x{1F469}\x{1F9D3}\x{1F474}\x{1F475}\x{1F46E}\x{1F575}\x{1F482}\x{1F477}\x{1F934}\x{1F478}\x{1F473}\x{1F472}\x{1F9D5}\x{1F9D4}\x{1F471}\x{1F935}\x{1F470}\x{1F930}\x{1F931}\x{1F47C}\x{1F385}\x{1F936}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F483}\x{1F57A}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F6C0}\x{1F6CC}\x{1F574}\x{1F3C7}\x{1F3C2}\x{1F3CC}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{26F9}\x{1F3CB}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93D}\x{1F93E}\x{1F939}\x{1F933}\x{1F4AA}\x{1F9B5}\x{1F9B6}\x{1F448}\x{1F449}\x{261D}\x{1F446}\x{1F595}\x{1F447}\x{270C}\x{1F91E}\x{1F596}\x{1F918}\x{1F919}\x{1F590}\x{270B}\x{1F44C}\x{1F44D}\x{1F44E}\x{270A}\x{1F44A}\x{1F91B}\x{1F91C}\x{1F91A}\x{1F44B}\x{1F91F}\x{270D}\x{1F44F}\x{1F450}\x{1F64C}\x{1F932}\x{1F64F}\x{1F485}\x{1F442}\x{1F443}](?:\x{1F3FC})|[\x{1F476}\x{1F9D2}\x{1F466}\x{1F467}\x{1F9D1}\x{1F468}\x{1F469}\x{1F9D3}\x{1F474}\x{1F475}\x{1F46E}\x{1F575}\x{1F482}\x{1F477}\x{1F934}\x{1F478}\x{1F473}\x{1F472}\x{1F9D5}\x{1F9D4}\x{1F471}\x{1F935}\x{1F470}\x{1F930}\x{1F931}\x{1F47C}\x{1F385}\x{1F936}\x{1F9D9}\x{1F9DA}\x{1F9DB}\x{1F9DC}\x{1F9DD}\x{1F64D}\x{1F64E}\x{1F645}\x{1F646}\x{1F481}\x{1F64B}\x{1F647}\x{1F926}\x{1F937}\x{1F486}\x{1F487}\x{1F6B6}\x{1F3C3}\x{1F483}\x{1F57A}\x{1F9D6}\x{1F9D7}\x{1F9D8}\x{1F6C0}\x{1F6CC}\x{1F574}\x{1F3C7}\x{1F3C2}\x{1F3CC}\x{1F3C4}\x{1F6A3}\x{1F3CA}\x{26F9}\x{1F3CB}\x{1F6B4}\x{1F6B5}\x{1F938}\x{1F93D}\x{1F93E}\x{1F939}\x{1F933}\x{1F4AA}\x{1F9B5}\x{1F9B6}\x{1F448}\x{1F449}\x{261D}\x{1F446}\x{1F595}\x{1F447}\x{270C}\x{1F91E}\x{1F596}\x{1F918}\x{1F919}\x{1F590}\x{270B}\x{1F44C}\x{1F44D}\x{1F44E}\x{270A}\x{1F44A}\x{1F91B}\x{1F91C}\x{1F91A}\x{1F44B}\x{1F91F}\x{270D}\x{1F44F}\x{1F450}\x{1F64C}\x{1F932}\x{1F64F}\x{1F485}\x{1F442}\x{1F443}](?:\x{1F3FB})|[\x{1F1E6}\x{1F1E7}\x{1F1E8}\x{1F1E9}\x{1F1F0}\x{1F1F2}\x{1F1F3}\x{1F1F8}\x{1F1F9}\x{1F1FA}](?:\x{1F1FF})|[\x{1F1E7}\x{1F1E8}\x{1F1EC}\x{1F1F0}\x{1F1F1}\x{1F1F2}\x{1F1F5}\x{1F1F8}\x{1F1FA}](?:\x{1F1FE})|[\x{1F1E6}\x{1F1E8}\x{1F1F2}\x{1F1F8}](?:\x{1F1FD})|[\x{1F1E6}\x{1F1E7}\x{1F1E8}\x{1F1EC}\x{1F1F0}\x{1F1F2}\x{1F1F5}\x{1F1F7}\x{1F1F9}\x{1F1FF}](?:\x{1F1FC})|[\x{1F1E7}\x{1F1E8}\x{1F1F1}\x{1F1F2}\x{1F1F8}\x{1F1F9}](?:\x{1F1FB})|[\x{1F1E6}\x{1F1E8}\x{1F1EA}\x{1F1EC}\x{1F1ED}\x{1F1F1}\x{1F1F2}\x{1F1F3}\x{1F1F7}\x{1F1FB}](?:\x{1F1FA})|[\x{1F1E6}\x{1F1E7}\x{1F1EA}\x{1F1EC}\x{1F1ED}\x{1F1EE}\x{1F1F1}\x{1F1F2}\x{1F1F5}\x{1F1F8}\x{1F1F9}\x{1F1FE}](?:\x{1F1F9})|[\x{1F1E6}\x{1F1E7}\x{1F1EA}\x{1F1EC}\x{1F1EE}\x{1F1F1}\x{1F1F2}\x{1F1F5}\x{1F1F7}\x{1F1F8}\x{1F1FA}\x{1F1FC}](?:\x{1F1F8})|[\x{1F1E6}\x{1F1E7}\x{1F1E8}\x{1F1EA}\x{1F1EB}\x{1F1EC}\x{1F1ED}\x{1F1EE}\x{1F1F0}\x{1F1F1}\x{1F1F2}\x{1F1F3}\x{1F1F5}\x{1F1F8}\x{1F1F9}](?:\x{1F1F7})|[\x{1F1E6}\x{1F1E7}\x{1F1EC}\x{1F1EE}\x{1F1F2}](?:\x{1F1F6})|[\x{1F1E8}\x{1F1EC}\x{1F1EF}\x{1F1F0}\x{1F1F2}\x{1F1F3}](?:\x{1F1F5})|[\x{1F1E6}\x{1F1E7}\x{1F1E8}\x{1F1E9}\x{1F1EB}\x{1F1EE}\x{1F1EF}\x{1F1F2}\x{1F1F3}\x{1F1F7}\x{1F1F8}\x{1F1F9}](?:\x{1F1F4})|[\x{1F1E7}\x{1F1E8}\x{1F1EC}\x{1F1ED}\x{1F1EE}\x{1F1F0}\x{1F1F2}\x{1F1F5}\x{1F1F8}\x{1F1F9}\x{1F1FA}\x{1F1FB}](?:\x{1F1F3})|[\x{1F1E6}\x{1F1E7}\x{1F1E8}\x{1F1E9}\x{1F1EB}\x{1F1EC}\x{1F1ED}\x{1F1EE}\x{1F1EF}\x{1F1F0}\x{1F1F2}\x{1F1F4}\x{1F1F5}\x{1F1F8}\x{1F1F9}\x{1F1FA}\x{1F1FF}](?:\x{1F1F2})|[\x{1F1E6}\x{1F1E7}\x{1F1E8}\x{1F1EC}\x{1F1EE}\x{1F1F2}\x{1F1F3}\x{1F1F5}\x{1F1F8}\x{1F1F9}](?:\x{1F1F1})|[\x{1F1E8}\x{1F1E9}\x{1F1EB}\x{1F1ED}\x{1F1F1}\x{1F1F2}\x{1F1F5}\x{1F1F8}\x{1F1F9}\x{1F1FD}](?:\x{1F1F0})|[\x{1F1E7}\x{1F1E9}\x{1F1EB}\x{1F1F8}\x{1F1F9}](?:\x{1F1EF})|[\x{1F1E6}\x{1F1E7}\x{1F1E8}\x{1F1EB}\x{1F1EC}\x{1F1F0}\x{1F1F1}\x{1F1F3}\x{1F1F8}\x{1F1FB}](?:\x{1F1EE})|[\x{1F1E7}\x{1F1E8}\x{1F1EA}\x{1F1EC}\x{1F1F0}\x{1F1F2}\x{1F1F5}\x{1F1F8}\x{1F1F9}](?:\x{1F1ED})|[\x{1F1E6}\x{1F1E7}\x{1F1E8}\x{1F1E9}\x{1F1EA}\x{1F1EC}\x{1F1F0}\x{1F1F2}\x{1F1F3}\x{1F1F5}\x{1F1F8}\x{1F1F9}\x{1F1FA}\x{1F1FB}](?:\x{1F1EC})|[\x{1F1E6}\x{1F1E7}\x{1F1E8}\x{1F1EC}\x{1F1F2}\x{1F1F3}\x{1F1F5}\x{1F1F9}\x{1F1FC}](?:\x{1F1EB})|[\x{1F1E6}\x{1F1E7}\x{1F1E9}\x{1F1EA}\x{1F1EC}\x{1F1EE}\x{1F1EF}\x{1F1F0}\x{1F1F2}\x{1F1F3}\x{1F1F5}\x{1F1F7}\x{1F1F8}\x{1F1FB}\x{1F1FE}](?:\x{1F1EA})|[\x{1F1E6}\x{1F1E7}\x{1F1E8}\x{1F1EC}\x{1F1EE}\x{1F1F2}\x{1F1F8}\x{1F1F9}](?:\x{1F1E9})|[\x{1F1E6}\x{1F1E8}\x{1F1EA}\x{1F1EE}\x{1F1F1}\x{1F1F2}\x{1F1F3}\x{1F1F8}\x{1F1F9}\x{1F1FB}](?:\x{1F1E8})|[\x{1F1E7}\x{1F1EC}\x{1F1F1}\x{1F1F8}](?:\x{1F1E7})|[\x{1F1E7}\x{1F1E8}\x{1F1EA}\x{1F1EC}\x{1F1F1}\x{1F1F2}\x{1F1F3}\x{1F1F5}\x{1F1F6}\x{1F1F8}\x{1F1F9}\x{1F1FA}\x{1F1FB}\x{1F1FF}](?:\x{1F1E6})|[\x{00A9}\x{00AE}\x{203C}\x{2049}\x{2122}\x{2139}\x{2194}-\x{2199}\x{21A9}-\x{21AA}\x{231A}-\x{231B}\x{2328}\x{23CF}\x{23E9}-\x{23F3}\x{23F8}-\x{23FA}\x{24C2}\x{25AA}-\x{25AB}\x{25B6}\x{25C0}\x{25FB}-\x{25FE}\x{2600}-\x{2604}\x{260E}\x{2611}\x{2614}-\x{2615}\x{2618}\x{261D}\x{2620}\x{2622}-\x{2623}\x{2626}\x{262A}\x{262E}-\x{262F}\x{2638}-\x{263A}\x{2640}\x{2642}\x{2648}-\x{2653}\x{2660}\x{2663}\x{2665}-\x{2666}\x{2668}\x{267B}\x{267E}-\x{267F}\x{2692}-\x{2697}\x{2699}\x{269B}-\x{269C}\x{26A0}-\x{26A1}\x{26AA}-\x{26AB}\x{26B0}-\x{26B1}\x{26BD}-\x{26BE}\x{26C4}-\x{26C5}\x{26C8}\x{26CE}-\x{26CF}\x{26D1}\x{26D3}-\x{26D4}\x{26E9}-\x{26EA}\x{26F0}-\x{26F5}\x{26F7}-\x{26FA}\x{26FD}\x{2702}\x{2705}\x{2708}-\x{270D}\x{270F}\x{2712}\x{2714}\x{2716}\x{271D}\x{2721}\x{2728}\x{2733}-\x{2734}\x{2744}\x{2747}\x{274C}\x{274E}\x{2753}-\x{2755}\x{2757}\x{2763}-\x{2764}\x{2795}-\x{2797}\x{27A1}\x{27B0}\x{27BF}\x{2934}-\x{2935}\x{2B05}-\x{2B07}\x{2B1B}-\x{2B1C}\x{2B50}\x{2B55}\x{3030}\x{303D}\x{3297}\x{3299}\x{1F004}\x{1F0CF}\x{1F170}-\x{1F171}\x{1F17E}-\x{1F17F}\x{1F18E}\x{1F191}-\x{1F19A}\x{1F201}-\x{1F202}\x{1F21A}\x{1F22F}\x{1F232}-\x{1F23A}\x{1F250}-\x{1F251}\x{1F300}-\x{1F321}\x{1F324}-\x{1F393}\x{1F396}-\x{1F397}\x{1F399}-\x{1F39B}\x{1F39E}-\x{1F3F0}\x{1F3F3}-\x{1F3F5}\x{1F3F7}-\x{1F3FA}\x{1F400}-\x{1F4FD}\x{1F4FF}-\x{1F53D}\x{1F549}-\x{1F54E}\x{1F550}-\x{1F567}\x{1F56F}-\x{1F570}\x{1F573}-\x{1F57A}\x{1F587}\x{1F58A}-\x{1F58D}\x{1F590}\x{1F595}-\x{1F596}\x{1F5A4}-\x{1F5A5}\x{1F5A8}\x{1F5B1}-\x{1F5B2}\x{1F5BC}\x{1F5C2}-\x{1F5C4}\x{1F5D1}-\x{1F5D3}\x{1F5DC}-\x{1F5DE}\x{1F5E1}\x{1F5E3}\x{1F5E8}\x{1F5EF}\x{1F5F3}\x{1F5FA}-\x{1F64F}\x{1F680}-\x{1F6C5}\x{1F6CB}-\x{1F6D2}\x{1F6E0}-\x{1F6E5}\x{1F6E9}\x{1F6EB}-\x{1F6EC}\x{1F6F0}\x{1F6F3}-\x{1F6F9}\x{1F910}-\x{1F93A}\x{1F93C}-\x{1F93E}\x{1F940}-\x{1F945}\x{1F947}-\x{1F970}\x{1F973}-\x{1F976}\x{1F97A}\x{1F97C}-\x{1F9A2}\x{1F9B0}-\x{1F9B9}\x{1F9C0}-\x{1F9C2}\x{1F9D0}-\x{1F9FF}]/u', '', $text );

		// Removes Square Box Hex fe0f.
		$regex_emoticons = '/[\x{fe0f}]/u';
		$clean_text      = preg_replace( $regex_emoticons, '', $clean_text );

		return $clean_text;
	}

	/**
	 * Perform a cURL request to the specified URL.
	 *
	 * @param string $url The URL to request.
	 *
	 * @return mixed The response content or null on failure.
	 */
	public function curl( $url ) {
		$curl = curl_init( $url ); // phpcs:ignore WordPress.WP.AlternativeFunctions.curl_curl_init
		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true ); // phpcs:ignore WordPress.WP.AlternativeFunctions.curl_curl_setopt
		curl_setopt( $curl, CURLOPT_HEADER, 0 ); // phpcs:ignore WordPress.WP.AlternativeFunctions.curl_curl_setopt
		curl_setopt( $curl, CURLOPT_USERAGENT, '' ); // phpcs:ignore WordPress.WP.AlternativeFunctions.curl_curl_setopt
		curl_setopt( $curl, CURLOPT_TIMEOUT, 10 ); // phpcs:ignore WordPress.WP.AlternativeFunctions.curl_curl_setopt
		$response = curl_exec( $curl ); // phpcs:ignore WordPress.WP.AlternativeFunctions.curl_curl_exec

		if ( 0 !== curl_errno( $curl ) || 200 !== curl_getinfo( $curl, CURLINFO_HTTP_CODE ) ) { // phpcs:ignore WordPress.WP.AlternativeFunctions.curl,WordPress.WP.AlternativeFunctions.curl,WordPress.WP.AlternativeFunctions.curl,WordPress.WP.AlternativeFunctions.curl_curl_errno_curl_getinfo_curl_getinfo_curl_errno
			$response = null;
		}

		curl_close( $curl ); // phpcs:ignore WordPress.WP.AlternativeFunctions.curl_curl_close

		return $response;
	}

	/**
	 * Check if a slug is available for use.
	 *
	 * @param string $full_slug The full slug to check.
	 * @param string $id        Optional ID to exclude from check.
	 *
	 * @return bool True if the slug is available, false otherwise.
	 */
	public function spp_slugIsAvailable( $full_slug, $id = '' ) { // phpcs:ignore WordPress.NamingConventions.ValidFunctionName.MethodNameInvalid
		global $wpdb;

		$slug = $full_slug;

		$limit = 1;
		// Check slug uniqueness against posts, pages and categories.
		$postname = $wpdb->get_var( $wpdb->prepare( "SELECT post_name FROM {$wpdb->posts} WHERE post_name=%s LIMIT %d", $slug, $limit ) ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
		$taxonomy = $wpdb->get_var( $wpdb->prepare( "SELECT taxonomy FROM {$wpdb->term_taxonomy} WHERE taxonomy=%s LIMIT %d", $slug, $limit ) ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching

		// If anything was returned for these two calls then the slug has been taken.
		if ( $postname || $taxonomy ) {
			return false;
		}

		// Check slug against files on the root WordPress install.
		$root_dir = opendir( ABSPATH );

		while ( ( $file = readdir( $root_dir ) ) !== false ) { // phpcs:ignore Generic.CodeAnalysis.AssignmentInCondition.FoundInWhileCondition
			$haystack = strtolower( $file );
			if ( $haystack === $slug ) {
				return false;
			}
		}

		$table_spp_links = $wpdb->prefix . 'spp_links';
		if ( ! is_null( $id ) && ! empty( $id ) && is_numeric( $id ) ) {
			$link_slug = $wpdb->get_var( $wpdb->prepare( "SELECT spp_slug FROM {$table_spp_links} WHERE spp_slug=%s AND id <> %d", $full_slug, $id ) ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
		} else {
			$link_slug = $wpdb->get_var( $wpdb->prepare( "SELECT spp_slug FROM {$table_spp_links} WHERE spp_slug=%s", $full_slug ) ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
		}

		if ( $link_slug === $full_slug ) {
			return false;
		}

		$pre_slug_slug = $this->get_permalink_pre_slug_uri( true, true );

		if ( $full_slug === $pre_slug_slug ) {
			return false;
		}

		return true;
	}

	/**
	 * Get the pre-slug URI.
	 *
	 * @param bool $force Force the retrieval even if not set.
	 * @param bool $trim  Trim the pre-slug URI.
	 *
	 * @return string The pre-slug URI.
	 */
	public function get_permalink_pre_slug_uri( $force = false, $trim = false ) {
		if ( $force ) {
			preg_match( '#^([^%]*?)%#', get_option( 'permalink_structure' ), $struct );
			$pre_slug_uri = $struct[1];
			if ( $trim ) {
				$pre_slug_uri = trim( $pre_slug_uri );
				$pre_slug_uri = preg_replace( '#^/#', '', $pre_slug_uri );
				$pre_slug_uri = preg_replace( '#/$#', '', $pre_slug_uri );
			}
			return $pre_slug_uri;
		} else {
			return '/';
		}
	}

	/**
	 * Setting Metabox Function.
	 *
	 * @param WP_Post $post The current post object.
	 */
	public function spp_setting_metabox_fn( $post ) {
		$btn_spp_custom_name6 = get_post_meta( $post->ID, '_btn_spp_custom_name6', true );
		$btn_spp_custom_url   = get_post_meta( $post->ID, '_btn_spp_custom_url6', true );
		?>
	<div class="well well-1" style="padding-top: 0px;">
		<div class="row-fluid row-fluid-4">
			<h3 style="margin-top:0px padding-top:0px"> <small>This will override the existing Custom Button 6</small> </h3>
			<span class="span4" style="height: auto ! important; margin-left:0px;">
				<input type="text" name="_btn_spp_custom_name6" id="btn_spp_custom_name6" style="width:220px" value="<?php echo esc_html( $btn_spp_custom_name6 ); ?>" placeholder="Button Name ...">
			</span>
			<span class="span3" style="height: auto ! important; width: 19%;">
				<input type="text" name="_btn_spp_custom_url6" id="btn_spp_custom_url6" style="width:250px" value="<?php echo esc_html( $btn_spp_custom_url ); ?>" placeholder="Enter Button URL ...">
			</span>
		</div>
	</div>
		<?php
		do_action( 'spp_setting_metabox' );
	}

	/**
	 * Audio URL Metabox Function.
	 *
	 * @param WP_Post $post The current post object.
	 */
	public function spp_audiourl_metabox_fn( $post ) {
		$spp_audiourl_frombox     = rawurldecode( get_post_meta( $post->ID, '_audiourl', true ) );
		$spp_useraudiourl_frombox = rawurldecode( get_post_meta( $post->ID, '_useraudiourl', true ) );
		?>
	<div class="well well-1" style="padding-top: 0px;">
		<div class="row-fluid row-fluid-4">
			<strong>Audio URL:</strong> <?php echo esc_html( $spp_audiourl_frombox ); ?><br><br>
			<strong>User Audio URL:</strong> <?php echo esc_html( $spp_useraudiourl_frombox ); ?><br>
		</div>
	</div>
		<?php
		do_action( 'spp_setting_metabox' );
	}

	/**
	 * Metabox Function.
	 *
	 * @param WP_Post $post The current post object.
	 */
	public function spp_metabox_fn( $post ) {
		$spptranscript = get_post_meta( $post->ID, '_spptranscript', true );
		?>
	<div class="mab-spp-transcript">
		<p>Enter your transcript or show notes here. To insert the transcript on your page, add the [spp-transcript] shortcode in the main post editor above.</p>
		<textarea name="_spptranscript" rows="10" style="width: 100%"><?php echo esc_html( $spptranscript ); ?></textarea>
	</div>
		<?php
	}


	/**
	 * Save metabox data.
	 *
	 * @param int $post_id The ID of the post being saved.
	 * @return void
	 */
	public function spp_metabox_save( $post_id ) {
		// Bail if we're doing an auto save.
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return;
		}

		// if our current user can't edit this post, bail.
		if ( ! current_user_can( 'edit_posts' ) ) {
			return;
		}

		// Now we can actually save the data.

		if ( isset( $_POST['_spptranscript'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$transcript_text = wpautop( wp_unslash( $_POST['_spptranscript'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			update_post_meta( $post_id, '_spptranscript', $transcript_text );
		}

		if ( isset( $_POST['_btn_spp_custom_name6'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$btn_spp_custom_name6 = sanitize_text_field( wp_unslash( $_POST['_btn_spp_custom_name6'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			update_post_meta( $post_id, '_btn_spp_custom_name6', $btn_spp_custom_name6 );
		}

		if ( isset( $_POST['_btn_spp_custom_url6'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$btn_spp_custom_url6 = sanitize_text_field( wp_unslash( $_POST['_btn_spp_custom_url6'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Missing
			update_post_meta( $post_id, '_btn_spp_custom_url6', $btn_spp_custom_url6 );
		}

		do_action( 'spp_setting_metabox_save', $post_id );
	}
	/**
	 * Get iTunes ID from URL.
	 *
	 * @param string $url The URL to extract iTunes ID from.
	 * @return string|false iTunes ID if found, false otherwise.
	 */
	public function get_itunes_ID( $url ) { // phpcs:ignore WordPress.NamingConventions.ValidFunctionName.MethodNameInvalid
		if ( ! $url ) {
			return false;
		}

		if ( strpos( $url, '.apple' ) === false ) {
			$headers = get_headers( $url, 1 );
			if ( isset( $headers['Location'][1] ) ) {
				$url = $headers['Location'][1];
			}
		}

		$idstyles       = array(
			'?id=',
			'&id=',
			'/id',
		);
		$count_id_style = count( $idstyles );
		for ( $counter = 0; $counter < $count_id_style; $counter++ ) {
			if ( strpos( $url, $idstyles[ $counter ] ) !== false ) {
				$idstyle  = $idstyles[ $counter ];
				$valid_id = "So, how are you holding up? Because I'm a potato!";
				break;
			}
		}

		preg_match( '/[0-9]+/', $url, $podid, 0, strpos( $url, $idstyle ) );

		return $podid[0];
	}

	/**
	 * Save plugin settings and trigger save action.
	 */
	public function spp_save_plugin_settings() {
		$this->save_spp_settings();
		$this->generate_buttons_css();
		$this->generate_options_css();
		$this->generate_barui_css();
		$this->generate_classic_css();

		do_action( 'spp_save_settings' );
	}

	/**
	 * Add scheme (http://) to URL if missing.
	 *
	 * @param string $url The URL to check and modify.
	 * @return string Modified URL with scheme.
	 */
	public function addScheme( $url ) { // phpcs:ignore WordPress.NamingConventions.ValidFunctionName.MethodNameInvalid
		if ( empty( $url ) ) {
			return $url;
		}

		if ( ! preg_match( '~^(?:f|ht)tps?://~i', $url ) ) {
			$url = 'http://' . $url;
		}
		return $url;
	}

	/**
	 * Check if the provided URL is a valid podcast URL.
	 *
	 * @param string $url The URL to check.
	 * @return bool True if the URL is a valid podcast URL, false otherwise.
	 */
	public function CheckPodcastUrl( $url ) { // phpcs:ignore WordPress.NamingConventions.ValidFunctionName.MethodNameInvalid
		$xml         = $this->url_get_contents( $url );
		$itunes_xml  = preg_replace( '/(<\/?)(\w+):([^>]*>)/', '$1$2$3', $xml );
		$podcast_xml = simplexml_load_string( $itunes_xml );
		return is_object( $podcast_xml ) ? true : false;
	}

	/**
	 * Save podcast settings.
	 */
	public function spp_save_podcast_settings() {
		do_action( 'spp_save_settings' );
		global $wpdb;
		$table_spp_podcast = $wpdb->prefix . 'spp_podcast';

		$variable = isset( $_POST['yourpostname'] ) ? wp_unslash( $_POST['yourpostname'] ) : ''; // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		$nonce    = isset( $_POST['wpnonce'] ) ? wp_unslash( $_POST['wpnonce'] ) : ''; // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		if ( ! wp_verify_nonce( $nonce, 'settings' ) ) {
			// This nonce is not valid.
			die( 'Security check' );
		} else {
			$this->spp_settings = get_option( 'spp_settings' );

			if ( 'Cancel' === $variable ) {
				// Reset settings and delete tables.
				$this->spp_settings['itunes_id']      = '';
				$this->spp_settings['btn_itunes_url'] = '';
				$this->spp_settings['itunes_url']     = '';
				$this->spp_settings['podcast_url']    = '';
				$this->spp_settings['total_podcasts'] = 0;
				$this->spp_settings['reviews']        = 0;
				$this->spp_settings['pp_counter']     = 0;

				update_option( 'spp_settings', $this->spp_settings );

				$table_spp_podcast = $wpdb->prefix . 'spp_podcast';
				$table_spp_reviews = $wpdb->prefix . 'spp_reviews';

				$sql0 = 'DROP TABLE IF EXISTS ' . $table_spp_podcast;
				$wpdb->query( $sql0 ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching

				$sql1 = 'DROP TABLE IF EXISTS ' . $table_spp_reviews;
				$wpdb->query( $sql1 ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching

				$this->delete_spp_settings();

				die();
			}

			if ( 'Continue' === $variable ) {
				$itunes_url = isset( $_POST['podcast_url'] ) ? sanitize_text_field( wp_unslash( $_POST['podcast_url'] ) ) : '';
				$itunes_url = $this->addScheme( $itunes_url );

				if ( strpos( $itunes_url, '?' ) ) {
					$itunes_url = substr( $itunes_url, 0, strpos( $itunes_url, '?' ) );
				}

				$podcast_xml = isset( $_POST['podcast_xml'] ) ? sanitize_text_field( wp_unslash( $_POST['podcast_xml'] ) ) : '';

				if ( ! empty( $podcast_xml ) ) {
					$podcast_xml = $this->addScheme( $podcast_xml );

					if ( ! $this->CheckPodcastUrl( $podcast_xml ) ) {
						die( ' Invalid Podcast Feed' );
					}
				}
				$itunes_url_redirect = false;
				$podcastURL          = false; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

				if ( ! empty( $itunes_url ) ) {
					if ( strpos( $itunes_url, '.apple' ) === false ) {
						$headers = get_headers( $itunes_url, 1 );
						if ( isset( $headers['Location'][1] ) ) {
							$itunes_url_redirect = $headers['Location'][1];
							$itunes_url          = $itunes_url_redirect;
						}
					}

					$itunes_ID = $this->get_itunes_ID( $itunes_url ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					if ( empty( $itunes_ID ) ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
						die( ' Invalid Apple Podcast URL' );
					}
					$this->spp_settings['itunes_id']      = $itunes_ID; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$this->spp_settings['btn_itunes_url'] = $itunes_url;
					$this->spp_settings['itunes_url']     = $itunes_url;

					// Json Parsing.
					$get_podcast = 'https://itunes.apple.com/lookup?id=' . $itunes_ID; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$jsonOBJ     = $this->url_get_contents( $get_podcast ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$jsonOBJ     = json_decode( $jsonOBJ ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$podcastURL  = $jsonOBJ->results[0]->feedUrl; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$parse       = wp_parse_url( $podcastURL ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$domain_name = $parse['host'];
					if ( strpos( $domain_name, 'blogtalkradio' ) !== false ) {
						$podcastURL = $podcastURL . '?more=true'; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					}
					$this->spp_settings['podcast_url'] = $podcastURL; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					unset( $jsonOBJ ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				} elseif ( ! empty( $podcast_xml ) ) {
					$parse       = wp_parse_url( $podcast_xml );
					$domain_name = ! empty( $parse['host'] ) ? $parse['host'] : '';
					if ( strpos( $domain_name, 'blogtalkradio' ) !== false ) {
						$podcast_xml = $podcast_xml . '?more=true';
					}
					$this->spp_settings['podcast_url'] = $podcast_xml;
					$podcastURL                        = $podcast_xml; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				} else {
					$this->spp_settings['is_third_party_feed'] = 0;
				}

				if ( $podcastURL ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$parse       = wp_parse_url( $podcastURL ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$domain_name = ! empty( $parse['host'] ) ? $parse['host'] : '';

					if ( is_multisite() ) {
						$spp_current_site        = get_current_site();
						$spp_current_site_domain = $spp_current_site->domain;
					} else {
						$spp_current_site_domain = isset( $_SERVER['SERVER_NAME'] ) ? $_SERVER['SERVER_NAME'] : ''; // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.MissingUnslash,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
					}
					$this->spp_settings['spp_site_domain'] = $spp_current_site_domain;
					$this->spp_settings['spp_feed_domain'] = $domain_name;
					update_option( 'spp_settings', $this->spp_settings );
					// Install DB and Build Table of Episodes for all types of feeds.
					$this->install_spp_db();
					// Check if Feed domain matches current domain of site (self hosted). Set flag to FALSE.
					if ( $domain_name === $spp_current_site_domain ) {
						$this->spp_settings['is_third_party_feed'] = 0;
					} else {
						$this->spp_settings['is_third_party_feed'] = 1;
					}

					// Start of getting and saving podcast channel art.
					$xml        = $this->url_get_contents( $podcastURL ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$itunes_xml = preg_replace( '/(<\/?)(\w+):([^>]*>)/', '$1$2$3', $xml );

					$podcastXML        = simplexml_load_string( $itunes_xml ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$channel_image_url = ! empty( $podcastXML->channel->itunesimage->attributes()->href ) ? (string) $podcastXML->channel->itunesimage->attributes()->href : ''; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					unset( $podcastXML ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$upload_dir = wp_upload_dir();
					$image_data = $this->url_get_contents( $channel_image_url );

					if ( ( ! $image_data ) || ( null === $image_data ) ) {
						// Get the default black image if we can't access the artwork for some reason.
						$channel_image_url = SPPRESS_PLUGIN_URL . '/responsive_audio_player/image/noartwork.png';
						$image_data        = $this->url_get_contents( $channel_image_url );

					}

					$image_array = explode( '.', $channel_image_url );
					$ext         = end( $image_array );
					if ( ( 'png' !== $ext ) || ( 'jpg' !== $ext ) || ( null === $ext ) ) {
						$ext = 'jpg';
					}

					unset( $image_array );
					$new_file = 'podcast_channel_artwork.' . $ext;
					$filename = basename( $new_file );
					if ( wp_mkdir_p( $upload_dir['path'] ) ) {
						$file = $upload_dir['path'] . '/' . $filename;
					} else {
						$file = $upload_dir['basedir'] . '/' . $filename;
					}
					$spp_channelart_upload = file_put_contents( $file, $image_data ); // phpcs:ignore WordPress.WP.AlternativeFunctions.file_system_operations_file_put_contents
					if ( ! $spp_channelart_upload ) {
						global $wp_filesystem;
						// protect if the the global filesystem isn't setup yet.
						if ( is_null( $wp_filesystem ) ) {
							WP_Filesystem();
						}
						$wp_filesystem->put_contents( $file, $image_data, FS_CHMOD_FILE );
					}

					$wp_filetype = wp_check_filetype( $filename, null );
					$attachment  = array(
						'post_mime_type' => $wp_filetype['type'],
						'post_title'     => sanitize_file_name( $filename ),
						'post_content'   => '',
						'post_status'    => 'inherit',
					);
					$attach_id   = wp_insert_attachment( $attachment, $file );
					require_once ABSPATH . 'wp-admin/includes/image.php';
					// Improve speed by removing and running later.
					$uploaded_file = get_attached_file( $attach_id );

					// Get editor, resize and overwrite file.
					$image_editor = wp_get_image_editor( $uploaded_file );
					$image_editor->resize( 640, 640 );
					$saved = $image_editor->save( $uploaded_file );

					$attach_data = wp_generate_attachment_metadata( $attach_id, $file );
					wp_update_attachment_metadata( $attach_id, $attach_data );
					$channel_image                       = wp_get_attachment_url( $attach_id );
					$this->spp_settings['channel_image'] = $channel_image;

					// End of getting and saving podcast channel art.
					$pp_counter       = $this->spp_get_option( 'pp_counter' );
					$podcast_imported = $this->spp_get_option( 'podcast_imported' );

					if ( 0 === $pp_counter ) {
						$this->spp_save_podcast_xml( $podcastURL, false, true ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					}

					// Must do at the end.  If iTunes URL entered, then Save Reviews.
					if ( $itunes_url ) {
						$this->spp_save_reviews( $itunes_ID ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					}
				} else {
					$this->spp_settings['podcast_url'] = 'none';
				}

				$this->remove_duplicate_images();
				$this->spp_settings['podcast_imported'] = 0;
				$this->spp_settings['total_podcasts']   = 0;
				$this->spp_settings['pp_counter']       = 0;
				$this->spp_settings['reviews']          = 0;
				update_option( 'spp_settings', $this->spp_settings );
				// (Removed for brevity).
			}
			if ( 'Save Changes' === $variable ) {
				// Save changes.
				$this->save_spp_settings();
				$this->generate_buttons_css();
				$this->generate_options_css();
				$this->generate_barui_css();
				$this->generate_classic_css();

				do_action( 'spp_save_css' );
			}

			if ( 'Import New Episodes' === $variable ) {
				// Import new episodes.
				$this->save_spp_settings();
				$this->generate_post_from_spp_table();
			}

			if ( 'Update Episodes Table' === $variable ) {
				// Update episodes table.
				$this->save_spp_settings();
				$this->spp_episodes_cron_fn();
			}

			if ( 'Check For New Reviews Now' === $variable ) {
				// Check for new reviews.
				$this->save_spp_settings();
				$this->spp_save_reviews();
			}
		}
	}

	/**
	 * Remove duplicate images.
	 */
	public function remove_duplicate_images() {
		global $wpdb;
	
		$table_name = $wpdb->prefix . 'posts';
	
		// Prepare the query to select duplicate attachment titles
		$query = $wpdb->prepare(
			"SELECT post_title, post_date, CONCAT(post_date, post_title) AS WHOLENAME 
			FROM {$table_name} 
			WHERE post_type = %s 
			GROUP BY WHOLENAME 
			HAVING COUNT(WHOLENAME) > 1", 
			'attachment'
		);
	
		// Execute the query to get duplicate titles
		$duplicate_titles = $wpdb->get_results($query);
	
		// Loop through duplicate titles and delete duplicates
		foreach ($duplicate_titles as $title) {
	
			// Prepare and execute the query to get post IDs for the duplicate titles
			$post_ids = $wpdb->get_col(
				$wpdb->prepare(
					"SELECT ID 
					FROM {$table_name} 
					WHERE post_title = %s AND post_date = %s",
					$title->post_title,
					$title->post_date
				)
			);
	
			// Skip the first occurrence and delete the rest
			foreach (array_slice($post_ids, 1) as $post_id) {
				wp_delete_post($post_id, true); // Force delete this post
			}
		}
	}
	

	/**
	 * Generate posts with cron job.
	 *
	 * This function generates posts based on podcast data using a cron job.
	 * It checks if the license is valid, and if the import option is enabled.
	 *
	 * @return void
	 */
	public function generate_post_with_cron() {

		$is_license_valid = get_option( 'sppress_ls' );

		if ( ( 'valid' !== $is_license_valid ) && ( 'trial' !== $is_license_valid ) ) {
			return;
		}

		$no_import = $this->spp_get_option( 'spp_import_select' );

		if ( '' === $no_import ) {
			$no_import = -1;
		} else {
			$no_import = (int) $no_import;
		}

		$no_import_legacy = $this->spp_get_option( 'spp_no_import' );
		// If the legacy no import option is enabled then override new import setting to disable.
		if ( $no_import_legacy ) {
			$no_import = 1;
		}

		$podcast_u_r_l_feed = $this->spp_get_option( 'is_third_party_feed' );
		$podcast_u_r_l      = $this->spp_get_option( 'podcast_url' );

		// If no podcast URL, then cron needs to do nothing.
		if ( 'none' === $podcast_u_r_l ) {
			return;
		} else {
			$this->spp_save_podcast_xml( $podcast_u_r_l );
		}

		// Only generate posts for Third Party Feeds and No Import option is OFF.
		if ( ( 1 === $podcast_u_r_l_feed ) && ( 0 === $no_import || 2 === $no_import ) && ( 'none' !== $podcast_u_r_l ) ) {
			$this->generate_post_from_spp_table();
		}
	}

	/**
	 * Delete and reset SPP plugin settings.
	 *
	 * This function resets various SPP plugin settings to their default values and updates the options in the database.
	 * It clears sensitive data such as iTunes ID, URLs, and counters.
	 *
	 * @return void
	 */
	public function delete_spp_settings() {
		// Reset various plugin settings.
		$this->spp_settings['itunes_id']           = '';
		$this->spp_settings['btn_itunes_url']      = '';
		$this->spp_settings['channel_keywords']    = '';
		$this->spp_settings['is_third_party_feed'] = '';
		$this->spp_settings['channel_image']       = '';
		$this->spp_settings['podcast_url']         = '';
		$this->spp_settings['itunes_url']          = '';
		$this->spp_settings['podcast_url_feed']    = '';
		$this->spp_settings['reviews_activated']   = '';
		$this->spp_settings['total_podcasts']      = 0;
		$this->spp_settings['reviews']             = 0;
		$this->spp_settings['pp_counter']          = 0;

		// Update the options in the database.
		update_option( 'spp_settings', $this->spp_settings );
	}

	/**
	 * Delete all SPP plugin settings and options.
	 *
	 * This function deletes various SPP plugin settings and options from the database.
	 * It removes options related to player customization, URLs, colors, and other preferences.
	 *
	 * @return void
	 */
	public function delete_all_spp_settings() {
		// List of options to be deleted.
		delete_option( 'reviews_activated' );
		delete_option( 'itunes_id' );
		delete_option( 'itunes_url' );
		delete_option( 'channel_image' );
		delete_option( 'channel_keywords' );
		delete_option( 'podcast_url' );
		delete_option( 'is_third_party_feed' );
		delete_option( 'podcast_url_feed' );
		delete_option( 'transcript_position' );
		delete_option( 'audio_player_position' );
		delete_option( 'spp-twitter-handle' );
		delete_option( 'episode_short_link' );
		delete_option( 'spp_author' );
		delete_option( 'spp_post_category' );
		delete_option( 'spp_cpt_select' );
		delete_option( 'optin_old_code' );
		delete_option( 'spp_auto_resp_url' );
		delete_option( 'spp_auto_resp_heading' );
		delete_option( 'spp_auto_resp_sub_heading' );
		delete_option( 'spp_auto_resp_hidden' );
		delete_option( 'spp_auto_resp_name' );
		delete_option( 'spp_auto_resp_email' );
		delete_option( 'spp_auto_resp_submitt' );
		delete_option( 'select_audio_player' );
		delete_option( 'player_button_color' );
		delete_option( 'select_audio_player_skin' );
		delete_option( 'replace_pp_with_spp' );
		delete_option( 'replace_stp_with_spp' );
		delete_option( 'replace_ppplayer_with_spp' );
		delete_option( 'replace_wp_with_spp' );
		delete_option( 'select_audio_player' );
		delete_option( 'spp_optin_box' );
		delete_option( 'btn_download' );
		delete_option( 'btn_g_play' );
		delete_option( 'btn_share' );
		delete_option( 'btn_download_color' );
		delete_option( 'btn_download_name' );
		delete_option( 'btn_spplisten_name' );
		delete_option( 'btn_sppreview_name' );
		delete_option( 'btn_spprss_name' );
		delete_option( 'btn_sppandroid_name' );
		delete_option( 'btn_itunes_name' );
		delete_option( 'btn_stitcher_name' );
		delete_option( 'btn_soundcloud_name' );
		delete_option( 'btn_spprss' );
		delete_option( 'btn_spprss_color' );
		delete_option( 'btn_sppreview' );
		delete_option( 'btn_sppreview_color' );
		delete_option( 'btn_spplisten' );
		delete_option( 'btn_spplisten_color' );
		delete_option( 'btn_sppandroid' );
		delete_option( 'btn_sppandroid_color' );
		delete_option( 'btn_itunes' );
		delete_option( 'btn_itunes_color' );
		delete_option( 'transcript' );
		delete_option( 'transcript_txt' );
		delete_option( 'spp_listen_text' );
		delete_option( 'transcript_color' );
		delete_option( 'btn_stiticher_color' );
		delete_option( 'btn_stiticher_url' );
		delete_option( 'btn_stiticher' );
		delete_option( 'btn_soundcloud_color' );
		delete_option( 'btn_soundcloud_url' );
		delete_option( 'btn_soundcloud' );
		delete_option( 'spp_auto_publish' );
		delete_option( 'spp_insert_feat_image' );
		delete_option( 'spp_email_on_draft' );
		delete_option( 'spp_player_over_image' );
		delete_option( 'spp_description' );
		delete_option( 'spp_disable_thumbs' );
		delete_option( 'spp_sync_description' );
		delete_option( 'ep_art_select' );
		delete_option( 'disable_url_shortner' );
		delete_option( 'disable_opengraph' );
		delete_option( 'twitter_text_color' );
		delete_option( 'headline_text_color' );
		delete_option( 'submit_button_color' );
		delete_option( 'submit_button_text' );
		delete_option( 'opt-container_color' );
		delete_option( 'btn_style_round' );
		delete_option( 'spp_remove_timecode' );
		delete_option( 'spp_flat_player' );
		delete_option( 'spp_post_inserted' );
		delete_option( 'spp_featured_image_detected' );
		delete_option( 'spp_check_shortcode' );
		delete_option( 'spp_site_domain' );
		delete_option( 'spp_feed_domain' );
		delete_option( 'is_third_party_feed' );
		delete_option( 'spp_channel_image' );
		delete_option( 'spp_hide_player_from_excerpt' );
		delete_option( 'container_width' );
		delete_option( 'minbutton_width' );
		delete_option( 'spp_disable_poweredby' );
		delete_option( 'spp_optin_hide_chk' );
		delete_option( 'spp_disable_spp_player_script' );
		delete_option( 'spp_autoplay_podcast' );
		delete_option( 'spp_rewind_button' );
		delete_option( 'spp_speed_button' );
		delete_option( 'spp_volume_button' );
		delete_option( 'spp_poweredby_url' );
		delete_option( 'spp_optshow_xseconds' );
		delete_option( 'spp_force_player_home' );
		delete_option( 'spp_disable_auto_timestamp' );
		delete_option( 'spp_remove_brackets_timestamp' );
		delete_option( 'spp_resolve_player_conflict' );
		delete_option( 'btn_spp_custom1' );
		delete_option( 'btn_spp_custom2' );
		delete_option( 'btn_spp_custom3' );
		delete_option( 'btn_spp_custom4' );
		delete_option( 'btn_spp_custom6' );
		delete_option( 'spp_LeadBox_btn_checkbox' );
		delete_option( 'btn_spp_custom_color1' );
		delete_option( 'btn_spp_custom_color2' );
		delete_option( 'btn_spp_custom_color3' );
		delete_option( 'btn_spp_custom_color4' );
		delete_option( 'btn_spp_custom_color5' );
		delete_option( 'btn_spp_custom_color6' );
		delete_option( 'custom_button_icon1' );
		delete_option( 'custom_button_icon2' );
		delete_option( 'custom_button_icon3' );
		delete_option( 'custom_button_icon4' );
		delete_option( 'custom_button_icon5' );
		delete_option( 'custom_button_icon6' );
		delete_option( 'spp_LeadBox_btn_color' );
		delete_option( 'btn_spp_custom_name1' );
		delete_option( 'btn_spp_custom_name2' );
		delete_option( 'btn_spp_custom_name3' );
		delete_option( 'btn_spp_custom_name4' );
		delete_option( 'btn_spp_custom_name5' );
		delete_option( 'btn_spp_custom_name6' );
		delete_option( 'btn_spp_custom_url1' );
		delete_option( 'btn_spp_custom_url2' );
		delete_option( 'btn_spp_custom_url3' );
		delete_option( 'btn_spp_custom_url4' );
		delete_option( 'btn_spp_custom_url5' );
		delete_option( 'btn_spp_custom_url6' );
		delete_option( 'spp_LeadBox_btn_code' );
		delete_option( 'btn_g_play_url' );
		delete_option( 'spp_imgsize_select' );
		delete_option( 'spp_imgalign_select' );
		delete_option( 'ep_specific_date' );
		delete_option( 'audio_player_image' );
		delete_option( 'btn_g_play_color' );
		delete_option( 'btn_share_color' );
		delete_option( 'btn_ClammrIT_color' );
		delete_option( 'player_bar_color' );
		delete_option( 'progress_bar_color' );
		delete_option( 'player_text_color' );
		delete_option( 'btn_g_play_name' );
		delete_option( 'btn_share_name' );
		delete_option( 'rewindspeed_button_color' );
		delete_option( 'btn_ClammrIT_checkbox' );
		delete_option( 'spp_pre_roll_checkbox' );
		delete_option( 'spp_pre_roll_url' );
		delete_option( 'direct_download_button' );
		delete_option( 'fix_importing_images' );
		delete_option( 'spp_import_html_desc' );
		delete_option( 'hide_elements_athome' );
		delete_option( 'spp_button_order' );
	}

	/**
	 * Migrate SPP plugin settings to an array and update options in the database.
	 *
	 * This function retrieves various SPP plugin settings from the database and stores them in
	 * the class property $spp_settings. It then updates the options in the database with the new values.
	 * Additionally, it performs other necessary actions after the migration is complete.
	 *
	 * @return void
	 */
	public function migrate_spp_settings_to_array() {

		$migrationcomplete = get_option( 'spp_migration_complete' );

		if ( ! $migrationcomplete ) {
			$this->spp_settings['itunes_id']                 = get_option( 'itunes_id' );
			$this->spp_settings['itunes_url']                = get_option( 'itunes_url' );
			$this->spp_settings['channel_image']             = get_option( 'channel_image' );
			$this->spp_settings['channel_keywords']          = get_option( 'channel_keywords' );
			$this->spp_settings['podcast_url']               = get_option( 'podcast_url' );
			$this->spp_settings['is_third_party_feed']       = get_option( 'is_third_party_feed' );
			$this->spp_settings['audio_player_position']     = get_option( 'audio_player_position' );
			$this->spp_settings['spp_twitter_handle']        = get_option( 'spp-twitter-handle' );
			$this->spp_settings['episode_short_link']        = get_option( 'episode_short_link' );
			$this->spp_settings['spp_author']                = get_option( 'spp_author' );
			$this->spp_settings['spp_post_category']         = get_option( 'spp_post_category' );
			$this->spp_settings['spp_cpt_select']            = get_option( 'spp_cpt_select' );
			$this->spp_settings['spp_two_step_optin']        = get_option( 'spp_two_step_optin' );
			$this->spp_settings['optin_old_code']            = get_option( 'optin_old_code' );
			$this->spp_settings['spp_auto_resp_url']         = get_option( 'spp_auto_resp_url' );
			$this->spp_settings['spp_auto_resp_heading']     = get_option( 'spp_auto_resp_heading' );
			$this->spp_settings['spp_auto_resp_hidden']      = get_option( 'spp_auto_resp_heading' );
			$this->spp_settings['spp_auto_resp_name']        = get_option( 'spp_auto_resp_name' );
			$this->spp_settings['spp_auto_resp_email']       = get_option( 'spp_auto_resp_email' );
			$this->spp_settings['spp_auto_resp_submitt']     = get_option( 'spp_auto_resp_submitt' );
			$this->spp_settings['select_audio_player']       = get_option( 'select_audio_player' );
			$this->spp_settings['player_button_color']       = get_option( 'player_button_color' );
			$this->spp_settings['select_audio_player_skin']  = get_option( 'select_audio_player_skin' );
			$this->spp_settings['replace_pp_with_spp']       = get_option( 'replace_pp_with_spp' );
			$this->spp_settings['replace_stp_with_spp']      = get_option( 'replace_stp_with_spp' );
			$this->spp_settings['replace_ppplayer_with_spp'] = get_option( 'replace_ppplayer_with_spp' );
			$this->spp_settings['replace_wp_with_spp']       = get_option( 'replace_wp_with_spp' );
			$this->spp_settings['select_audio_player']       = get_option( 'select_audio_player' );
			$this->spp_settings['spp_optin_box']             = get_option( 'spp_optin_box' );
			$this->spp_settings['btn_download']              = get_option( 'btn_download' );
			$this->spp_settings['btn_g_play']                = get_option( 'btn_g_play' );
			$this->spp_settings['btn_share']                 = get_option( 'btn_share' );
			$this->spp_settings['btn_download_color']        = get_option( 'btn_download_color' );
			$this->spp_settings['btn_download_name']         = get_option( 'btn_download_name' );
			$this->spp_settings['btn_spplisten_name']        = get_option( 'btn_spplisten_name' );
			$this->spp_settings['btn_sppreview_name']        = get_option( 'btn_sppreview_name' );
			$this->spp_settings['btn_ClammrIT_name']         = get_option( 'btn_ClammrIT_name' );
			$this->spp_settings['btn_spprss_name']           = get_option( 'btn_spprss_name' );
			$this->spp_settings['btn_sppandroid_name']       = get_option( 'btn_sppandroid_name' );
			$this->spp_settings['btn_itunes_name']           = get_option( 'btn_itunes_name' );
			$this->spp_settings['btn_stitcher_name']         = get_option( 'btn_stitcher_name' );
			$this->spp_settings['btn_soundcloud_name']       = get_option( 'btn_soundcloud_name' );
			$this->spp_settings['btn_spprss']                = get_option( 'btn_spprss' );
			$this->spp_settings['btn_spprss_color']          = get_option( 'btn_spprss_color' );
			$this->spp_settings['btn_sppreview']             = get_option( 'btn_sppreview' );
			$this->spp_settings['btn_sppreview_color']       = get_option( 'btn_sppreview_color' );
			$this->spp_settings['btn_spplisten']             = get_option( 'btn_spplisten' );
			$this->spp_settings['btn_spplisten_color']       = get_option( 'btn_spplisten_color' );
			$this->spp_settings['btn_sppandroid']            = get_option( 'btn_sppandroid' );
			$this->spp_settings['btn_sppandroid_color']      = get_option( 'btn_sppandroid_color' );
			$this->spp_settings['btn_itunes']                = get_option( 'btn_itunes' );
			$this->spp_settings['btn_embed_color']           = get_option( 'btn_embed_color' );
			$this->spp_settings['btn_itunes_color']          = get_option( 'btn_itunes_color' );
			$this->spp_settings['transcript']                = get_option( 'transcript' );
			$this->spp_settings['transcript_txt']            = get_option( 'transcript_txt' );
			$this->spp_settings['spp_listen_text']           = get_option( 'spp_listen_text' );
			$this->spp_settings['transcript_color']          = get_option( 'transcript_color' );
			$this->spp_settings['btn_stiticher_color']       = get_option( 'btn_stiticher_color' );
			$this->spp_settings['btn_stiticher_url']         = get_option( 'btn_stiticher_url' );
			$this->spp_settings['btn_stiticher']             = get_option( 'btn_stiticher' );
			$this->spp_settings['btn_soundcloud_color']      = get_option( 'btn_soundcloud_color' );
			$this->spp_settings['btn_soundcloud_url']        = get_option( 'btn_soundcloud_url' );
			$this->spp_settings['btn_soundcloud']            = get_option( 'btn_soundcloud' );
			$this->spp_settings['spp_auto_publish']          = get_option( 'spp_auto_publish' );
			$this->spp_settings['spp_insert_feat_image']     = get_option( 'spp_insert_feat_image' );
			$this->spp_settings['spp_email_on_draft']        = get_option( 'spp_email_on_draft' );
			$this->spp_settings['spp_player_over_image']     = get_option( 'spp_player_over_image' );
			$this->spp_settings['spp_description']           = get_option( 'spp_description' );
			$this->spp_settings['spp_sync_description']      = get_option( 'spp_sync_description' );
			$this->spp_settings['ep_art_select']             = get_option( 'ep_art_select' );
			$dus = get_option( 'disable_url_shortner', -1 );
			if ( ( 1 === $dus ) || ( 0 === $dus ) ) {
				$this->spp_settings['disable_url_shortner'] = $dus;
			}
			$this->spp_settings['disable_opengraph']             = get_option( 'disable_opengraph' );
			$this->spp_settings['twitter_text_color']            = get_option( 'twitter_text_color' );
			$this->spp_settings['headline_text_color']           = get_option( 'headline_text_color' );
			$this->spp_settings['submit_button_color']           = get_option( 'submit_button_color' );
			$this->spp_settings['submit_button_text']            = get_option( 'submit_button_text' );
			$this->spp_settings['opt_container_color']           = get_option( 'opt-container_color' );
			$this->spp_settings['btn_style_round']               = get_option( 'btn_style_round' );
			$this->spp_settings['spp_remove_timecode']           = get_option( 'spp_remove_timecode' );
			$this->spp_settings['spp_flat_player']               = get_option( 'spp_flat_player' );
			$this->spp_settings['spp_post_inserted']             = get_option( 'spp_post_inserted' );
			$this->spp_settings['spp_featured_image_detected']   = get_option( 'spp_featured_image_detected' );
			$this->spp_settings['spp_check_shortcode']           = get_option( 'spp_check_shortcode' );
			$this->spp_settings['spp_site_domain']               = get_option( 'spp_site_domain' );
			$this->spp_settings['spp_feed_domain']               = get_option( 'spp_feed_domain' );
			$this->spp_settings['is_third_party_feed']           = get_option( 'is_third_party_feed' );
			$this->spp_settings['spp_channel_image']             = get_option( 'spp_channel_image' );
			$this->spp_settings['spp_hide_player_from_excerpt']  = get_option( 'spp_hide_player_from_excerpt' );
			$this->spp_settings['container_width']               = get_option( 'container_width' );
			$this->spp_settings['minbutton_width']               = get_option( 'minbutton_width' );
			$this->spp_settings['spp_disable_poweredby']         = get_option( 'spp_disable_poweredby' );
			$this->spp_settings['spp_optin_hide_chk']            = get_option( 'spp_optin_hide_chk' );
			$this->spp_settings['spp_disable_spp_player_script'] = get_option( 'spp_disable_spp_player_script' );
			$this->spp_settings['spp_rewind_button']             = get_option( 'spp_rewind_button' );
			$this->spp_settings['spp_speed_button']              = get_option( 'spp_speed_button' );
			$this->spp_settings['spp_volume_button']             = get_option( 'spp_volume_button' );
			$this->spp_settings['spp_poweredby_url']             = get_option( 'spp_poweredby_url' );
			$this->spp_settings['spp_optshow_xseconds']          = get_option( 'spp_optshow_xseconds' );
			$this->spp_settings['spp_force_player_home']         = get_option( 'spp_force_player_home' );
			$this->spp_settings['spp_disable_auto_timestamp']    = get_option( 'spp_disable_auto_timestamp' );
			$this->spp_settings['spp_remove_brackets_timestamp'] = get_option( 'spp_remove_brackets_timestamp' );
			$this->spp_settings['spp_resolve_player_conflict']   = get_option( 'spp_resolve_player_conflict' );
			$this->spp_settings['btn_spp_custom1']               = get_option( 'btn_spp_custom1' );
			$this->spp_settings['btn_spp_custom2']               = get_option( 'btn_spp_custom2' );
			$this->spp_settings['btn_spp_custom3']               = get_option( 'btn_spp_custom3' );
			$this->spp_settings['btn_spp_custom4']               = get_option( 'btn_spp_custom4' );
			$this->spp_settings['btn_spp_custom5']               = get_option( 'btn_spp_custom5' );
			$this->spp_settings['btn_spp_custom6']               = get_option( 'btn_spp_custom6' );
			$this->spp_settings['spp_LeadBox_btn_checkbox']      = get_option( 'spp_LeadBox_btn_checkbox' );
			$this->spp_settings['btn_spp_custom_color1']         = get_option( 'btn_spp_custom_color1' );
			$this->spp_settings['btn_spp_custom_color2']         = get_option( 'btn_spp_custom_color2' );
			$this->spp_settings['btn_spp_custom_color3']         = get_option( 'btn_spp_custom_color3' );
			$this->spp_settings['btn_spp_custom_color4']         = get_option( 'btn_spp_custom_color4' );
			$this->spp_settings['btn_spp_custom_color5']         = get_option( 'btn_spp_custom_color5' );
			$this->spp_settings['btn_spp_custom_color6']         = get_option( 'btn_spp_custom_color6' );
			$this->spp_settings['custom_button_icon1']           = get_option( 'custom_button_icon1' );
			$this->spp_settings['custom_button_icon2']           = get_option( 'custom_button_icon2' );
			$this->spp_settings['custom_button_icon3']           = get_option( 'custom_button_icon3' );
			$this->spp_settings['custom_button_icon4']           = get_option( 'custom_button_icon4' );
			$this->spp_settings['custom_button_icon5']           = get_option( 'custom_button_icon5' );
			$this->spp_settings['custom_button_icon6']           = get_option( 'custom_button_icon6' );
			$this->spp_settings['spp_LeadBox_btn_color']         = get_option( 'spp_LeadBox_btn_color' );
			$this->spp_settings['btn_spp_custom_name1']          = get_option( 'btn_spp_custom_name1' );
			$this->spp_settings['btn_spp_custom_name2']          = get_option( 'btn_spp_custom_name2' );
			$this->spp_settings['btn_spp_custom_name3']          = get_option( 'btn_spp_custom_name3' );
			$this->spp_settings['btn_spp_custom_name4']          = get_option( 'btn_spp_custom_name4' );
			$this->spp_settings['btn_spp_custom_name5']          = get_option( 'btn_spp_custom_name5' );
			$this->spp_settings['btn_spp_custom_name6']          = get_option( 'btn_spp_custom_name6' );
			$this->spp_settings['btn_spp_custom_url1']           = get_option( 'btn_spp_custom_url1' );
			$this->spp_settings['btn_spp_custom_url2']           = get_option( 'btn_spp_custom_url2' );
			$this->spp_settings['btn_spp_custom_url3']           = get_option( 'btn_spp_custom_url3' );
			$this->spp_settings['btn_spp_custom_url4']           = get_option( 'btn_spp_custom_url4' );
			$this->spp_settings['btn_spp_custom_url5']           = get_option( 'btn_spp_custom_url5' );
			$this->spp_settings['btn_spp_custom_url6']           = get_option( 'btn_spp_custom_url6' );
			$this->spp_settings['spp_LeadBox_btn_code']          = get_option( 'spp_LeadBox_btn_code' );
			$this->spp_settings['btn_g_play_url']                = get_option( 'btn_g_play_url' );
			$this->spp_settings['spp_imgsize_select']            = get_option( 'spp_imgsize_select' );
			$this->spp_settings['spp_imgalign_select']           = get_option( 'spp_imgalign_select' );
			$this->spp_settings['ep_specific_date']              = get_option( 'ep_specific_date' );
			$this->spp_settings['audio_player_image']            = get_option( 'audio_player_image' );
			$this->spp_settings['btn_g_play_color']              = get_option( 'btn_g_play_color' );
			$this->spp_settings['btn_share_color']               = get_option( 'btn_share_color' );
			$this->spp_settings['player_bar_color']              = get_option( 'player_bar_color' );
			$this->spp_settings['progress_bar_color']            = get_option( 'progress_bar_color' );
			$this->spp_settings['player_text_color']             = get_option( 'player_text_color' );
			$this->spp_settings['btn_g_play_name']               = get_option( 'btn_g_play_name' );
			$this->spp_settings['btn_share_name']                = get_option( 'btn_share_name' );
			$this->spp_settings['rewindspeed_button_color']      = get_option( 'rewindspeed_button_color' );
			$this->spp_settings['powered_by_svp_link']           = get_option( 'powered_by_svp_link' );
			$this->spp_settings['spp_pre_roll_checkbox']         = get_option( 'spp_pre_roll_checkbox' );
			$this->spp_settings['spp_pre_roll_url']              = get_option( 'spp_pre_roll_url' );
			$this->spp_settings['direct_download_button']        = get_option( 'direct_download_button' );
			$this->spp_settings['fix_importing_images']          = get_option( 'fix_importing_images' );
			$this->spp_settings['spp_import_html_desc']          = get_option( 'spp_import_html_desc' );
			$this->spp_settings['hide_elements_athome']          = get_option( 'hide_elements_athome' );
			$this->spp_settings['spp_button_order']              = get_option( 'spp_button_order' );
			update_option( 'spp_settings', $this->spp_settings );
			update_option( 'spp_migration_complete', 1 );
			$this->generate_buttons_css();
			$this->generate_options_css();
			$this->generate_barui_css();
			$this->generate_classic_css();
		} //end if
	}
	/**
	 * Save SPP plugin settings function.
	 *
	 * @return void
	 */
	public function save_spp_settings() {
		global $wpdb;
		$btn_spp_custom1               = '';
		$btn_spp_custom2               = '';
		$btn_spp_custom3               = '';
		$btn_spp_custom4               = '';
		$btn_spp_custom5               = '';
		$btn_spp_custom6               = '';
		$btn_spp_custom_color1         = '';
		$btn_spp_custom_color2         = '';
		$btn_spp_custom_color3         = '';
		$btn_spp_custom_color4         = '';
		$btn_spp_custom_color5         = '';
		$btn_spp_custom_color6         = '';
		$transcript_position           = '';
		$btn_spp_custom_name1          = '';
		$btn_spp_custom_name2          = '';
		$btn_spp_custom_name3          = '';
		$btn_spp_custom_name4          = '';
		$btn_spp_custom_name5          = '';
		$btn_spp_custom_name6          = '';
		$custom_button_icon1           = '';
		$custom_button_icon2           = '';
		$custom_button_icon3           = '';
		$custom_button_icon4           = '';
		$custom_button_icon5           = '';
		$custom_button_icon6           = '';
		$btn_spp_custom_url1           = '';
		$btn_spp_custom_url2           = '';
		$btn_spp_custom_url3           = '';
		$btn_spp_custom_url4           = '';
		$btn_spp_custom_url5           = '';
		$btn_spp_custom_url6           = '';
		$disable_url_shortner          = false;
		$disable_opengraph             = false;
		$replace_pp_with_spp           = false;
		$replace_wp_with_spp           = false;
		$replace_stp_with_spp          = false;
		$replace_ppplayer_with_spp     = false;
		$select_audio_player           = 'simplepodcastpressblack';
		$player_button_color           = 'sppwhite';
		$select_audio_player_skin      = 'soundmanager2_player';
		$powered_by_svp_link           = 'on';
		$spp_force_player_home         = false;
		$spp_disable_auto_timestamp    = false;
		$spp_remove_brackets_timestamp = false;
		$spp_resolve_player_conflict   = false;
		$spp_LeadBox_btn_checkbox      = false; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		$spp_LeadBox_btn_color         = ''; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		$spp_LeadBox_btn_code          = ''; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

		if ( isset( $_POST['spp_two_step_optin'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$spp_settings['spp_two_step_optin'] = wp_unslash( $_POST['spp_two_step_optin'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['spp_privacy_page'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$spp_settings['spp_privacy_page'] = wp_unslash( $_POST['spp_privacy_page'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}

		if ( isset( $_POST['replace_ppplayer_with_spp'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$replace_ppplayer_with_spp = wp_unslash( $_POST['replace_ppplayer_with_spp'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['replace_wp_with_spp'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$replace_wp_with_spp = wp_unslash( $_POST['replace_wp_with_spp'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['replace_stp_with_spp'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$replace_stp_with_spp = wp_unslash( $_POST['replace_stp_with_spp'] );// phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom1'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom1 = wp_unslash( $_POST['btn_spp_custom1'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom2'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom2 = wp_unslash( $_POST['btn_spp_custom2'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom3'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom3 = wp_unslash( $_POST['btn_spp_custom3'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom4'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom4 = wp_unslash( $_POST['btn_spp_custom4'] ); // phpcs:ignore Wo,WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing rdPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom5'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom5 = wp_unslash( $_POST['btn_spp_custom5'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom6'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom6 = wp_unslash( $_POST['btn_spp_custom6'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['spp_LeadBox_btn_checkbox'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$spp_LeadBox_btn_checkbox = wp_unslash( $_POST['spp_LeadBox_btn_checkbox'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		}

		if ( isset( $_POST['transcript_color'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$transcript_color = wp_unslash( $_POST['transcript_color'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_color1'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_color1 = wp_unslash( $_POST['btn_spp_custom_color1'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_color2'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_color2 = wp_unslash( $_POST['btn_spp_custom_color2'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_color3'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_color3 = wp_unslash( $_POST['btn_spp_custom_color3'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_color4'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_color4 = wp_unslash( $_POST['btn_spp_custom_color4'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_color5'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_color5 = wp_unslash( $_POST['btn_spp_custom_color5'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_color6'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_color6 = wp_unslash( $_POST['btn_spp_custom_color6'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['custom_button_icon1'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$custom_button_icon1 = wp_unslash( $_POST['custom_button_icon1'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['custom_button_icon2'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$custom_button_icon2 = wp_unslash( $_POST['custom_button_icon2'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['custom_button_icon3'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$custom_button_icon3 = wp_unslash( $_POST['custom_button_icon3'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['custom_button_icon4'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$custom_button_icon4 = wp_unslash( $_POST['custom_button_icon4'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['custom_button_icon5'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$custom_button_icon5 = wp_unslash( $_POST['custom_button_icon5'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['custom_button_icon6'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$custom_button_icon6 = wp_unslash( $_POST['custom_button_icon6'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['spp_LeadBox_btn_color'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$spp_LeadBox_btn_color = wp_unslash( $_POST['spp_LeadBox_btn_color'] ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}

		if ( isset( $_POST['btn_spp_custom_name1'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_name1 = wp_unslash( $_POST['btn_spp_custom_name1'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_name2'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_name2 = wp_unslash( $_POST['btn_spp_custom_name2'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_name3'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_name3 = wp_unslash( $_POST['btn_spp_custom_name3'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_name4'] ) ) {// phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_name4 = wp_unslash( $_POST['btn_spp_custom_name4'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_name5'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_name5 = wp_unslash( $_POST['btn_spp_custom_name5'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_name6'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_name6 = wp_unslash( $_POST['btn_spp_custom_name6'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}

		if ( isset( $_POST['btn_spp_custom_url1'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_url1 = wp_unslash( $_POST['btn_spp_custom_url1'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_url2'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_url2 = wp_unslash( $_POST['btn_spp_custom_url2'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_url3'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_url3 = wp_unslash( $_POST['btn_spp_custom_url3'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_url4'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$btn_spp_custom_url4 = wp_unslash( $_POST['btn_spp_custom_url4'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_url5'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_url5 = wp_unslash( $_POST['btn_spp_custom_url5'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spp_custom_url6'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$btn_spp_custom_url6 = wp_unslash( $_POST['btn_spp_custom_url6'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['spp_LeadBox_btn_code'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_LeadBox_btn_code = stripslashes( htmlspecialchars( wp_unslash( $_POST['spp_LeadBox_btn_code'] ), ENT_QUOTES ) ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		// Saving Custom Button 1.
		if ( 'on' === $replace_ppplayer_with_spp ) {
			$spp_settings['replace_ppplayer_with_spp'] = 1;
		} else {
			$spp_settings['replace_ppplayer_with_spp'] = 2;
		}

		if ( 'on' === $replace_wp_with_spp ) {
			$spp_settings['replace_wp_with_spp'] = 1;
		} else {
			$spp_settings['replace_wp_with_spp'] = 0;
		}

		if ( 'on' === $replace_stp_with_spp ) {
			$spp_settings['replace_stp_with_spp'] = 1;
		} else {
			$spp_settings['replace_stp_with_spp'] = 0;
		}

		if ( 'on' === $btn_spp_custom1 ) {
			$spp_settings['btn_spp_custom1'] = 1;
		} else {
			$spp_settings['btn_spp_custom1'] = 0;
		}
		$spp_settings['btn_spp_custom_color1'] = $btn_spp_custom_color1;
		$spp_settings['custom_button_icon1']   = $custom_button_icon1;
		$spp_settings['btn_spp_custom_name1']  = $btn_spp_custom_name1;
		$spp_settings['btn_spp_custom_url1']   = $btn_spp_custom_url1;
		// Saving Custom Button 2.
		if ( 'on' === $btn_spp_custom2 ) {
			$spp_settings['btn_spp_custom2'] = 1;
		} else {
			$spp_settings['btn_spp_custom2'] = 0;
		}
		$spp_settings['btn_spp_custom_color2'] = $btn_spp_custom_color2;
		$spp_settings['custom_button_icon2']   = $custom_button_icon2;
		$spp_settings['btn_spp_custom_name2']  = $btn_spp_custom_name2;
		$spp_settings['btn_spp_custom_url2']   = $btn_spp_custom_url2;
		// Saving Custom Button 3.
		if ( 'on' === $btn_spp_custom3 ) {
			$spp_settings['btn_spp_custom3'] = 1;
		} else {
			$spp_settings['btn_spp_custom3'] = 0;
		}
		$spp_settings['btn_spp_custom_color3'] = $btn_spp_custom_color3;
		$spp_settings['custom_button_icon3']   = $custom_button_icon3;
		$spp_settings['btn_spp_custom_name3']  = $btn_spp_custom_name3;
		$spp_settings['btn_spp_custom_url3']   = $btn_spp_custom_url3;

		// Saving Custom Button 4.
		if ( 'on' === $btn_spp_custom4 ) {
			$spp_settings['btn_spp_custom4'] = 1;
		} else {
			$spp_settings['btn_spp_custom4'] = 0;
		}
		$spp_settings['btn_spp_custom_color4'] = $btn_spp_custom_color4;
		$spp_settings['custom_button_icon4']   = $custom_button_icon4;
		$spp_settings['btn_spp_custom_name4']  = $btn_spp_custom_name4;
		$spp_settings['btn_spp_custom_url4']   = $btn_spp_custom_url4;
		// Saving Custom Button 5.
		if ( 'on' === $btn_spp_custom5 ) {
			$spp_settings['btn_spp_custom5'] = 1;
		} else {
			$spp_settings['btn_spp_custom5'] = 0;
		}
		$spp_settings['btn_spp_custom_color5'] = $btn_spp_custom_color5;
		$spp_settings['custom_button_icon5']   = $custom_button_icon5;
		$spp_settings['btn_spp_custom_name5']  = $btn_spp_custom_name5;
		$spp_settings['btn_spp_custom_url5']   = $btn_spp_custom_url5;
		// Saving Custom Button 6.
		if ( 'on' === $btn_spp_custom6 ) {
			$spp_settings['btn_spp_custom6'] = 1;
		} else {
			$spp_settings['btn_spp_custom6'] = 0;
		}
		$spp_settings['btn_spp_custom_color6'] = $btn_spp_custom_color6;
		$spp_settings['custom_button_icon6']   = $custom_button_icon6;
		$spp_settings['btn_spp_custom_name6']  = $btn_spp_custom_name6;
		$spp_settings['btn_spp_custom_url6']   = $btn_spp_custom_url6;
		// Saving  SPP LeadBox Button.
		if ( 'on' === $spp_LeadBox_btn_checkbox ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$spp_settings['spp_LeadBox_btn_checkbox'] = 1;
		} else {
			$spp_settings['spp_LeadBox_btn_checkbox'] = 0;
		}
		$spp_settings['spp_LeadBox_btn_color'] = $spp_LeadBox_btn_color; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		$spp_settings['spp_LeadBox_btn_code']  = $spp_LeadBox_btn_code; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		if ( isset( $_POST['btn_g_play_url'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$spp_settings['btn_g_play_url'] = wp_unslash( $_POST['btn_g_play_url'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_itunes_url'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized

			$itunes_url = wp_unslash( $_POST['btn_itunes_url'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			if ( ! empty( $itunes_url ) ) {
				$itunes_ID                      = $this->get_itunes_ID( $itunes_url ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$spp_settings['btn_itunes_url'] = $itunes_url;
				$spp_settings['itunes_id']      = $itunes_ID; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			}
		}
		if ( isset( $_POST['container_width'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['container_width'] = wp_unslash( $_POST['container_width'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['minbutton_width'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['minbutton_width'] = wp_unslash( $_POST['minbutton_width'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}

		if ( isset( $_POST['ep_art_select'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['ep_art_select'] = wp_unslash( $_POST['ep_art_select'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['spp_import_select'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_import_select'] = wp_unslash( $_POST['spp_import_select'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['spp_imgsize_select'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_imgsize_select'] = wp_unslash( $_POST['spp_imgsize_select'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}

		if ( isset( $_POST['spp_imgalign_select'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_imgalign_select'] = wp_unslash( $_POST['spp_imgalign_select'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['ep_specific_date'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['ep_specific_date'] = wp_unslash( $_POST['ep_specific_date'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}

		$podcastURL_Feed = $this->spp_get_option( 'is_third_party_feed' ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

		if ( isset( $_POST['transcript_position'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['transcript_position'] = wp_unslash( $_POST['transcript_position'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['audio_player_image'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['audio_player_image'] = wp_unslash( $_POST['audio_player_image'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['audio_player_position'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['audio_player_position'] = wp_unslash( $_POST['audio_player_position'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['spp-twitter-handle'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spptwitterhandle                   = wp_unslash( $_POST['spp-twitter-handle'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$spptwitterhandle                   = str_replace( '@', '', wp_strip_all_tags( stripslashes( $spptwitterhandle ) ) );
			$spp_settings['spp-twitter-handle'] = $spptwitterhandle;
		}
		if ( isset( $_POST['episode_short_link'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['episode_short_link'] = wp_unslash( $_POST['episode_short_link'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['spp_cat'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_category                      = wp_unslash( $_POST['spp_cat'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_post_category'] = $spp_category;
		}
		if ( isset( $_POST['spp_author'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_author'] = wp_unslash( $_POST['spp_author'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['spp_cpt_select'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_cpt_select'] = wp_unslash( $_POST['spp_cpt_select'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['spp_cpt_select'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_post_format_select'] = wp_unslash( $_POST['spp_post_format_select'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotValidated
		}

		if ( isset( $_POST['btn_download_color'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_download_color'] = wp_unslash( $_POST['btn_download_color'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_g_play_color'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_g_play_color'] = wp_unslash( $_POST['btn_g_play_color'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_share_color'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_share_color'] = wp_unslash( $_POST['btn_share_color'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_spplisten_color'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_spplisten_color'] = wp_unslash( $_POST['btn_spplisten_color'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_spprss_color'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_spprss_color'] = wp_unslash( $_POST['btn_spprss_color'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_sppreview_color'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_sppreview_color'] = wp_unslash( $_POST['btn_sppreview_color'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_sppandroid_color'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_sppandroid_color'] = wp_unslash( $_POST['btn_sppandroid_color'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['player_bar_color'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['player_bar_color'] = wp_unslash( $_POST['player_bar_color'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['progress_bar_color'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['progress_bar_color'] = wp_unslash( $_POST['progress_bar_color'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['player_text_color'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['player_text_color'] = wp_unslash( $_POST['player_text_color'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_download_name'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_download_name'] = wp_unslash( $_POST['btn_download_name'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_g_play_name'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_g_play_name'] = wp_unslash( $_POST['btn_g_play_name'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_share_name'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_share_name'] = wp_unslash( $_POST['btn_share_name'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_spplisten_name'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_spplisten_name'] = wp_unslash( $_POST['btn_spplisten_name'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_sppreview_name'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_sppreview_name'] = wp_unslash( $_POST['btn_sppreview_name'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_spprss_name'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_spprss_name'] = wp_unslash( $_POST['btn_spprss_name'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_sppandroid_name'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_sppandroid_name'] = wp_unslash( $_POST['btn_sppandroid_name'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_itunes_name'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_itunes_name'] = wp_unslash( $_POST['btn_itunes_name'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_embed_name'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_embed_name'] = wp_unslash( $_POST['btn_embed_name'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_stitcher_name'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_stitcher_name'] = wp_unslash( $_POST['btn_stitcher_name'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_soundcloud_name'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_soundcloud_name'] = wp_unslash( $_POST['btn_soundcloud_name'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['btn_spotify_name'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_spotify_name'] = wp_unslash( $_POST['btn_spotify_name'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['optin_old_code'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$optin_old_code                 = stripslashes( htmlspecialchars( wp_unslash( $_POST['optin_old_code'] ), ENT_QUOTES ) );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['optin_old_code'] = $optin_old_code;
		}
		if ( isset( $_POST['spp_auto_resp_url'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_auto_resp_url                 = htmlspecialchars( wp_unslash( $_POST['spp_auto_resp_url'] ), ENT_QUOTES ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$spp_settings['spp_auto_resp_url'] = $spp_auto_resp_url;
		}
		if ( isset( $_POST['spp_auto_resp_heading'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_auto_resp_heading                 = wp_unslash( $_POST['spp_auto_resp_heading'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_auto_resp_heading'] = stripslashes( $spp_auto_resp_heading );
		}
		if ( isset( $_POST['spp_auto_resp_sub_heading'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_auto_resp_sub_heading                 = wp_unslash( $_POST['spp_auto_resp_sub_heading'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_auto_resp_sub_heading'] = stripslashes( $spp_auto_resp_sub_heading );
		}
		if ( isset( $_POST['spp_auto_resp_hidden'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_auto_resp_hidden                 = stripslashes( htmlspecialchars( wp_unslash( $_POST['spp_auto_resp_hidden'] ), ENT_QUOTES ) );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_auto_resp_hidden'] = $spp_auto_resp_hidden;
		}
		if ( isset( $_POST['spp_auto_resp_name'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_auto_resp_name'] = wp_unslash( $_POST['spp_auto_resp_name'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['spp_auto_resp_email'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_auto_resp_email'] = wp_unslash( $_POST['spp_auto_resp_email'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['spp_auto_resp_submitt'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_auto_resp_submitt'] = wp_unslash( $_POST['spp_auto_resp_submitt'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}

		if ( isset( $_POST['disable_url_shortner'] ) && ( 'on' === $_POST['disable_url_shortner'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['disable_url_shortner'] = 1;
		} else {
			$spp_settings['disable_url_shortner'] = 0;
		}

		if ( isset( $_POST['spp_force_player_home'] ) && ( 'on' === $_POST['spp_force_player_home'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_force_player_home'] = 1;
		} else {
			$spp_settings['spp_force_player_home'] = 0;
		}

		if ( isset( $_POST['spp_disable_auto_timestamp'] ) && ( 'on' === $_POST['spp_disable_auto_timestamp'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_disable_auto_timestamp'] = 1;
		} else {
			$spp_settings['spp_disable_auto_timestamp'] = 0;
		}
		if ( isset( $_POST['spp_remove_brackets_timestamp'] ) && ( 'on' === $_POST['spp_remove_brackets_timestamp'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_remove_brackets_timestamp'] = 1;
		} else {
			$spp_settings['spp_remove_brackets_timestamp'] = 0;
		}

		if ( isset( $_POST['spp_resolve_player_conflict'] ) && ( 'on' === $_POST['spp_resolve_player_conflict'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_resolve_player_conflict'] = 1;
		} else {
			$spp_settings['spp_resolve_player_conflict'] = 0;
		}
		if ( isset( $_POST['force_ssl_for_audio'] ) && ( 'on' === $_POST['force_ssl_for_audio'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['force_ssl_for_audio'] = 1;
		} else {
			$spp_settings['force_ssl_for_audio'] = 0;
		}

		if ( isset( $_POST['disable_opengraph'] ) && 'on' === $_POST['disable_opengraph'] ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['disable_opengraph'] = 1;
		} else {
			$spp_settings['disable_opengraph'] = 0;
		}

		$select_audio_player = 'simplepodcastpressblack';

		if ( isset( $_POST['select_audio_player'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$select_audio_player = wp_unslash( $_POST['select_audio_player'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}

		if ( isset( $_POST['player_button_color'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['player_button_color'] = wp_unslash( $_POST['player_button_color'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}

		if ( isset( $_POST['rewindspeed_button_color'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['rewindspeed_button_color'] = wp_unslash( $_POST['rewindspeed_button_color'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}

		if ( isset( $_POST['select_audio_player_skin'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['select_audio_player_skin'] = wp_unslash( $_POST['select_audio_player_skin'] );  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}

		switch ( $select_audio_player ) {
			case 'powerpressplayer':
				$spp_settings['select_audio_player'] = $select_audio_player;
				$spp_settings['replace_pp_with_spp'] = 0;

				break;
			case 'simplepodcastpressblack':
				$spp_settings['select_audio_player'] = $select_audio_player;

				// If Raw podcast URL is Libsyn or Soundcloud, just update colour of player and break.
				if ( $podcastURL_Feed ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$spp_settings['replace_pp_with_spp'] = 0;
				} else {
					$spp_settings['replace_pp_with_spp'] = 1;

				}
				break;
			case 'simplepodcastpresswhite':
				$spp_settings['select_audio_player'] = $select_audio_player;

				// If Raw podcast URL is Libsyn or Soundcloud, just update colour of player and break.
				if ( $podcastURL_Feed ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$spp_settings['replace_pp_with_spp'] = 0;

				} else {
					$spp_settings['replace_pp_with_spp'] = 1;

				}

				break;
			case 'simplepodcastpresscustomcolor':
				$spp_settings['select_audio_player'] = $select_audio_player;

				// If Raw podcast URL is Libsyn or Soundcloud, just update colour of player and break.
				if ( $podcastURL_Feed ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$spp_settings['replace_pp_with_spp'] = 0;
				} else {
					$spp_settings['replace_pp_with_spp'] = 1;

				}

				break;

		}
		if ( isset( $_POST['replace_pp_with_spp'] ) && 'on' === $_POST['replace_pp_with_spp'] ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
			$spp_settings['replace_pp_with_spp'] = 1;

		} elseif ( isset( $_POST['replace_pp_with_spp'] ) && 'on' === $_POST['replace_pp_with_spp'] ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing

			$spp_settings['replace_pp_with_spp'] = 0;
			if ( isset( $_POST['powered_by_svp_link'] ) ) {  // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
				$powered_by_svp_link = wp_unslash( $_POST['powered_by_svp_link'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			}
		}
		if ( isset( $_POST['spp_optin_box'] ) && 'on' === $_POST['spp_optin_box'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_optin_box'] = 1;
		} else {
			$spp_settings['spp_optin_box'] = 0;
		}
		if ( isset( $_POST['btn_download'] ) && 'on' === $_POST['btn_download'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_download'] = 1;
		} else {
			$spp_settings['btn_download'] = 0;
		}
		if ( isset( $_POST['btn_g_play'] ) && 'on' === $_POST['btn_g_play'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_g_play'] = 1;
		} else {
			$spp_settings['btn_g_play'] = 0;
		}
		if ( isset( $_POST['btn_share'] ) && 'on' === $_POST['btn_share'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_share'] = 1;
		} else {
			$spp_settings['btn_share'] = 0;
		}

		if ( isset( $_POST['btn_spplisten'] ) && 'on' === $_POST['btn_spplisten'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_spplisten'] = 1;
		} else {
			$spp_settings['btn_spplisten'] = 0;
		}
		if ( isset( $_POST['btn_spprss'] ) && 'on' === $_POST['btn_spprss'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_spprss'] = 1;
		} else {
			$spp_settings['btn_spprss'] = 0;
		}
		if ( isset( $_POST['btn_sppreview'] ) && 'on' === $_POST['btn_sppreview'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_sppreview'] = 1;
		} else {
			$spp_settings['btn_sppreview'] = 0;
		}
		if ( isset( $_POST['btn_sppandroid'] ) && 'on' === $_POST['btn_sppandroid'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_sppandroid'] = 1;
		} else {
			$spp_settings['btn_sppandroid'] = 0;
		}

		if ( isset( $_POST['btn_itunes_color'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_itunes_color'] = wp_unslash( $_POST['btn_itunes_color'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_embed_color'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_embed_color'] = wp_unslash( $_POST['btn_embed_color'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['transcript'] ) && 'on' === $_POST['transcript'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['transcript'] = 1;
		} else {
			$spp_settings['transcript'] = 0;
		}
		if ( isset( $_POST['transcript_txt'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['transcript_txt'] = wp_unslash( $_POST['transcript_txt'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['spp_listen_text'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_listen_text'] = wp_unslash( $_POST['spp_listen_text'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['transcript_color'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['transcript_color'] = wp_unslash( $_POST['transcript_color'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_itunes'] ) && 'on' === $_POST['btn_itunes'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_itunes'] = 1;
		} else {
			$spp_settings['btn_itunes'] = 0;
		}
		if ( isset( $_POST['btn_embed'] ) && 'on' === $_POST['btn_embed'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_embed'] = 1;
		} else {
			$spp_settings['btn_embed'] = 0;
		}
		if ( isset( $_POST['btn_stiticher_color'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_stiticher_color'] = wp_unslash( $_POST['btn_stiticher_color'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_stiticher_url'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_stiticher_url'] = wp_unslash( $_POST['btn_stiticher_url'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_stiticher'] ) && 'on' === $_POST['btn_stiticher'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_stiticher'] = 1;
		} else {
			$spp_settings['btn_stiticher'] = 0;
		}

		if ( isset( $_POST['spp_optshow_xseconds'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_optshow_xseconds'] = wp_unslash( $_POST['spp_optshow_xseconds'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['spp_poweredby_url'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_poweredby_url'] = wp_unslash( $_POST['spp_poweredby_url'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['spp_optin_hide_chk'] ) && 'on' === $_POST['spp_optin_hide_chk'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_optin_hide_chk'] = 1;
		} else {
			$spp_settings['spp_optin_hide_chk'] = 0;
		}
		if ( isset( $_POST['spp_disable_poweredby'] ) && 'on' === $_POST['spp_disable_poweredby'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_disable_poweredby'] = 1;
		} else {
			$spp_settings['spp_disable_poweredby'] = 0;
		}
		if ( isset( $_POST['spp_pre_roll_checkbox'] ) && 'on' === $_POST['spp_pre_roll_checkbox'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_pre_roll_checkbox'] = 1;
			if ( isset( $_POST['spp_pre_roll_url'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
				$spp_settings['pp_pre_roll_url'] = wp_unslash( $_POST['spp_pre_roll_url'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			}
		} else {
			$spp_settings['spp_pre_roll_checkbox'] = 0;
			$spp_settings['spp_pre_roll_url']      = '';
		}
		if ( isset( $_POST['spp_disable_spp_player_script'] ) && 'on' === $_POST['spp_disable_spp_player_script'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_disable_spp_player_script'] = 1;
		} else {
			$spp_settings['spp_disable_spp_player_script'] = 0;
		}

		if ( isset( $_POST['disable_jquery_model'] ) && 'on' === $_POST['disable_jquery_model'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['disable_jquery_model'] = 1;
		} else {
			$spp_settings['disable_jquery_model'] = 0;
		}
		if ( isset( $_POST['direct_download_button'] ) && 'on' === $_POST['direct_download_button'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['direct_download_button'] = 1;
		} else {
			$spp_settings['direct_download_button'] = 0;
		}

		if ( isset( $_POST['spp_import_html_desc'] ) && 'on' === $_POST['spp_import_html_desc'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_import_html_desc'] = 1;
		} else {
			$spp_settings['spp_import_html_desc'] = 0;
		}

		if ( isset( $_POST['spp_rewind_button'] ) && 'on' === $_POST['spp_rewind_button'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_rewind_button'] = 1;
		} else {
			$spp_settings['spp_rewind_button'] = 0;
		}
		if ( isset( $_POST['spp_speed_button'] ) && 'on' === $_POST['spp_speed_button'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_speed_button'] = 1;
		} else {
			$spp_settings['spp_speed_button'] = 0;
		}
		if ( isset( $_POST['spp_volume_button'] ) && 'on' === $_POST['spp_volume_button'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_volume_button'] = 1;
		} else {
			$spp_settings['spp_volume_button'] = 0;
		}

		if ( isset( $_POST['spp_autoplay_podcast'] ) && 'on' === $_POST['spp_autoplay_podcast'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_autoplay_podcast'] = 1;
		} else {
			$spp_settings['spp_autoplay_podcast'] = 0;
		}

		if ( isset( $_POST['twitter_text_color'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['twitter_text_color'] = wp_unslash( $_POST['twitter_text_color'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['privacy_policy_text'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			$spp_settings['privacy_policy_text'] = wp_unslash( $_POST['privacy_policy_text'] ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized,WordPress.Security.NonceVerification.Missing
		}
		if ( isset( $_POST['headline_text_color'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['headline_text_color'] = wp_unslash( $_POST['headline_text_color'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['submit_button_color'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['submit_button_color'] = wp_unslash( $_POST['submit_button_color'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['submit_button_text'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['submit_button_text'] = wp_unslash( $_POST['submit_button_text'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['opt_container_color'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['opt_container_color'] = wp_unslash( $_POST['opt_container_color'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_soundcloud_color'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_soundcloud_color'] = wp_unslash( $_POST['btn_soundcloud_color'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_soundcloud_url'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_soundcloud_url'] = wp_unslash( $_POST['btn_soundcloud_url'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spotify_color'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_spotify_color'] = wp_unslash( $_POST['btn_spotify_color'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		if ( isset( $_POST['btn_spotify_url'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_spotify_url'] = wp_unslash( $_POST['btn_spotify_url'] ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}

		if ( isset( $_POST['btn_style_round'] ) && 'on' === $_POST['btn_style_round'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_style_round'] = 1;
		} else {
			$spp_settings['btn_style_round'] = 0;
		}

		if ( isset( $_POST['spp_remove_timecode'] ) && 'on' === $_POST['spp_remove_timecode'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_remove_timecode'] = 1;
		} else {
			$spp_settings['spp_remove_timecode'] = 0;
		}

		if ( isset( $_POST['spp_flat_player'] ) && 'on' === $_POST['spp_flat_player'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_flat_player'] = 1;
		} else {
			$spp_settings['spp_flat_player'] = 0;
		}

		if ( isset( $_POST['disable_player_excerpt'] ) && 'on' === $_POST['disable_player_excerpt'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_hide_player_from_excerpt'] = 1;
		} else {
			$spp_settings['spp_hide_player_from_excerpt'] = 0;
		}
		if ( isset( $_POST['hide_elements_athome'] ) && 'on' === $_POST['hide_elements_athome'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['hide_elements_athome'] = 1;
		} else {
			$spp_settings['hide_elements_athome'] = 0;
		}
		if ( isset( $_POST['off_scrolling_text'] ) && 'on' === $_POST['off_scrolling_text'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['off_scrolling_text'] = 1;
		} else {
			$spp_settings['off_scrolling_text'] = 0;
		}

		if ( isset( $_POST['btn_soundcloud'] ) && 'on' === $_POST['btn_soundcloud'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_soundcloud'] = 1;
		} else {
			$spp_settings['btn_soundcloud'] = 0;
		}

		if ( isset( $_POST['btn_spotify'] ) && 'on' === $_POST['btn_spotify'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['btn_spotify'] = 1;
		} else {
			$spp_settings['btn_spotify'] = 0;
		}

		if ( isset( $_POST['spp_auto_publish'] ) && 'on' === $_POST['spp_auto_publish'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_auto_publish'] = 1;
		} else {
			$spp_settings['spp_auto_publish'] = 0;
		}

		if ( isset( $_POST['spp_insert_feat_image'] ) && 'on' === $_POST['spp_insert_feat_image'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_insert_feat_image'] = 1;
		} else {
			$spp_settings['spp_insert_feat_image'] = 0;
		}

		if ( isset( $_POST['spp_email_on_draft'] ) && 'on' === $_POST['spp_email_on_draft'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_email_on_draft'] = 1;
		} else {
			$spp_settings['spp_email_on_draft'] = 0;
		}

		if ( isset( $_POST['spp_player_over_image'] ) && 'on' === $_POST['spp_player_over_image'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_player_over_image'] = 1;
		} else {
			$spp_settings['spp_player_over_image'] = 0;
		}

		if ( isset( $_POST['spp_description'] ) && 'on' === $_POST['spp_description'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_description'] = 1;
		} else {
			$spp_settings['spp_description'] = 0;
		}
		// Disabling of option to turn on and off thumbnail import.
		// For now never import thumbnail - set it to 1 always.

		if ( isset( $_POST['spp_sync_description'] ) && 'on' === $_POST['spp_sync_description'] ) {// phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$spp_settings['spp_sync_description'] = 1;
		} else {
			$spp_settings['spp_sync_description'] = 0;
		}
		do_action( 'spp_options_save' );
		
		$this->spp_save_option( $spp_settings );

		// Only save reviews when the user first types in iTunes URL.
		$itunes_url_exists = $this->spp_get_option( 'itunes_url' );

		if ( ( ! empty( $itunes_url ) ) && ( false === $itunes_url_exists ) ) {

			$spp_settings['itunes_url'] = $itunes_url;
			$this->spp_save_option( $spp_settings );

			$this->spp_save_reviews( $itunes_ID ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase,WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		}
	}

	/**
	 * Get the last page number from a given URL.
	 *
	 * @param string $pageurl The URL to extract the page number from.
	 *
	 * @return string The extracted page number.
	 */
	public function get_last_page( $pageurl ) {
		$page_number = '1';
		$pageurl     = explode( '/', $pageurl );

		if ( isset( $pageurl[6] ) ) {
			$pageurl     = $pageurl[6];
			$page_number = str_replace( 'page=', '', $pageurl );
		}

		return $page_number;
	}
	/**
	 * Get the full country name based on the short country code.
	 *
	 * @param string $short The short country code.
	 *
	 * @return string The full country name.
	 */
	public function get_full_country_name( $short ) {
		switch ( $short ) {
			case 'us':
				$fullname = 'United States';
				break;
			case 'ca':
				$fullname = 'Canada';
				break;
			case 'la':
				$fullname = 'Puerto Rico"';
				break;
			case 'mx':
				$fullname = 'Mexico';
				break;
			case 'gb':
				$fullname = 'United Kingdom';
				break;
			case 'br':
				$fullname = 'Brazil';
				break;
			case 'tr':
				$fullname = 'Turkey';
				break;
			case 'se':
				$fullname = 'Sweden';
				break;
			case 'fi':
				$fullname = 'Finland';
				break;
			case 'ch':
				$fullname = 'Switzerland';
				break;
			case 'si':
				$fullname = 'Slovenia';
				break;
			case 'at':
				$fullname = 'Austria';
				break;
			case 'pl':
				$fullname = 'Poland';
				break;
			case 'pt':
				$fullname = 'Portugal';
				break;
			case 'ro':
				$fullname = 'Romania';
				break;
			case 'ru':
				$fullname = 'Russia';
				break;
			case 'sk':
				$fullname = 'Slovak Republic';
				break;
			case 'no':
				$fullname = 'Norway';
				break;
			case 'nl':
				$fullname = 'Netherlands';
				break;
			case 'md':
				$fullname = 'Moldova';
				break;
			case 'mt':
				$fullname = 'Malta';
				break;
			case 'hu':
				$fullname = 'Hungary';
				break;
			case 'mk':
				$fullname = 'Macedonia';
				break;
			case 'lu':
				$fullname = 'Luxembourg';
				break;
			case 'lt':
				$fullname = 'Lithuania';
				break;
			case 'lv':
				$fullname = 'Latvia';
				break;
			case 'it':
				$fullname = 'Italy';
				break;
			case 'ie':
				$fullname = 'Ireland';
				break;
			case 'hr':
				$fullname = 'Croatia';
				break;
			case 'gr':
				$fullname = 'Greece';
				break;
			case 'fr':
				$fullname = 'France';
				break;
			case 'es':
				$fullname = 'Spain';
				break;
			case 'ee':
				$fullname = 'Estonia';
				break;
			case 'de':
				$fullname = 'Germany';
				break;
			case 'dk':
				$fullname = 'Denmark';
				break;
			case 'cz':
				$fullname = 'Czech Republic';
				break;
			case 'bg':
				$fullname = 'Bulgaria';
				break;
			case 'be':
				$fullname = 'Belgium';
				break;
			case 'vn':
				$fullname = 'Vietnam';
				break;
			case 'th':
				$fullname = 'Thailand';
				break;
			case 'tw':
				$fullname = 'Taiwan';
				break;
			case 'sg':
				$fullname = 'Singapore';
				break;
			case 'ph':
				$fullname = 'Philippines';
				break;
			case 'nz':
				$fullname = 'New Zealand';
				break;
			case 'my':
				$fullname = 'Malaysia';
				break;
			case 'kr':
				$fullname = 'South Korea';
				break;
			case 'jp':
				$fullname = 'Japan';
				break;
			case 'id':
				$fullname = 'Indonesia';
				break;
			case 'hk':
				$fullname = 'Hong Kong';
				break;
			case 'cn':
				$fullname = 'China';
				break;
			case 'au':
				$fullname = 'Australia';
				break;
			case 'ae':
				$fullname = 'United Arab Emirates';
				break;
			case 'ug':
				$fullname = 'Uganda';
				break;
			case 'tn':
				$fullname = 'Tunisia';
				break;
			case 'za':
				$fullname = 'South Africa';
				break;
			case 'sn':
				$fullname = 'Senegal';
				break;
			case 'sa':
				$fullname = 'Saudi Arabia';
				break;
			case 'om':
				$fullname = 'Oman';
				break;
			case 'ng':
				$fullname = 'Nigeria';
				break;
			case 'ne':
				$fullname = 'Niger';
				break;
			case 'mz':
				$fullname = 'Mozambique';
				break;
			case 'mu':
				$fullname = 'Mauritius';
				break;
			case 'ml':
				$fullname = 'Mali';
				break;
			case 'mg':
				$fullname = 'Madagascar';
				break;
			case 'kw':
				$fullname = 'Kuwait';
				break;
			case 'ke':
				$fullname = 'Kenya';
				break;
			case 'jo':
				$fullname = 'Jordan';
				break;
			case 'il':
				$fullname = 'Israel';
				break;
			case 'in':
				$fullname = 'India';
				break;
			case 'gw':
				$fullname = 'Guinea Bissau';
				break;
			case 'eg':
				$fullname = 'Egypt';
				break;
			case 'bw':
				$fullname = 'Botswana';
				break;
			case 'bh':
				$fullname = 'Bahrain';
				break;
			case 'am':
				$fullname = 'Armenia';
				break;

		} //end switch
		return $fullname;
	}


	// Apple podcast reviews using JSON - Not required now.
	/**
	 * Save reviews data as JSON for a specified podcast.
	 *
	 * @param int|null $podcast_id The ID of the podcast. Default is null.
	 *
	 * @return void
	 */
	public function spp_save_reviews_json( $podcast_id = null ) {
		$is_license_valid = get_option( 'sppress_ls' );

		if ( ! $podcast_id ) {
			$podcast_id = $this->spp_get_option( 'itunes_id' );
		}

		if ( ( 'valid' === $is_license_valid && 'trial' !== $is_license_valid ) || false === $podcast_id ) {
			return;
		}

		global $wpdb;
		$table_spp_reviews = $wpdb->prefix . 'spp_reviews';

		$countries = 'us, ca, la, br, tr, se, fi, ch, si, at, pl, pt, ro, ru, sk, no, nl, md, mt, hu, mk, lu, lt, lv, it, ie, hr, gr, fr, es, ee, de, dk, cz, bg, be, vn, th, tw, sg, ph, nz, my, kr, jp, id, hk, cn, ae, ug, tn, za, sn, sa, om, ng, ne, mz, mu, ml, mg, kw, ke, jo, il, in, gw, eg, bw, bh, am, gb, mx, au';
		$countries = explode( ',', $countries );
		foreach ( $countries as $country ) {
			$country     = trim( str_replace( ' ', '', $country ) );
			$reviews_url = 'http://itunes.apple.com/' . $country . '/rss/customerreviews/id=' . $podcast_id . '/sortBy=mostRecent/json';

			$reviews_json_content = $this->url_get_contents( $reviews_url );
			$reviews_json_array   = json_decode( $reviews_json_content, true );
			$entry_count_check    = 0;

			if ( ! isset( $reviews_json_array['feed']['entry'] ) ) {
				continue;
			}

			foreach ( $reviews_json_array['feed']['entry'] as $entry ) {
				++$entry_count_check;
				break; // found an entry which means that URL worked.
			}

			if ( 0 === $entry_count_check ) {
				$reviews_url = 'http://itunes.apple.com/' . $country . '/rss/customerreviews/page=1/id=' . $podcast_id . '/sortBy=mostRecent/json';

				$reviews_json_content = $this->url_get_contents( $reviews_url );
				$reviews_json_array   = json_decode( $reviews_json_content, true );

				foreach ( $reviews_json_array['feed']['entry'] as $entry ) {
					++$entry_count_check;
					break; // found an entry which means that URL worked.
				}

				if ( 0 === $entry_count_check ) {
					$reviews_url = 'http://itunes.apple.com/' . $country . '/rss/customerreviews/id=' . $podcast_id . '/sortBy=mostRecent/json?urlDesc=/customerreviews/page=1/id=' . $podcast_id . '/sortby=mostrecent/json';

					$reviews_json_content = $this->url_get_contents( $reviews_url );
					$reviews_json_array   = json_decode( $reviews_json_content, true );

					foreach ( $reviews_json_array['feed']['entry'] as $entry ) {
						++$entry_count_check;
						break; // found an entry which means that URL worked.
					}
				}
			}

			$pd_last_page = $this->get_last_page( (string) $reviews_json_array['feed']['link'][3]['attributes']['href'] );

			for ( $i = 1; $i <= $pd_last_page; $i++ ) {

				if ( ( 0 === $entry_count_check ) || ( $i > 1 ) ) {

					$reviews = 'http://itunes.apple.com/' . $country . '/rss/customerreviews/page=' . $i . '/id=' . $podcast_id . '/sortBy=mostRecent/json';

					$reviews_json_content = $this->url_get_contents( $reviews_url );
					$reviews_json_array   = json_decode( $reviews_json_content, true );
					$entry_count_check    = 0;

					foreach ( $reviews_json_array['feed']['entry'] as $entry ) {
						++$entry_count_check;
						break; // found an entry which means that URL worked.
					}

					if ( 0 === $entry_count_check ) {
						usleep( 1000000 );
						$reviews = 'http://itunes.apple.com/' . $country . '/rss/customerreviews/page=' . $i . '/id=' . $podcast_id . '/sortBy=mostRecent/json?urlDesc=/customerreviews/page=' . $i . '/id=' . $podcast_id . '/sortby=mostrecent/json';

						$reviews_json_content = $this->url_get_contents( $reviews_url );
						$reviews_json_array   = json_decode( $reviews_json_content, true );

						foreach ( $reviews_json_array['feed']['entry'] as $entry ) {
							++$entry_count_check;
							break; // found an entry which means that URL worked.
						}
					}
				}

				$entry_count = 0;

				foreach ( $reviews_json_array['feed']['entry'] as $entry ) {

					$rw_id = ! empty( $entry['id']['label'] ) ? $entry['id']['label'] : '';

					if ( $entry_count > 0 ) {
						$is_in_db = $wpdb->get_var( $wpdb->prepare( 'SELECT rw_id FROM '. $table_spp_reviews .' WHERE rw_id=%s', $rw_id ) ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
						if ( ! $is_in_db ) {

							$title    = ! empty( $entry['title']['label'] ) ? $entry['title']['label'] : '';
							$content  = ! empty( $entry['content']['label'] ) ? $entry['content']['label'] : '';
							$imrating = ! empty( $entry['im:rating']['label'] ) ? $entry['im:rating']['label'] : '';
							$author   = ! empty( $entry['author']['name']['label'] ) ? $entry['author']['name']['label'] : '';

							$data = array(
								'rw_id'             => $rw_id,
								'rw_coutry'         => $country,
								'rw_published_date' => '',
								'rw_title'          => $title,
								'rw_text'           => $content,
								'rw_ratings'        => $imrating,
								'rw_author'         => $author,
							);

							$inserted = $wpdb->insert( $table_spp_reviews, $data ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching,WordPress.DB.DirectDatabaseQuery.DirectQuery
						}
					}

					++$entry_count;
				}
			}
		}

		unset( $podcast_reviews );
		$this->sppress_tcheck();
	}



	// Apple podcast reviews using XML - Use this version.
	/**
	 * Save reviews data for a specified podcast.
	 *
	 * @param int|null $podcast_id The ID of the podcast. Default is null.
	 *
	 * @return void
	 */
	public function spp_save_reviews( $podcast_id = null ) {
		$is_license_valid = get_option( 'sppress_ls' );

		if ( ! $podcast_id ) {
			$podcast_id = $this->spp_get_option( 'itunes_id' );
		}

		if ( ( 'valid' !== $is_license_valid && 'trial' !== $is_license_valid ) || false === $podcast_id ) {
			return;
		}

		global $wpdb;
		$table_spp_reviews = $wpdb->prefix . 'spp_reviews';

		$countries = 'us, ca, la, br, tr, se, fi, ch, si, at, pl, pt, ro, ru, sk, no, nl, md, mt, hu, mk, lu, lt, lv, it, ie, hr, gr, fr, es, ee, de, dk, cz, bg, be, vn, th, tw, sg, ph, nz, my, kr, jp, id, hk, cn, ae, ug, tn, za, sn, sa, om, ng, ne, mz, mu, ml, mg, kw, ke, jo, il, in, gw, eg, bw, bh, am, gb, mx, au';
		$countries = explode( ',', $countries );
		foreach ( $countries as $country ) {
			$country     = trim( str_replace( ' ', '', $country ) );
			$reviews_url = 'http://itunes.apple.com/' . $country . '/rss/customerreviews/id=' . $podcast_id . '/sortBy=mostRecent/xml';

			$xml = $this->url_get_contents( $reviews_url );

			$xml             = preg_replace( '/(<\/?)(\w+):([^>]*>)/', '$1$2$3', $xml );
			$podcast_reviews = simplexml_load_string( $xml );

			$entry_count_check = 0;

			if ( isset( $podcast_reviews->entry ) ) {
				foreach ( $podcast_reviews->entry as $entry ) {
					++$entry_count_check;
					break; // found an entry which means that URL worked.
				}
			}

			if ( 0 === $entry_count_check ) {
				$reviews_url = 'http://itunes.apple.com/' . $country . '/rss/customerreviews/page=1/id=' . $podcast_id . '/sortBy=mostRecent/xml';

				$xml             = $this->url_get_contents( $reviews_url );
				$xml             = preg_replace( '/(<\/?)(\w+):([^>]*>)/', '$1$2$3', $xml );
				$podcast_reviews = simplexml_load_string( $xml );

				if ( isset( $podcast_reviews->entry ) ) {
					foreach ( $podcast_reviews->entry as $entry ) {
						++$entry_count_check;
						break; // found an entry which means that URL worked.
					}
				}

				if ( 0 === $entry_count_check ) {
					$reviews_url = 'http://itunes.apple.com/' . $country . '/rss/customerreviews/id=' . $podcast_id . '/sortBy=mostRecent/xml?urlDesc=/customerreviews/page=1/id=' . $podcast_id . '/sortby=mostrecent/xml';

					$xml             = $this->url_get_contents( $reviews_url );
					$xml             = preg_replace( '/(<\/?)(\w+):([^>]*>)/', '$1$2$3', $xml );
					$podcast_reviews = simplexml_load_string( $xml );

					if ( isset( $podcast_reviews->entry ) ) {
						foreach ( $podcast_reviews->entry as $entry ) {
							++$entry_count_check;
							break; // found an entry which means that URL worked.
						}
					}
				}
			}

			if ( isset( $podcast_reviews->link[3] ) ) {
				$pd_last_page = $this->get_last_page( (string) $podcast_reviews->link[3]->attributes()->href );
			}

			$reviews_count = 0;
			for ( $i = 1; $i <= $pd_last_page; $i++ ) {

				if ( ( 0 === $entry_count_check ) || ( $i > 1 ) ) {

					$reviews         = 'http://itunes.apple.com/' . $country . '/rss/customerreviews/page=' . $i . '/id=' . $podcast_id . '/sortBy=mostRecent/xml';
					$xml             = $this->url_get_contents( $reviews );
					$xml             = preg_replace( '/(<\/?)(\w+):([^>]*>)/', '$1$2$3', $xml );
					$podcast_reviews = simplexml_load_string( $xml );

					$entry_count_check = 0;

					if ( isset( $podcast_reviews->entry ) ) {
						foreach ( $podcast_reviews->entry as $entry ) {
							++$entry_count_check;
							break; // found an entry which means that URL worked.
						}
					}

					if ( 0 === $entry_count_check ) {
						usleep( 1000000 );
						$reviews = 'http://itunes.apple.com/' . $country . '/rss/customerreviews/page=' . $i . '/id=' . $podcast_id . '/sortBy=mostRecent/xml?urlDesc=/customerreviews/page=' . $i . '/id=' . $podcast_id . '/sortby=mostrecent/xml';

						$xml             = $this->url_get_contents( $reviews );
						$xml             = preg_replace( '/(<\/?)(\w+):([^>]*>)/', '$1$2$3', $xml );
						$podcast_reviews = simplexml_load_string( $xml );

						if ( isset( $podcast_reviews->entry ) ) {
							foreach ( $podcast_reviews->entry as $entry ) {
								++$entry_count_check;
								break; // found an entry which means that URL worked.
							}
						}
					}
				}

				if ( isset( $podcast_reviews->entry ) ) {

					foreach ( $podcast_reviews->entry as $entry ) {

						$rw_id    = (int) $entry->id;
						$is_in_db = $wpdb->query( $wpdb->prepare( "SELECT rw_id FROM {$table_spp_reviews} WHERE rw_id=%d", $rw_id ) ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
						if ( ! $is_in_db ) {
							$updated  = (string) $entry->updated;
							$title    = (string) $entry->title;
							$content  = (string) $entry->content[0];
							$imrating = (string) $entry->imrating;
							$author   = (string) $entry->author->name;

							$data     = array(
								'rw_id'             => $rw_id,
								'rw_coutry'         => $country,
								'rw_published_date' => $updated,
								'rw_title'          => $title,
								'rw_text'           => $content,
								'rw_ratings'        => $imrating,
								'rw_author'         => $author,
							);
							$inserted = $wpdb->insert( $table_spp_reviews, $data ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
							++$reviews_count;
						}
					}
				}
			}
		}

		unset( $podcast_reviews );
		$this->sppress_tcheck();
	}


	/**
	 * Convert a SimpleXMLElement object to an associative array.
	 *
	 * @param SimpleXMLElement $xmlobj The SimpleXMLElement object to convert.
	 *
	 * @return array The converted associative array.
	 */
	public function simplexml_to_array( $xmlobj ) {
		$a = array();
		foreach ( $xmlobj->children() as $node ) {
			if ( is_array( $node ) ) {
				$a[ $node->getName() ] = simplexml_to_array( $node );
			} else {
				$a[ $node->getName() ] = (string) $node;
			}
		}
		return $a;
	}


	/**
	 * Replace the first occurrence of a substring in a string.
	 *
	 * This function searches for the first occurrence of $search in $subject and replaces it with $replace.
	 *
	 * @param string $search  The substring to search for.
	 * @param string $replace The string to replace $search with.
	 * @param string $subject The original string.
	 *
	 * @return string The modified string with the first occurrence of $search replaced by $replace.
	 */
	public function str_replace_first( $search, $replace, $subject ) {
		$pos = strpos( $subject, $search );
		if ( false !== $pos ) {
			return substr_replace( $subject, $replace, $pos, strlen( $search ) );
		}
		return $subject;
	}


		/**
		 * Save podcast XML data to the database.
		 *
		 * This function retrieves XML data from a provided URL, processes it,
		 * and inserts relevant information into the WordPress database.
		 *
		 * @param string $podcastURL    The URL of the podcast XML feed.
		 * @param bool   $manul_import  Flag indicating manual import.
		 * @param bool   $initial       Flag indicating initial import.
		 *
		 * @return int The number of podcasts inserted into the database.
		 */
	public function spp_save_podcast_xml( $podcastURL, $manul_import = false, $initial = false ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		global $wpdb;
		$inserted      = false;
		$duplicate     = false;
		$html          = '';
		$episode_image = '';

		$table_spp_podcast = $wpdb->prefix . 'spp_podcast';

		$xml        = $this->url_get_contents( $podcastURL ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		$itunes_xml = preg_replace( '/(<\/?)(\w+):([^>]*>)/', '$1$2$3', $xml );

		$podcastXML = simplexml_load_string( $itunes_xml ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		if ( ! is_object( $podcastXML ) ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			return;
		}
		
		$channel_title                       = (string) $podcastXML->channel->title; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		$this->spp_settings['channel_title'] = $channel_title;
		update_option( 'spp_settings', $this->spp_settings );
		$channel_keywords                       = $podcastXML->channel->ituneskeywords; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		$channel_keywords                       = explode( ',', $channel_keywords );
		$channel_keywords                       = serialize( $channel_keywords ); // phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.serialize_serialize
		$this->spp_settings['channel_keywords'] = $channel_keywords;
		update_option( 'spp_settings', $this->spp_settings );
		$channel_image     = (string) $podcastXML->channel->itunesimage->attributes()->href; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		$channel_image_url = (string) $podcastXML->channel->itunesimage->attributes()->href; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		$import_html_desc  = $this->spp_get_option( 'spp_import_html_desc' );

		$ep_specific_date  = strtotime( $this->spp_get_option( 'ep_specific_date' ) );
		$spp_import_select = $this->spp_get_option( 'spp_import_select' );
		$timeoffset        = $this->get_unix_timeoffset();

		$LibsynPostImage = $this->spp_get_option( 'ep_art_select' ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		if ( '2' === $LibsynPostImage ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$useLibsynPostImage = true; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		} else {
			$useLibsynPostImage = false; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		}

		// For special hosts like Anchor and Omny.
		$parseurl = wp_parse_url( $podcastURL ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

		$omniHost = false;  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		if ( strpos( $podcastURL, 'omny' ) ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$omniHost = true; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		}

		$acast = false;
		if ( strpos( $podcastURL, 'acast' ) ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$acast = true;
		}

		$captivatefm = false;
		if ( strpos( $podcastURL, 'captivate.fm' ) ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$captivatefm = true;
		}

		$redCircle = false; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		if ( ( strpos( $podcastURL, 'podcache' ) ) || ( strpos( $podcastURL, 'redcircle' ) ) ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$redCircle = true; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		}

		$soundon = false;
		if ( strpos( $podcastURL, 'soundon.fm' ) ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$soundon = true;
		}

		$simplecast = false;
		if ( strpos( $podcastURL, 'simplecast.com' ) ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$simplecast = true;
		}

		$pp_counter     = 0;
		$total_podcasts = $this->spp_get_option( 'total_podcasts' );

		if ( ( isset( $total_podcasts ) ) && ( 0 === $total_podcasts && false === $manul_import ) ) {

			$total_podcasts                       = count( $podcastXML->channel->item ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$this->spp_settings['total_podcasts'] = $total_podcasts;
			update_option( 'spp_settings', $this->spp_settings );
			if ( 1 === $total_podcasts ) {
				++$total_podcasts;
			}
			echo esc_html( $total_podcasts );
			exit;
		}
		foreach ( $podcastXML->channel->item as $item ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$item2 = $this->simplexml_to_array( $item );

			$publishedate = strtotime( $item2['pubDate'] ) - $timeoffset;
			// $date_time = date("Y-m-d H:i:s", $item2['pubDate']);
			// Add debugging statements
			if ((2 === (int)$spp_import_select) && (false === $initial)) {
				if ($publishedate <= $ep_specific_date) {
					continue;
				}
			}
			if ((3 === (int)$spp_import_select) && (false === $initial)) {
				
				if ($publishedate >= $ep_specific_date) {
					continue;
				}
			}
			$title          = isset( $item2['title'] ) ? $item2['title'] : ' ';
			$pub_date       = isset( $item2['pubDate'] ) ? $item2['pubDate'] : ' ';
			$btrooffsiteurl = isset( $item2['btroffsiteplayerurl'] ) ? $item2['btroffsiteplayerurl'] : '';
			$btr_link       = isset( $item2['link'] ) ? $item2['link'] : ' ';

			if ( $import_html_desc ) {
				$description = isset( $item2['contentencoded'] ) ? $item2['contentencoded'] : ' ';
				if ( empty( $description ) ) {
					$description = isset( $item2['itunessummary'] ) ? $item2['itunessummary'] : ' ';
				}
			} else {
				$description = isset( $item2['description'] ) ? $item2['description'] : ' ';
			}

			if ( $item->enclosure ) {
				$audio_file = (string) $item->enclosure->attributes()->url;
				// Don't strip out query strings any more when storing to database (Libsyn and AudioBoom do this).
				// For classic player and when checking for duplicates we ignore the query parameter.
			} else {
				continue;
			}
			$audio_duration = (string) $item->itunesduration;
			$audio_length   = (string) $item->enclosure->attributes()->length;
			if ( is_null( $audio_length ) ) {
				$audio_length = 0;
			}
			$episode_keywords = isset( $item2['ituneskeywords'] ) ? $item2['ituneskeywords'] : ' ';
			$pc_libsyn_image  = '';

			if ( $item->itunesimage ) {
				$episode_image = (string) $item->itunesimage->attributes()->href;
			}

			// Anchor specific check to avoid duplicates.
			if ( 'anchor.fm' === $parseurl['host'] ) {
				$a               = wp_parse_url( $audio_file );
				$p               = $a['path'];
				$pa              = explode( '/', $p );
				$base_audio_file = $pa[5];
			} elseif ( $omniHost ) { //phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				// Specific case for Omny podcast host.
				$stripped_audio_file = substr( $audio_file, strpos( $audio_file, 'omny.fm' ) );
				if ( strpos( $stripped_audio_file, '?' ) ) {
					$base_audio_file = substr( $stripped_audio_file, 0, strpos( $stripped_audio_file, '?' ) );
				} else {
					$base_audio_file = $stripped_audio_file;
				}
			} elseif ( $acast ) {
				// Specific case for Acast podcast host.
				$stripped_audio_file = substr( $audio_file, strpos( $audio_file, 'acast.com' ) );
				if ( strpos( $stripped_audio_file, '?' ) ) {
					$base_audio_file = substr( $stripped_audio_file, 0, strpos( $stripped_audio_file, '?' ) );
				} else {
					$base_audio_file = $stripped_audio_file;
				}
			} elseif ( $captivatefm ) {
				// Specific case for Captivate.fm podcast host.
				$stripped_audio_file = substr( $audio_file, strpos( $audio_file, 'captivate.fm' ) );
				if ( strpos( $stripped_audio_file, '?' ) ) {
					$base_audio_file = substr( $stripped_audio_file, 0, strpos( $stripped_audio_file, '?' ) );
				} else {
					$base_audio_file = $stripped_audio_file;
				}
			} elseif ( $simplecast ) {
				// Specific case for simplecast podcast host.
				$stripped_audio_file = substr( $audio_file, strpos( $audio_file, 'simplecast.com' ) );
				if ( strpos( $stripped_audio_file, '?' ) ) {
					$base_audio_file = substr( $stripped_audio_file, 0, strpos( $stripped_audio_file, '?' ) );
				} else {
					$base_audio_file = $stripped_audio_file;
				}
			} elseif ( $soundon ) {
				// Specific case for Soundon podcast host.
				$stripped_audio_file = substr( $audio_file, strpos( $audio_file, 'soundon.fm' ) );
				if ( strpos( $stripped_audio_file, '?' ) ) {
					$base_audio_file = substr( $stripped_audio_file, 0, strpos( $stripped_audio_file, '?' ) );
				} else {
					$base_audio_file = $stripped_audio_file;
				}
			} elseif ( $redCircle ) { //  phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				// Specific case for RedCircle podcast host.
				$stripped_audio_file = substr( $audio_file, strpos( $audio_file, 'redcircle.com' ) );
				if ( strpos( $stripped_audio_file, '?' ) ) {
					$base_audio_file = substr( $stripped_audio_file, 0, strpos( $stripped_audio_file, '?' ) );
				} else {
					$base_audio_file = $stripped_audio_file;
				}
			} else { // Rest of podcast hosts.
				// Remove query arguments from MP3 file when checking for duplicates (we still store query params to table).
				$stripped_audio_file = wp_parse_url( $audio_file );
				$base_audio_file     = $stripped_audio_file['path'];
			}

			$duplicate = $wpdb->get_row( // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
				$wpdb->prepare(
					"SELECT * FROM {$table_spp_podcast} WHERE pc_audio_file LIKE %s", // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
					'%' . $wpdb->esc_like( $base_audio_file ) . '%'
				)
			);

			// Logic for Libsyn image only.
			if ( ( $useLibsynPostImage ) && ( $manul_import ) ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$episode_link = $item->link;
				if ( strpos( $episode_link, '.mp3' ) ) {
					break;
				}
				include_once 'tools/simple_html_dom.php';
				$response = wp_safe_remote_get( $episode_link );
				$html     = wp_remote_retrieve_body( $response );
				$html     = str_get_html( $html );

				if ( $html ) {
					$pc_libsyn_image = $html->find( 'img[class=postImage]', 0 )->src;
					$html->clear();
				}

				if ( ( $manul_import ) && ( $duplicate ) ) {
					if ( empty( $duplicate->pc_libsyn_image ) ) {
						$updated = $wpdb->update( // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
							$table_spp_podcast,
							array(
								'pc_libsyn_image' => $pc_libsyn_image,
							),
							array(
								'pc_audio_file' => $duplicate->pc_audio_file,
							)
						);
					} //end if $pc_libsyn_image
					continue;
				} //end if $manual_import
			}

			if ( empty( $duplicate->pc_audio_file ) ) {

				$pc_data  = array(
					'pc_title'            => $this->remove_emoji( $title ),
					'pc_published_date'   => $pub_date,
					'pc_description'      => $this->remove_emoji( $description ),
					'pc_audio_file'       => $audio_file,
					'pc_audio_duration'   => $audio_duration,
					'pc_audio_length'     => $audio_length,
					'pc_episode_keywords' => $episode_keywords,
					'pc_episode_image'    => $episode_image,
					'pc_libsyn_image'     => $pc_libsyn_image,
					'pc_btr_offsite_url'  => $btrooffsiteurl,
					'pc_btr_link'         => $btr_link,
				);
				
				$inserted = $wpdb->insert( // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
					$table_spp_podcast,
					$pc_data,
					array(
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
					)
				);
			}

			if ( $inserted ) {
				++$pp_counter;
			}
		} //end foreach
		$this->spp_settings['pp_counter']       = $pp_counter;
		$this->spp_settings['podcast_imported'] = 1;

		update_option( 'spp_settings', $this->spp_settings );
		unset( $podcastXML ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase,WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

		return $pp_counter;
	}
	/**
	 * Register and enqueue scripts and styles for the admin area.
	 *
	 * @param string $hook The current admin page hook.
	 */
	public function register_spp_admin_scripts( $hook ) {
		if ( ( 'post.php' === $hook ) || ( 'post-new.php' === $hook ) ) {
			// Enqueue script for post.php and post-new.php pages.
			wp_enqueue_script(
				'spp-optin-script',
				SPPRESS_PLUGIN_URL . '/spp_view/js/spp-optin.js',
				array( 'jquery' ),
				'1.0', // Replace with the actual version number you're using.
				true
			);
		}

		if ( ( isset( $_GET['page'] ) ) && ( in_array( $_GET['page'], array( 'spp-podcast-settings', 'spp_reviews', 'spp-url-shortner', 'spp-podcasts-table' ) ) ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended,WordPress.PHP.StrictInArray.MissingTrueStrict
			// Enqueue scripts and styles for specific admin pages.
			wp_enqueue_style( 'wp-color-picker' );
			wp_enqueue_script( 'jquery-ui-datepicker' );
			wp_enqueue_media();
			wp_enqueue_script( 'media-grid' );
			wp_enqueue_script( 'media' );
			wp_enqueue_script( 'thickbox' ); // Responsible for managing the modal window.
			wp_enqueue_style( 'thickbox' ); // Provides the styles needed for this window.

			wp_enqueue_style(
				'jquery-ui-css',
				'https://ajax.googleapis.com/ajax/libs/jqueryui/1.8.2/themes/smoothness/jquery-ui.css',
				array(),
				gmdate( 'Ymd' )
			);

			// Enqueue custom admin script.
			wp_enqueue_script(
				'spp-admin-script',
				SPPRESS_PLUGIN_URL . '/spp_view/js/spp_admin_scripts.js',
				array( 'wp-color-picker' ),
				'1.0', // Placeholder version.
				true
			);

			// Localize script.
			$params = array(
				'spp_ajax_url' => '/wp-admin/admin-ajax.php',
				'nonce'        => wp_create_nonce( 'button_order' ),
			);
			wp_localize_script( 'spp-admin-script', 'sppajaxurl', $params );

			wp_register_style( 'spp_wp_admin_css_bootstrap', SPPRESS_PLUGIN_URL . '/spp_view/css/bootstrap.css', false, '1.0.0' );
			wp_register_style( 'spp_wp_admin_css_bootstrap_responsive', SPPRESS_PLUGIN_URL . '/spp_view/css/bootstrap-responsive.css', false, '1.0.0' );
			wp_register_style( 'spp_wp_admin_css_common', SPPRESS_PLUGIN_URL . '/spp_view/css/common.css', false, '1.0.0' );
			wp_register_style( 'spp_wp_admin_css_fontawesome', SPPRESS_PLUGIN_URL . '/spp_view/css/fontawesome.css', false, '1.0.0' );
			wp_register_style( 'spp_wp_admin_css', SPPRESS_PLUGIN_URL . '/spp_view/css/spp-admin.css', false, '1.0.0' );
			wp_enqueue_style( 'spp_wp_admin_css_bootstrap' );
			wp_enqueue_style( 'spp_wp_admin_css_bootstrap_responsive' );
			wp_enqueue_style( 'spp_wp_admin_css_common' );
			wp_enqueue_style( 'spp_wp_admin_css_fontawesome' );
			wp_enqueue_style( 'spp_wp_admin_css_project' );
			wp_enqueue_style( 'spp_wp_admin_css' );
			// Enqueue custom admin script.
			wp_enqueue_script(
				'spp_wp_admin_js_bootstrap_min',
				SPPRESS_PLUGIN_URL . '/spp_view/js/bootstrap.min.js',
				false,
				'1.0.0',
				true
			);

			// Trigger an action for additional scripts.
			do_action( 'spp_scripts' );
		}

		if ( 'widgets.php' !== $hook ) {
			return;
		}
	}

	/**
	 * Update post dates based on the release date meta field.
	 *
	 * @param WP_Post $post The post object.
	 */
	public function preserve_draft_date( $post ) {

		$release_date = get_post_meta( $post->ID, '_release_date', true );

		if ( $release_date && gmdate( 'Y-m-d H:i:s', strtotime( $release_date ) ) !== $post->post_date ) {
			$post->edit_date = gmdate( 'Y-m-d H:i:s', strtotime( $release_date ) );
			$post->post_date = gmdate( 'Y-m-d H:i:s', strtotime( $release_date ) );
			wp_update_post( $post );
		}
	}
	/**
	 * Converts URLs, email addresses, and www addresses to clickable links.
	 *
	 * @param string $text The input text.
	 * @return string The text with clickable links.
	 */
	public function makeClickableLinks( $text ) { // phpcs:ignore WordPress.NamingConventions.ValidFunctionName.MethodNameInvalid
		$text = preg_replace( '/(((f|ht){1}tp:\/\/)[-a-zA-Z0-9@:%_\+.~#?&\/\/=]+)/i', '<a href="\\1" rel="nofollow">\\1</a>' . PHP_EOL, $text );
		$text = preg_replace( '/([[:space:]()[{}])(www.[-a-zA-Z0-9@:%_\+.~#?&\/\/=]+)/i', '\\1<a href="http://\\2" rel="nofollow">\\2</a>' . PHP_EOL, $text );
		$text = preg_replace( '/([_\.0-9a-z-]+@([0-9a-z][0-9a-z-]+\.)+[a-z]{2,3})/i', '<a href="mailto:\\1" rel="nofollow">\\1</a>' . PHP_EOL, $text );
		return $text;
	}

	/**
	 * Removes specific styles from the input tag source.
	 *
	 * @param string $tag_source The source of the HTML tag.
	 * @return string The sanitized tag source.
	 */
	public function remove_evil_styles_v2( $tag_source ) {
		$evil_styles       = array(
			'font',
			'font-family',
			'font-face',
			'font-size',
			'font-size-adjust',
			'font-stretch',
			'font-variant',
			'line-height',
			'margin-top',
			'margin-bottom',
		);
		$evil_style_pttrns = array();
		foreach ( $evil_styles as $v ) {
			$evil_style_pttrns[] = '/' . $v . '\s*:\s*[^;"]*;?/';
		}
		return preg_replace( $evil_style_pttrns, '', $tag_source );
	}

	/**
	 * Get the time zone offset in UNIX seconds.
	 *
	 * @return int The time zone offset.
	 */
	public static function get_unix_timeoffset() {
		// Determine TimeZone Offset in UNIX seconds.
		$timeLocal  = (int) current_time( 'timestamp', 0 ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase,WordPress.DateTime.CurrentTimeTimestamp.Requested
		$timeGmt    = (int) time(); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		$timeOffset = ( $timeGmt - $timeLocal ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		return $timeOffset; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
	}
	/**
	 * Generate a post from the spp_table.
	 *
	 * @return int|WP_Error The generated post ID on success, or a WP_Error object on failure.
	 */
	public function generate_post_from_spp_table() {
		global $wpdb;
		$disable_thumbs        = $this->spp_get_option( 'spp_disable_thumbs' );
		$auto_post             = $this->spp_get_option( 'spp_auto_publish' );
		$no_import             = $this->spp_get_option( 'spp_import_select' );
		$cpt_select            = $this->spp_get_option( 'spp_cpt_select' );
		$spp_post_format       = $this->spp_get_option( 'spp_post_format_select' );
		$author                = $this->spp_get_option( 'spp_author' );
		$episode_short_link    = $this->spp_get_option( 'episode_short_link' );
		$category              = $this->spp_get_option( 'spp_post_category' );
		$disable_url_shortner  = $this->spp_get_option( 'disable_url_shortner' );
		$ep_art_select         = $this->spp_get_option( 'ep_art_select' );
		$spp_description       = $this->spp_get_option( 'spp_description' );
		$audio_player_position = $this->spp_get_option( 'audio_player_position' );
		$transcript_position   = $this->spp_get_option( 'transcript_position' );
		$table_spp_podcast     = $wpdb->prefix . "spp_podcast";
		$counterposts          = 0;
		$podcast_URL           = $this->spp_get_option( 'podcast_url' ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		$spp_channel_image     = $this->spp_get_option( 'channel_image' );
		$this->spp_save_podcast_xml( $podcast_URL, true ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		$spp_import_select = $this->spp_get_option( 'spp_import_select' );
		$ep_specific_date  = strtotime( $this->spp_get_option( 'ep_specific_date' ) );
		$timeoffset        = $this->get_unix_timeoffset();

		include_once ABSPATH . 'wp-admin/includes/image.php';

		$starttime = strtotime( 'now' );

		if ( $wpdb->get_var( "SHOW TABLES LIKE '{$table_spp_podcast}'" ) != $table_spp_podcast ) { // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
			return;
		}
		$resultresponse = isset( $_REQUEST['resultresponse'] ) ? intval( $_REQUEST['resultresponse'] ) : 0; // phpcs:ignore WordPress.Security.NonceVerification.Recommended,WordPress.Security.NonceVerification.Recommended
		$totalvids      = $wpdb->get_var( 'SELECT COUNT(*) FROM ' . $table_spp_podcast ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
		if ( $totalvids > 0 ) {
			$podcast_items = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$table_spp_podcast} LIMIT %d,%d", $resultresponse, $totalvids ) ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching,WordPress.DB.PreparedSQL.InterpolatedNotPrepared		
		}

		// Additional check for funny hosts like Anchor and Omny.
		$parseurl = wp_parse_url( $podcast_URL );  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		$omniHost = false;  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		if ( strpos( $podcast_URL, 'omny' ) ) {  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$omniHost = true;  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		}

		$acast = false;
		if ( strpos( $podcast_URL, 'acast' ) ) {  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$acast = true;
		}

		$captivatefm = false;
		if ( strpos( $podcast_URL, 'captivate.fm' ) ) {  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$captivatefm = true;
		}

		$redCircle = false;  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		if ( ( strpos( $podcast_URL, 'podcache' ) ) || ( strpos( $podcast_URL, 'redcircle' ) ) ) {  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$redCircle = true;  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
		}

		$soundon = false;
		if ( strpos( $podcast_URL, 'soundon.fm' ) ) {  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$soundon = true;
		}

		$simplecast = false;
		if ( strpos( $podcast_URL, 'simplecast.com' ) ) {  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$simplecast = true;
		}

		foreach ( $podcast_items as $podcast_item ) {

			$publishedate = strtotime( $podcast_item->pc_published_date ) - $timeoffset;

			// Add debugging statements
			if ( (2 == (int)$spp_import_select) ) {
				if ($publishedate <= $ep_specific_date) {
					continue;
				}
			}
			if ( (3 == (int)$spp_import_select) ) {
				if ($publishedate >= $ep_specific_date) {
					continue;
				}
			}
			$audio_duration = $podcast_item->pc_audio_duration;
			$audio_file     = $podcast_item->pc_audio_file;

			// Anchor specific check to avoid duplicates.
			if ( 'anchor.fm' === $parseurl['host'] ) {
				$a               = wp_parse_url( $audio_file );
				$p               = $a['path'];
				$pa              = explode( '/', $p );
				$base_audio_file = $pa[5];
			} elseif ( $omniHost ) {  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				// Specific case for Omny podcast host.
				$stripped_audio_file = substr( $audio_file, strpos( $audio_file, 'omny.fm' ) );
				if ( strpos( $stripped_audio_file, '?' ) ) {
					$base_audio_file = substr( $stripped_audio_file, 0, strpos( $stripped_audio_file, '?' ) );
				} else {
					$base_audio_file = $stripped_audio_file;
				}
			} elseif ( $acast ) {
				// Specific case for Acast podcast host.
				$stripped_audio_file = substr( $audio_file, strpos( $audio_file, 'acast.com' ) );
				if ( strpos( $stripped_audio_file, '?' ) ) {
					$base_audio_file = substr( $stripped_audio_file, 0, strpos( $stripped_audio_file, '?' ) );
				} else {
					$base_audio_file = $stripped_audio_file;
				}
			} elseif ( $captivatefm ) {
				// Specific case for Captivate.fm podcast host.
				$stripped_audio_file = substr( $audio_file, strpos( $audio_file, 'captivate.fm' ) );
				if ( strpos( $stripped_audio_file, '?' ) ) {
					$base_audio_file = substr( $stripped_audio_file, 0, strpos( $stripped_audio_file, '?' ) );
				} else {
					$base_audio_file = $stripped_audio_file;
				}
			} elseif ( $simplecast ) {
				// Specific case for simplecast podcast host.
				$stripped_audio_file = substr( $audio_file, strpos( $audio_file, 'simplecast.com' ) );
				if ( strpos( $stripped_audio_file, '?' ) ) {
					$base_audio_file = substr( $stripped_audio_file, 0, strpos( $stripped_audio_file, '?' ) );
				} else {
					$base_audio_file = $stripped_audio_file;
				}
			} elseif ( $soundon ) {
				// Specific case for Soundon podcast host.
				$stripped_audio_file = substr( $audio_file, strpos( $audio_file, 'soundon.fm' ) );
				if ( strpos( $stripped_audio_file, '?' ) ) {
					$base_audio_file = substr( $stripped_audio_file, 0, strpos( $stripped_audio_file, '?' ) );
				} else {
					$base_audio_file = $stripped_audio_file;
				}
			} elseif ( $redCircle ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				// Specific case for RedCircle podcast host.
				$stripped_audio_file = substr( $audio_file, strpos( $audio_file, 'redcircle.com' ) );
				if ( strpos( $stripped_audio_file, '?' ) ) {
					$base_audio_file = substr( $stripped_audio_file, 0, strpos( $stripped_audio_file, '?' ) );
				} else {
					$base_audio_file = $stripped_audio_file;
				}
			} else {
				// Rest of podcast hosts.
				$stripped_audio_file = wp_parse_url( $audio_file );
				$base_audio_file     = $stripped_audio_file['path'];
			}

			$post_meta = $wpdb->get_row( 'SELECT * FROM ' . $wpdb->prefix . "postmeta WHERE meta_value LIKE '%" . $wpdb->_real_escape( $base_audio_file ) . "%' " ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching,WordPress.DB.PreparedSQL.NotPrepared
			if ( ! empty( $post_meta->meta_value ) ) {
					continue;
			}
			$pcTitle = htmlspecialchars_decode( $podcast_item->pc_title ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

			if ( 0 === $spp_description ) {
				$pc_description = $podcast_item->pc_description;
				$pc_description = $this->remove_evil_styles_v2( $pc_description );
				$pc_description = make_clickable( $pc_description );
			} //end if
			$spp_description = htmlspecialchars_decode( $spp_description );
			$audioUrl        = $podcast_item->pc_audio_file; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

			switch ( $ep_art_select ) {
				case 1:
					$thumbnail = $podcast_item->pc_episode_image;
					break;
				case 2:
					$thumbnail = $podcast_item->pc_libsyn_image;
					break;
				case 3:
					$thumbnail = $spp_channel_image;
					break;
			}
			$tags = explode( ',', $podcast_item->pc_episode_keywords );

			$publish_date = gmdate( 'Y-m-d H:i:s', $publishedate );

			if ( $auto_post ) {
				$my_post = array(
					'post_title'    => $pcTitle, // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					'post_content'  => $pc_description,
					'post_date'     => $publish_date,
					'post_status'   => 'publish',
					'post_category' => array(
						$category,
					),
					'post_author'   => $author,
					'post_type'     => $cpt_select,
					'tags_input'    => $tags,
				);
			} else {
				$my_post = array(
					'post_title'    => $pcTitle, // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					'post_content'  => $pc_description,
					'post_date'     => $publish_date,
					'post_status'   => 'draft',
					'post_category' => array(
						$category,
					),
					'post_author'   => $author,
					'post_type'     => $cpt_select,
					'tags_input'    => $tags,
				);
			}
			// Insert the post into the database.
			$post_id = wp_insert_post( $my_post );
			// fix The URL shortener post meta section is always empty when SPP creates a post (either Draft or Published).  It only gets populated when you click "Update", "Save Draft", "Publish" -  Please have it auto set the post meta automatically during auto post creation.
			$this->spp_link_metabox_save( $post_id, true );

			set_post_format( $post_id, $spp_post_format );

			add_post_meta( $post_id, '_audiourl', $audioUrl, true ) || update_post_meta( $post_id, '_audiourl', $audioUrl ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

			if ( ! $disable_url_shortner ) {
				$permellink      = get_permalink( $post_id );
				$table_spp_links = $wpdb->prefix . 'spp_links';

				$pcTitle   = html_entity_decode( get_the_title( $post_id ) );  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$slugarray = preg_split( '/[\s,]+/', $pcTitle ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				foreach ( $slugarray as $slugelement ) {
					$episode_number = preg_replace( '~[^0-9]~', '', $slugelement );
					$episode_number = intval( $episode_number );
					if ( 0 !== $episode_number ) {
						break;
					}
				}

				$slug         = $episode_short_link . $episode_number;
				$is_available = $this->spp_slugIsAvailable( $slug );

				$data = array();

				if ( $is_available ) {
					$data = array(

						'spp_name'    => $pcTitle,  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
						'spp_url'     => $permellink,
						'spp_slug'    => $slug,
						'spp_post_id' => $post_id,

					);

					$slugAlreadyInDB = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$table_spp_links} WHERE spp_slug = %s", $slug ) );  // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching

					if ( $slugAlreadyInDB ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
						$wpdb->update( // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
							$table_spp_links,
							$data,
							array(
								'spp_slug' => $slug,
							)
						);
					} else {
						$wpdb->insert( $table_spp_links, $data ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
					}
				}
			} //end if
			add_post_meta( $post_id, '_audioduration', $audio_duration, true ) || update_post_meta( $post_id, '_audioduration', $audio_duration );
			++$counterposts;

			add_post_meta( $post_id, '_release_date', $publish_date, true ) || update_post_meta( $post_id, '_release_date', $publish_date );

			// Set FB Open Graph Image.  Overright with thumbnail below if it exists.
			$fbImage = $this->spp_get_option( 'channel_image' ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

			if ( ! empty( $thumbnail ) && ! $this->spp_image_exist( $thumbnail ) ) {

				// Set feature image for post.
				$upload_dir = wp_upload_dir();
				$image_data = $this->url_get_contents( $thumbnail );
				if ( ( ! $image_data ) || ( null === $image_data ) ) {
					// Get the default black image if we can't access the artwork for some reason.
					$thumbnail  = SPPRESS_PLUGIN_URL . '/responsive_audio_player/image/noartwork.png';
					$image_data = $this->url_get_contents( $thumbnail );

				}

				$image_array = explode( '.', $thumbnail );
				$ext         = end( $image_array );
				if ( ( 'png' !== $ext ) || ( 'jpg' !== $ext ) || ( null === $ext ) ) {
					$ext = 'png';
				}
				unset( $image_array );

				$pcTitleWithDashes = preg_replace( '/\%/', ' percentage', $pcTitle ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$pcTitleWithDashes = preg_replace( '/\@/', ' at ', $pcTitleWithDashes ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$pcTitleWithDashes = preg_replace( '/\&/', ' and ', $pcTitleWithDashes ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$pcTitleWithDashes = preg_replace( '/\]/', ' - ', $pcTitleWithDashes ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$pcTitleWithDashes = preg_replace( '/\[/', ' - ', $pcTitleWithDashes ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$pcTitleWithDashes = preg_replace( '/\+/', ' - ', $pcTitleWithDashes ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$pcTitleWithDashes = preg_replace( '/\s[\s]+/', '-', $pcTitleWithDashes ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

				// Strip off multiple spaces.
				$pcTitleWithDashes = preg_replace( '/[\s\W]+/', '-', $pcTitleWithDashes ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

				// Strip off spaces and non-alpha-numeric.
				$pcTitleWithDashes = preg_replace( '/^[\-]+/', '', $pcTitleWithDashes ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

				// Strip off the starting hyphens.
				$pcTitleWithDashes = preg_replace( '/[\-]+$/', '', $pcTitleWithDashes ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

				// Strip off the ending hyphens.
				$pcTitleWithDashes = strtolower( $pcTitleWithDashes ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$new_file          = $pcTitleWithDashes . '_thumbnail.' . $ext; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				// If Libsyn Image Import then always add PNG extension.
				if ( 2 === $ep_art_select ) {
					$new_file = $pcTitleWithDashes . '_thumbnail.png'; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				}
				$filename = basename( $new_file );

				if ( wp_mkdir_p( $upload_dir['path'] ) ) {
					$file = $upload_dir['path'] . '/' . $filename;
				} else {
					$file = $upload_dir['basedir'] . '/' . $filename;
				}
				$spp_channelart_upload = file_put_contents( $file, $image_data ); // phpcs:ignore WordPress.WP.AlternativeFunctions.file_system_operations_file_put_contents
				if ( ! $spp_channelart_upload ) {
					global $wp_filesystem;
					// protect if the the global filesystem isn't setup yet.
					if ( is_null( $wp_filesystem ) ) {
						WP_Filesystem();
					}
					$wp_filesystem->put_contents( $file, $image_data, FS_CHMOD_FILE );
				}

				$wp_filetype = wp_check_filetype( $filename, null );
				$attachment  = array(
					'post_mime_type' => $wp_filetype['type'],
					'post_title'     => sanitize_file_name( $filename ),
					'post_content'   => '',
					'post_status'    => 'inherit',
				);
				$attach_id   = wp_insert_attachment( $attachment, $file, $post_id );
				include_once ABSPATH . 'wp-admin/includes/image.php';
				$uploaded_file = get_attached_file( $attach_id );

				// Get editor, resize and overwrite file.
				$image_editor = wp_get_image_editor( $uploaded_file );
				$image_editor->resize( 640, 640 );
				$saved = $image_editor->save( $uploaded_file );

				// Improve speed by removing and running later.
				$attach_data = wp_generate_attachment_metadata( $attach_id, $file );
				wp_update_attachment_metadata( $attach_id, $attach_data );

				set_post_thumbnail( $post_id, $attach_id );
				// End of setting feature image.

				$fbImage = $thumbnail; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

				$attach_data = null;
				$image_data  = null;
				unset( $attachment );
			} else {
				$attachment = $this->spp_image_exist( $thumbnail );
				set_post_thumbnail( $post_id, $attachment );
			}

			$sppogp   = get_post_meta( $post_id, 'OGP', true );
			$ogptitle = get_the_title( $post_id );
			$ogpimage = $fbImage; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$ogparr   = array(
				'title' => $ogptitle,
				'image' => $ogpimage,
			);

			update_post_meta( $post_id, 'OGP', $ogparr );
			apply_filters( 'spp_post_insert', $post_id, $audioUrl ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			// End of Setting Facebook Image Post-specific OpenGraph Tags.
			if ( ! defined( 'DOING_CRON' ) ) {
				$endtime = strtotime( 'now' );
				if ( $endtime - $starttime >= 10 ) {
					echo esc_html( $counterposts );
					die();
				}
			}
		} //end foreach

		echo esc_html( $counterposts );
		die();
	} //end function
	/**
	 * Check if an image exists in the WordPress media library.
	 *
	 * @param string $image_url The URL of the image to check.
	 * @return int|false The attachment ID if image exists, false otherwise.
	 */
	public function spp_image_exist( $image_url ) {
		global $wpdb;

		// Use WordPress function to convert image URL to attachment ID.
		$attach_id = attachment_url_to_postid( $image_url );

		// Check if attachment ID is not empty.
		if ( ! empty( $attach_id ) ) {
			return $attach_id;
		} else {
			return false;
		}
	}

	/**
	 * Panel function for handling SPP settings.
	 */
	public function spp_pannel() {
		// Retrieve poweredby URL from options.
		$refID = $this->spp_get_option( 'spp_poweredby_url' ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

		// Trigger 'load_spp_settings' action with spp_settings.
		do_action( 'load_spp_settings', $this->spp_settings );

		if ( isset( $_POST['spp_cbsn_reset'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			global $wpdb;

			// Define table names.
			$table_spp_podcast = $wpdb->prefix . 'spp_podcast';
			$table_spp_reviews = $wpdb->prefix . 'spp_reviews';
			$table_spp_episode = $wpdb->prefix . 'spp_episode';

			// SQL queries to drop tables if they exist.
			$sql0 = 'DROP TABLE IF EXISTS ' . $table_spp_podcast;
			$wpdb->query( $sql0 ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
			$sql1 = 'DROP TABLE IF EXISTS ' . $table_spp_reviews;
			$wpdb->query( $sql1 ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
			$sql2 = 'DROP TABLE IF EXISTS ' . $table_spp_episode;
			$wpdb->query( $sql2 ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching

			// Delete spp settings.
			$this->delete_spp_settings();
		}

		// Global $wpdb (WordPress database object).

		// Retrieve various SPP options.
		$spp_auto_publish      = $this->spp_get_option( 'spp_auto_publish' );
		$spp_insert_feat_image = $this->spp_get_option( 'spp_insert_feat_image' );
		$spp_email_on_draft    = $this->spp_get_option( 'spp_email_on_draft' );
		$spp_player_over_image = $this->spp_get_option( 'spp_player_over_image' );

		$spp_description      = $this->spp_get_option( 'spp_description' );
		$spp_category         = $this->spp_get_option( 'spp_post_category' );
		$spp_sync_description = $this->spp_get_option( 'spp_sync_description' );
		$spp_autoplay_podcast = $this->spp_get_option( 'spp_autoplay_podcast' );
		$spp_rewind_button    = $this->spp_get_option( 'spp_rewind_button' );
		$spp_speed_button     = $this->spp_get_option( 'spp_speed_button' );
		$spp_volume_button    = $this->spp_get_option( 'spp_volume_button' );
		$spp_optin_box1       = $this->spp_get_option( 'spp_optin_box' );
		$spp_privacy_page     = $this->spp_get_option( 'spp_privacy_page' );
		$spp_two_step_optin   = $this->spp_get_option( 'spp_two_step_optin' );

		// Generate selected attributes based on $spp_two_step_optin value.
		$spp_two_step_optin_selected1 = ( 1 === $spp_two_step_optin ) ? 'selected=selected' : '';
		$spp_two_step_optin_selected2 = ( 2 === $spp_two_step_optin ) ? 'selected=selected' : '';
		$spp_two_step_optin_selected3 = ( 3 === $spp_two_step_optin ) ? 'selected=selected' : '';
		$spp_two_step_optin_selected4 = ( 4 === $spp_two_step_optin ) ? 'selected=selected' : '';

		// Conditional display style for privacy policy text.
		$privacy_policy_text_display = '';
		if ( 'disabled' === $spp_privacy_page ) {
			$privacy_policy_text_display = 'display:none;';
		}

		// start custom post type dropdown.
		$post_types                 = get_post_types(
			array(
				'_builtin' => false,
				'public'   => true,
			)
		);
			$spp_cpt_select         = $this->spp_get_option( 'spp_cpt_select' );
			$spp_post_format_select = $this->spp_get_option( 'spp_post_format_select' );

			$spp_cpt_dropdown = '<select name="spp_cpt_select" id="spp_cpt_select">';
		if ( 'post' === $spp_cpt_select || empty( $spp_cpt_select ) ) {
			$spp_cpt_dropdown .= "<option value='post' selected='selected'>Standard Post (Default)</option>";
		} else {
			$spp_cpt_dropdown .= "<option value='post'>Standard Post (Default)</option>";
		}

		foreach ( $post_types as $post_type ) {
			if ( $spp_cpt_select === $post_type ) {

				$spp_cpt_dropdown .= "<option value='" . esc_html( $post_type ) . "' selected='selected'>" . esc_html( $post_type ) . '</option>';

			} else {
				$spp_cpt_dropdown .= "<option value='" . esc_html( $post_type ) . "'>" . esc_html( $post_type ) . '</option>';

			}
		}
			$spp_cpt_dropdown .= '</select>';
			$podcast_xml       = $this->spp_get_option( 'podcast_url' );
			$btn_itunes_url    = $this->spp_get_option( 'btn_itunes_url' );
			$is_license_valid  = get_option( 'sppress_ls' );

		if ( ( 'valid' === $is_license_valid ) || ( 'trial' === $is_license_valid ) ) {
			if ( empty( $podcast_xml ) ) {
				$html  = '
        <div class="container-fluid">                    
              <div class="well well-1">
        <h2>Simple Podcast Press</h2>
        <br class="clear" />';
				$html .= '                        
                                <p><b>' . __( 'Enter Apple Podcast Url:&nbsp; &nbsp;&nbsp;&nbsp;&nbsp;', 'spp' ) . ' </b><input class="span5" type="text" name="podcast_url" value="" size="61" placeholder="https://podcasts.apple.com/us/podcast/eofire-with-john-lee-dumas/id564001633?mt=2"><br>*Highly recommended especially if you would like to see your world-wide Apple Podcast reviews<br><br><p>OR
                                <p><b>' . __( 'Enter Podcast Feed:&nbsp;', 'spp' ) . ' </b><input class="span5" type="text" name="podcast_xml" value="" size="61" placeholder="This must be a valid podcast feed"></p>
                                &nbsp;&nbsp;
                                <p><b>Don\'t have a podcast feed yet? Leave the fields above blank and click the Continue button.</b></p>
                                <div style="margin-top: 25px;margin-bottom:15px">
                                <table>
                                <tr><td>
                                <input type="submit" id="podcast_url" class="btn btn-primary" name="save_podcast_url" value="' . __( 'Continue', 'spp' ) . '"/>
                                <input style="display:none;" type="submit" id="delete_settings" class="button button-secondary" name="delete_settings" value="' . __( 'Delete', 'spp' ) . '"/>
                                <input type="submit" class="btn btn-primary" name="cbsn_save" id="cbsn_save" value="Save Changes" style="display:none;">
                                <input type="submit" class="btn" name="chk_new_vids" id="chk_new_vids" value="Import New Episodes" style="display:none;">
                                <input type="submit" name="manual_update" class="btn" id="fetch_comments" value="Import New Comments Now" style="display:none;">
                                <input type="submit" name="cancel_save_podcast" class="btn" id="cancel_save_podcast" value="Cancel" style="display:none; color:red;">

                                </td><td class="save_access_spinning"></td>
                                </tr>
                                </table>
                                <div id="save_podcast_message" class="alert alert-1" style="display:none; margin-top:20px;"><button type="button" class="btn close">x</button>Analyzing Podcast Feed.</div></span>
                                </div>
                                <strong>Note: Depending on how many episodes and reviews you have, it may take several minutes to analyze your Podcast Channel.</strong>
                        </div></div>
                        ';
			} else {

				$powerpressplayer        = '';
				$simplepodcastpressblack = '';
				$spp_disable_thumbs      = $this->spp_get_option( 'spp_disable_thumbs' );
				$spp_auto_publish1       = ( 1 === $spp_auto_publish ) ? 'checked' : '';
				$spp_insert_feat_image1  = ( 1 === $spp_insert_feat_image ) ? 'checked' : '';

				$disable_url_shortner = $this->spp_get_option( 'disable_url_shortner' );
				$disable_url_shortner = ( 1 === $disable_url_shortner ) ? 'checked' : '';

				$spp_force_player_home      = $this->spp_get_option( 'spp_force_player_home' );
				$spp_force_player_home      = ( 1 === $spp_force_player_home ) ? 'checked' : '';
				$spp_disable_auto_timestamp = $this->spp_get_option( 'spp_disable_auto_timestamp' );
				$spp_disable_auto_timestamp = ( 1 === $spp_disable_auto_timestamp ) ? 'checked' : '';

				$spp_remove_brackets_timestamp = $this->spp_get_option( 'spp_remove_brackets_timestamp' );
				$spp_remove_brackets_timestamp = ( 1 === $spp_remove_brackets_timestamp ) ? 'checked' : '';

				$spp_resolve_player_conflict = $this->spp_get_option( 'spp_resolve_player_conflict' );
				$spp_resolve_player_conflict = ( 1 === $spp_resolve_player_conflict ) ? 'checked' : '';

				$force_ssl_for_audio = $this->spp_get_option( 'force_ssl_for_audio' );
				$force_ssl_for_audio = ( 1 === $force_ssl_for_audio ) ? 'checked' : '';

				$disable_opengraph     = $this->spp_get_option( 'disable_opengraph' );
				$disable_opengraph     = ( 1 === $disable_opengraph ) ? 'checked' : '';
				$spp_draft_row_display = ( 1 === $spp_auto_publish ) ? 'display:none;' : '';
				$spp_email_on_draft    = ( 1 === $spp_email_on_draft ) ? 'checked' : '';
				$spp_player_over_image = ( 1 === $spp_player_over_image ) ? 'checked' : '';
				$spp_description       = ( 1 === $spp_description ) ? 'checked' : '';

				$replace_pp_with_spp          = $this->spp_get_option( 'replace_pp_with_spp' );
				$replace_pp_with_spp          = ( 1 === $replace_pp_with_spp ) ? 'checked' : '';
				$spp_replace_with_spp_display = ( 0 === $replace_pp_with_spp ) ? 'display:none;' : '';
				$replace_ppplayer_with_spp    = $this->spp_get_option( 'replace_ppplayer_with_spp' );
				$replace_ppplayer_with_spp    = ( 1 === $replace_ppplayer_with_spp ) ? 'checked' : '';
				$replace_wp_with_spp          = $this->spp_get_option( 'replace_wp_with_spp' );
				$replace_wp_with_spp          = ( 1 === $replace_wp_with_spp ) ? 'checked' : '';
				$replace_stp_with_spp         = $this->spp_get_option( 'replace_stp_with_spp' );
				$replace_stp_with_spp         = ( 1 === $replace_stp_with_spp ) ? 'checked' : '';
				$spp_autoplay_podcast         = ( 1 === $spp_autoplay_podcast ) ? 'checked' : '';
				$spp_rewind_button            = ( 1 === $spp_rewind_button ) ? 'checked' : '';
				$spp_speed_button             = ( 1 === $spp_speed_button ) ? 'checked' : '';
				$spp_volume_button            = ( 1 === $spp_volume_button ) ? 'checked' : '';

				$spp_sync_description = ( 1 === $spp_sync_description ) ? 'checked' : '';
				$spp_disable_thumbs   = ( 1 === $spp_disable_thumbs ) ? 'checked' : '';

				$btn_g_play = $this->spp_get_option( 'btn_g_play' );
				$btn_g_play = ( 1 === $btn_g_play ) ? 'checked' : '';

				$btn_share = $this->spp_get_option( 'btn_share' );
				$btn_share = ( 1 === $btn_share ) ? 'checked' : '';

				$btn_download = $this->spp_get_option( 'btn_download' );
				$btn_download = ( 1 === $btn_download ) ? 'checked' : '';

				$btn_spplisten = $this->spp_get_option( 'btn_spplisten' );
				$btn_spplisten = ( 1 === $btn_spplisten ) ? 'checked' : '';
				$btn_spprss    = $this->spp_get_option( 'btn_spprss' );
				$btn_spprss    = ( 1 === $btn_spprss ) ? 'checked' : '';
				$btn_sppreview = $this->spp_get_option( 'btn_sppreview' );
				$btn_sppreview = ( 1 === $btn_sppreview ) ? 'checked' : '';

				$btn_sppandroid = $this->spp_get_option( 'btn_sppandroid' );
				$btn_sppandroid = ( 1 === $btn_sppandroid ) ? 'checked' : '';

				$btn_itunes     = $this->spp_get_option( 'btn_itunes' );
				$btn_itunes     = ( 1 === $btn_itunes ) ? 'checked' : '';
				$btn_itunes_url = $this->spp_get_option( 'btn_itunes_url' );

				$btn_embed = $this->spp_get_option( 'btn_embed' );
				$btn_embed = ( 1 === $btn_embed ) ? 'checked' : '';

				$btn_stiticher = $this->spp_get_option( 'btn_stiticher' );
				$btn_stiticher = ( 1 === $btn_stiticher ) ? 'checked' : '';

				$spp_optin_hide_chk = $this->spp_get_option( 'spp_optin_hide_chk' );
				$spp_optin_hide_chk = ( 1 === $spp_optin_hide_chk ) ? 'checked' : '';

				$spp_disable_poweredby = $this->spp_get_option( 'spp_disable_poweredby' );
				$spp_disable_poweredby = ( 1 === $spp_disable_poweredby ) ? 'checked' : '';

				$spp_pre_roll_url              = $this->spp_get_option( 'spp_pre_roll_url' );
				$spp_pre_roll_checkbox         = $this->spp_get_option( 'spp_pre_roll_checkbox' );
				$spp_pre_roll_checkbox         = ( 1 === $spp_pre_roll_checkbox ) ? 'checked' : '';
				$spp_disable_spp_player_script = $this->spp_get_option( 'spp_disable_spp_player_script' );
				$spp_disable_spp_player_script = ( 1 === $spp_disable_spp_player_script ) ? 'checked' : '';
				$disable_jquery_model          = $this->spp_get_option( 'disable_jquery_model' );
				$disable_jquery_model          = ( 1 === $disable_jquery_model ) ? 'checked' : '';

				$direct_download_button = $this->spp_get_option( 'direct_download_button' );
				$direct_download_button = ( 1 === $direct_download_button ) ? 'checked' : '';
				$spp_import_html_desc   = $this->spp_get_option( 'spp_import_html_desc' );
				$spp_import_html_desc   = ( 1 === $spp_import_html_desc ) ? 'checked' : '';
				$btn_soundcloud         = $this->spp_get_option( 'btn_soundcloud' );
				$btn_soundcloud         = ( 1 === $btn_soundcloud ) ? 'checked' : '';
				$btn_spotify            = $this->spp_get_option( 'btn_spotify' );
				$btn_spotify            = ( 1 === $btn_spotify ) ? 'checked' : '';
				$btn_style_round        = $this->spp_get_option( 'btn_style_round' );
				$btn_style_round        = ( 1 === $btn_style_round ) ? 'checked' : '';
				$spp_remove_timecode    = $this->spp_get_option( 'spp_remove_timecode' );
				$spp_remove_timecode    = ( 1 === $spp_remove_timecode ) ? 'checked' : '';
				$spp_flat_player        = $this->spp_get_option( 'spp_flat_player' );
				$spp_flat_player        = ( 1 === $spp_flat_player ) ? 'checked' : '';
				$disable_player_excerpt = $this->spp_get_option( 'spp_hide_player_from_excerpt' );
				$disable_player_excerpt = ( 1 === $disable_player_excerpt ) ? 'checked' : '';
				$hide_elements_athome   = $this->spp_get_option( 'hide_elements_athome' );
				$hide_elements_athome   = ( 1 === $hide_elements_athome ) ? 'checked' : '';
				$off_scrolling_text     = $this->spp_get_option( 'off_scrolling_text' );
				$off_scrolling_text     = ( 1 === $off_scrolling_text ) ? 'checked' : '';
				$transcript             = $this->spp_get_option( 'transcript' );
				$transcript             = ( 1 === $transcript ) ? 'checked' : '';
				$player_bar_color       = $this->spp_get_option( 'player_bar_color' );
				$progress_bar_color     = $this->spp_get_option( 'progress_bar_color' );
				$player_text_color      = $this->spp_get_option( 'player_text_color' );
				$twitter_text_color     = $this->spp_get_option( 'twitter_text_color' );

				$privacy_policy_text = $this->spp_get_option( 'privacy_policy_text' );
				$privacy_policy_text = ! empty( $privacy_policy_text ) ? $privacy_policy_text : 'I have read and agreed to your Privacy Policy';
				$headline_text_color = $this->spp_get_option( 'headline_text_color' );
				$submit_button_color = $this->spp_get_option( 'submit_button_color' );
				$submit_button_text  = $this->spp_get_option( 'submit_button_text' );
				$opt_container_color = $this->spp_get_option( 'opt_container_color' );
				$btn_g_play_name     = $this->spp_get_option( 'btn_g_play_name' );
				$btn_share_name      = $this->spp_get_option( 'btn_share_name' );
				$btn_download_name   = $this->spp_get_option( 'btn_download_name' );
				$btn_spplisten_name  = $this->spp_get_option( 'btn_spplisten_name' );
				$btn_sppreview_name  = $this->spp_get_option( 'btn_sppreview_name' );
				$btn_spprss_name     = $this->spp_get_option( 'btn_spprss_name' );
				$btn_sppandroid_name = $this->spp_get_option( 'btn_sppandroid_name' );
				$btn_itunes_name     = $this->spp_get_option( 'btn_itunes_name' );
				$btn_embed_name      = $this->spp_get_option( 'btn_embed_name' );
				$btn_stitcher_name   = $this->spp_get_option( 'btn_stitcher_name' );
				$btn_soundcloud_name = $this->spp_get_option( 'btn_soundcloud_name' );
				$btn_spotify_name    = $this->spp_get_option( 'btn_spotify_name' );

				$btn_g_play_url       = $this->spp_get_option( 'btn_g_play_url' );
				$btn_g_play_color     = $this->spp_get_option( 'btn_g_play_color' );
				$btn_share_color      = $this->spp_get_option( 'btn_share_color' );
				$btn_download_color   = $this->spp_get_option( 'btn_download_color' );
				$btn_spplisten_color  = $this->spp_get_option( 'btn_spplisten_color' );
				$btn_spprss_color     = $this->spp_get_option( 'btn_spprss_color' );
				$btn_sppreview_color  = $this->spp_get_option( 'btn_sppreview_color' );
				$btn_sppandroid_color = $this->spp_get_option( 'btn_sppandroid_color' );
				$btn_itunes_color     = $this->spp_get_option( 'btn_itunes_color' );
				$btn_embed_color      = $this->spp_get_option( 'btn_embed_color' );
				$btn_stiticher_color  = $this->spp_get_option( 'btn_stiticher_color' );
				$btn_soundcloud_color = $this->spp_get_option( 'btn_soundcloud_color' );
				$btn_spotify_color    = $this->spp_get_option( 'btn_spotify_color' );
				$transcript_txt       = $this->spp_get_option( 'transcript_txt' );
				if ( empty( $transcript_txt ) ) {
					$transcript_txt = 'Read Full Transcript';
				}
				$spp_listen_text = $this->spp_get_option( 'spp_listen_text' );

				$transcript_color     = $this->spp_get_option( 'transcript_color' );
				$btn_stiticher_url    = $this->spp_get_option( 'btn_stiticher_url' );
				$spp_poweredby_url    = $this->spp_get_option( 'spp_poweredby_url' );
				$spp_optshow_xseconds = $this->spp_get_option( 'spp_optshow_xseconds' );
				$btn_soundcloud_url   = $this->spp_get_option( 'btn_soundcloud_url' );
				$btn_spotify_url      = $this->spp_get_option( 'btn_spotify_url' );

				$spp_optin_box1                = $this->spp_get_option( 'spp_optin_box' );
				$spp_optin_box                 = ( 1 === $spp_optin_box1 ) ? 'checked' : '';
				$spp_optin_row_display         = ( 0 === $spp_optin_box1 ) ? '' : '';
				$optin_old_code_get            = $this->spp_get_option( 'optin_old_code' );
				$spp_auto_resp_url_get         = $this->spp_get_option( 'spp_auto_resp_url' );
				$spp_auto_resp_heading_get     = $this->spp_get_option( 'spp_auto_resp_heading' );
				$spp_auto_resp_sub_heading_get = $this->spp_get_option( 'spp_auto_resp_sub_heading' );
				$spp_auto_resp_hidden_get      = $this->spp_get_option( 'spp_auto_resp_hidden' );
				$spp_auto_resp_name_get        = $this->spp_get_option( 'spp_auto_resp_name' );
				$spp_auto_resp_email_get       = $this->spp_get_option( 'spp_auto_resp_email' );
				$spp_auto_resp_submitt_get     = $this->spp_get_option( 'spp_auto_resp_submitt' );
				$spptwitterhandle              = $this->spp_get_option( 'spp-twitter-handle' );
				$episode_short_link            = $this->spp_get_option( 'episode_short_link' );

				$btn_spp_custom1          = $this->spp_get_option( 'btn_spp_custom1' );
				$btn_spp_custom1          = ( 1 === $btn_spp_custom1 ) ? 'checked' : '';
				$btn_spp_custom2          = $this->spp_get_option( 'btn_spp_custom2' );
				$btn_spp_custom2          = ( 1 === $btn_spp_custom2 ) ? 'checked' : '';
				$btn_spp_custom3          = $this->spp_get_option( 'btn_spp_custom3' );
				$btn_spp_custom3          = ( 1 === $btn_spp_custom3 ) ? 'checked' : '';
				$btn_spp_custom4          = $this->spp_get_option( 'btn_spp_custom4' );
				$btn_spp_custom4          = ( 1 === $btn_spp_custom4 ) ? 'checked' : '';
				$btn_spp_custom5          = $this->spp_get_option( 'btn_spp_custom5' );
				$btn_spp_custom5          = ( 1 === $btn_spp_custom5 ) ? 'checked' : '';
				$btn_spp_custom6          = $this->spp_get_option( 'btn_spp_custom6' );
				$btn_spp_custom6          = ( 1 === $btn_spp_custom6 ) ? 'checked' : '';
				$spp_LeadBox_btn_checkbox = $this->spp_get_option( 'spp_LeadBox_btn_checkbox' ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$spp_LeadBox_btn_checkbox = ( 1 === $spp_LeadBox_btn_checkbox ) ? 'checked' : ''; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

				$btn_spp_custom_color1 = $this->spp_get_option( 'btn_spp_custom_color1' );
				$btn_spp_custom_color2 = $this->spp_get_option( 'btn_spp_custom_color2' );
				$btn_spp_custom_color3 = $this->spp_get_option( 'btn_spp_custom_color3' );
				$btn_spp_custom_color4 = $this->spp_get_option( 'btn_spp_custom_color4' );
				$btn_spp_custom_color5 = $this->spp_get_option( 'btn_spp_custom_color5' );
				$btn_spp_custom_color6 = $this->spp_get_option( 'btn_spp_custom_color6' );

				$custom_button_icon1 = $this->spp_get_option( 'custom_button_icon1' );
				$custom_button_icon2 = $this->spp_get_option( 'custom_button_icon2' );
				$custom_button_icon3 = $this->spp_get_option( 'custom_button_icon3' );
				$custom_button_icon4 = $this->spp_get_option( 'custom_button_icon4' );
				$custom_button_icon5 = $this->spp_get_option( 'custom_button_icon5' );
				$custom_button_icon6 = $this->spp_get_option( 'custom_button_icon6' );

				$spp_LeadBox_btn_color = $this->spp_get_option( 'spp_LeadBox_btn_color' ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

				$btn_spp_custom_url1  = $this->spp_get_option( 'btn_spp_custom_url1' );
				$btn_spp_custom_url2  = $this->spp_get_option( 'btn_spp_custom_url2' );
				$btn_spp_custom_url3  = $this->spp_get_option( 'btn_spp_custom_url3' );
				$btn_spp_custom_url4  = $this->spp_get_option( 'btn_spp_custom_url4' );
				$btn_spp_custom_url5  = $this->spp_get_option( 'btn_spp_custom_url5' );
				$btn_spp_custom_url6  = $this->spp_get_option( 'btn_spp_custom_url6' );
				$spp_LeadBox_btn_code = $this->spp_get_option( 'spp_LeadBox_btn_code' ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

				$btn_spp_custom_name1 = $this->spp_get_option( 'btn_spp_custom_name1' );
				$btn_spp_custom_name2 = $this->spp_get_option( 'btn_spp_custom_name2' );
				$btn_spp_custom_name3 = $this->spp_get_option( 'btn_spp_custom_name3' );
				$btn_spp_custom_name4 = $this->spp_get_option( 'btn_spp_custom_name4' );
				$btn_spp_custom_name5 = $this->spp_get_option( 'btn_spp_custom_name5' );
				$btn_spp_custom_name6 = $this->spp_get_option( 'btn_spp_custom_name6' );

				$ep_art_select     = $this->spp_get_option( 'ep_art_select' );
				$ep_art_select1    = ( 0 == $ep_art_select ) ? 'selected=selected' : '';
				$ep_art_select2    = ( 1 == $ep_art_select ) ? 'selected=selected' : '';
				$ep_art_select3    = ( 2 == $ep_art_select ) ? 'selected=selected' : '';
				$ep_art_select4    = ( 3 == $ep_art_select ) ? 'selected=selected' : '';
				$spp_import_select = $this->spp_get_option( 'spp_import_select' );
				$playerbuttoncolor = '';

				$noImportLegacy = $this->spp_get_option( 'spp_no_import' ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				if ( $noImportLegacy ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$spp_import_select                   = 1;
					$this->spp_settings['spp_no_import'] = 0;
					update_option( 'spp_settings', $this->spp_settings );
				}
				$spp_import_selected1 = ( 0 == $spp_import_select ) ? 'selected=selected' : '';
				$spp_import_selected2 = ( 1 == $spp_import_select ) ? 'selected=selected' : '';
				$spp_import_selected3 = ( 2 == $spp_import_select ) ? 'selected=selected' : '';
				$spp_import_selected4 = ( 3 == $spp_import_select ) ? 'selected=selected' : '';

				$spp_imgsize_select    = $this->spp_get_option( 'spp_imgsize_select' );
				$spp_imgsize_selected1 = ( 'thumbnail' == $spp_imgsize_select ) ? 'selected=selected' : '';
				$spp_imgsize_selected2 = ( 'medium' == $spp_imgsize_select ) ? 'selected=selected' : '';
				$spp_imgsize_selected3 = ( 'large' == $spp_imgsize_select ) ? 'selected=selected' : '';
				$spp_imgsize_selected4 = ( 'full' == $spp_imgsize_select ) ? 'selected=selected' : '';

				$spp_imgalign_select    = $this->spp_get_option( 'spp_imgalign_select' );
				$spp_imgalign_selected1 = ( 'alignleft' === $spp_imgalign_select ) ? 'selected=selected' : '';
				$spp_imgalign_selected2 = ( 'aligncenter' === $spp_imgalign_select ) ? 'selected=selected' : '';
				$spp_imgalign_selected3 = ( 'alignright' === $spp_imgalign_select ) ? 'selected=selected' : '';
				$spp_imgalign_selected4 = ( 'alignnone' === $spp_imgalign_select ) ? 'selected=selected' : '';

				$ep_specific_date = $this->spp_get_option( 'ep_specific_date' );
				if ( 2 == $spp_import_select || 3 == $spp_import_select ) {
					$date_textbox_display = 'display:block;';
				} else {
					$date_textbox_display = 'display:none;';
				}
				$audio_player_image  = $this->spp_get_option( 'audio_player_image' );
				$audio_player_image1 = ( 'itunes' === $audio_player_image ) ? 'selected=selected' : '';
				$audio_player_image2 = ( 'featured' === $audio_player_image ) ? 'selected=selected' : '';
				$audio_player_image3 = ( 'disabled' === $audio_player_image ) ? 'selected=selected' : '';

				$audio_player_position  = $this->spp_get_option( 'audio_player_position' );
				$audio_player_position1 = ( 'above' === $audio_player_position ) ? 'selected=selected' : '';
				$audio_player_position2 = ( 'below' === $audio_player_position ) ? 'selected=selected' : '';
				$audio_player_position3 = ( 'disabled' === $audio_player_position ) ? 'selected=selected' : '';

				$select_audio_player_skin = $this->spp_get_option( 'select_audio_player_skin' );
				$audioimagestyle          = ( 'original_player' === $select_audio_player_skin ) ? 'display:none;' : '';
				$progressbarcolor         = ( 'soundmanager2_player' === $select_audio_player_skin ) ? 'display:none;' : '';

				$select_audio_player_skin1 = ( 'original_player' === $select_audio_player_skin ) ? 'selected=selected' : '';
				$select_audio_player_skin3 = ( 'soundmanager2_player' === $select_audio_player_skin ) ? 'selected=selected' : '';

				$player_button_color = $this->spp_get_option( 'player_button_color' );

				$player_button_color1 = ( 'sppwhite' === $player_button_color ) ? 'selected=selected' : '';
				$player_button_color2 = ( 'sppblack' === $player_button_color ) ? 'selected=selected' : '';

				$rewindspeed_button_color = $this->spp_get_option( 'rewindspeed_button_color' );

				$rewindspeed_button_color1 = ( 'sppwhite' === $rewindspeed_button_color ) ? 'selected=selected' : '';
				$rewindspeed_button_color2 = ( 'sppblack' === $rewindspeed_button_color ) ? 'selected=selected' : '';

				$select_audio_player           = $this->spp_get_option( 'select_audio_player' );
				$select_audio_player1          = ( 'powerpressplayer' === $select_audio_player ) ? 'selected=selected' : '';
				$select_audio_player2          = ( 'simplepodcastpressblack' === $select_audio_player ) ? 'selected=selected' : '';
				$select_audio_player3          = ( 'simplepodcastpresswhite' === $select_audio_player ) ? 'selected=selected' : '';
				$select_audio_player4          = ( 'simplepodcastpresscustomcolor' === $select_audio_player ) ? 'selected=selected' : '';
				$sppress_custom_player_display = ( 'simplepodcastpresscustomcolor' === $select_audio_player ) ? 'display:block;' : 'display:none;';

				if ( 'original_player' === $select_audio_player_skin ) {
					$rewindspeed_button_color = 'display:none;';
					$player_button_color      = 'display:none;';
				}
				$transcript_position  = $this->spp_get_option( 'transcript_position' );
				$transcript_position1 = ( 'above' === $transcript_position ) ? 'selected=selected' : '';
				$transcript_position2 = ( 'below' === $transcript_position ) ? 'selected=selected' : '';
				$spp_allcategories    = get_terms(
					array(
						'taxonomy'   => 'category',
						'hide_empty' => 0,
						'orderby'    => 'name',
					)
				);
				$spp_catselechtml     = '<select id="cat" name="spp_cat" class="postform" >';
				foreach ( $spp_allcategories as $term ) {
					$selectedcat = '';
					if ( $spp_category == $term->term_id ) {
						$selectedcat = 'selected=selected';
					}
					$spp_catselechtml .= '<option  value="' . esc_html( $term->term_id ) . '" ' . esc_html( $selectedcat ) . '>' . esc_html( $term->name ) . '</option>';
				}
				$spp_catselechtml .= '</select>';
				$spp_authorlist    = '<select id="spp_author" name="spp_author" class="postform" >';
				$spp_author        = $this->spp_get_option( 'spp_author' );
				$authors           = get_users( array( 'role__in' => array( 'administrator', 'author', 'contributor', 'editor' ) ) );
				$i                 = 0;
				// get all users list.
				foreach ( $authors as $author ) {
					$selectedauthor = '';
					if ( $spp_author === $author->data->ID ) {
						$selectedauthor = 'selected=selected';
					}
					$spp_authorlist .= '<option  value="' . esc_html( $author->data->ID ) . '" ' . esc_html( $selectedauthor ) . '>' . esc_html( $author->data->display_name ) . '</option>';
				}
				$spp_authorlist   .= '</select>';
				$spp_channel_image = $this->spp_get_option( 'channel_image' );
				$container_width   = $this->spp_get_option( 'container_width' );
				$minbutton_width   = $this->spp_get_option( 'minbutton_width' );
				$podcastURL_Feed   = $this->spp_get_option( 'is_third_party_feed' ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$html              = ' 
				<div class="container-fluid">
					<h1></h1>

					<div class="well well-1">
						<div class="row-fluid">
							<span class="span8">
								<div class="row-fluid">
									<div class="spp-logo">
										<!-- <img src="../wp-content/plugins/simple-podcast-press/spp_view/img/spp-logo-lg.png" class="image" style="margin-top:20px;"> -->
									</div>
									<span class="span12">
										<h4 style="background-color:#fceaee;">Want to distribute your Podcast to YouTube, Facebook, LinkedIn, Instagram, and Twitter?<br>Try our latest <a href="https://repurpose.io/?aff=spp" target="_new">content distribution platform for FREE today</a>.</h4>
									</span>
									<span class="span10"><h5>
										Need Help? Check out the <a href="https://www.simplepodcastpress.com/quickstartguide.pdf" target="_new">Quick Start Guide</a> or <a href="https://support.repurpose.io/en/category/simple-podcast-press-d23rlk/" target="_new">Contact Our Support Team</a>.  We are here for you.<br><br>
										Love It? Please share on <a href="https://www.facebook.com/sharer/sharer.php?u=https://simplepodcastpress.com/?ref=' . esc_html( $refID ) . '" target="_blank">Facebook</a> and <a href="https://twitter.com/intent/tweet?text=I%20love%20%23SimplePodcastPress%20a%20%23podcast%20player%20that%20builds%20your%20list%20and%20grows%20your%20audience%20on%20autopilot%20%2D&url=https://simplepodcastpress.com/?ref=' . $refID . '" target="_blank">Twitter</a> (automatically shares your affiliate link)</h5>'; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$html             .= '					     
									</span>
								</div>
							</span>
							<span class="span4">
								<div class="row-fluid">
									<span class="span8">
										<p></p>
									</span>
									<span class="span4 userimage" > 
										<!-- <img src="' . $spp_channel_image . '" class="image" style="width:100px;"> -->
										<img src="' . $spp_channel_image . '" class="imagenotround" style="width:100px;">
									</span>
								</div>
							</span>
						</div>
					</div>';

				$spp_post_format = '<select name="spp_post_format_select">';

				$post_formats = array(
					'0'       => 'Standard (Default)',
					'aside'   => 'Aside',
					'chat'    => 'Chat',
					'gallery' => 'Gallery',
					'link'    => 'Link',
					'quote'   => 'Quote',
					'status'  => 'Status',
					'video'   => 'Video',
					'audio'   => 'Audio',
				);
				foreach ( $post_formats as $key => $format ) {
					if ( $spp_post_format_select === $key ) {

						$spp_post_format .= "<option value='" . $key . "' selected='selected'>" . $format . '</option>';

					} else {
						$spp_post_format .= "<option value='" . $key . "'>" . $format . '</option>';

					}
				}
				$spp_post_format .= '</select>';
				if ( $podcastURL_Feed ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
						$html .= '
							<div class="well well-1">
								<div class="page-header page-header-2">
									<h1> <small>Publish Settings</small></h1>
								</div>
							
								<div class="row-fluid">
									<span class="span6">
										Select which episodes to automatically import
									</span>
									<span class="span2" style="height:10px; width: 16%;">
										<select id="spp_import_select" name="spp_import_select" style="width: 200px;">
											<option value="0" ' . $spp_import_selected1 . '>All (Default)</option>
											<option value="1" ' . $spp_import_selected2 . '>Disable Import</option>
											<option value="2" ' . $spp_import_selected3 . '>Only After Specific Date</option>
											<option value="3" ' . $spp_import_selected4 . '>Only Before Specific Date</option>
										</select>
									</span>
									<span class="span3 date_textbox" style="' . $date_textbox_display . '">
										<input class="ep_specific_date" type="text" placeholder="Select Date ...." value="' . $ep_specific_date . '" name="ep_specific_date" style="width:128px">
									</span>
								</div>
								<br>
							
								<div class="row-fluid">
									<span class="span6">
										<label class="checkbox">
											<input type="checkbox" name="spp_auto_publish" id="spp_auto_publish"  ' . $spp_auto_publish1 . ' />
											<span></span>
											<span>Automatically publish posts (uncheck to import posts as drafts)</span>
											<br>
										</label>
									</span>
									<span class="span6">
										<div class="row-fluid">
											<span class="span12">
												<a data-content="When this option is selected, Simple Podcast Press will automatically publish the new podcast blog posts. If this is deselected, then the podcast blog posts will remain in the draft state." data-original-title="Auto Publish Podcast Posts" rel="popover" class="icon icon-info-sign spp_icon-auto-publish">&nbsp;</a>
											</span>
										</div>
									</span>
								</div>
							
								<div class="row-fluid spp_emaildraft" style="' . $spp_draft_row_display . '">
									<span class="span6">
										<label class="checkbox">
											<input type="checkbox" name="spp_email_on_draft" id="spp_email_on_draft" ' . $spp_email_on_draft . '  />
											<span></span>
											<span>Email me each time a <strong>draft</strong> podcast post is created</span>
											<br>
										</label>
									</span>
									<span class="span6">
										<div class="row-fluid">
											<span class="span12">
												<a data-content="When this option is selected, you will receive an email each time a draft podcast blog post is created by Simple Podcast Press. Note that if the auto publish option is enabled, you will not receive any emails." data-original-title="Draft Notification" rel="popover" class="icon icon-info-sign spp_icon-draft-post">&nbsp;</a>
											</span>
										</div>
									</span>
								</div>
								<br>
							
								<div class="row-fluid row-fluid-1">
									<span class="span6">
										Import podcast episode image and set as featured image
									</span>
									<span class="span6">
										<select name="ep_art_select" style="width: 200px;">
											<option value="0" ' . $ep_art_select1 . '>Off (Default)</option>
											<option value="1" ' . $ep_art_select2 . '>Episode Specific Image</option>
											<option value="3" ' . $ep_art_select4 . '>Podcast Cover Art</option>
											<option value="2" ' . $ep_art_select3 . '>From Your Libsyn Website Page (advanced users only)</option>
										</select>
									</span>
								</div>
							
								<div class="row-fluid">
									<span class="span6">
										<label class="checkbox">
											<input type="checkbox" name="spp_insert_feat_image" id="spp_insert_feat_image"  ' . $spp_insert_feat_image1 . ' />
											<span></span>
											<span>Automatically insert featured image on top of post</span>
											<br>
										</label>
									</span>
								</div>
							
								<div class="spp_player_over_image_section">
									<div class="row-fluid">
										<span class="span6">
											Featured image size and alignment
										</span>
										<span class="span2" style="height:10px; width: 16%;">
											<select id="spp_imgsize_select" name="spp_imgsize_select" style="width: 200px;">
												<option value="thumbnail" ' . $spp_imgsize_selected1 . '>Thumbnail (150 x 150px max)</option>
												<option value="medium" ' . $spp_imgsize_selected2 . '>Medium (300 x 300px max)</option>
												<option value="large" ' . $spp_imgsize_selected3 . '>Large (640 x 640p max)</option>
												<option value="full" ' . $spp_imgalign_selected4 . '>Full (original size max)</option>
											</select>
										</span>
										<span class="span2" style="height:10px; width: 16%;">
											<select id="spp_imgalign_select" name="spp_imgalign_select" style="width: 200px;">
												<option value="alignleft" ' . $spp_imgalign_selected1 . '>Left</option>
												<option value="aligncenter" ' . $spp_imgalign_selected2 . '>Center</option>
												<option value="alignright" ' . $spp_imgalign_selected3 . '>Right</option>
												<option value="alignnone" ' . $spp_imgalign_selected4 . '>None</option>
											</select>
										</span>
									</div>
									<br>
								</div>
							
								<div class="row-fluid">
									<span class="span6">
										WordPress category
									</span>
									<span class="span6">
										<div class="row-fluid">
											<span class="span7">
												' . $spp_catselechtml . '
											</span>
											<span class="span5"></span>
										</div>
									</span>
								</div>
							
								<div class="row-fluid">
									<span class="span6">
										Author
									</span>
									<span class="span6">
										<div class="row-fluid">
											<span class="span7">
												' . $spp_authorlist . '
											</span>
											<span class="span5"></span>
										</div>
									</span>
								</div>
							
								<div class="row-fluid">
									<span class="span6">
										Custom post type (if supported by your theme)
									</span>
									<span class="span6">
										<div class="row-fluid">
											<span class="span7">
												' . $spp_cpt_dropdown . '
											</span>
										</div>
									</span>
								</div>
							
								<div class="row-fluid">
									<span class="span6">
										Post format (if supported by your theme)
									</span>
									<span class="span6">
										<div class="row-fluid">
											<span class="span7">
												' . $spp_post_format . '
											</span>
										</div>
									</span>
								</div>
							</div>
							';
				}
					$html            .= '
					<div class="well">
						<div class="page-header page-header-1">
							<h1> <small>Podcast Player Settings</small></h1>
						</div>
						
						<div class="row-fluid">
							<span class="span6">
								<label class="span6">
									<span>Audio player type
										<br>
									</span>
								</label>
							</span>
							<span class="span7" style="height: auto ! important; width: 19%;">
								<select id="select_audio_player_skin" name="select_audio_player_skin">
									<option value="soundmanager2_player"  ' . $select_audio_player_skin3 . '>Pro Player</option>
									<option value="original_player"  ' . $select_audio_player_skin1 . '>Classic Player</option>
								</select>
							</span>
						</div>

						<div class="row-fluid">
							<span class="span6">
								<label class="span6">
									<span>Audio player colors
										<br>
									</span>
								</label>
							</span>
							<span class="span7" style="height: auto ! important; width: 19%;">
								<select id="select_audio_player" name="select_audio_player">
									<option value="simplepodcastpressblack" ' . $simplepodcastpressblack . ' ' . $select_audio_player2 . '>Black</option>
									<option value="simplepodcastpresswhite" ' . $simplepodcastpressblack . ' ' . $select_audio_player3 . '>White</option>
									<option value="simplepodcastpresscustomcolor"  ' . $select_audio_player4 . '>Custom</option>
								</select>
							</span>
						</div>

						<div class="sppress_customplayer_settings" style="' . $sppress_custom_player_display . '">
							<div class="row-fluid player_button_color" style="' . $player_button_color . '" >
								<span class="span4" style="height:auto !important;">
									<label class="checkbox">
										<span>Play button color
											<br><br>
										</span>
									</label>
								</span>
								<span class="span8">
									<select style="width: 110px;" id="player_button_color" name="player_button_color">
										<option value="sppwhite" ' . $player_button_color1 . '>White</option>
										<option value="sppblack" ' . $player_button_color2 . '>Black</option>
									</select>
								</span>
							</div>
							
							<div class="row-fluid rewindspeed_button_color" style="' . $rewindspeed_button_color . '">
								<span class="span4" style="height:auto !important;">
									<label class="checkbox">
										<span>Rewind / Speed / Playlist / Volume Button Color
											<br><br>
										</span>
									</label>
								</span>
								<span class="span8">
									<select style="width: 110px;" id="rewindspeed_button_color" name="rewindspeed_button_color">
										<option value="sppwhite" ' . $rewindspeed_button_color1 . '>White</option>
										<option value="sppblack" ' . $rewindspeed_button_color2 . '>Black</option>
									</select>
								</span>
							</div>

							<div class="row-fluid">
								<span class="span4" style="height:auto !important;">
									<label class="checkbox">
										<span>Player Bar Color
											<br><br>
										</span>
									</label>
								</span>
								<span class="span8">
									<input type="text" name="player_bar_color" id="player_bar_color" style="width:90px" value="' . $player_bar_color . '" >
								</span>
							</div>

							<div class="row-fluid progressbarcolor" style="' . $progressbarcolor . '">
								<span class="span4" style="height:auto !important; ">
									<label class="checkbox">
										<span>Progress Bar Color
											<br><br>
										</span>
									</label>
								</span>
								<span class="span8">
									<input type="text" name="progress_bar_color" id="progress_bar_color" style="width:90px" value="' . $progress_bar_color . '" >
								</span>
							</div>

							<div class="row-fluid">
								<span class="span4" style="height:auto !important;">
									<label class="checkbox">
										<span>Player Text Color
											<br><br>
										</span>
									</label>
								</span>
								<span class="span8">
									<input type="text" name="player_text_color" id="player_text_color" style="width:90px" value="' . $player_text_color . '" >
									<br><br>
								</span>
							</div>
						</div>

						<div class="row-fluid audioimageoption" style="' . $audioimagestyle . '">
							<span class="span6">
								<label class="span6">
									<span>Audio Player Image
										<br>
									</span>
								</label>
							</span>
							<span class="span7" style="height: auto ! important; width: 19%;">
								<select name="audio_player_image">
									<option value="itunes" ' . $audio_player_image1 . '>Podcast Cover Art (default)</option>
									<option value="featured" ' . $audio_player_image2 . '>Featured Image</option>
									<option value="disabled" ' . $audio_player_image3 . '>Disabled</option>
								</select>
							</span>
						</div>
					';
					$powerpressexists = get_option( 'powerpress_general' );
					$powerpressactive = is_plugin_active( 'powerpress/powerpress.php' );

				if ( ! $podcastURL_Feed ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
						$html .= '
							';
					if ( ! $powerpressactive ) {
							$html .= '

									<div class="row-fluid row-fluid-1">
										<span class="span5">
											<span><strong>Prevent SPP from replacing the following players:</strong><br><br></span>
										</span>
									</div>

									<div class="row-fluid row-fluid-1">
										<span class="span5">
											<label class="checkbox">
												<input type="checkbox" name="replace_wp_with_spp" id="replace_wp_with_spp" ' . $replace_wp_with_spp . ' />
												<span>WordPress Audio Player<br><br></span>
											</label>
										</span>
									</div>

									<div class="row-fluid row-fluid-1">
										<span class="span5">
											<label class="checkbox">
												<input type="checkbox" name="replace_stp_with_spp" id="replace_stp_with_spp" ' . $replace_stp_with_spp . ' />
												<span>Smart Podcast Player Single Track Player<br><br></span>
											</label>
										</span>
									</div>
									';
					} else {
							$html .= '
									<div class="row-fluid row-fluid-1">
										<span class="span5">
											<span><strong>SPP has replaced the PowerPress player on all your posts.<br>Prevent SPP from replacing the following players:</strong><br><br></span>
										</span>
									</div>

									<div class="row-fluid row-fluid-1">
										<span class="span5">
											<label class="checkbox">
												<input type="checkbox" name="replace_wp_with_spp" id="replace_wp_with_spp" ' . $replace_wp_with_spp . ' />
												<span>WordPress Audio Player<br><br></span>
											</label>
										</span>
									</div>

									<div class="row-fluid row-fluid-1">
										<span class="span5">
											<label class="checkbox">
												<input type="checkbox" name="replace_stp_with_spp" id="replace_stp_with_spp" ' . $replace_stp_with_spp . ' />
												<span>Smart Podcast Player Single Track Player<br><br></span>
											</label>
										</span>
									</div>
									';
					}
				} else {
						$html .= '

							<div class="row-fluid">
								<span class="span6">
									<label class="span6">
										<span>Audio player default location<br><br></span>
									</label>
								</span>
								<span class="span7" style="height: auto ! important; width: 19%;">
									<select name="audio_player_position">
										<option value="above" ' . $audio_player_position1 . '>Above page content</option>
										<option value="below" ' . $audio_player_position2 . '>Below page content</option>
										<option value="disabled" ' . $audio_player_position3 . '>Disabled</option>
									</select>
								</span>
							</div>

							<div class="spp_player_over_image_section">

								<div class="row-fluid row-fluid-1">
									<span class="span6">
										<label class="checkbox">
											<input type="checkbox" name="spp_player_over_image" id="spp_player_over_image" ' . $spp_player_over_image . '  />
											<span>Force player on top of featured image</span><br><br>
										</label>
									</span>
									<span class="span6">
										<div class="row-fluid">
											<span class="span12"> <a data-content="By default, the featured image will be inserted above everything in the post including the player.  If you want the player to be on top of the featured image, then enable this option." data-original-title="Force Player On Top Of Image" rel="popover" class="icon icon-info-sign spp_icon-draft-post">&nbsp;</a>
											</span>
										</div>
									</span>
								</div>

							</div>
							';
				}
					$html .= '
						<div class="row-fluid row-fluid-1" id="spp_replace">
							<span class="span6">
								<label class="checkbox">
									<input type="checkbox" name="replace_pp_with_spp" id="replace_pp_with_spp" ' . $replace_pp_with_spp . ' />
									<span></span>
									<span>Automatically replace all your existing audio players with the SPP player</br>
										<br>
									</span>
								</label>
							</span>
							<span class="span6">
								<div class="row-fluid">
									<span class="span12"> 
										<a data-content="Supports PowerPress, Smart Podcast Player Single Track Player, and Native WordPress audio players. Turning this option ON will automatically replace all instances of the existing audio player with the Simple Podcast Press player at the same location.  It will also add the call-to-action buttons or opt-in boxes as specified in your settings. Turning this option OFF will automatically revert back to the original player.Your podcast feed is untouched." data-original-title="Use Simple Podcast Player instead of existing player" rel="popover" class="icon icon-info-sign spp_icon-auto-publish">&nbsp;</a>
									</span>
								</div>
							</span>
						</div>

						<div class="sppress_donotreplace_player" style="' . $spp_replace_with_spp_display . '">
							<div class="row-fluid row-fluid-1" style="margin-left: 25px;">
								<span class="span5">
									<span><strong>Do NOT replace the following podcast players:</strong></br>
										<br>
									</span>
								</span>
							</div>

							<div class="row-fluid row-fluid-1" style="margin-left: 25px;">
								<span class="span5">
									<label class="checkbox">
										<input type="checkbox" name="replace_wp_with_spp" id="replace_wp_with_spp" ' . $replace_wp_with_spp . ' />
										<span>WordPress Audio Player</br>
											<br>
										</span>
									</label>
								</span>
							</div>

							<div class="row-fluid row-fluid-1" style="margin-left: 25px;">
								<span class="span5">
									<label class="checkbox">
										<input type="checkbox" name="replace_stp_with_spp" id="replace_stp_with_spp" ' . $replace_stp_with_spp . ' />
										<span>Smart Podcast Player Single Track Player</br>
											<br>
										</span>
									</label>
								</span>
							</div>
						</div>
					';
					$html .= '
			<br><div class="row-fluid row-fluid-1">
				
				<span class="span3" style="height:auto !important;">
					<label class="checkbox">
						<input type="checkbox" name="spp_remove_timecode" id="spp_remove_timecode" ' . $spp_remove_timecode . ' >
						<span>Hide time from text above player<br><br></span>
					</label>
				</span>
			
			</div>

			<div class="row-fluid">
				<span class="span6" style="height:auto !important;">
					<label class="span6">
						<span>Customize text above player<br><br></span>
					</label>
				</span>
				<span class="span7"  style="height: auto ! important; width: 19%;">
					<input type="text" name="spp_listen_text" id="spp_listen_text" style="width:350px" value="' . $spp_listen_text . '" placeholder="i.e Listen to the Episode. Leave blank for no text.">
				</span>
			</div>  

			<div class="row-fluid row-fluid-1">
				<span style="height:auto !important;" class="span3">
					<label>
						<span>Audio player width<br><br></span>
					</label>
				</span>
				<span class="span3">
					<input type="text" placeholder="Auto" value="' . $container_width . '" name="container_width" style="width:105px"> px
				</span>
				<span class="span6">
					<div class="row-fluid">
						<span class="span12">  
							<a data-content="Set a custom width for the audio player (including the call to action buttons and opt-in if enabled).  Note that when a value is entered, it may affect the look of the player on mobile sites.  To set an automatic width, simply delete the contents of the input box." data-original-title="Audio Player Width" rel="popover" class="icon icon-info-sign spp_icon-auto-publish">&nbsp;</a>
						</span>
					</div>
				</span>
			</div>  

			<div class="row-fluid row-fluid-4" >
				<span class="span6" style="height:auto !important;">
					<label class="checkbox">
						<input type="checkbox" name="spp_autoplay_podcast" id="spp_autoplay_podcast" ' . $spp_autoplay_podcast . '>
						<span></span>
						<span>Enable auto play  (not recommended)<br><br></span>
					</label>
				</span>
				<span class="span1">
					<div class="row-fluid">
						<span class="span7"> 
							<a data-content="When this option is selected, the Simple Podcast Press will automatically start playing a podcast as soon as the page loads." data-original-title="Automatically Play Podcasts" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
						</span>
					</div>
				</span>
			</div>  

			<div class="row-fluid row-fluid-4" >
				<span class="span6" style="height:auto !important;">
					<label class="checkbox">
						<input type="checkbox" name="spp_rewind_button" id="spp_rewind_button" ' . $spp_rewind_button . '>
						<span></span>
						<span>Hide 15 second rewind button<br><br></span>
					</label>
				</span>
				<span class="span1">
					<div class="row-fluid">
						<span class="span7"> 
							<a data-content="When this option is selected, the Simple Podcast Press will hide the 15-second rewind button on player." data-original-title="15 Second Rewind" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
						</span>
					</div>
				</span>
			</div>  

			<div class="row-fluid row-fluid-4" >
				<span class="span6" style="height:auto !important;">
					<label class="checkbox">
						<input type="checkbox" name="spp_speed_button" id="spp_speed_button" ' . $spp_speed_button . '>
						<span></span>
						<span>Hide speed control button<br><br></span>
					</label>
				</span>
				<span class="span1">
					<div class="row-fluid">
						<span class="span7"> 
							<a data-content="When this option is selected, the Simple Podcast Press will hide the 1x speed control button on player." data-original-title="Speed Control Button" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
						</span>
					</div>
				</span>
			</div>  

			<div class="row-fluid row-fluid-4" >
				<span class="span6" style="height:auto !important;">
					<label class="checkbox">
						<input type="checkbox" name="spp_volume_button" id="spp_volume_button" ' . $spp_volume_button . '>
						<span></span>
						<span>Show volume control button<br><br></span>
					</label>
				</span>
				<span class="span1">
					<div class="row-fluid">
						<span class="span7"> 
							<a data-content="When this option is selected, the Simple Podcast Press will show volume control button on player." data-original-title="Volume Control Button" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
						</span>
					</div>
				</span>
			</div>  

			<div class="row-fluid row-fluid-1" >
				
				<span class="span3" style="height:auto !important;">
					<label class="checkbox">
						<input type="checkbox" name="btn_style_round" id="btn_style_round" ' . $btn_style_round . ' >
						<span>Make player and buttons round<br><br></span>
					</label>
				</span>
			
			</div>

			<div class="row-fluid row-fluid-4">
				<span class="span6">
					<label class="checkbox">
						<input type="checkbox" name="disable_player_excerpt" id="disable_player_excerpt" ' . $disable_player_excerpt . ' >
						<span>Disable player and text from home, blog listing, archive, and all user generated pages<br><br></span>
					</label>
				</span>
				<span class="span3">
					<div class="row-fluid">
						<span class="span12"> 
							<a data-content="When this option is selected, the Simple Podcast Press audio player will not appear on the home, blog or archive pages.  This option is also used to remove the random player text that appears on some home or archive pages on some themes" data-original-title="Disable Player on Blog Pages" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
						</span>
					</div>
				</span>
			</div>

			<div class="row-fluid row-fluid-4">
				<span class="span6">
					<label class="checkbox">
						<input type="checkbox" name="hide_elements_athome" id="hide_elements_athome" ' . $hide_elements_athome . ' >
						<span>Hide buttons and email optin on home, blog listing, or archive Pages<br><br></span>
					</label>
				</span>
				<span class="span3">
					<div class="row-fluid">
						<span class="span12"> 
							<a data-content="When this option is selected, then just Simple Podcast Press audio player appear on the home, blog or archive pages. Not Buttons and Email Optin Below Player. " data-original-title="Disable Player on Blog Pages" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
						</span>
					</div>
				</span>
			</div>  

			<div class="row-fluid row-fluid-4">
				<span class="span6">
					<label class="checkbox">
						<input type="checkbox" name="off_scrolling_text" id="off_scrolling_text" ' . $off_scrolling_text . ' >
						<span>Disable scrolling text on player<br><br></span>
					</label>
				</span>
				<span class="span3">
					<div class="row-fluid">
						<span class="span12"> 
							<a data-content="When this option is selected, Text on player will be not scroll. " data-original-title="Disable scrolling text on player." rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
						</span>
					</div>
				</span>
			</div>            
			';

				if ( 'valid' === $is_license_valid ) {
						$html .= '<div class="row-fluid row-fluid-4">
					<span class="span6" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="spp_optin_hide_chk" id="spp_optin_hide_chk" ' . $spp_optin_hide_chk . '>
							<span></span>
							<span>Delay display of buttons and email optin until user clicks play (Pro Player only)<br><br></span>
						</label>
					</span>
					<span class="span5" style="height: auto ! important; width: 30%;">
						<input type="text" name="spp_optshow_xseconds" id="spp_optshow_xseconds" style="width:90px" value="' . $spp_optshow_xseconds . '" placeholder="Default is 3" > secs
					</span>
				</div>  

				<div class="row-fluid row-fluid-4">
					<span class="span6" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="spp_disable_poweredby" id="spp_disable_poweredby" ' . $spp_disable_poweredby . '>
							<span></span>
							<span>Disable "Powered by" link below player<br><br></span>
						</label>
					</span>
					<span class="span5" style="height: auto ! important; width: 30%;">
						<input type="text" name="spp_poweredby_url" id="spp_poweredby_url" style="width:350px" value="' . $spp_poweredby_url . '" placeholder="Enter Your Affiliate ID Number ONLY (i.e 1)" ><br><a href="https://simplepodcastpress.com/affiliate-area/" target="_blank">Click Here To Get Your Affilliate ID.</a>
					</span>
				</div>  
				</br>
				<!-- Preroll
				<div class="row-fluid row-fluid-4" >
				<span class="span6" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="spp_pre_roll_checkbox" id="spp_pre_roll_checkbox" ' . $spp_pre_roll_checkbox . '>
							<span></span>
							<span>Pre Roll.<br><br></span>
						</label>
					</span>
					<span class="span5" style="height: auto ! important; width: 19%;">
						<input type="text" name="spp_pre_roll_url" id="spp_pre_roll_url" style="width:350px" value="' . $spp_pre_roll_url . '" placeholder="Enter Your Pre Roll Mp3 URL)" >
					</span>
				</div> 
				-->
				</br>
				';
				}

					$html                                 .= '
			</div>
			<div class="well">
				<div class="page-header page-header-1">
					<h1> <small>Button Settings</small> 
					</h1>
				</div>

				<strong>New: You can change the order of the buttons by dragging and dropping each row.</strong>
				</br></br>
				<ul id="sortable">';
					$buttons_array['btn_g_play']           = '<li id="btn_g_play">
				<div class="row-fluid row-fluid-2" >
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="btn_g_play" id="btn_g_play" ' . $btn_g_play . ' >
							<span>Google Podcasts Button
								</br></br>
							</span>
						</label>
					</span>
					<span class="span2" style="height: auto !important;">
						<input type="text" name="btn_g_play_color" id="btn_g_play_color" value="' . $btn_g_play_color . '" style="width:90px" >
					</span>
					<span class="span3">
						<input type="text" name="btn_g_play_name" id="btn_g_play_name" value="' . $btn_g_play_name . '" style="width:220px" placeholder="Google Podcast" >
					</span>
					<!-- <span class="span2" style="height: auto;">
						<input type="text" name="btn_g_play_url" id="btn_g_play_url"  value="' . $btn_g_play_url . '"  style="width:250px" placeholder="Enter your Google Play URL ..." >
					</span> -->
				</div>
			</li>';
					$buttons_array['btn_share']            = '<li id="btn_share">
				<div class="row-fluid row-fluid-2" >
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="btn_share" id="btn_share" ' . $btn_share . ' >
							<span>Share Button
								</br></br>
							</span>
						</label>
					</span>
					<span class="span2" style="height: auto !important;">
						<input type="text" name="btn_share_color" id="btn_share_color" value="' . $btn_share_color . '" style="width:90px" >
					</span>
					<span class="span3">
						<input type="text" name="btn_share_name" id="btn_share_name" value="' . $btn_share_name . '" style="width:220px" placeholder="Share" >
					</span>
				</div>
			</li>';
					$buttons_array['btn_download']         = '<li id="btn_download">
				<div class="row-fluid row-fluid-2" >
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="btn_download" id="btn_download" ' . $btn_download . ' >
							<span>Download Button
								</br></br>
							</span>
						</label>
					</span>
					<span class="span2" style="height: auto !important;">
						<input type="text" name="btn_download_color" id="btn_download_color" value="' . $btn_download_color . '" style="width:90px" >
					</span>
					<span class="span3">
						<input type="text" name="btn_download_name" id="btn_download_name" value="' . $btn_download_name . '" style="width:220px" placeholder="Download" >
					</span>
				</div>
			</li>';
					$buttons_array['btn_spplisten']        = '<li id="btn_spplisten">
				<div class="row-fluid row-fluid-1" >
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="btn_spplisten" id="btn_spplisten" ' . $btn_spplisten . '>
							<span></span>
							<span>Listen In New Window Button
								</br></br>
							</span>
						</label>
					</span>
					<span class="span2" style="height: auto !important;">
						<input type="text" name="btn_spplisten_color" id="btn_spplisten_color" style="width:90px" value="' . $btn_spplisten_color . '" >
					</span>
					<span class="span3">
						<input type="text" name="btn_spplisten_name" id="btn_spplisten_name" value="' . $btn_spplisten_name . '" style="width:220px" placeholder="Listen in a New Window" >
					</span>
				</div>
			</li>';
					$buttons_array['btn_sppreview']        = '<li id="btn_sppreview">
			<div class="row-fluid row-fluid-1" >
				<span class="span3" style="height:auto !important;">
					<label class="checkbox">
						<input type="checkbox" name="btn_sppreview" id="btn_sppreview" ' . $btn_sppreview . '>
						<span></span>
						<span>Leave a Review Button
							</br></br>
						</span>
					</label>
				</span>
				<span class="span2" style="height: auto !important;">
					<input type="text" name="btn_sppreview_color" id="btn_sppreview_color" style="width:90px" value="' . $btn_sppreview_color . '" >
				</span>
				<span class="span3">
					<input type="text" name="btn_sppreview_name" id="btn_sppreview_name" value="' . $btn_sppreview_name . '" style="width:220px" placeholder="Leave a Review" >
				</span>
			</div>
			</li>';
					$buttons_array['btn_spprss']           = '<li id="btn_spprss">
			<div class="row-fluid row-fluid-1" >
				<span class="span3" style="height:auto !important;">
					<label class="checkbox">
						<input type="checkbox" name="btn_spprss" id="btn_spprss" ' . $btn_spprss . '>
						<span></span>
						<span>Subscribe Via RSS Button
							</br></br>
						</span>
					</label>
				</span>
				<span class="span2" style="height: auto !important;">
					<input type="text" name="btn_spprss_color" id="btn_spprss_color" style="width:90px" value="' . $btn_spprss_color . '" >
				</span>
				<span class="span3">
					<input type="text" name="btn_spprss_name" id="btn_spprss_name" value="' . $btn_spprss_name . '" style="width:220px" placeholder="Subscribe via RSS" >
				</span>
			</div>
			</li>';
					$buttons_array['btn_sppandroid']       = '<li id="btn_sppandroid">
			<div class="row-fluid row-fluid-1" >
				<span class="span3" style="height:auto !important;">
					<label class="checkbox">
						<input type="checkbox" name="btn_sppandroid" id="btn_sppandroid" ' . $btn_sppandroid . '>
						<span></span>
						<span>Subscribe On Android Button
							</br></br>
						</span>
					</label>
				</span>
				<span class="span2" style="height: auto !important;">
					<input type="text" name="btn_sppandroid_color" id="btn_sppandroid_color" style="width:90px" value="' . $btn_sppandroid_color . '" >
				</span>
				<span class="span4">
					<input type="text" name="btn_sppandroid_name" id="btn_sppandroid_name" value="' . $btn_sppandroid_name . '" style="width:220px" placeholder="Subscribe on Android" >
				</span>
			</div>
			</li>';
					$buttons_array['btn_itunes']           = '<li id="btn_itunes">
			<div class="row-fluid row-fluid-1" >
				<span class="span3" style="height:auto !important;">
					<label class="checkbox">
						<input type="checkbox" name="btn_itunes" id="btn_itunes" ' . $btn_itunes . ' >
						<span>Apple Podcasts Button
							</br></br>
						</span>
					</label>
				</span>
				<span class="span2" style="height: auto !important;">
					<input type="text" name="btn_itunes_color" id="btn_itunes_color" style="width:90px" value="' . $btn_itunes_color . '" >
				</span>
				<span class="span3" style="height: auto !important;">
					<input type="text" name="btn_itunes_name" id="btn_itunes_name" value="' . $btn_itunes_name . '" style="width:220px" placeholder="Apple Podcast" >
				</span>
				<span class="span2" style="height: auto;">
					<input type="text" name="btn_itunes_url" id="btn_itunes_url"  value="' . $btn_itunes_url . '"  style="width:250px" placeholder="Enter your Apple Podcast URL to overwrite default ..." >
				</span>
			</div>
			</li>';
					$buttons_array['btn_stiticher']        = '<li id="btn_stiticher">
				<div class="row-fluid row-fluid-1" >
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="btn_stiticher" id="btn_stiticher" ' . $btn_stiticher . '>
							<span></span>
							<span>Stitcher Button
								</br></br>
							</span>
						</label>
					</span>
					<span class="span2" style="height: auto !important;">
						<input type="text" name="btn_stiticher_color" id="btn_stiticher_color" style="width:90px" value="' . $btn_stiticher_color . '" >
					</span>
					<span class="span3">
						<input type="text" name="btn_stitcher_name" id="btn_stitcher_name" value="' . $btn_stitcher_name . '" style="width:220px" placeholder="Stitcher" >
					</span>
					<span class="span2" style="height: auto;">
						<input type="text" name="btn_stiticher_url" id="btn_stiticher_url"  value="' . $btn_stiticher_url . '"  style="width:250px" placeholder="Enter your Stitcher URL ..." >
					</span>
				</div>
			</li>';
					$buttons_array['btn_soundcloud']       = '<li id="btn_soundcloud">
				<div class="row-fluid row-fluid-1" >
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="btn_soundcloud" id="btn_soundcloud" ' . $btn_soundcloud . ' >
							<span>SoundCloud Button
								</br></br>
							</span>
						</label>
					</span>
					<span class="span2" style="height: auto !important;">
						<input type="text" name="btn_soundcloud_color" id="btn_soundcloud_color" style="width:90px" value="' . $btn_soundcloud_color . '" >
					</span>
					<span class="span3" style="height: auto !important;">
						<input type="text" name="btn_soundcloud_name" id="btn_soundcloud_name" value="' . $btn_soundcloud_name . '" style="width:220px" placeholder="SoundCloud" >
					</span>
					<span class="span2" style="height: auto;">
						<input type="text" name="btn_soundcloud_url" id="btn_soundcloud_url"   style="width:250px" value="' . $btn_soundcloud_url . '" placeholder="Enter your SoundCloud URL ..." >
					</span>
				</div>
			</li>';
					$buttons_array['btn_spotify']          = '<li id="btn_spotify">
				<div class="row-fluid row-fluid-1" >
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="btn_spotify" id="btn_spotify" ' . $btn_spotify . ' >
							<span>Spotify Button
								</br></br>
							</span>
						</label>
					</span>
					<span class="span2" style="height: auto !important;">
						<input type="text" name="btn_spotify_color" id="btn_spotify_color" style="width:90px" value="' . $btn_spotify_color . '" >
					</span>
					<span class="span3" style="height: auto !important;">
						<input type="text" name="btn_spotify_name" id="btn_spotify_name" value="' . $btn_spotify_name . '" style="width:220px" placeholder="Spotify" >
					</span>
					<span class="span2" style="height: auto;">
						<input type="text" name="btn_spotify_url" id="btn_spotify_url"   style="width:250px" value="' . $btn_spotify_url . '" placeholder="Enter your Spotify URL ..." >
					</span>
				</div>
			</li>';
					$buttons_array['btn_spp_custom1']      = '<li id="btn_spp_custom1">
				<div class="row-fluid row-fluid-4" >
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="btn_spp_custom1" id="btn_spp_custom1" ' . $btn_spp_custom1 . ' >
							<span>Custom Button 1
								</br></br>
							</span>
						</label>
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" name="btn_spp_custom_color1" id="btn_spp_custom_color1" style="width:90px" value="' . $btn_spp_custom_color1 . '" >
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" placeholder="icon code" value="' . $custom_button_icon1 . '" name="custom_button_icon1" style="width:90px;margin-right: 20px;"><a data-content="Enter your font awesoness icon code (example f1bc). Please visit the cheat sheet here for a complete list of icons you can use: https://fontawesome.com/cheatsheet  " data-original-title="Custom Button Icon" rel="popover" class="icon icon-info-sign spp_icon-auto-publish">&nbsp;</a>
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" name="btn_spp_custom_name1" id="btn_spp_custom_name1" style="width:130px" value="' . $btn_spp_custom_name1 . '" placeholder="Button Label ..." >
					</span>
					<span class="span3"  style="height: auto ! important; width: 19%;">
						<input type="text" name="btn_spp_custom_url1" id="btn_spp_custom_url1" style="width:170px" value="' . $btn_spp_custom_url1 . '" placeholder="Button URL ..." >
					</span>
				</div>
			</li>';
					$buttons_array['btn_spp_custom2']      = '<li id="btn_spp_custom2"> 
				<div class="row-fluid row-fluid-4" >
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="btn_spp_custom2" id="btn_spp_custom2" ' . $btn_spp_custom2 . ' >
							<span>Custom Button 2
								</br></br>
							</span>
						</label>
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" name="btn_spp_custom_color2" id="btn_spp_custom_color2" style="width:90px" value="' . $btn_spp_custom_color2 . '" >
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" placeholder="icon code" value="' . $custom_button_icon2 . '" name="custom_button_icon2" style="width:90px;margin-right: 20px;"><a data-content="Enter your font awesoness icon code (example f1bc). Please visit the cheat sheet here for a complete list of icons you can use: https://fontawesome.com/cheatsheet  " data-original-title="Custom Button Icon" rel="popover" class="icon icon-info-sign spp_icon-auto-publish">&nbsp;</a>
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" name="btn_spp_custom_name2" id="btn_spp_custom_name2" style="width:130px" value="' . $btn_spp_custom_name2 . '" placeholder="Button Label ..." >
					</span>
					<span class="span2"  style="height: auto ! important; width: 19%;">
						<input type="text" name="btn_spp_custom_url2" id="btn_spp_custom_url2" style="width:170px" value="' . $btn_spp_custom_url2 . '" placeholder="Button URL ..." >
					</span>
				</div>
			</li>';
					$buttons_array['btn_spp_custom3']      = '<li id="btn_spp_custom3">
				<div class="row-fluid row-fluid-4" >
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="btn_spp_custom3" id="btn_spp_custom3" ' . $btn_spp_custom3 . ' >
							<span>Custom Button 3
								</br></br>
							</span>
						</label>
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" name="btn_spp_custom_color3" id="btn_spp_custom_color3" style="width:90px" value="' . $btn_spp_custom_color3 . '" >
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" placeholder="icon code" value="' . $custom_button_icon3 . '" name="custom_button_icon3" style="width:90px;margin-right: 20px;"><a data-content="Enter your font awesoness icon code (example f1bc). Please visit the cheat sheet here for a complete list of icons you can use: https://fontawesome.com/cheatsheet  " data-original-title="Custom Button Icon" rel="popover" class="icon icon-info-sign spp_icon-auto-publish">&nbsp;</a>
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" name="btn_spp_custom_name3" id="btn_spp_custom_name3" style="width:130px" value="' . $btn_spp_custom_name3 . '" placeholder="Button Label ..." >
					</span>
					<span class="span2"  style="height: auto ! important; width: 19%;">
						<input type="text" name="btn_spp_custom_url3" id="btn_spp_custom_url3" style="width:170px" value="' . $btn_spp_custom_url3 . '" placeholder="Button URL ..." >
					</span>
				</div>
			</li>';
					$buttons_array['btn_spp_custom4']      = '<li id="btn_spp_custom4">
				<div class="row-fluid row-fluid-4" >
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="btn_spp_custom4" id="btn_spp_custom4" ' . $btn_spp_custom4 . ' >
							<span>Custom Button 4
								</br></br>
							</span>
						</label>
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" name="btn_spp_custom_color4" id="btn_spp_custom_color4" style="width:90px" value="' . $btn_spp_custom_color4 . '" >
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" placeholder="icon code" value="' . $custom_button_icon4 . '" name="custom_button_icon4" style="width:90px;margin-right: 20px;"><a data-content="Enter your font awesoness icon code (example f1bc). Please visit the cheat sheet here for a complete list of icons you can use: https://fontawesome.com/cheatsheet  " data-original-title="Custom Button Icon" rel="popover" class="icon icon-info-sign spp_icon-auto-publish">&nbsp;</a>
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" name="btn_spp_custom_name4" id="btn_spp_custom_name4" style="width:130px" value="' . $btn_spp_custom_name4 . '" placeholder="Button Label ..." >
					</span>
					<span class="span2"  style="height: auto ! important; width: 19%;">
						<input type="text" name="btn_spp_custom_url4" id="btn_spp_custom_url4" style="width:170px" value="' . $btn_spp_custom_url4 . '" placeholder="Button URL ..." >
					</span>
				</div>
			</li>';
					$buttons_array['btn_spp_custom5']      = '<li id="btn_spp_custom5">
				<div class="row-fluid row-fluid-4" >
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="btn_spp_custom5" id="btn_spp_custom5" ' . $btn_spp_custom5 . ' >
							<span>Custom Button 5
								</br></br>
							</span>
						</label>
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" name="btn_spp_custom_color5" id="btn_spp_custom_color5" style="width:90px" value="' . $btn_spp_custom_color5 . '" >
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" placeholder="icon code" value="' . $custom_button_icon5 . '" name="custom_button_icon5" style="width:90px;margin-right: 20px;"><a data-content="Enter your font awesoness icon code (example f1bc). Please visit the cheat sheet here for a complete list of icons you can use: https://fontawesome.com/cheatsheet  " data-original-title="Custom Button Icon" rel="popover" class="icon icon-info-sign spp_icon-auto-publish">&nbsp;</a>
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" name="btn_spp_custom_name5" id="btn_spp_custom_name5" style="width:130px" value="' . $btn_spp_custom_name5 . '" placeholder="Button Label ..." >
					</span>
					<span class="span2"  style="height: auto ! important; width: 19%;">
						<input type="text" name="btn_spp_custom_url5" id="btn_spp_custom_url5" style="width:170px" value="' . $btn_spp_custom_url5 . '" placeholder="Button URL ..." >
					</span>
				</div>
			</li>';
					$buttons_array['btn_spp_custom6']      = '<li id="btn_spp_custom6">
				<div class="row-fluid row-fluid-4" >
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="btn_spp_custom6" id="btn_spp_custom6" ' . $btn_spp_custom6 . ' >
							<span>Custom Button 6
								</br></br>
							</span>
						</label>
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" name="btn_spp_custom_color6" id="btn_spp_custom_color6" style="width:90px" value="' . $btn_spp_custom_color6 . '" >
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" placeholder="icon code" value="' . $custom_button_icon6 . '" name="custom_button_icon6" style="width:90px;margin-right: 20px;"><a data-content="Enter your font awesoness icon code (example f1bc). Please visit the cheat sheet here for a complete list of icons you can use: https://fontawesome.com/cheatsheet  " data-original-title="Custom Button Icon" rel="popover" class="icon icon-info-sign spp_icon-auto-publish">&nbsp;</a>
					</span>                                                                                
					<span class="span2" style="height: auto ! important;">
						<input type="text" name="btn_spp_custom_name6" id="btn_spp_custom_name6" style="width:130px" value="' . $btn_spp_custom_name6 . '" placeholder="Button Label ..." >
					</span>
					<span class="span2"  style="height: auto ! important;">
						<input type="text" name="btn_spp_custom_url6" id="btn_spp_custom_url6" style="width:170px" value="' . $btn_spp_custom_url6 . '" placeholder="Button URL ..." >
					</span>
				</div>
			</li>';
				$buttons_array['spp_LeadBox_btn_checkbox'] = '<li id="spp_LeadBox_btn_checkbox">
			<div class="row-fluid row-fluid-4" >
				<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="spp_LeadBox_btn_checkbox" id="spp_LeadBox_btn_checkbox" ' . $spp_LeadBox_btn_checkbox . ' >
						  <span>LeadPages LeadBox Button
							</br></br>
						  </span> </label>
						</span>
						<span class="span2" style="height: auto ! important;">
							<input type="text" name="spp_LeadBox_btn_color" id="spp_LeadBox_btn_color" style="width:90px" value="' . $spp_LeadBox_btn_color . '" >
						</span>
						<span class="span3"  style="height: auto ! important; width: 19%;">
							<textarea id="spp_LeadBox_btn_code" name="spp_LeadBox_btn_code" style="width: 576px; height: 117px;" class="span11" placeholder=" Paste LeadPages LeadBox HTML code here.">' . htmlspecialchars_decode( $spp_LeadBox_btn_code, ENT_QUOTES ) . '</textarea>
	  
						</span>
			 </div></li>';

					// Temporarily remove the Embed Button until we fix the issue.
					$buttons_array['btn_embed'] = '<li id="btn_embed">
				<div class="row-fluid row-fluid-1" >
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<input type="checkbox" name="btn_embed" id="btn_embed" ' . $btn_embed . ' >
							<span>Embed Player Button
								</br></br>
							</span>
						</label>
					</span>
					<span class="span2" style="height: auto !important;">
						<input type="text" name="btn_embed_color" id="btn_embed_color" style="width:90px" value="' . $btn_embed_color . '" >
					</span>
					<span class="span3" style="height: auto !important;">
						<input type="text" name="btn_embed_name" id="btn_embed_name" value="' . $btn_embed_name . '" style="width:220px" placeholder="Player Embed" >
					</span>
				</div>
			</li>';

					$buttons_order = $this->spp_get_option( 'spp_button_order' );

					// If buttons_order is not set, use default order.
				if ( ! $buttons_order ) {
					$buttons_order = 'btn_itunes,btn_g_play,btn_embed,btn_share,btn_sppreview,btn_spplisten,btn_download,btn_soundcloud,btn_stiticher,btn_sppandroid,btn_spprss,btn_spotify,btn_spp_custom1,btn_spp_custom2,btn_spp_custom3,btn_spp_custom4,btn_spp_custom5,btn_spp_custom6,spp_LeadBox_btn_checkbox';
				}

					$button_order = explode( ',', $buttons_order );

					$ButtonsSorted = array_replace( array_flip( $button_order ), $buttons_array ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase

				foreach ( $ButtonsSorted as $button => $key ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
					$html .= $key;
				}
					$html .= '</ul>
				<div class="row-fluid row-fluid-1">
					<span style="height:auto !important;" class="span3">
						<label>
							<span>Minimum Button Width
								<br><br>
							</span>
						</label>
					</span>
					<span class="span3">
						<input type="text" placeholder="130" value="' . $minbutton_width . '" name="minbutton_width" style="width:105px"> px
					</span>
					<span class="span6">
						<div class="row-fluid">
							<span class="span12">
								<a data-content="Set a minimum button width for the audio player buttons. This is just a minimum size. The actual size of the button may be larger depending on the amount of text inside the button. To set it back to the default value, simply delete the contents of the input box." data-original-title="Button Minimum Width" rel="popover" class="icon icon-info-sign spp_icon-auto-publish">&nbsp;</a>
							</span>
						</div>
					</span>
				</div>  
						
				<div class="row-fluid row-fluid-4">
					<span class="span3" style="height:auto !important;">
						<span>Clutter-Free Transcript Title
							</br></br>
						</span>
					</span>
					<span class="span2" style="height: auto ! important;">
						<input type="text" name="transcript_color" id="transcript_color" style="width:90px" value="' . $transcript_color . '" >
					</span>
					<span class="span7" style="height: auto ! important; width: 19%;">
						<input type="text" name="transcript_txt" id="transcript_txt" style="width:350px" value="' . $transcript_txt . '" placeholder="Enter Transcript Title ..." >
					</span>
				</div> 
					
			</div>';

					$args = array(
						'post_type'      => 'page',
						'posts_per_page' => -1,
						'order'          => 'ASC',
						'orderby'        => 'menu_order',
					);

					$parent = new WP_Query( $args );
					$pages  = '
			<div class="row-fluid">
				<span class="span3" style="height:auto !important;">
					<label class="checkbox">
						<span>Select Privacy Page
							</br></br>
						</span>
					</label>
				</span>
				<span class="span3">
					<select name="spp_privacy_page" id="spp_privacy_page">';

					$pages .= '<option value="disabled"   selected=selected>Disabled</option>';

					if ( $parent->have_posts() ) :
						while ( $parent->have_posts() ) :
							$parent->the_post();
							if ( get_the_ID() === $spp_privacy_page ) {
								$selectedpage = 'selected=selected';
							} else {
								$selectedpage = '';
							}
							$pages .= '<option value="' . get_the_ID() . '"   ' . $selectedpage . '>' . get_the_title() . '</option>';
						endwhile;
			endif;

					$pages .= '</select>
				</span>
			</div>';
					wp_reset_postdata();
					$optin_setting = '
			<div class="well well-1">
				<div class="page-header page-header-1">
					<h1> <small>Opt-in Box Settings</small> 
					</h1>
				</div>
				<div class="row-fluid row-fluid-4">
					<span class="span12">
						<label class="checkbox">
							<input type="checkbox" name="spp_optin_box" id="spp_optin_box" ' . $spp_optin_box . ' />
							<span></span>
							<span>Automatically Add Opt-in Box Below Player
								<br>
							</span>
						</label>
					</span>
				</div>
				' . $pages . '
				<div class="row-fluid" >
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<span>Opt-in Style
								</br></br>
							</span> 
						</label>
					</span>
					<span class="span3">
						<select name="spp_two_step_optin" id="spp_two_step_optin">
							<option value="1" ' . $spp_two_step_optin_selected1 . '>Name and Email</option>
							<option value="2" ' . $spp_two_step_optin_selected2 . '>Email Address Only</option>
							<option value="3" ' . $spp_two_step_optin_selected3 . '>Two Step Opt-in Name and Email</option>
							<option value="4" ' . $spp_two_step_optin_selected4 . '>Two Step Opt-in Email Only</option>
						</select>
					</span>
				</div>  
				<div class="row-fluid privacy_policy_text" style="' . $privacy_policy_text_display . '">
					<span class="span3" style="height:auto !important;">
						<label class="checkbox">
							<span>Privacy Policy Text
								</br></br>
							</span> 
						</label>
					</span>
					<span class="span4">
						<textarea id="privacy_policy_text" name="privacy_policy_text" class="span11" placeholder="I have read and agreed to your Privacy Policy">' . $privacy_policy_text . '</textarea>
					</span>
				</div> 
				<div class="optsettings" style="' . $spp_optin_row_display . '" >
					<div class="row-fluid">
						<span class="span3" style="height:auto !important;">
							<label class="checkbox">
								<span>Headline Text Color
									</br></br>
								</span> 
							</label>
						</span>
						<span class="span4">
							<input type="text" name="headline_text_color" id="headline_text_color" style="width:90px" value="' . $headline_text_color . '" >
						</span>
					</div>  
					<div class="row-fluid">
						<span class="span3" style="height:auto !important;">
							<label class="checkbox">
								<span>Submit Button Color
									</br></br>
								</span> 
							</label>
						</span>
						<span class="span4">
							<input type="text" name="submit_button_color" id="submit_button_color" style="width:90px" value="' . $submit_button_color . '" >
						</span>
					</div>  
					<div class="row-fluid" >
						<span class="span3" style="height:auto !important;">
							<label class="checkbox">
								<span>Submit Button Text Color
									</br></br>
								</span> 
							</label>
						</span>
						<span class="span4">
							<input type="text" name="submit_button_text" id="submit_button_text" style="width:90px" value="' . $submit_button_text . '" >
						</span>
					</div>  
					<div class="row-fluid" >
						<span class="span3" style="height:auto !important;">
							<label class="checkbox">
								<span>Opt-in Box Container Color
									</br></br>
								</span> 
							</label>
						</span>
						<span class="span4">
							<input type="text" name="opt_container_color" id="opt_container_color" style="width:90px" value="' . $opt_container_color . '" >
						</span>
					</div>  
					</br>
					<div class="row-fluid">
						<span class="span5">
							<label class="control-label">Opt-in box headline</label>
							<input class="textinput span12" type="text" name="spp_auto_resp_heading" placeholder="Enter headline here" value="' . $spp_auto_resp_heading_get . '">
						</span>
						<span class="span7">
							<div class="row-fluid">
								<span class="span9 offset1">
									<label class="control-label">Opt-in box sub headline</label>
									<input class="textinput span12" type="text"  placeholder="Enter sub headline here" value="' . $spp_auto_resp_sub_heading_get . '" name="spp_auto_resp_sub_heading" />
								</span>
							</div>
						</span>
					</div>
					<label class="control-label control-label-2">Opt-in form HTML code</label>
					<textarea id="jwsqz_autocode" name="optin_old_code" class="span11" placeholder="Paste your HTML code here.  Opt-in form should only include First Name and Email Address fields.">' . htmlspecialchars_decode( esc_textarea( $optin_old_code_get ), ENT_QUOTES ) . '</textarea>
					<input type="hidden" name="spp_auto_resp_url" id="jwsqz_autoformurl" value="' . $spp_auto_resp_url_get . '">
					<textarea style="display:none;" name="spp_auto_resp_hidden" cols="70" rows="5" id="jwsqz_autohidden">' . htmlspecialchars_decode( $spp_auto_resp_hidden_get, ENT_QUOTES ) . '</textarea>
					<input type="hidden" name="spp_auto_resp_name" id="jwsqz_arnameinput" value="' . htmlspecialchars_decode( $spp_auto_resp_name_get, ENT_QUOTES ) . '">
					<input type="hidden" name="spp_auto_resp_email" id="jwsqz_aremailinput" value="' . htmlspecialchars_decode( $spp_auto_resp_email_get, ENT_QUOTES ) . '"><br>
					<div class="row-fluid">
						<span class="span12">
							<label class="control-label">Submit button text</label>
							<input class="textinput span3" type="text"  placeholder="Enter submit button text here" value="' . stripslashes( $spp_auto_resp_submitt_get ) . '" name="spp_auto_resp_submitt">
						</span>
					</div>
				</div>
			</div>';

					$html .= apply_filters( 'optin_setting', $optin_setting );

					$html .= '<div class="well">
             <div class="page-header page-header-1">
                  <h1> <small>Share Settings</small> 
                  </h1>
                </div>
          
             <div class="row-fluid row-fluid-2">
                <span class="span2" style="height:auto !important;">
                    <label>
                      <span>Tweet This 
                       </br></br>
                      </span> 
                    </label>
                </span>
                <span class="span2" style="height:auto !important;">
                    <input type="text" name="twitter_text_color" id="twitter_text_color" style="width:90px" value="' . $twitter_text_color . '" >
                </span>
                <span class="span3">
                    <input type="text" name="spp-twitter-handle" value="' . $spptwitterhandle . '" placeholder="Twitter Name without @ ..." />
                </span>
                <span class="span3">
                    <div class="row-fluid">
                        <span class="span12">  
                            <a data-content="Choose the color of the stylized Tweet This box and enter Twitter username, without the @ symbol.  The phrase via @username will be added at the end of the clickable tweet quote when shared on Twitter." data-original-title="Tweetable Twitter Name" rel="popover" class="icon icon-info-sign spp_icon-auto-publish">&nbsp;</a>
                        </span>
                    </div>
                </span>
             </div>  
        <div class="row-fluid row-fluid-2">
            <span class="span2" style="height:auto !important;">
                <label>
                    <span>Podcast Episode URL Shortener Prefix
                        </br></br>
                    </span> 
                </label>
            </span>
            <span class="span5">
                <input type="text" name="episode_short_link" value="' . $episode_short_link . '" placeholder="e.g. EPISODE or EP ..." />
            </span>
            <span class="span3">
                <div class="row-fluid">
                    <span class="span12">  
                        <a data-content="This prefix will be automatically added to your shortened URL just before the episode number. For example, if you want all your episode shortened URLs to be yoursite.com/episode1, then enter the prefix \'episode\' in this box.  If you leave it blank, the shortened URL will simply be www.yoursite.com/1." data-original-title="Episode URL Shortener Prefix" rel="popover" class="icon icon-info-sign spp_icon-auto-publish">&nbsp;</a>
                    </span>
                </div>
            </span>
        </div>  
        <div class="row-fluid row-fluid-4">
            <span class="span7">
                <label class="checkbox">
                    <input type="checkbox" name="disable_url_shortner" id="disable_url_shortner" ' . $disable_url_shortner . ' />
                    <span></span>
                    <span>Disable URL Shortener
                        <br>
                    </span>
                </label>
            </span>
            <span class="span1">
                <div class="row-fluid">
                    <span class="span7"> 
                        <a data-content="When this option is selected, the Simple Podcast Press URL shortener is disabled." data-original-title="Disable SPP URL shortener" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
                    </span>
                </div>
            </span>
        </div>
        <div class="row-fluid row-fluid-1">
            <span class="span7">
                <label class="checkbox">
                    <input type="checkbox" name="disable_opengraph" id="disable_opengraph" ' . $disable_opengraph . ' />
                    <span></span>
                    <span>Disable Facebook meta tags generated by SPP
                        <br>
                    </span>
                </label>
            </span>
            <span class="span1">
                <div class="row-fluid">
                    <span class="span7"> 
                        <a data-content="When this option is selected, the Simple Podcast Press will not output the Facebook Meta data.  This is usually disabled when your theme or another plugin already generates the required Facebook meta data." data-original-title="Disable Facebook Open Graph" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
                    </span>
                </div>
            </span>
        </div>          
    </div>
        
    <div class="well well-1">
        <div class="page-header page-header-1">
            <h1> <small>Advanced Tweaks (Most Setups Don\'t Require These Settings)</small></h1>
        </div>
        
        <div class="row-fluid" >
            <span class="span6" style="height:auto !important;">
                <label class="checkbox">
                    <input type="checkbox" name="direct_download_button" id="direct_download_button" ' . $direct_download_button . '>
                    <span></span>
                    <span>Direct Download Button (not compatible with all hosting companies)
                        </br></br>
                    </span> 
                </label>
            </span>
            <span class="span1">
                <div class="row-fluid">
                    <span class="span7"> 
                        <a data-content="When this option is selected, the download button will directly download the MP3 file to listener\'s computer.  Note that some hosting companies have their servers configured to block this option.  If this option doesn\'t work on your site, please uncheck this option" data-original-title="Direct Download URL" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
                    </span>
                </div>
            </span>
        </div>  
        <div class="row-fluid" >
            <span class="span6" style="height:auto !important;">
                <label class="checkbox">
                    <input type="checkbox" name="spp_import_html_desc" id="spp_import_html_desc" ' . $spp_import_html_desc . '>
                    <span></span>
                    <span>Import Full HTML Description
                        </br></br>
                    </span> 
                </label>
            </span>
            <span class="span1">
                <div class="row-fluid">
                    <span class="span7"> 
                        <a data-content="If you notice that the posts are not importing the full HTML description from your podcast feed, then enabled this option.  Note that you will need to delete the existing post, empty trash, RESET the PLUGIN, and attempt to import the post again from the SPP Settings"data-original-title="Import Full HTML Description" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
                    </span>
                </div>
            </span>
        </div>  
        <div class="row-fluid">
            <span class="span6">
                <label class="checkbox">
                    <input type="checkbox" name="spp_force_player_home" id="spp_force_player_home" ' . $spp_force_player_home . ' />
                    <span></span>
                    <span>Force Audio Player on Home Page
                        </br></br>
                    </span>
                </label>
            </span>
            <span class="span1">
                <div class="row-fluid">
                    <span class="span7"> 
                        <a data-content="When this option is selected, the Simple Podcast Press Player is forced to appear on the home page.  This is only required on some themes that do not display the player on the home page. You can choose the location of the player (above or below the post excerpt) in the Podcast Player Styles Settings above or the PowerPress Media Appearance settings if you are replacing the PowerPress player with SPP)" data-original-title="Force SPP Player" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
                    </span>
                </div>
            </span>
        </div>
        <div class="row-fluid">
            <span class="span6">
                <label class="checkbox">
                    <input type="checkbox" name="spp_disable_auto_timestamp" id="spp_disable_auto_timestamp" ' . $spp_disable_auto_timestamp . ' />
                    <span></span>
                    <span>Disable Auto Timestamp Detection
                        </br></br>
                    </span>
                </label>
            </span>
            <span class="span1">
                <div class="row-fluid">
                    <span class="span7"> 
                        <a data-content="When this option is selected, timestamps on the page will no longer be automatically detected and made clickable to advance the player to a specific time.  You can still use the Clickable Timestamp toolbar button and shortcode to make individual timestamps clickable." data-original-title="Disable Auto Timestamp" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
                    </span>
                </div>
            </span>
        </div>
        <div class="row-fluid">
            <span class="span6">
                <label class="checkbox">
                    <input type="checkbox" name="spp_remove_brackets_timestamp" id="spp_remove_brackets_timestamp" ' . $spp_remove_brackets_timestamp . ' />
                    <span></span>
                    <span>Remove Brackets From Auto Detected Timestamps
                        </br></br>
                    </span>
                </label>
            </span>
            <span class="span1">
                <div class="row-fluid">
                    <span class="span7"> 
                        <a data-content="Disable the brackets that automatically to the timestamps which get added by SPP by default." data-original-title="Remove Brackets From Auto Detected Timestamps" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
                    </span>
                </div>
            </span>
        </div>  
        <div class="row-fluid">
            <span class="span6">
                <label class="checkbox">
                    <input type="checkbox" name="spp_resolve_player_conflict" id="spp_resolve_player_conflict" ' . $spp_resolve_player_conflict . ' />
                    <span></span>
                    <span>Fix Conflict With Other Podcast Players
                        </br></br>
                    </span>
                </label>
            </span>
            <span class="span1">
                <div class="row-fluid">
                    <span class="span7"> 
                        <a data-content="This option attempts to fix conflicts with Smart Podcast Player and other podcast players so they can work together with SPP. Do not use unless have been instructed by our support team." data-original-title="Fix Conflict With Other Podcast Players" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
                    </span>
                </div>
            </span>
        </div>  
        <div class="row-fluid">
            <span class="span6">
                <label class="checkbox">
                    <input type="checkbox" name="force_ssl_for_audio" id="force_ssl_foraudio" ' . $force_ssl_for_audio . ' />
                    <span></span>
                    <span>Force SSL for audio files
                        </br></br>
                    </span>
                </label>
            </span>
            <span class="span1">
                <div class="row-fluid">
                    <span class="span7"> 
                        <a data-content="Force SPP to use the secure (HTTPS) vesrion of your audio files.  Required when your site uses HTTPS but the feed your connected original is not secure." data-original-title="Force HTTPS for audio files" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
                    </span>
                </div>
            </span>
        </div>
        <div class="row-fluid" >
            <span class="span6" style="height:auto !important;">
                <label class="checkbox">
                    <input type="checkbox" name="disable_jquery_model" id="disable_jquery_model" ' . $disable_jquery_model . '>
                    <span></span>
                    <span>Disable All jQuery Modals 
                        </br></br>
                    </span> 
                </label>
            </span>
            <span class="span1">
                <div class="row-fluid">
                    <span class="span7"> 
                        <a data-content="Disable all jQuery Modals. Only enable if you have a conflict with other popups on your site" data-original-title="Disable jQuery Modals" rel="popover" class="icon icon-info-sign icon-spp-description">&nbsp;</a>
                    </span>
                </div>
            </span>
        </div>  
    </div> 
              
    <div class="well">
        <div class="row-fluid">
            <span class="span7">
                <table>
                    <tr>
                        <td>
                            <input type="submit" class="btn btn-primary" name="spp_cbsn_save" id="spp_cbsn_save" value="Save Changes"> 
                        </td>
                        <td>
                            <span class="saved"></span>
                        </td>
                    </tr>
                </table>
            </span>
            <span class="span3">
                <table>
                    <tr>
                        <td>
                            <input type="submit" class="btn" name="spp_cbsn_reset" id="spp_cbsn_reset" value="Reconnect My Podcast Feed">
                    	</td>
                    	<td>
                            <span class="reset"></span>
                    	</td>
                    </tr>
                </table>
            </span>
            </div>
            </div>              
			<div class="alert alert-1" id="saved" style="display:none;"> </div>';
					if ( $podcastURL_Feed ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
						$html .= '
                          <div class="well well-1 well-2">
                            <div class="page-header page-header-1">
                              <h1> <small>Manual Import (Required for Initial Import)</small> 
                              </h1>
                            </div>
                            <p><strong>The first time you install and configure this plugin, you will need to manually import all your podcasts.  This can be done using the button below.</strong>
                    </br></br>Under most circumstances, you won\'t need to use this button. &nbsp;Simple Podcast Press with automatically check for new podcasts (once an hour) &nbsp;and import&nbsp;them automatically. &nbsp;If you need to import new podcasts sooner than that, you can use the button below.</p><p><strong>Note: Depending on how many podcasts you have, it may take several minutes for the import to complete.</strong></p>
                            <div class="row-fluid">
                              <span class="span7">
                                <table>
                                <tr>
                                <td>
                                <input type="submit" class="btn" name="spp_chk_new_vids" id="spp_chk_new_vids" value="Import New Episodes">
                                </td>
                                <td>
                                        <span  id="fetchresults"></span>
                                </td>
                                <td>
                                <input style="display:none; color:red;" type="submit" class="btn" name="cancel_import" id="cancel_import" value="Cancel Import New Episodes">
                                </td>
                                </tr>
                                </table>
                              </span>
                            <span class="span3">
                                <table>
                                <tr>
                                <td>
                                <input style="" type="submit" class="btn" name="update_episode_table" id="update_episode_table" value="Update Episodes Table">
                                </td>
                                <td>
                                <span  id="fetchresultsepisode"></span>
                                </td>
                                </tr>
                                </table>
                              </span>
                            </div>
                          </div>
                          <div class="alert fetchresults fetchcomments" style="display:none;">      
                        </div>
                    ';
					}
			}
		} else {
			$html = '
        Oops. Your Simple Podcast Press License Hasn\'t Been Activated Yet.  <a href="admin.php?page=spp-license">Click Here to Activate It Now.</a></br></br>
        Don\'t Have a Full License Yet? <a target="_blank" href="https://simplepodcastpress.com">Grab One from Here.</a></br></br>
        Need Help? Send us an email to <a href="mailto:support@simplepodcastpress.com">support@simplepodcastpress.com</a> and we will be more than happy to help you.<br><br>
        Love It? Please share on <a href="https://www.facebook.com/sharer/sharer.php?u=https://simplepodcastpress.com/?ref=' . $refID . '" target="_blank">Facebook</a> and <a href="https://twitter.com/intent/tweet?text=I%20love%20%23SimplePodcastPress%20a%20%23podcast%20player%20that%20builds%20your%20list%20and%20grows%20your%20audience%20on%20autopilot%20%2D&url=https://simplepodcastpress.com/?ref=' . $refID . '" target="_blank">Twitter</a> (automatically shares your affiliate link)'; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase,WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			// Load the License Activator Page if no valid license is found.

		}
		echo apply_filters( 'spp_panel_html', $html ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
	}
	/**
	 * Display the plugin review page.
	 */
	public function spp_plugin_review_page() {
		// Check if the license is valid or in trial mode.
		$is_license_valid = get_option( 'sppress_ls' );

		if ( ( 'valid' === $is_license_valid ) || ( 'trial' === $is_license_valid ) ) {
			global $wpdb;
			$refID             = $this->spp_get_option( 'spp_poweredby_url' ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$table_spp_reviews = $wpdb->prefix . 'spp_reviews';
			$spp_channel_image = $this->spp_get_option( 'channel_image' );

			// Retrieve reviews from the database.
			$spp_reviews   = $wpdb->get_results( 'SELECT * FROM ' . $table_spp_reviews . ' ORDER BY rw_id DESC' );
			$reviews_count = count( $spp_reviews );

			// Generate HTML for the review page.
			$html  = '
			<div class="container-fluid">
				<div class="well well-1">
					<div class="row-fluid">
						<span class="span8">
							<div class="row-fluid">
								<div class="spp-logo">
									<!-- <img src="../wp-content/plugins/simple-podcast-press/spp_view/img/spp-logo-lg.png" class="image" style="margin-top:20px;"> -->
								</div>
						
								<span class="span10">
									Need Help? Check out the <a href="https://www.simplepodcastpress.com/quickstartguide.pdf" target="_new">Quick Start Guide</a> or <a href="mailto:support@simplepodcastpress.com">Email Our Support Team</a>.  We are here for you.<br><br>
									Love It? Please share on <a href="https://www.facebook.com/sharer/sharer.php?u=https://simplepodcastpress.com/?ref=' . $refID . '" target="_blank">Facebook</a> and <a href="https://twitter.com/intent/tweet?text=I%20love%20%23SimplePodcastPress%20a%20%23podcast%20player%20that%20builds%20your%20list%20and%20grows%20your%20audience%20on%20autopilot%20%2D&url=https://simplepodcastpress.com/?ref=' . $refID . '" target="_blank">Twitter</a> (automatically shares your affiliate link)'; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
			$html .= '					</span>
							</div>
						</span>
						<span class="span4">
							<div class="row-fluid">
								<span class="span8">
									<p></p>
								</span>
								<span class="span4 userimage">
									<!-- <img src="' . $spp_channel_image . '" class="image" style="width:100px;"> -->
									<img src="' . $spp_channel_image . '" class="imagenotround" style="width:100px;">
								</span>
							</div>
						</span>
					</div>
				</div>
				<div class="well well-1">
					<div class="page-header page-header-2">
						<h1><small>International Apple Podcast Reviews (' . $reviews_count . ')</small></h1>
						<h3><small>Your Apple Podcast Reviews will update automatically every 24 hours</small></h3>
					</div>
					<div class="row-fluid">';

					$counter = 0;
			foreach ( $spp_reviews as $spp_review ) {

				$html  .= '<table><tr><td>';
				$html  .= '<strong>' . $spp_review->rw_title . '</strong>&nbsp;&nbsp;&nbsp;&nbsp;';
				$rating = $spp_review->rw_ratings;
				switch ( $rating ) {
					case 5:
						$html .= '<img src="' . SPPRESS_PLUGIN_URL . '/icons/rating_stars/5star.png" />';
						break;
					case 4:
						$html .= '<img src="' . SPPRESS_PLUGIN_URL . '/icons/rating_stars/5star.png" />';
						break;
					case 3:
						$html .= '<img src="' . SPPRESS_PLUGIN_URL . '/icons/rating_stars/3star.png" />';
						break;
					case 2:
						$html .= '<img src="' . SPPRESS_PLUGIN_URL . '/icons/rating_stars/2star.png" />';
						break;
					case 1:
						$html .= '<img src="' . SPPRESS_PLUGIN_URL . '/icons/rating_stars/1star.png" />';
						break;
					default:
						$html .= '<img src="' . SPPRESS_PLUGIN_URL . '/icons/rating_stars/00star.png" />';
						break;
				}
				$publishedat = $spp_review->rw_published_date;
				$PublishDate = gmdate( 'F j, Y', strtotime( $publishedat ) ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$html       .= '</td></tr><tr><td>';
				$html       .= $PublishDate . ' by <b>' . $spp_review->rw_author . '</b> from ' . $this->get_full_country_name( $spp_review->rw_coutry ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$html       .= '</td></tr><tr><td>';
				$html       .= $spp_review->rw_text . '</br></br>';
				$html       .= '</td></tr></table>';
			}

					$html .= '</div>
				</div>
			</div>';

			// Output the HTML with applied filters.
			echo apply_filters( 'spp_reviews_panel_html', $html ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		} else {
			// Display a message if the license is not valid or not in trial mode.
			$html = '
            </br>Oops. Your Simple Podcast Press License Hasn\'t Been Activated Yet.  <a href="admin.php?page=spp-license">Click Here to Activate It Now.</a>
        ';

			// Load the License Activator Page if no valid license is found.
			echo apply_filters( 'spp_reviews_panel_html', $html ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		}
	}

	/**
	 * Function to handle setting and updating the Simple Podcast Press Post Type.
	 */
	public function spp_post_type() {
		if ( isset( $_POST['posttype_txt_box'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Missing
			// Sanitize and set the post type option.
			$this->spp_settings['posttype_txt_box'] = sanitize_text_field( wp_unslash( $_POST['posttype_txt_box'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Missing,WordPress.Security.NonceVerification.Missing
			$posttype_txt_box                       = str_replace( ' ', '', strtolower( $this->spp_settings['posttype_txt_box'] ) );
			$this->spp_settings['spp_cpt_select']   = $posttype_txt_box;
			update_option( 'spp_settings', $this->spp_settings );
		}

		// Get and set the post type.
		$posttype_txt_box = $this->spp_get_option( 'posttype_txt_box' );
		$posttype_txt_box = ( empty( $posttype_txt_box ) ? 'podcast' : $posttype_txt_box );

		// Build the HTML form.
		$html  = '<h2>Simple Podcast Press Post Type</h2><br class="clear" />';
		$html .= '<form action="' . esc_url( get_bloginfo( 'wpurl' ) . '/wp-admin/admin.php?page=spp-post-type' ) . '" method="post" id="spp_url_shortner">';
		$html .= '<p><b>' . esc_html( __( 'Enter New Post Type:&nbsp; &nbsp;&nbsp;&nbsp;&nbsp;', 'spp' ) ) . ' </b><input class="span5" type="text" name="posttype_txt_box" value="' . esc_attr( $posttype_txt_box ) . '" size="61" placeholder="podcast"></p>';
		$html .= '<div style="margin-top: 25px;margin-bottom:15px"><table><tr><td>';
		$html .= '<input type="submit" id="posttype_submit" class="button button-primary" name="posttype_submit" value="' . esc_attr( __( 'Submit', 'spp' ) ) . '"/>';
		$html .= '</td></tr></table></div></form>';

		// Output the HTML form.
		echo $html; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
	}

	/**
	 * Function to handle displaying Simple Podcast Press podcasts page.
	 */
	public function spp_spp_podcasts_page() {
		global $wpdb;
		$table_spp_podcast = $wpdb->prefix . 'spp_podcast';
		$post_meta_table   = $wpdb->prefix . 'postmeta';
		$s_html            = '';
		$podcast_items = $wpdb->get_results('SELECT * FROM ' . $table_spp_podcast . ' ORDER BY STR_TO_DATE(pc_published_date, "%a, %d %b %Y %H:%i:%s") DESC');

		$dbsearchhtml      = '<table>
        <thead>
            <th>Full or Partial Audio URL</th>
        </thead>
        <tbody>
        <tr>
            <td>
                <form method="get" action="#dbsearch">
                    <input type="hidden" value="spp-podcasts-table" name="page">
                    <input type="text" name="search" >
               </form> 
            </td>
        </tr>
            </tbody>
        </table>
        ';

		// Check if search is performed.
		if ( isset( $_GET['search'] ) ) {
			$s             = sanitize_text_field( wp_unslash( $_GET['search'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$results       = $wpdb->get_results( $wpdb->prepare( 'SELECT * FROM '.$post_meta_table .' WHERE meta_value LIKE %s', '%' . $s . '%' ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended,WordPress.DB.DirectDatabaseQuery.DirectQuery, phpcs:ignore WordPress.DB.DirectDatabaseQuery.NoCaching
			$dbsearchhtml .= '<table id="mytable" class="table table-bordred table-striped"> // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
                        <thead>
                            <th>Post ID</th>
                            <th>Meta Key</th>
                            <th>Meta Value</th> 
                        </thead> 
                        <tbody>';
			foreach ( $results as $result ) {
				$postid        = $result->post_id;
				$meta_key      = $result->meta_key;
				$meta_value    = $result->meta_value;
				$dbsearchhtml .= '
                <tr>
                    <td><a href="' . esc_url( get_site_url() ) . '/wp-admin/post.php?post=' . $postid . '&action=edit">' . $postid . '</a></td>
                    <td>' . $meta_key . '</td>
                    <td>' . $meta_value . '</td>
                </tr>
                ';
			}
			$dbsearchhtml .= '</tbody></table>';
		}

		$podcasttable = '
        <div class="container">
            <div class="row">
                <div class="col-md-12">
                    <div class="table-responsive">
                        <table id="mytable" class="table table-bordred table-striped">
                            <thead>
                                <th>Title</th>
                                <th class="col-md-3">Publish Date</th>
                                <th>Description</th>
                                <th>Keywords</th>
                                <th>Image</th>
                                <th>Episode URL</th>
                            </thead>
                            <tbody>';

		// Loop through podcast items.
		foreach ( $podcast_items as $p_item ) {
			$podcasttable .= ' <tr>
                <td>' . $p_item->pc_title . '</td>
                <td>' . $p_item->pc_published_date . '</td>
                <td>' . $p_item->pc_description . '</td>
                <td>' . $p_item->pc_episode_keywords . '</td>
                <td><img src="' . $p_item->pc_episode_image . '" width="200" /></td>
                <td>' . $p_item->pc_audio_file . '</td>
            </tr>';
		}

		$podcasttable .= '
                            </tbody>
                        </table>
                    </div>
                </div>
            </div>
        </div>';

		// Build the HTML content.
		$html = '
        <div id="tabs">
            <ul>
                <li><a href="#podcast-table">Podcast Episodes</a></li>
                <li><a href="#dbsearch">Search</a></li>
            </ul>
            <div id="podcast-table">' . $podcasttable . '</div>
            <div id="dbsearch">' . $dbsearchhtml . '</div>
        </div>
        <script type="text/javascript">
            jQuery(document).ready(function(){
                jQuery( "#tabs" ).tabs();
            });
        </script>
    ';

		// Output the HTML content.
		echo $html; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
	}
	public function spp_url_shortner_page() {
		$spp_channel_image = $this->spp_get_option( 'channel_image' );
		$refID             = $this->spp_get_option( 'spp_poweredby_url' );
		$fixed             = $this->spp_get_option( 'spp_links_table_fixed' );
		?>
<script data-cfasync="false" type="text/javascript">
jQuery(document).ready(function($){
	$("#spp_url_shortner_button").click(function() {
				var plug_url = "
				<?php
				echo SPPRESS_PLUGIN_URL;
				?>
				";
			$('.saved').html('<img src="'+plug_url+'/spp_view/img/loading1.gif" title="loading" style="padding-left: 15px;">'); 
				var data = jQuery('#spp_url_shortner').serialize();
			<?php
			$nonce = wp_create_nonce( 'urlshortner_settings' );
			?>
			data += '&wpnonce_urlshortner=
			<?php
			echo $nonce;
			?>
			' ;
		
				$.post(ajaxurl, data, function(response) {
	
				
				$('body').load(window.location.href);
				$('.saved').html(''); 
				});
		
		return false;
		});
});
</script>
		
<form action="
		<?php
		echo get_bloginfo( 'wpurl' );
		?>/wp-admin/admin.php?page=spp-url-shortner" method="post" id="spp_url_shortner">
		<input type="hidden" name="action" value="spp_url_shortner" />
		<?php
		$html = '
<div class="container-fluid">
      <div class="well well-1">
        <div class="row-fluid">
          <span class="span8">
            <div class="row-fluid">
              <div class="spp-logo">
                <!-- <img src="../wp-content/plugins/simple-podcast-press/spp_view/img/spp-logo-lg.png" class="image" style="margin-top:20px;"> -->
              </div>
              
          
        
          
          
              <span class="span10">
                                 Need Help? Check out the <a href="https://www.simplepodcastpress.com/quickstartguide.pdf" target="_new">Quick Start Guide</a> or <a href="mailto:support@simplepodcastpress.com">Email Our Support Team</a>.  We are here for you.<br><br>
                                 Love It? Please share on <a href="https://www.facebook.com/sharer/sharer.php?u=https://simplepodcastpress.com/?ref=' . $refID . '" target="_blank">Facebook</a> and <a href="https://twitter.com/intent/tweet?text=I%20love%20%23SimplePodcastPress%20a%20%23podcast%20player%20that%20builds%20your%20list%20and%20grows%20your%20audience%20on%20autopilot%20%2D&url=https://simplepodcastpress.com/?ref=' . $refID . '" target="_blank">Twitter</a> (automatically shares your affiliate link)
              </span>
            </div>
          </span>
          <span class="span4">
            <div class="row-fluid">
            <span class="span8">
                <p></p>
              </span>
              <span class="span4 userimage" > 
                <!-- <img src="' . $spp_channel_image . '" class="image" style="width:100px;"> -->
              </span>
            </div>
          </span>
        </div>
      </div>
      <div class="well well-1">
        <div class="page-header page-header-2">
          <h1> <small>URL Shortener</small> 
          </h1>
        </div>';

		if ( ! isset( $_GET['edit'] ) ) {
			$html .= '<div class="row-fluid"><span class="span7">
                    <table>
                    <tr>
                    <td>
                       <input type="submit" class="btn btn-primary" name="spp_url_shortner_button" id="spp_url_shortner_button" value="Automatically Fix URL Shortener Links">
                    </td>
                    <td>
                            <span class="saved"></span>
                    </td>
                    </tr>
                    </table>
                  </span></div></form>';
		}
		echo $html;
		echo '<div class="row-fluid">';
		$this->spp_links();
		if ( empty( $_GET['edit'] ) ) {
			/**Display the data into the Dashboard*/
			$this->spp_manage_links();
		} else {
			/**Display a form to add or update the data*/
			$this->spp_add_link();
		}
		echo '</div></div>';
	}
	/**
	 * Redirects to the pretty link if found.
	 */
	public function spp_redirect() {
		// Remove the trailing slash if there is one.
		if ( isset( $_SERVER['REQUEST_URI'] ) ) {
			$request_uri = preg_replace( '#/$#', '', urldecode( wp_unslash( $_SERVER['REQUEST_URI'] ) ) ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}
		$link_info = '';
		if ( $link_info = $this->spp_is_pretty_link( $request_uri, false ) ) {
			$params = isset( $link_info['pretty_link_params'] ) ? $link_info['pretty_link_params'] : '';
			if (is_object($link_info['pretty_link_found']) && isset($link_info['pretty_link_found']->spp_slug)) {
				$this->spp_link_redirect_from_slug($link_info['pretty_link_found']->spp_slug);
			} 
		}
	}

	/**
	 * Checks if the provided URL is a pretty link.
	 *
	 * @param string $url           The URL to check.
	 * @param bool   $check_domain  Whether to check the domain.
	 * @return array|bool           An array with pretty link info if found, false otherwise.
	 */
	public function spp_is_pretty_link( $url, $check_domain = true ) {
		$pretty_link_params = '';
		$spp_blogurl        = ( ( get_option( 'home' ) ) ? get_option( 'home' ) : get_option( 'siteurl' ) );

		if ( ! $check_domain || preg_match( '#^' . preg_quote( $spp_blogurl, '#' ) . '#', $url ) ) {
			$uri = preg_replace( '#' . preg_quote( $spp_blogurl, '#' ) . '#', '', $url );

			// Resolve WP installs in sub-directories.
			preg_match( '#^(https?://.*?)(/.*)$#', $spp_blogurl, $subdir );

			$struct     = $this->spp_get_permalink_pre_slug_regex();
			$subdir_str = ( isset( $subdir[2] ) ? $subdir[2] : '' );
			$match_str  = '#^' . $subdir_str . '(' . $struct . ')([^\?]*?)([\?].*?)?$#';

			if ( preg_match( $match_str, $uri, $match_val ) ) {
				// Match longest slug -- this is the most common.
				$params            = ( isset( $match_val[3] ) ? $match_val[3] : '' );
				$pretty_link_found = '';
				
				if ( $pretty_link_found == $this->spp_is_pretty_link_slug( $match_val[2] ) ) {
					return compact( 'pretty_link_found', 'pretty_link_params' );
				}
				
				// Trim down the matched link.
				$matched_link = preg_replace( '#/[^/]*?$#', '', $match_val[2], 1 );
				// Cycle through the links (maximum depth 25 folders so we don't get out of control).

				for ( $i = 0; $i < 25 && $matched_link && ! empty( $matched_link ); $i++ ) {
					$new_match_str = "#^{$subdir_str}({$struct})({$matched_link})(.*?)?$#";
					$params        = ( isset( $match_val[3] ) ? $match_val : '' );
					$pretty_link_found = $this->spp_is_pretty_link_slug( $match_val[2] );
					if ( $pretty_link_found ) {
						return compact( 'pretty_link_found', 'pretty_link_params' );
					}

					// Trim down the matched link and try again.
					$matched_link = preg_replace( '#/[^/]*$#', '', $match_val[2], 1 );
				}
			}
		}

		return false;
	}

	/**
	 * Checks if the provided slug is a pretty link.
	 *
	 * @param string $slug The slug to check.
	 * @return mixed|object The pretty link object if found, or false if not found.
	 */
	public function spp_is_pretty_link_slug( $slug ) {
		return apply_filters( 'spp_check_if_slug', $this->getOneFromSlug( urldecode( $slug ) ), urldecode( $slug ) );
	}

	/**
	 * Retrieves a link from the database based on its slug.
	 *
	 * @param string $slug The slug of the link.
	 * @return mixed|object The link object if found, or false if not found.
	 */
	public function getOneFromSlug( $slug ) { // phpcs:ignore WordPress.NamingConventions.ValidFunctionName.MethodNameInvalid
		global $wpdb;

		$table_spp_links = $wpdb->prefix . 'spp_links';
		$query           = $wpdb->prepare( "SELECT * FROM {$table_spp_links} WHERE spp_slug='%s'", $slug ); // phpcs:ignore WordPress.DB.PreparedSQLPlaceholders.QuotedSimplePlaceholder,WordPress.DB.PreparedSQL.InterpolatedNotPrepared
		$link            = $wpdb->get_row( $query ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching

		return $link;
	}

	/**
	 * Retrieves the pre-slug URI from the permalink structure.
	 *
	 * @param bool $force Force retrieving the pre-slug URI.
	 * @param bool $trim Trim the pre-slug URI.
	 * @return string The pre-slug URI.
	 */
	public function spp_get_permalink_pre_slug_uri( $force = false, $trim = false ) {
		global $prli_options;

		if ( $force ) {
			preg_match( '#^([^%]*?)%#', get_option( 'permalink_structure' ), $struct );
			$pre_slug_uri = $struct[1];

			if ( $trim ) {
				$pre_slug_uri = trim( $pre_slug_uri );
				$pre_slug_uri = preg_replace( '#^/#', '', $pre_slug_uri );
				$pre_slug_uri = preg_replace( '#/$#', '', $pre_slug_uri );
			}

			return $pre_slug_uri;
		} else {
			return '/';
		}
	}

	/**
	 * Retrieves the pre-slug URI regex pattern from the permalink structure.
	 *
	 * @return string The pre-slug URI regex pattern.
	 */
	public function spp_get_permalink_pre_slug_regex() {
		$pre_slug_uri = $this->spp_get_permalink_pre_slug_uri( true );

		if ( empty( $pre_slug_uri ) ) {
			return '/';
		} else {
			return "{$pre_slug_uri}|/";
		}
	}
	/**
	 * Redirects to a link based on its slug.
	 *
	 * @param string $slug The slug of the link.
	 * @return void
	 */
	public function spp_link_redirect_from_slug( $slug ) {
		$link = $this->getOneFromSlug( urldecode( $slug ) );

		if ( isset( $link->spp_slug ) && ! empty( $link->spp_slug ) ) {
			$custom_get = $_GET; // phpcs:ignore WordPress.Security.NonceVerification.Recommended

			$this->spp_track_link( $link->spp_slug, $custom_get );
			exit;
		}
	}

	/**
	 * Tracks link and performs a redirect.
	 *
	 * @param string $slug The slug of the link.
	 * @return void
	 */
	public function spp_track_link( $slug ) {
		global $wpdb;
		$table_spp_links = $wpdb->prefix . 'spp_links';

		$query       = $wpdb->prepare( "SELECT * FROM {$table_spp_links} WHERE spp_slug=%s LIMIT 1", $slug );
		$spp_link    = $wpdb->get_row( $query ); // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
		$link_target = apply_filters(
			'spp_target_url',
			array(
				'spp_url' => $spp_link->spp_url,
				'link_id' => $spp_link->spp_id,
			)
		);
		$link_url    = $link_target['spp_url'];
		header( 'HTTP/1.1 301 Moved Permanently' );
		header( 'Location: ' . $link_url );
	}

	/**
	 * Sets up cron schedule for podcast updates.
	 *
	 * @param array $schedules Existing cron schedules.
	 * @return array Modified cron schedules.
	 */
	public function podcast_cron_setup( $schedules ) {
		$schedules['simplepodcastpress-cron'] = array(
			'interval' => 60,
			'display'  => __( 'Every Minute' ),
		);
		return $schedules;
	}

	/**
	 * Installs the database tables for Simple Podcast Press.
	 *
	 * @return void
	 */
	public function install_spp_db() {
		global $wpdb;
		$podcast_feed_url = $this->spp_get_option( 'podcast_url' );

		include_once ABSPATH . 'wp-admin/includes/upgrade.php';

		if ( $podcast_feed_url ) {
			$sppress_collate         = ' COLLATE utf8_general_ci';
			$table_spp_podcast       = $wpdb->prefix . 'spp_podcast';
			$table_spp_podcast_query = "CREATE TABLE IF NOT EXISTS {$table_spp_podcast} (
            pc_id bigint(20) NOT NULL AUTO_INCREMENT,
            pc_title text NOT NULL,
            pc_published_date varchar(50) NOT NULL,
            pc_description longtext NOT NULL,
            pc_audio_file varchar(500) NOT NULL,
            pc_audio_duration varchar(100) NOT NULL,
            pc_audio_length varchar(100) NOT NULL,
            pc_episode_keywords varchar(500) NOT NULL,
            pc_episode_image varchar(500) NOT NULL,
            pc_libsyn_image varchar(500) NOT NULL,
            pc_btr_offsite_url varchar(500) NOT NULL,
            pc_btr_link varchar(500) NOT NULL,
            PRIMARY KEY (pc_id)
        )" . $sppress_collate . ';';

			dbDelta( $table_spp_podcast_query );
		}

		$table_spp_links       = $wpdb->prefix . 'spp_links';
		$table_spp_links_query = "CREATE TABLE IF NOT EXISTS {$table_spp_links} (
        spp_id int(11) NOT NULL auto_increment,
        spp_name varchar(255) default NULL,
        spp_url text default NULL,
        spp_slug varchar(255) default NULL,
        spp_post_id bigint(20) default NULL,
        PRIMARY KEY (spp_id)
    )" . $sppress_collate . ';';

		dbDelta( $table_spp_links_query );

		$table_spp_reviews       = $wpdb->prefix . 'spp_reviews';
		$table_spp_reviews_query = "CREATE TABLE IF NOT EXISTS {$table_spp_reviews} (
        id int(11) NOT NULL auto_increment,
        rw_id bigint default NULL,
        rw_coutry varchar(50) default NULL,
        rw_published_date varchar(50) default NULL,
        rw_title varchar(255) default NULL,
        rw_text longtext default NULL,
        rw_ratings varchar(255) default NULL,
        rw_author varchar(255) default NULL,
        PRIMARY KEY (id)
    )" . $sppress_collate . ';';

		dbDelta( $table_spp_reviews_query );

		$table_spp_episode       = $wpdb->prefix . 'spp_episode';
		$table_spp_episode_query = "CREATE TABLE IF NOT EXISTS {$table_spp_episode} (
        id int(11) NOT NULL auto_increment,
        postid bigint default NULL,
        post_date varchar(50) default NULL,
        posttitle varchar(255) default NULL,
        downloadurl varchar(500) default NULL,
        podcast_table_id bigint default NULL,
        PRIMARY KEY (id)
    )" . $sppress_collate . ';';

		dbDelta( $table_spp_episode_query );
	}

	/**
	 * Checks trial expiration and updates options accordingly.
	 *
	 * @return void
	 */
	public function sppress_tcheck() {
		$sppress_ls  = get_option( 'sppress_ls' );
		$sppress_vdr = get_option( 'sppress_vdr' );
		$sppress_ted = get_option( 'sppress_ted' );
		$sppress_ky  = get_option( 'edd_sppress_license_key' );
		$domain      = get_site_url();
		$domain      = wp_parse_url( $domain );
		$domain      = $domain['host'];

		if ( $sppress_vdr && $sppress_ted && ! $sppress_ls && $domain !== $sppress_vdr && ! $sppress_ky ) {
			$ct = time();
			update_option( 'sppress_ed', (int) $ct + (int) $sppress_ted );
		}

		$ed = get_option( 'sppress_ed' );

		if ( $ed ) {
			$ct = time();

			if ( $ct < $ed ) {
				update_option( 'sppress_ls', 'trial' );
				$this->spp_settings['spp_disable_poweredby'] = 0;
				$this->spp_settings['spp_poweredby_url']     = 0;
				update_option( 'spp_settings', $this->spp_settings );

				add_action(
					'admin_notices',
					array(
						$this,
						'trial_expire_soon',
					)
				);
			} else {
				update_option( 'sppress_ls', 'trialexpired' );

				add_action(
					'admin_notices',
					array(
						$this,
						'trial_expired',
					)
				);
			}
		}
	}

	/**
	 * Update the database if needed.
	 *
	 * @return void
	 */
	public function sppress_updatedb() {
		global $wpdb, $spp_db_version;
		$table_spp_podcast = $wpdb->prefix . 'spp_podcast';

		$installed_version = get_option( 'spp_db_version' );

		if ( $installed_version !== $spp_db_version ) {
			$myPodcastTable = $wpdb->get_row( 'SELECT * FROM ' . $table_spp_podcast ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase,WordPress.DB.PreparedSQL.NotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching

			// Add columns if missing.
			if ( ! isset( $myPodcastTable->pc_libsyn_image ) ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$wpdb->query( "ALTER TABLE {$table_spp_podcast} ADD pc_libsyn_image varchar(255) NOT NULL" ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.SchemaChange,WordPress.DB.PreparedSQL.InterpolatedNotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
			}
			if ( ! isset( $myPodcastTable->pc_btr_offsite_url ) ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$wpdb->query( "ALTER TABLE {$table_spp_podcast} ADD pc_btr_offsite_url varchar(255) NOT NULL" ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.SchemaChange,WordPress.DB.PreparedSQL.InterpolatedNotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
			}
			if ( ! isset( $myPodcastTable->pc_btr_url ) ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase
				$wpdb->query( "ALTER TABLE {$table_spp_podcast} ADD pc_btr_url varchar(255) NOT NULL" ); // phpcs:ignore WordPress.DB.DirectDatabaseQuery.SchemaChange,WordPress.DB.PreparedSQL.InterpolatedNotPrepared,WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching
			}

			update_option( 'spp_db_version', $spp_db_version );
		}
	}
	/**
	 * Adjusts the brightness of a given hexadecimal color code.
	 *
	 * @param string $hex   The hexadecimal color code (e.g., #RRGGBB).
	 * @param int    $steps The number of steps to adjust the brightness.
	 *                      Negative values darken the color, positive values lighten it.
	 *
	 * @return string The adjusted hexadecimal color code.
	 */
	public static function sppressAdjBrightness( $hex, $steps ) { // phpcs:ignore WordPress.NamingConventions.ValidFunctionName.MethodNameInvalid
		// Steps should be between -255 and 255. Negative = darker, positive = lighter.
		$steps = max( -255, min( 255, $steps ) );

		// Format the hex color string.
		$hex = str_replace( '#', '', $hex );
		if ( strlen( $hex ) === 3 ) {
			$hex = str_repeat( substr( $hex, 0, 1 ), 2 ) . str_repeat( substr( $hex, 1, 1 ), 2 ) . str_repeat( substr( $hex, 2, 1 ), 2 );
		}

		// Get decimal values.
		$r = hexdec( substr( $hex, 0, 2 ) );
		$g = hexdec( substr( $hex, 2, 2 ) );
		$b = hexdec( substr( $hex, 4, 2 ) );

		// Adjust number of steps and keep it inside 0 to 255.
		$r = max( 0, min( 255, $r + ( $r * ( $steps / 255 ) ) ) );
		$g = max( 0, min( 255, $g + $steps ) );
		$b = max( 0, min( 255, $b + $steps ) );

		$r_hex = str_pad( dechex( (int) $r ), 2, '0', STR_PAD_LEFT );
		$g_hex = str_pad( dechex( (int) $g ), 2, '0', STR_PAD_LEFT );
		$b_hex = str_pad( dechex( (int) $b ), 2, '0', STR_PAD_LEFT );

		return '#' . $r_hex . $g_hex . $b_hex;
	}

	/**
	 * Activate the Simple Podcast Press plugin.
	 *
	 * @return void
	 */
	public function simplepodcastpress_activate() {
		$this->sppress_updatedb();
		$this->generate_buttons_css();
		$this->generate_options_css();
		$this->generate_barui_css();
		$this->generate_classic_css();
	}

	/**
	 * Deactivate the Simple Podcast Press plugin.
	 *
	 * @return void
	 */
	public function simplepodcastpress_deactivate() {
		$timestamp = wp_next_scheduled( 'simplepodcastpress_fetch' );
		wp_unschedule_event( $timestamp, 'hourly', 'simplepodcastpress_fetch' );
		$timestamp = wp_next_scheduled( 'simplepodcastpress_reviews_fetch' );
		wp_unschedule_event( $timestamp, 'daily', 'simplepodcastpress_reviews_fetch' );

		deactivate_plugins( 'spp-smart-podcast-player/spp-smart-podcast-player.php' );
		deactivate_plugins( 'spp-multi-itunes-addon/spp-multi-itunes-addon.php' );
		deactivate_plugins( 'spp-blog-talk-radio/spp-blog-talk-radio.php' );
	}
}//end class
?>